@@ -30,16 +30,16 @@ InitCancelCache(void)
     struct shmid_ds ds;
 
     if (sid < 0) {
-	syslog(LOG_WARNING, "InitCancelCache cannot allocate sysv shared memory");
+	logit(LOG_WARNING, "InitCancelCache cannot allocate sysv shared memory");
     } else {
 	CCacheAry = (hash_t *)shmat(sid, NULL, SHM_R|SHM_W);
 	if (shmctl(sid, IPC_STAT, &ds) < 0 || shmctl(sid, IPC_RMID, &ds) < 0) {
-	    syslog(LOG_CRIT, "sysv shmctl stat/rmid failed");
+	    logit(LOG_CRIT, "sysv shmctl stat/rmid failed");
 	    exit(1);
 	}
 	if (CCacheAry == (hash_t *)-1) {
 	    CCacheAry = NULL;
-	    syslog(LOG_CRIT, "sysv shared memory map failed");
+	    logit(LOG_CRIT, "sysv shared memory map failed");
 	    exit(1);
 	}
 	if (CCacheAry)
