@@ -638,7 +638,7 @@ CtlCancel(Connection *conn, Control *ctl, const char **cmd, const char *art, con
 		 * Get article contents, set isValidCancel to 1 or -1
 		 */
 		int hlen;
-		const char *data = GetOverRecord(ov, artNo, &hlen, NULL, NULL);
+		const char *data = GetOverRecord(ov, artNo, &hlen, NULL, NULL, NULL);
 		if (data != NULL) {
 		    int cxLen;
 		    const char *cxtrace = locateXTrace(data, hlen, &cxLen);
