@@ -46,10 +46,14 @@ struct Timer;
  * a spool thread before we back off to a higher d_Pri.
  */
 
-#define THREAD_QSIZE	4	/* for spool access		*/
+#define THREAD_QSIZE	16	/* for spool access		*/
 #define THREAD_PSIZE	2	/* for posting			*/
 #define THREAD_LIMSIZE	(((THREAD_QSIZE > THREAD_PSIZE) ? THREAD_QSIZE : THREAD_PSIZE) * 2)
 
+#ifndef INET6
+typedef struct sockaddr sockaddr_storage;
+#endif
+
 /*
  * ForkDesc - structure used to manage descriptors for select() threads
  */
@@ -67,8 +71,14 @@ typedef struct ForkDesc {
     int		d_Count;	/* active subthreads	*/
     int		d_Pri;
     void	*d_Data;
+    struct sockaddr_storage d_SaveRSin;
     char	*d_LocalSpool;	/* direct spool access	*/
-    struct sockaddr_in d_SaveRSin;
+    int		d_Cache;	/* is spooler to be locally cached */
+    int		d_CacheMin;	/* min article size to be cached */
+    int		d_CacheMax;	/* max article size to be cached */
+    int		d_CacheableTime;	/* oldest article to be cached */
+    double	d_ReadNewRatio;	/* read/new articles ratio to reach to cache a group */
+    double	d_CacheReadRatio;	/* cachehits/read articles ratio to reach to cache a group */
 #ifdef	DREADER_CLIENT_TIMING
     clock_t	d_utime;
     clock_t	d_stime;
@@ -79,6 +89,18 @@ typedef struct ForkDesc {
  * Authentication structures
  */
 
+/*
+ * The current output mode of the server to the client
+ */
+#define	DRBC_NONE	0
+#define	DRBC_ARTICLE	1
+#define	DRBC_HEAD	2
+#define	DRBC_BODY	3
+#define	DRBC_LIST	4
+#define	DRBC_XOVER	5
+#define	DRBC_XHDR	6
+#define DRBC_XXXX	7	/* The last of the DRBC_ defines */
+
 #define	OPT_NONE	0
 #define	OPT_VSERVER	1
 #define	OPT_GROUPS	2
@@ -96,12 +118,13 @@ typedef struct Vserver {		/* Define a virtual server	*/
     char vs_Org[128];
     char vs_AbuseTo[64];
     char vs_CryptPw[32];
-    long vs_Interface;
+    struct sockaddr_storage vs_Interface;
     char vs_AccessFile[128];
-    char vs_AccessDb[128];
     int vs_NoXrefHostUpdate;
     int vs_NoReadPath;
     char vs_Welcome[256];
+    char vs_Comments[256];
+    char vs_PostTrailer[256];
 } Vserver;
 
 typedef struct GroupDef {		/* Define a group access list	*/
@@ -121,8 +144,10 @@ typedef struct AuthDef {		/* Define an authentication type */
     char au_User[255];
     char au_Pass[255];
     char au_Realm[255];
+    char au_AddRealm[255];
     char au_NetRemote[255];
     char au_LDAP[255];
+    char au_Perl[255];
     char au_PAM[255];
     int au_Ident;
 } AuthDef;
@@ -144,8 +169,9 @@ typedef struct ReaderDef {		/* Define a reader group	*/
     int rd_MaxConnPerHost;
     int rd_MaxConnPerUser;
     int rd_MaxConnPerGroup;
+    int rd_MaxConnUniqueHostLimit;
     int rd_MaxConnPerVs;
-    int rd_RateLimit;
+    int rd_RateLimit[DRBC_XXXX];
     int rd_RateLimitTax;
     int rd_ByteLimit;
     int rd_PathComponents;
@@ -197,8 +223,8 @@ typedef struct AccessMap {		/* Define an access mapping	*/
  */
 
 typedef struct DnsReq {
-    struct sockaddr_in dr_LSin;		/* local interface		*/
-    struct sockaddr_in dr_RSin;		/* remote ip:port		*/
+    struct sockaddr_storage dr_LSin;	/* local interface		*/
+    struct sockaddr_storage dr_RSin;	/* remote ip:port		*/
     char dr_AuthUser[64];		/* authinfo user		*/
     char dr_AuthPass[16];		/* authinfo pass		*/
     int dr_ResultFlags;			/* see #defines below		*/
@@ -208,6 +234,13 @@ typedef struct DnsReq {
     double dr_PostBytes;		/* how many bytes posted 	*/
     int dr_PostCount;			/* number of articles posted	*/
     int dr_PostFailCount;		/* number of failed postings	*/
+    double dr_ByteCountArticle;		/* Bytes sent for art fetches	*/
+    double dr_ByteCountHead;		/* Bytes sent for head fetches	*/
+    double dr_ByteCountBody;		/* Bytes sent for body fetches	*/
+    double dr_ByteCountList;		/* Bytes sent for list fetches	*/
+    double dr_ByteCountXover;		/* Bytes sent for xover fetches	*/
+    double dr_ByteCountXhdr;		/* Bytes sent for xhdr fetches	*/
+    double dr_ByteCountOther;		/* Bytes sent for other fetches	*/
     time_t dr_SessionLength;		/* duration of session in secs	*/
 } DnsReq;
 
@@ -228,10 +261,16 @@ typedef struct DnsRes {
     int dr_PostFailCount;		/* how many posts failed	*/
     double dr_ByteCount;		/* how many bytes sent		*/
     int dr_GrpCount;			/* how many groups accessed	*/
+    double dr_ByteCountArticle;		/* Bytes sent for art fetches	*/
+    double dr_ByteCountHead;		/* Bytes sent for head fetches	*/
+    double dr_ByteCountBody;		/* Bytes sent for body fetches	*/
+    double dr_ByteCountList;		/* Bytes sent for list fetches	*/
+    double dr_ByteCountXover;		/* Bytes sent for xover fetches	*/
+    double dr_ByteCountXhdr;		/* Bytes sent for xhdr fetches	*/
+    double dr_ByteCountOther;		/* Bytes sent for other fetches	*/
     time_t dr_TimeStart;		/* Time connection was made	*/
     int dr_ReaderPid;			/* process ID of reader process	*/
-    struct in_addr dr_Addr;		/* remote IP address		*/
-    int dr_Port;			/* remote port for logging	*/
+    struct sockaddr_storage dr_Addr;	/* remote IP address and port	*/
     char dr_Host[64];			/* the remote hostname		*/
     char dr_ReaderName[DEFNAMELEN];	/* the reader group name	*/
     char dr_VServer[DEFNAMELEN];	/* the virtual server		*/
@@ -241,7 +280,7 @@ typedef struct DnsRes {
     int dr_DnsMismatch;			/* Fwd/Rev DNS don't match	*/
     int dr_ResultFlags;			/* see #defines above		*/
     int dr_StaticAuth;			/* don't change auth details	*/
-    int dr_CurrRateLimit;		/* current rate limit		*/
+    int dr_ConnCount;			/* for rate limit calculation	*/
     ReaderDef *dr_ReaderDef;		/* reader group	pointer		*/
     Vserver *dr_VServerDef;		/* vserver pointer		*/
     GroupDef *dr_GroupDef;		/* group list pointer		*/
@@ -327,8 +366,12 @@ typedef struct TimeRestrict {
 #define BLOCK_ERROR	1
 #define BLOCK_END	2
 
+#ifdef __linux__
+#define USE_AIO		1
+#define USE_LINUX_KAIO	0
+#else
 #define USE_AIO		0
-#define USE_LINUX_KAIO	1
+#endif
 
 typedef struct DirectFileAccess {
 #if USE_AIO
@@ -354,6 +397,11 @@ typedef struct DirectFileAccess {
  *	    THREAD_POST connection.
  */
 
+#define CACHE_OFF		0x0000
+#define CACHE_ON		0x0001
+#define CACHE_LAZY		0x0002
+#define CACHE_SCOREBOARD	0x0003
+
 typedef struct Connection {
     ForkDesc	*co_Desc;
     void	(*co_Func)(struct Connection *conn);
@@ -400,12 +448,16 @@ typedef struct Connection {
     time_t		co_LastServerLog;
     unsigned long	co_ServerByteCount;
     unsigned long	co_ServerArticleCount;
+    unsigned long	co_ServerArticleRequestedCount;
+    unsigned long	co_ServerArticleNotFoundErrorCount;
+    unsigned long	co_ServerArticleMiscErrorCount;
     double		co_ClientTotalByteCount;
     unsigned long	co_ClientTotalArticleCount;
     double		co_ClientGroupByteCount;
     unsigned long	co_ClientGroupArticleCount;
     unsigned long	co_ClientPostCount;
     unsigned long	co_ClientGroupCount;
+    int 		co_ByteCountType; /* Temp for by-type byte count */
 
     MBufHead	co_ArtBuf;	/* article buffer		*/
 } Connection;
@@ -426,6 +478,8 @@ typedef struct Connection {
 #define COF_ININIT	0x00100000	/* see server.c			*/
 #define COF_MAYNOTCLOSE 0x00200000	/* may not close the connection	*/
 #define	COF_READONLY	0x00400000	/* send ``mode readonly''	*/
+#define	COF_LOGIN	0x00800000	/* send ``authinfo'' to log in	*/
+#define	COF_POSTTOOBIG	0x01000000	/* posted article is too big	*/
 
 #define MBUF_SIZE	1024
 #define MBUF_HIWAT	(MBUF_SIZE*2-256)
@@ -457,9 +511,12 @@ typedef struct ServReq {
     Connection *sr_CConn;	/* client making request	*/
     Connection *sr_SConn;	/* server handling request	*/
     char	*sr_Group;	/* request related to group	*/
+    int		sr_GrpIter;	/* Group iteration, for cachehits hash */
+    int		sr_endNo;	/* last group article, for cachehits */
     char	*sr_MsgId;	/* request related to messageid	*/
     time_t	sr_Time;	/* time of request for timeout calc	*/
     FILE	*sr_Cache;	/* cache write (locked for duration)	*/
+    int		sr_TimeRcvd;	/* article received time (NNRetrieveHead) */
     int		sr_Rolodex;	/* see server.c		*/
     int		sr_NoPass;	/* see server.c		*/
 } ServReq;
@@ -525,6 +582,7 @@ typedef struct OverInfo {
     OverHead	*ov_Head;
     int		ov_Size;
     int		ov_Iter;
+    int		ov_endNo;
 } OverInfo;
 
 typedef struct ArtNumAss {
