@@ -167,7 +167,7 @@ void Sig_AIO(int signo, siginfo_t *info, void *context)
     Connection *conn = (Connection*) info->si_value.sival_ptr ;
     DirectFileAccess *dfa = conn->co_DirectFA;
     struct aiocb *aio=dfa->dfa_AIOcb;
-    const aiocb_t *const aiolist[1] = {aio} ;
+    const struct aiocb_t *const aiolist[1] = {aio};
     int ret;
 
     /* check IO */
@@ -271,7 +271,7 @@ NNSLA_Loop(Connection *conn)
                       if (fflush(sreq->sr_Cache) || ferror(sreq->sr_Cache))
                           AbortCache(fileno(sreq->sr_Cache), sreq->sr_MsgId, 0);
                       else
-                          CommitCache(fileno(sreq->sr_Cache), sreq->sr_MsgId, 0);
+                          CommitCache(conn, 0);
                       fclose(sreq->sr_Cache) ;
                       sreq->sr_Cache = NULL ;
                   }
