@@ -78,15 +78,17 @@ void
 NNTPIHave(Connection *conn, char **pptr)
 {
     if (conn->co_Flags & COF_SERVER) {
-	const char *msgid = MsgId(parseword(pptr, ""), NULL);
+	const char *origmsgid = "<>";
+	const char *msgid = MsgId(parseword(pptr, ""), &origmsgid);
 
 	if (strcmp(msgid, "<>") == 0) {
-	    MBLogPrintf(conn, &conn->co_TMBuf, "435 %s Bad Message-ID\r\n", msgid);
+	    MBLogPrintf(conn, &conn->co_TMBuf, "435 %s Bad Message-ID\r\n",
+								origmsgid);
 	    NNCommand(conn);
 	    return;
 	}
 
-	conn->co_Flags &= ~(COF_IHAVE|COF_POST);
+	conn->co_Flags &= ~(COF_IHAVE|COF_POST|COF_POSTTOOBIG);
 	conn->co_Flags |= COF_IHAVE;
 	MBLogPrintf(conn, &conn->co_TMBuf, "335\r\n");
 	NNAcceptArticleStart(conn, msgid);
@@ -99,11 +101,12 @@ NNTPIHave(Connection *conn, char **pptr)
 void 
 NNTPCheck(Connection *conn, char **pptr)
 {
-    const char *msgid = MsgId(parseword(pptr, " \t"), NULL);
+    const char *origmsgid = "<>";
+    const char *msgid = MsgId(parseword(pptr, " \t"), &origmsgid);
 
     if (conn->co_Flags & COF_SERVER) {
 	if (strcmp(msgid, "<>") == 0)
-	    MBLogPrintf(conn, &conn->co_TMBuf, "438 %s\r\n", msgid);
+	    MBLogPrintf(conn, &conn->co_TMBuf, "438 %s\r\n", origmsgid);
 	else
 	    MBLogPrintf(conn, &conn->co_TMBuf, "238 %s\r\n", msgid);
     } else {
@@ -115,12 +118,17 @@ NNTPCheck(Connection *conn, char **pptr)
 void 
 NNTPTakeThis(Connection *conn, char **pptr)
 {
+    const char *origmsgid = "<>";
     const char *msgid = MsgId(parseword(pptr, ""), NULL);
 
-    conn->co_Flags &= ~(COF_IHAVE|COF_POST);
+    conn->co_Flags &= ~(COF_IHAVE|COF_POST|COF_POSTTOOBIG);
 
     if (conn->co_Flags & COF_SERVER) {
-	NNAcceptArticleStart(conn, msgid);
+	if (strcmp(msgid, "<>") == 0) {
+	    MBLogPrintf(conn, &conn->co_TMBuf, "439 %s\r\n", origmsgid);
+	    NNCommand(conn);
+	} else
+	    NNAcceptArticleStart(conn, msgid);
     } else {
 	MBLogPrintf(conn, &conn->co_TMBuf, "500 What?\r\n");
 	NNCommand(conn);
@@ -247,15 +255,17 @@ NNAcceptArticle(Connection *conn)
 	 * whole body so we can pgp-verify it
 	 */
 
-	if ((conn->co_Flags & COF_INHEADER) || (conn->co_Flags & (COF_POST|COF_WASCONTROL))) {
+	conn->co_Auth.dr_PostBytes += len;
+	conn->co_ByteCounter += len;
+
+	if (DOpts.ReaderMaxArtSize > 0 && conn->co_ArtBuf.mh_Bytes > DOpts.ReaderMaxArtSize)
+	    conn->co_Flags |= COF_POSTTOOBIG;
+
+	else if ((conn->co_Flags & COF_INHEADER) || (conn->co_Flags & (COF_POST|COF_WASCONTROL))) {
 	    if (len > 0)
 		buf[len-1] = '\n';
-	    conn->co_ByteCounter += len;
 	    MBWrite(&conn->co_ArtBuf, buf, len);
-	} else if (!(conn->co_Flags & COF_HEADFEED) && (conn->co_Flags & COF_SERVER)) {
-	    conn->co_ByteCounter += len;
 	}
-	conn->co_Auth.dr_PostBytes += len;
     }
     if (len <= 0) {
 	if (len < 0)
