@@ -39,12 +39,12 @@ Prototype void NNFeedOverview(Connection *conn);
 Prototype void FlushOverCache(void);
 Prototype OverInfo *GetOverInfo(const char *group);
 Prototype void PutOverInfo(OverInfo *ov);
-Prototype const char *GetOverRecord(OverInfo *ov, int artno, int *plen, int *aleno, TimeRestrict *tr);
+Prototype const char *GetOverRecord(OverInfo *ov, int artno, int *plen, int *aleno, TimeRestrict *tr, int *TimeRcvd);
 Prototype OverInfo *FindCanceledMsg(const char *group, const char *msgid, int *partNo, int *pvalidGroups);
 Prototype int CancelOverArt(OverInfo *ov, int artNo);
 
 Prototype int NNTestOverview(Connection *conn);
-Prototype const char *NNRetrieveHead(Connection *conn, int *povlen, const char **msgid);
+Prototype const char *NNRetrieveHead(Connection *conn, int *povlen, const char **pmsgid, int *TimeRcvd, int *grpIter, int *endNo);
 
 const OverArt *GetOverArt(OverInfo *ov, int artNo, off_t *ppos);
 void AssignArticleNo(Connection *conn, ArtNumAss **pan, const char *group, const char *xref, int approved, const char *art, int artLen, const char *msgid);
@@ -122,7 +122,15 @@ NNFeedOverview(Connection *conn)
 	}
     }
 
-    if (newsgroups == NULL || msgid == NULL || subject == NULL || 
+    if (conn->co_Flags & COF_POSTTOOBIG) {
+	conn->co_Auth.dr_PostFailCount++;
+	if (conn->co_Flags & COF_IHAVE) {
+	    MBLogPrintf(conn, &conn->co_TMBuf, "437 Rejected, too big\r\n");
+	} else {
+	    MBLogPrintf(conn, &conn->co_TMBuf, "439 %s too big\r\n",  conn->co_IHaveMsgId);
+	}
+	conn->co_Flags &= ~COF_POSTTOOBIG;
+    } else if (newsgroups == NULL || msgid == NULL || subject == NULL || 
 	date == NULL || strcmp(msgid, "<>") == 0
     ) {
 	/*
@@ -147,7 +155,7 @@ NNFeedOverview(Connection *conn)
 	snprintf(logbuf, sizeof(logbuf), "%s cancel cache", msgid);
 	LogCmd(conn, '-', logbuf);
 	if (DRIncomingLogPat != NULL)
-	    LogIncoming("%s - %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+	    LogIncoming("%s - %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	if (conn->co_Flags & COF_IHAVE) {
 	    MBLogPrintf(conn, &conn->co_TMBuf, "437 Article Already Cancelled\r\n");
 	} else {
@@ -207,6 +215,12 @@ NNFeedOverview(Connection *conn)
 	    c = *ngroup;
 	    *ngroup = 0;
 
+	    /*
+	     * Skip groups with names that are too long
+	     */
+	    if (ngroup - group > MAXGNAME)
+		continue;
+
 	    /* Strip trailing space or tab from group name */
 	    whspptr = strpbrk(group, " \t");
 	    if (whspptr) {
@@ -250,6 +264,12 @@ NNFeedOverview(Connection *conn)
 	    c = *ngroup;
 	    *ngroup = 0;
 
+	    /*
+	     * Skip groups with names that are too long
+	     */
+	    if (ngroup - group > MAXGNAME)
+		continue;
+
 	    /* Strip trailing space or tab from group name */
 	    whspptr = strpbrk(group, " \t");
 	    if (whspptr) {
@@ -278,7 +298,7 @@ NNFeedOverview(Connection *conn)
 		printf("Control message: %s %s\n", msgid, newsgroups);
 	    LogCmd(conn, 'c', control);
 	    if (DRIncomingLogPat != NULL)
-		LogIncoming("%s c %s %s\n", conn->co_Auth.dr_Host,
+		LogIncoming("%s c %s %s", conn->co_Auth.dr_Host,
 							msgid, control);
 	    ExecuteControl(conn, control, art, artLen);
 	}
@@ -329,7 +349,7 @@ AssignArticleNo(Connection *conn, ArtNumAss **pan, const char *group, const char
 	 */
 	if ((rec = KPDBReadRecord(KDBActive, group, KP_LOCK, &recLen)) == NULL) {
 	    if (ValidGroupName(group) < 0) {
-		/* syslog(LOG_ERR, "group %s illegal", group); */
+		/* logit(LOG_ERR, "group %s illegal", group); */
 	    } else {
 		char tsBuf[64];
 
@@ -344,7 +364,7 @@ AssignArticleNo(Connection *conn, ArtNumAss **pan, const char *group, const char
 		snprintf(logbuf, sizeof(logbuf), "%s %s not in dactive", msgid, group);
 		LogCmd(conn, '-', logbuf);
 		if (DRIncomingLogPat != NULL)
-		    LogIncoming("%s - %s%s\n", conn->co_Auth.dr_Host,
+		    LogIncoming("%s - %s%s", conn->co_Auth.dr_Host,
 							"", logbuf);
 		return;
 	    }
@@ -357,7 +377,7 @@ AssignArticleNo(Connection *conn, ArtNumAss **pan, const char *group, const char
 	    snprintf(logbuf, sizeof(logbuf), "%s %s not in dactive", msgid, group);
 	    LogCmd(conn, '-', logbuf);
 	    if (DRIncomingLogPat != NULL)
-		LogIncoming("%s - %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+		LogIncoming("%s - %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	    return;
 	}
     }
@@ -377,7 +397,7 @@ AssignArticleNo(Connection *conn, ArtNumAss **pan, const char *group, const char
 		snprintf(logbuf, sizeof(logbuf), "%s %s unapproved", msgid, group);
 		LogCmd(conn, '-', logbuf);
 		if (DRIncomingLogPat != NULL)
-		    LogIncoming("%s - %s%s\n", conn->co_Auth.dr_Host,
+		    LogIncoming("%s - %s%s", conn->co_Auth.dr_Host,
 							"", logbuf);
 		return;
 	    }
@@ -418,7 +438,7 @@ AssignArticleNo(Connection *conn, ArtNumAss **pan, const char *group, const char
 	snprintf(logbuf, sizeof(logbuf), "%s %s no valid xref present in slave mode", msgid, group);
 	LogCmd(conn, '-', logbuf);
 	if (DRIncomingLogPat != NULL)
-	    LogIncoming("%s - %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+	    LogIncoming("%s - %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	return;
     }
 
@@ -537,20 +557,20 @@ WriteOverview(Connection *conn, ArtNumAss *an, const char *group, const char *xr
 	struct iovec iov[3];
 
 	if ((ov = GetOverInfo(group)) == NULL) {
-	    syslog(LOG_ERR, "Error in GetOverInfo(%s) msgid=%s", group, msgid);
+	    logit(LOG_ERR, "Error in GetOverInfo(%s) msgid=%s", group, msgid);
 	    LogCmd(conn, '-', logbuf);
 	    if (DRIncomingLogPat != NULL)
-		LogIncoming("%s - %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+		LogIncoming("%s - %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	    MBLogPrintf(conn, &conn->co_TMBuf, "400 Error writing data\r\n");
 	    NNTerminate(conn);
 	    return(1);
 	}
 
 	if (MakeOverHFile(ov, artNo, 1) == NULL) {
-	    syslog(LOG_ERR, "Error in MakeOverHFile() msgid=%s", msgid);
+	    logit(LOG_ERR, "Error in MakeOverHFile() msgid=%s", msgid);
 	    LogCmd(conn, '-', logbuf);
 	    if (DRIncomingLogPat != NULL)
-		LogIncoming("%s - %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+		LogIncoming("%s - %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	    MBLogPrintf(conn, &conn->co_TMBuf, "400 Error writing data\r\n");
 	    NNTerminate(conn);
 	    return(1);
@@ -612,7 +632,7 @@ WriteOverview(Connection *conn, ArtNumAss *an, const char *group, const char *xr
 	     */
 	    LogCmd(conn, 'd', logbuf);
 	    if (DRIncomingLogPat != NULL)
-		LogIncoming("%s d %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+		LogIncoming("%s d %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	    ; /* EMPTY */
 	} else if (DOpts.ReaderXOverMode == 0) {
 	    /*
@@ -620,10 +640,10 @@ WriteOverview(Connection *conn, ArtNumAss *an, const char *group, const char *xr
 	     * supported by the reader but may eventually be supported
 	     * in 100% nntp-cache mode if/when we develop it.
 	     */
-	    syslog(LOG_INFO, "ReaderXOverMode0 (%d:%s)", artNo, msgid);
+	    logit(LOG_INFO, "ReaderXOverMode0 (%d:%s)", artNo, msgid);
 	    LogCmd(conn, 'm', logbuf);
 	    if (DRIncomingLogPat != NULL)
-		LogIncoming("%s m %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+		LogIncoming("%s m %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	    ; /* EMPTY */
 	} else if (
 	    DOpts.ReaderXOverMode == 2 ||
@@ -656,13 +676,13 @@ WriteOverview(Connection *conn, ArtNumAss *an, const char *group, const char *xr
 	    hflock(ov->ov_OFd, 0, XLOCK_UN);
 	    LogCmd(conn, '+', logbuf);
 	    if (DRIncomingLogPat != NULL)
-		LogIncoming("%s + %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+		LogIncoming("%s + %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	} else {
 	    ftruncate(ov->ov_HCache->od_HFd, pos);
 	    logit(LOG_ERR, "error writing overview data file for %s", group);
 	    LogCmd(conn, '-', logbuf);
 	    if (DRIncomingLogPat != NULL)
-		LogIncoming("%s - %s%s\n", conn->co_Auth.dr_Host, "", logbuf);
+		LogIncoming("%s - %s%s", conn->co_Auth.dr_Host, "", logbuf);
 	    MBLogPrintf(conn, &conn->co_TMBuf, "400 Error writing data\r\n");
 	    NNTerminate(conn);
 	    err = 1;
@@ -714,6 +734,7 @@ GetOverInfo(const char *group)
 	struct stat st;
 	char *path;
 	int iter = 0;
+	int endNo = 0;
 
 	bzero(&st, sizeof(st));
 
@@ -755,6 +776,7 @@ GetOverInfo(const char *group)
 		return(NULL);
 	    }
 	    iter = strtol(KPDBGetField(rec, recLen, "ITER", NULL, "0"), NULL, 10);
+	    endNo = strtol(KPDBGetField(rec, recLen, "NE", NULL, "-1"), NULL, 10);
 	    KPDBUnlock(KDBActive, rec);
 	}
 
@@ -802,12 +824,12 @@ again:
 	     */
 	    r = (read(ov->ov_OFd, &oh, sizeof(oh)) != sizeof(oh));
 	    if (r == 0 && oh.oh_ByteOrder != OH_BYTEORDER) {
-		syslog(LOG_CRIT, "Incorrect overview byte order for %s (%s)",
+		logit(LOG_CRIT, "Incorrect overview byte order for %s (%s)",
 								group, path);
 		r = -1;
 	    }
 	    if (r == 0 && oh.oh_Version > OH_VERSION) {
-		syslog(LOG_CRIT, "Incorrect overview version for %s (%s)",
+		logit(LOG_CRIT, "Incorrect overview version for %s (%s)",
 								group, path);
 		r = -1;
 	    }
@@ -845,7 +867,7 @@ again:
 		    oh.oh_ByteOrder = OH_BYTEORDER;
 		    oh.oh_HeadSize = sizeof(oh);
 		    oh.oh_MaxArts = save.oe_InitArts;
-		    strncpy(oh.oh_Gname, group, sizeof(oh.oh_Gname));
+		    strncpy(oh.oh_Gname, group, sizeof(oh.oh_Gname) - 1);
 		    write(ov->ov_OFd, &oh, sizeof(oh));
 		    fsync(ov->ov_OFd);
 		}
@@ -864,6 +886,7 @@ again:
 		KPDBWrite(KDBActive, group, "ITER", tsBuf, 0);   
 	    }
 	    ov->ov_Iter = iter;
+	    ov->ov_endNo = endNo;
 	    ov->ov_Size = st.st_size;
 	    ov->ov_MaxArts = (st.st_size - oh.oh_HeadSize) / sizeof(OverArt);
 	    ov->ov_Head = xmap(NULL, ov->ov_Size, PROT_READ, MAP_SHARED, ov->ov_OFd, 0);
@@ -873,7 +896,7 @@ again:
 		FreeOverInfo(ov);
 		ov = NULL;
 	    } else {
-		xadvise(ov->ov_Head, ov->ov_Size, XADV_WILLNEED);
+		xadvise(ov->ov_Head, ov->ov_Size, XADV_SEQUENTIAL);
 		++NumOverInfo;
 		pov = &OvHash[shash(group) & OVHMASK];
 		ov->ov_Next = *pov;
@@ -910,7 +933,7 @@ GetOverArt(OverInfo *ov, int artno, off_t *ppos)
 }
 
 const char *
-GetOverRecord(OverInfo *ov, int artno, int *plen, int *alen, TimeRestrict *tr)
+GetOverRecord(OverInfo *ov, int artno, int *plen, int *alen, TimeRestrict *tr, int *TimeRcvd)
 {
     int hvpos;
     int xpos;
@@ -929,6 +952,9 @@ GetOverRecord(OverInfo *ov, int artno, int *plen, int *alen, TimeRestrict *tr)
     if (tr && tr->tr_Time > oa->oa_TimeRcvd)
 	return(NULL);
 
+    if (TimeRcvd)
+	*TimeRcvd = oa->oa_TimeRcvd;
+
     if (alen)
 	*alen = oa->oa_ArtSize;
 
@@ -1006,11 +1032,11 @@ GetOverRecord(OverInfo *ov, int artno, int *plen, int *alen, TimeRestrict *tr)
 	if (*r == 0)
 	    return(NULL);
 	if (xpos < hvpos && r[-1] != 0) {
-	    syslog(LOG_ERR, "corrupt overview entry for %s:%d", ov->ov_Group, artno);
+	    logit(LOG_ERR, "corrupt overview entry for %s:%d", ov->ov_Group, artno);
 	    return(NULL);
 	}
 	if (r[oa->oa_Bytes] != 0) {
-	    syslog(LOG_ERR, "corrupt overview entry for %s:%d", ov->ov_Group, artno);
+	    logit(LOG_ERR, "corrupt overview entry for %s:%d", ov->ov_Group, artno);
 	    return(NULL);
 	}
 	return(r);
@@ -1215,7 +1241,7 @@ NNTestOverview(Connection *conn)
     int r = -1;
 
     if ((ov = GetOverInfo(conn->co_GroupName)) != NULL) {
-	if (GetOverRecord(ov, conn->co_ArtNo, NULL, NULL, NULL) != NULL)
+	if (GetOverRecord(ov, conn->co_ArtNo, NULL, NULL, NULL, NULL) != NULL)
 	    r = 0;
 	PutOverInfo(ov);
     }
@@ -1223,7 +1249,7 @@ NNTestOverview(Connection *conn)
 }
 
 const char *
-NNRetrieveHead(Connection *conn, int *povlen, const char **pmsgid)
+NNRetrieveHead(Connection *conn, int *povlen, const char **pmsgid, int *TimeRcvd, int *grpIter, int *endNo)
 {
     OverInfo *ov;
     const char *res = NULL;
@@ -1237,7 +1263,11 @@ NNRetrieveHead(Connection *conn, int *povlen, const char **pmsgid)
     }
 
     if ((ov = GetOverInfo(conn->co_GroupName)) != NULL) {
-	if ((res = GetOverRecord(ov, conn->co_ArtNo, povlen, NULL, NULL)) != NULL) {
+	if (grpIter != NULL)
+	    *grpIter = ov->ov_Iter;
+	if (endNo != NULL)
+	    *endNo = ov->ov_endNo;
+	if ((res = GetOverRecord(ov, conn->co_ArtNo, povlen, NULL, NULL, TimeRcvd)) != NULL) {
 
 	    const char *scan = res;
 	    int scanLen = *povlen;
