@@ -505,15 +505,15 @@ NNTPListGroup(Connection *conn, char **pptr)
 	    KPDBUnlock(KDBActive, rec);
 	    if (conn->co_ListEndNo - conn->co_ListBegNo > ov->ov_Head->oh_MaxArts)
 		conn->co_ListBegNo = conn->co_ListEndNo - ov->ov_Head->oh_MaxArts;
-	    MBLogPrintf(conn, &conn->co_TMBuf, "211 Article list follows\n");
+	    MBLogPrintf(conn, &conn->co_TMBuf, "211 Article list follows\r\n");
 	    NNListGroup(conn);
 	} else {
-	    MBLogPrintf(conn, &conn->co_TMBuf, "411 No such group %s\n", conn->co_GroupName);
+	    MBLogPrintf(conn, &conn->co_TMBuf, "411 No such group %s\r\n", conn->co_GroupName);
 	    NNCommand(conn);
 	}
 	PutOverInfo(ov);
     } else {
-	MBLogPrintf(conn, &conn->co_TMBuf, "481 No group specified\n");
+	MBLogPrintf(conn, &conn->co_TMBuf, "481 No group specified\r\n");
 	NNCommand(conn);
     }
 }
@@ -533,7 +533,7 @@ NNListGroup(Connection *conn)
 	int resLen;
         int artSize;
 
-        if (GetOverRecord(ov, conn->co_ListBegNo, &resLen, &artSize, NULL) != NULL) {
+        if (GetOverRecord(ov, conn->co_ListBegNo, &resLen, &artSize, NULL, NULL) != NULL) {
 	    MBPrintf(&conn->co_TMBuf, "%d\r\n", conn->co_ListBegNo);
 	}
 	++conn->co_ListBegNo;
@@ -742,16 +742,24 @@ ListActiveGroups(Connection *conn, char *pat)
     int recOff;
     GroupList *patlist = NULL;
     GroupList *gl = NULL;
+    GroupList *allowedgroups = conn->co_Auth.dr_GroupDef->gr_Groups;
     char *ng;
  
     /*
      * Create a list of the patterns
      */
     for (ng = strtok(pat, ","); ng; ng = strtok(NULL, ",")) {
-	GroupList *tgl = zalloc(&conn->co_MemPool, sizeof(GroupList));
+	GroupList *tgl;
+	int wild = (strchr(ng, '*') || strchr(ng, '?'));
+
+	if (allowedgroups != NULL && !wild &&
+					GroupFindWild(ng, allowedgroups) == 0)
+	    continue;
+
+	tgl = zalloc(&conn->co_MemPool, sizeof(GroupList));
 	tgl->group = zallocStr(&conn->co_MemPool, ng);
 	tgl->next = NULL;
-	if (strchr(ng, '*') || strchr(ng, '?')) {
+	if (wild) {
 	    tgl->next = patlist;
 	    patlist = tgl;
 	} else {
@@ -772,12 +780,18 @@ ListActiveGroups(Connection *conn, char *pat)
 	    grpbuf[groupLen] = 0;
 	    for (gl = patlist; gl; gl = gl->next) {
 		if (WildCmp(gl->group, grpbuf) == 0) {
-		    GroupList *tgl = zalloc(&conn->co_MemPool, sizeof(GroupList));
+		    GroupList *tgl;
+
+		    if (allowedgroups != NULL &&
+				GroupFindWild(grpbuf, allowedgroups) == 0) {
+			continue;
+		    }
+
+		    tgl = zalloc(&conn->co_MemPool, sizeof(GroupList));
 		    tgl->group = zallocStr(&conn->co_MemPool, grpbuf);
 		    tgl->next = NULL;
 		    tgl->next = conn->co_ListCacheGroups;
 		    conn->co_ListCacheGroups = tgl;
-		    /* printf("NG:%s\n", grpbuf); */
 		}
 	    }
 	}
