@@ -53,7 +53,11 @@ void ValidateTcpBufferSize(int *psize);
 void deleteDnsResHash(DnsRes *dres, ForkDesc *desc);
 int getReaderForkSlot(void);
 void delReaderForkSlot(pid_t pid);
+#ifdef INET6
+void SendDns(int fd, struct sockaddr_storage *rsin, DnsRes *dres);
+#else
 void SendDns(int fd, struct sockaddr_in *rsin, DnsRes *dres);
+#endif
 void FinishRetain(int what);
 void DoListen(ForkDesc *desc, time_t t);
 void DoCommand(int fd);
@@ -115,12 +119,6 @@ pid_t *ReaderPids;
 MemPool *DnsResPool;
 DnsRes  *DnsResHash[DRHSIZE];
 
-typedef struct BindList {
-    struct BindList *bl_Next;
-    char	*bl_Port;
-    char 	*bl_Host;
-} BindList;
-
 BindList *BindBase = NULL;
 
 typedef struct PendingClose {
@@ -173,7 +171,7 @@ MemPool	*ReMemPool;
 void
 Usage(void)
 {
-    fprintf(stderr, "Usage: dreaderd -p pathhost [-A admin] [-B bindip[:port] [-b fd]\n");
+    fprintf(stderr, "Usage: dreaderd -p pathhost [-A admin] [-B bindip[:port] [-b fd][-C file]\n");
     fprintf(stderr, "                [-c delaysecs] [-d [debugval] [-D dnsprocs ] [-F feedprocs]\n");
     fprintf(stderr, "                [-f val] [-h hostname ] [-M readerprocs ] [-N readthreads]\n");
     fprintf(stderr, "                [-P [bindip:]port ] [-R rxbufsize ]\n");
@@ -185,6 +183,7 @@ Usage(void)
     fprintf(stderr, "   -A admin         Set the default newsadmin email for this host:\n");
     fprintf(stderr, "   -B bindip[:port] Bind to a specific interface\n");
     fprintf(stderr, "   -b fd            Provide an open socket to listen on\n");
+    fprintf(stderr, "	-C file		specify diablo.config to use\n");
     fprintf(stderr, "   -c delaysecs     Delay closing of failed connections\n");
     fprintf(stderr, "   -d [debugval]    Enable debugging\n");
     fprintf(stderr, "   -D dnsprocs      Specify the number of DNS processes to start\n");
@@ -245,12 +244,16 @@ main(int ac, char **av)
 
 		if (*ptr == 0)
 		    ptr = av[++i];
-		if (strrchr(ptr, ':')) {
-		    bl->bl_Port = strrchr(ptr, ':') + 1;
-		    bl->bl_Host = ptr;
-		    *strrchr(bl->bl_Host, ':') = 0;
+		if (*ptr == '[') {
+		    char *p = strchr(ptr, ']');
+		    bl->bl_Host = strdup(SanitiseAddr(ptr));
+		    if (p != NULL && (p = strrchr(p, ':')) != NULL)
+			bl->bl_Port = strdup(p + 1);
+		} else if (strrchr(ptr, ':')) {
+		    bl->bl_Port = strdup(strrchr(ptr, ':') + 1);
+		    bl->bl_Host = strdup(SanitiseAddr(ptr));
 		} else {
-		    bl->bl_Host = ptr;
+		    bl->bl_Host = strdup(SanitiseAddr(ptr));
 		}
 		bl->bl_Next = BindBase;
 		BindBase = bl;
@@ -296,12 +299,16 @@ main(int ac, char **av)
 
 		if (*ptr == 0)
 		    ptr = av[++i];
-		if (strrchr(ptr, ':')) {
-		    bl->bl_Port = strrchr(ptr, ':') + 1;
-		    bl->bl_Host = ptr;
-		    *strrchr(bl->bl_Host, ':') = 0;
+		if (*ptr == '[') {
+		    char *p = strchr(ptr, ']');
+		    bl->bl_Host = strdup(SanitiseAddr(ptr));
+		    if (p != NULL && (p = strrchr(p, ':')) != NULL)
+			bl->bl_Port = strdup(p + 1);
+		} else if (strrchr(ptr, ':')) {
+		    bl->bl_Port = strdup(strrchr(ptr, ':') + 1);
+		    bl->bl_Host = strdup(SanitiseAddr(ptr));
 		} else {
-		    bl->bl_Port = ptr;
+		    bl->bl_Port = strdup(ptr);
 		}
 		bl->bl_Next = BindBase;
 		BindBase = bl;
@@ -386,6 +393,16 @@ main(int ac, char **av)
 
     MyGroupHome = strdup(PatExpand(GroupHomePat));
 
+    if (BindBase == NULL) {
+	BindList *bl = calloc(sizeof(BindList), 1);
+
+	bl->bl_Host = DOpts.ReaderBindHost;
+	bl->bl_Port = DOpts.ReaderPort;
+
+	bl->bl_Next = BindBase;
+	BindBase = bl;
+    }
+
     /*
      * To stdout, not an error, no arguments == user requesting 
      * options list.  We can do this because the -p option is mandatory.
@@ -403,9 +420,36 @@ main(int ac, char **av)
 	BindList *bl;
 
 	for (bl = BindBase; bl; bl = bl->bl_Next) {
-	    int lfd = socket(AF_INET, SOCK_STREAM, 0);
+	    int lfd;
+
+#ifdef INET6
+	    int error;
+	    struct addrinfo hints;
+	    struct addrinfo *res;
+
+	    /*
+	     * Open a wildcard listening socket
+	     */
+	    memset(&hints, 0, sizeof(hints));
+	    hints.ai_flags = AI_PASSIVE;
+	    hints.ai_family = PF_UNSPEC;
+	    hints.ai_socktype = SOCK_STREAM;
+	    error = getaddrinfo(bl->bl_Host, bl->bl_Port, &hints, &res);
+	    if (error == EAI_NODATA) {
+		hints.ai_flags = 0;
+		error = getaddrinfo(bl->bl_Host, 0, &hints, &res);
+	    }
+	    if (error != 0) {
+		fprintf(stderr, "getaddrinfo: %s:%s: %s\n",
+					bl->bl_Host ? bl->bl_Host : "ALL",
+					bl->bl_Port, gai_strerror(error));
+		exit(1);
+	    }
+	    lfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+#else
 	    struct sockaddr_in lsin;
 
+	    lfd = socket(AF_INET, SOCK_STREAM, 0);
 	    bzero(&lsin, sizeof(lsin));
 	    if (bl->bl_Port == NULL)
 		bl->bl_Port = DOpts.ReaderPort;
@@ -438,6 +482,7 @@ main(int ac, char **av)
 		    }
 		}
 	    }
+#endif
 	    {
 		int on = 1;
 
@@ -447,11 +492,27 @@ main(int ac, char **av)
 	    setsockopt(lfd, SOL_SOCKET, SO_SNDBUF, (void *)&TxBufSize, sizeof(int));
 	    setsockopt(lfd, SOL_SOCKET, SO_RCVBUF, (void *)&RxBufSize, sizeof(int));
 
+#ifdef INET6
+	    if (bind(lfd, res->ai_addr, res->ai_addrlen) < 0) {
+		perror("bind");
+		exit(1);
+	    }
+	    freeaddrinfo(res);
+#else
 	    if (bind(lfd, (struct sockaddr *)&lsin, sizeof(lsin)) < 0) {
 		perror("bind");
 		exit(1);
 	    }
+#endif
 	    listen(lfd, 256);
+	    if (bl->bl_Host != NULL && strchr(bl->bl_Host, ':') != NULL)
+		logit(LOG_INFO, "Listening on [%s]:%s\n",
+					bl->bl_Host ? bl->bl_Host : "ALL",
+					bl->bl_Port);
+	    else
+		logit(LOG_INFO, "Listening on %s:%s\n",
+					bl->bl_Host ? bl->bl_Host : "ALL",
+					bl->bl_Port);
 	    AddThread("acceptor", lfd, -1, THREAD_LISTEN, -1, 0);
 	}
     } else {
@@ -514,27 +575,6 @@ main(int ac, char **av)
 #endif
     }
 
-    /*
-     * Save PID
-     */
-
-    {
-	int fd;
-	char buf[32];
-
-	sprintf(buf, "%d\n", (int)getpid());
-
-	PidFile = malloc(strlen(PatExpand(LogHomePat)) + 32);
-	sprintf(PidFile, "%s/dreaderd.pid", PatExpand(LogHomePat));
-	remove(PidFile);
-	if ((fd = open(PidFile, O_RDWR|O_CREAT|O_TRUNC, 0644)) < 0) {
-	    fprintf(stderr, "unable to create %s\n", PidFile);
-	    exit(1);
-	}
-	write(fd, buf, strlen(buf));
-	close(fd);
-    }
-
     /*
      * Ignore SIGPIPE, in case a write() fails we do not
      * want to segfault.
@@ -604,9 +644,30 @@ main(int ac, char **av)
      * open syslog
      */
 
-    openlog("dreaderd", (DebugOpt ? LOG_PERROR : 0) | LOG_NDELAY | LOG_PID, LOG_NEWS);
+    OpenLog("dreaderd", (DebugOpt ? LOG_PERROR : 0) | LOG_NDELAY | LOG_PID);
     SysLogDesc = "dreaderd";
 
+    /*
+     * Save PID
+     */
+
+    {
+	int fd;
+	char buf[32];
+
+	sprintf(buf, "%d\n", (int)getpid());
+
+	PidFile = malloc(strlen(PatExpand(LogHomePat)) + 32);
+	sprintf(PidFile, "%s/dreaderd.pid", PatExpand(LogHomePat));
+	remove(PidFile);
+	if ((fd = open(PidFile, O_RDWR|O_CREAT|O_TRUNC, 0644)) < 0) {
+	    logit(LOG_ERR, "unable to create %s\n", PidFile);
+	    exit(1);
+	}
+	write(fd, buf, strlen(buf));
+	close(fd);
+    }
+
 #ifdef	DREADER_CLIENT_TIMING
     ticksPerSecond=sysconf(_SC_CLK_TCK);
 #endif
@@ -762,7 +823,7 @@ main(int ac, char **av)
 			--NumDnsForks;
 			if (desc->d_FdPend >= 0) {
 			    logit(LOG_NOTICE, "pending descriptor from %s on dead DNS pid closed",
-				inet_ntoa(desc->d_SaveRSin.sin_addr)
+				NetAddrToSt(0, (struct sockaddr *)&desc->d_SaveRSin, 1, 1, 1)
 			    );
 			    close(desc->d_FdPend);
 			    desc->d_FdPend = -1;
@@ -877,11 +938,17 @@ main(int ac, char **av)
 				    TotalSpoolArticles += dres.dr_ArtCount;
 				    TotalSpoolBytes += dres.dr_ByteCount;
 				} else {
+#ifdef	INET6
+				    struct sockaddr_storage rsin;
+
+				    memcpy(&rsin, &dres.dr_Addr,
+					    sizeof(struct sockaddr_storage));
+#else
 				    struct sockaddr_in rsin;
 
 				    bzero(&rsin, sizeof(rsin));
-				    rsin.sin_addr = dres.dr_Addr;
-				    rsin.sin_port = dres.dr_Port;
+				    memcpy(&rsin, &dres.dr_Addr, sizeof(rsin));
+#endif
 				    if (dres.dr_ResultFlags == DR_REQUIRE_DNS) {
 					SendDns(recv_fd, &rsin, &dres);
 				    } else {
@@ -1023,7 +1090,11 @@ DoListen(ForkDesc *desc, time_t t)
      */
     {
         char errbuf[1024];
+#ifdef INET6
+	struct sockaddr_storage rsin;
+#else
 	struct sockaddr_in rsin;
+#endif
 	int rslen = sizeof(rsin);
 
 	int fd = accept(desc->d_Fd, (struct sockaddr *)&rsin, &rslen);
@@ -1076,6 +1147,7 @@ DoListen(ForkDesc *desc, time_t t)
 		    }
 		    pc->fd = fd;
 		    strncpy(pc->msgbuf, errbuf, sizeof(pc->msgbuf) - 1);
+		    pc->msgbuf[sizeof(pc->msgbuf) - 1] = '\0';
 		    pc->when = time(NULL) + DelayedClose;
 		    pc->quickclose = 1;
 		    pc->next = NULL;
@@ -1107,7 +1179,11 @@ DoCommand(int ufd)
 {
     int fd;
     int retain = 0;
+#ifdef INET6
+    struct sockaddr_storage asin;
+#else
     struct sockaddr_in asin;
+#endif
     ACCEPT_ARG3_TYPE alen = sizeof(asin);
 
     /*
@@ -1271,11 +1347,20 @@ DoCommand(int ufd)
     }
 }
 
+#ifdef INET6
+void
+SendDns(int fd, struct sockaddr_storage *rsin, DnsRes *dres)
+#else
 void
 SendDns(int fd, struct sockaddr_in *rsin, DnsRes *dres)
+#endif
 {
     ForkDesc *best;
+#ifdef INET6
+    struct sockaddr_storage lsin;
+#else
     struct sockaddr_in lsin;
+#endif
     int lslen = sizeof(lsin);
 
     /*
@@ -1283,7 +1368,11 @@ SendDns(int fd, struct sockaddr_in *rsin, DnsRes *dres)
      */
     InstallAccessCache();
 
-    getsockname(fd, (struct sockaddr *)&lsin, &lslen);
+    bzero((void *)&lsin, sizeof(lsin));
+
+    if (getsockname(fd, (struct sockaddr *)&lsin, &lslen) < 0) {
+	logit(LOG_ERR, "getsockname(%d) call failed: %s", fd, strerror(errno));
+    }
 
     ++ConnectCount;
 
@@ -1297,8 +1386,8 @@ SendDns(int fd, struct sockaddr_in *rsin, DnsRes *dres)
 	++best->d_Count;
 
 	bzero(&dreq, sizeof(dreq));
-	dreq.dr_RSin = *rsin;
-	dreq.dr_LSin = lsin;
+	bcopy(rsin, &dreq.dr_RSin, sizeof(dreq.dr_RSin));
+	bcopy(&lsin, &dreq.dr_LSin, sizeof(dreq.dr_LSin));
 	if (dres != NULL) {
 	    dreq.dr_ResultFlags = dres->dr_ResultFlags;
 	    strcpy(dreq.dr_AuthUser, dres->dr_AuthUser);
@@ -1309,11 +1398,18 @@ SendDns(int fd, struct sockaddr_in *rsin, DnsRes *dres)
 	    dreq.dr_PostBytes = dres->dr_PostBytes;
 	    dreq.dr_PostCount = dres->dr_PostCount;
 	    dreq.dr_PostFailCount = dres->dr_PostFailCount;
+	    dreq.dr_ByteCountArticle = dres->dr_ByteCountArticle;
+	    dreq.dr_ByteCountHead = dres->dr_ByteCountHead;
+	    dreq.dr_ByteCountBody = dres->dr_ByteCountBody;
+	    dreq.dr_ByteCountList = dres->dr_ByteCountList;
+	    dreq.dr_ByteCountXover = dres->dr_ByteCountXover;
+	    dreq.dr_ByteCountXhdr = dres->dr_ByteCountXhdr;
+	    dreq.dr_ByteCountOther = dres->dr_ByteCountOther;
 	    dreq.dr_SessionLength = time(NULL) - dres->dr_TimeStart;
 	}
 
         best->d_FdPend = fd;
-	best->d_SaveRSin = *rsin;
+	bcopy(rsin, &best->d_SaveRSin, sizeof(best->d_SaveRSin));
 	write(best->d_Fd, &dreq, sizeof(dreq));
 	/* 
 	 * ignore return code.  If DNS process died, it
@@ -1366,13 +1462,13 @@ ForkPipe(int *pfd, void (*task)(int fd, const char *id), const char *description
      * fork.
      */
 
-    closelog();
+    CloseLog(NULL, 1);
     pid = fork();
     if (pid == 0) {
-	openlog(description, (DebugOpt ? LOG_PERROR : 0) | LOG_NDELAY | LOG_PID, LOG_NEWS);
+	OpenLog(description, (DebugOpt ? LOG_PERROR : 0) | LOG_NDELAY | LOG_PID);
 	SysLogDesc = description;
     } else {
-	openlog(SysLogDesc, (DebugOpt ? LOG_PERROR : 0) | LOG_NDELAY | LOG_PID, LOG_NEWS);
+	OpenLog(SysLogDesc, (DebugOpt ? LOG_PERROR : 0) | LOG_NDELAY | LOG_PID);
     }
 
     if (pid == 0) {
@@ -1408,6 +1504,7 @@ HandleDnsThread(ForkDesc *desc)
     const char *error2 = NULL;
     char errbuf[1024];
     int r;
+    char tmpaddr[NI_MAXHOST];
 
     errno = 0;
     if ((r = read(desc->d_Fd, &dres, sizeof(dres))) != sizeof(dres)) {
@@ -1452,36 +1549,46 @@ HandleDnsThread(ForkDesc *desc)
 	return;
     }
 
-    if ((r = ReadAccessCache()) < 0) {
-	syslog(LOG_CRIT, "Fatal: Unable to load access cache");
-	exit(1);
-    }
-    SetAuthDetails(&dres, dres.dr_ReaderName);
-    dres.dr_TimeStart = time(NULL);
-    if (r == 1)
-	ClearOldAccessMap();
-
-    if (dres.dr_Code == 0) {
-	snprintf(errbuf, sizeof(errbuf), "502 %s: Access denied to your node%s%s\r\n",
-		dres.dr_VServerDef->vs_HostName,
-		*dres.dr_VServerDef->vs_NewsAdm ? " - " : "",
-		dres.dr_VServerDef->vs_NewsAdm);
+    if (isdigit(dres.dr_ReaderName[0]) && isdigit(dres.dr_ReaderName[1]) && isdigit(dres.dr_ReaderName[2])) {
+	snprintf(errbuf, sizeof(errbuf), "%s\r\n", dres.dr_ReaderName);
 	error1 = errbuf;
-	error2 = "Unauthorized";
+	/* Point error2 at errbuf + 4 to catch the specific error */
+	/* error2 = "CustomError"; */
+	error2 = errbuf + 4;
     }
 
-    if (dres.dr_Flags & DF_STATUS) {
-        snprintf(errbuf, sizeof(errbuf), "180 %s status: connect=%d failed=%d dns=%d/%d act=%d/%d\r\n", dres.dr_VServerDef->vs_HostName,
-		ConnectCount, FailCount, 
-		NumPending, DOpts.ReaderDns,
-		NumActive, MaxConnects
-		);
-	write(desc->d_FdPend, errbuf, strlen(errbuf));
-	close(desc->d_FdPend);
-	desc->d_FdPend = -1;
-	--NumActive;
-	deleteDnsResHash(&dres, desc);
-	return;
+    if (error1 == NULL) {
+        if ((r = ReadAccessCache()) < 0) {
+	    logit(LOG_CRIT, "Fatal: Unable to load access cache");
+	    exit(1);
+        }
+        SetAuthDetails(&dres, dres.dr_ReaderName);
+        dres.dr_TimeStart = time(NULL);
+        if (r == 1)
+	    ClearOldAccessMap();
+
+        if (dres.dr_Code == 0) {
+	    snprintf(errbuf, sizeof(errbuf), "502 %s: Access denied to your node%s%s\r\n",
+		    dres.dr_VServerDef->vs_HostName,
+		    *dres.dr_VServerDef->vs_NewsAdm ? " - " : "",
+		    dres.dr_VServerDef->vs_NewsAdm);
+	    error1 = errbuf;
+	    error2 = "Unauthorized";
+        }
+
+        if (dres.dr_Flags & DF_STATUS) {
+            snprintf(errbuf, sizeof(errbuf), "180 %s status: connect=%d failed=%d dns=%d/%d act=%d/%d\r\n", dres.dr_VServerDef->vs_HostName,
+		    ConnectCount, FailCount, 
+		    NumPending, DOpts.ReaderDns,
+		    NumActive, MaxConnects
+		    );
+	    write(desc->d_FdPend, errbuf, strlen(errbuf));
+	    close(desc->d_FdPend);
+	    desc->d_FdPend = -1;
+	    --NumActive;
+	    deleteDnsResHash(&dres, desc);
+	    return;
+        }
     }
 
     /*
@@ -1509,8 +1616,11 @@ HandleDnsThread(ForkDesc *desc)
     /*
      * Add DnsRes to hash table
      */
-    if (error1 == NULL) {
-	int hi = shash(inet_ntoa(dres.dr_Addr)) & DRHMASK;
+    strncpy(tmpaddr, NetAddrToSt(0, (struct sockaddr *)&dres.dr_Addr, 1, 0, 0),
+							sizeof(tmpaddr) - 1);
+    tmpaddr[sizeof(tmpaddr) - 1] = '\0';
+    if (error1 == NULL && tmpaddr != NULL) {
+	int hi = shash(tmpaddr) & DRHMASK;
 	int hmi;
 	DnsRes *dr;
 	int gcount = 0;
@@ -1525,7 +1635,7 @@ HandleDnsThread(ForkDesc *desc)
 	 * calculate the rate limit.
 	 */
 	for (dr = DnsResHash[hi]; dr; dr = dr->dr_HNext) {
-	    if (dres.dr_Addr.s_addr == dr->dr_Addr.s_addr) {
+	    if (strcmp(tmpaddr, NetAddrToSt(0, (struct sockaddr *)&dr->dr_Addr, 1, 0, 0)) == 0) {
 		++hcount;
 		if (*dres.dr_IdentUser && *dr->dr_IdentUser &&
 			strcmp(dres.dr_IdentUser, dr->dr_IdentUser) == 0)
@@ -1549,7 +1659,7 @@ HandleDnsThread(ForkDesc *desc)
 	/*
 	 * Tax the rate limit based on number of connections
 	 */
-        if (dres.dr_ReaderDef->rd_RateLimitTax) {
+	{
 	    int count = 0;
 	    if (ucount)
 		count = ucount;
@@ -1557,10 +1667,7 @@ HandleDnsThread(ForkDesc *desc)
 		count = icount;
 	    else if (hcount)
 		count = hcount;
-	    dres.dr_CurrRateLimit = dres.dr_ReaderDef->rd_RateLimit -
-			(dres.dr_ReaderDef->rd_RateLimitTax * count);
-	    if (dres.dr_CurrRateLimit < 100)
-		dres.dr_CurrRateLimit = 100;
+	    dres.dr_ConnCount = count;
 	}
 
 	/*
@@ -1574,7 +1681,7 @@ HandleDnsThread(ForkDesc *desc)
 	vcount++;
 
 	if ((dres.dr_Flags & DF_QUIET) == 0)
-	    syslog(LOG_INFO, "counters %s%s%s%s%s %s %s tcount=%d/%d vcount=%d/%d gcount=%d/%d hcount=%d/%d ucount=%d/%d icount=%d rate=%d/%d byte=%d",
+	    logit(LOG_INFO, "counters %s%s%s%s%s %s %s tcount=%d/%d vcount=%d/%d gcount=%d/%d hcount=%d/%d ucount=%d/%d icount=%d byte=%d",
 			(dres.dr_AuthUser[0] ? dres.dr_AuthUser : ""),
 			(dres.dr_AuthUser[0] ? "/" : ""),
 			(dres.dr_IdentUser[0] ? dres.dr_IdentUser : ""),
@@ -1586,9 +1693,7 @@ HandleDnsThread(ForkDesc *desc)
 			gcount, dres.dr_ReaderDef->rd_MaxConnPerGroup,
 			hcount, dres.dr_ReaderDef->rd_MaxConnPerHost,
 			ucount, dres.dr_ReaderDef->rd_MaxConnPerUser,
-			icount, dres.dr_CurrRateLimit,
-			dres.dr_ReaderDef->rd_RateLimit, 
-			dres.dr_ReaderDef->rd_ByteLimit);
+			icount, dres.dr_ReaderDef->rd_ByteLimit);
 
 	if (dres.dr_ReaderDef->rd_MaxConnTotal &&
 		NumActive > dres.dr_ReaderDef->rd_MaxConnTotal) {
@@ -1601,7 +1706,7 @@ HandleDnsThread(ForkDesc *desc)
 #ifdef	READER_BAN_LISTS
 	    if (GlobalBan.BanCount > 0)
 		IPListAdd(&GlobalBan.BanList,
-				inet_ntoa(desc->d_SaveRSin.sin_addr),
+		NetAddrToSt(0, (struct sockaddr *)&desc->d_SaveRSin, 1, 0, 1),
 				GlobalBan.BanCount,
 				time(NULL) + GlobalBan.BanTime,
 				GlobalBan.BanHashSize,
@@ -1621,7 +1726,7 @@ HandleDnsThread(ForkDesc *desc)
 
 	if (dres.dr_ReaderDef->rd_MaxConnPerGroup &&
 		gcount > dres.dr_ReaderDef->rd_MaxConnPerGroup) {
-	    snprintf(errbuf, sizeof(errbuf), "400 %s: Your per-group connection limit reached%s%s\r\n",
+	    snprintf(errbuf, sizeof(errbuf), "400 %s: You have reached the connection limit for your access classification%s%s\r\n",
 				dres.dr_VServerDef->vs_HostName,
 				*dres.dr_VServerDef->vs_NewsAdm ? " - " : "",
 				dres.dr_VServerDef->vs_NewsAdm);
@@ -1630,7 +1735,7 @@ HandleDnsThread(ForkDesc *desc)
 #ifdef	READER_BAN_LISTS
 	    if (GroupBan.BanCount > 0)
 		IPListAdd(&GroupBan.BanList,
-				inet_ntoa(desc->d_SaveRSin.sin_addr),
+		NetAddrToSt(0, (struct sockaddr *)&desc->d_SaveRSin, 1, 0, 1),
 				GroupBan.BanCount,
 				time(NULL) + GroupBan.BanTime,
 				GroupBan.BanHashSize,
@@ -1649,7 +1754,7 @@ HandleDnsThread(ForkDesc *desc)
 #ifdef	READER_BAN_LISTS
 	    if (HostBan.BanCount > 0)
 		IPListAdd(&HostBan.BanList,
-				inet_ntoa(desc->d_SaveRSin.sin_addr),
+		NetAddrToSt(0, (struct sockaddr *)&desc->d_SaveRSin, 1, 0, 1),
 				HostBan.BanCount,
 				time(NULL) + HostBan.BanTime,
 				HostBan.BanHashSize,
@@ -1668,7 +1773,7 @@ HandleDnsThread(ForkDesc *desc)
 #ifdef	READER_BAN_LISTS
 	    if (UserBan.BanCount > 0)
 		IPListAdd(&UserBan.BanList,
-				dres.dr_AuthUser,
+		NetAddrToSt(0, (struct sockaddr *)&desc->d_SaveRSin, 1, 0, 1),
 				UserBan.BanCount,
 				time(NULL) + UserBan.BanTime,
 				UserBan.BanHashSize,
@@ -1693,13 +1798,12 @@ HandleDnsThread(ForkDesc *desc)
 	if ((dres.dr_Flags & DF_QUIET) == 0) {
 	    logit(
 		LOG_NOTICE, 
-		"connection to %s from %s%s%s (%s:%d) rejected: %s", 
+		"connection to %s from %s%s%s (%s) rejected: %s", 
 		dres.dr_VServerDef->vs_Name,
 		(dres.dr_IdentUser[0] ? dres.dr_IdentUser : ""),
 		(dres.dr_IdentUser[0] ? "@" : ""),
 		dres.dr_Host,
-		inet_ntoa(desc->d_SaveRSin.sin_addr),
-		ntohs(desc->d_SaveRSin.sin_port),
+		NetAddrToSt(0, (struct sockaddr *)&desc->d_SaveRSin, 1, 1, 1),
 		error2
 	    );
 	}
@@ -1726,6 +1830,7 @@ HandleDnsThread(ForkDesc *desc)
 	}
 	pc->fd = desc->d_FdPend;
 	strncpy(pc->msgbuf, error1, sizeof(pc->msgbuf) - 1);
+	pc->msgbuf[sizeof(pc->msgbuf) - 1] = '\0';
 	pc->when = time(NULL) + DelayedClose;
 	pc->quickclose = 0;
 	pc->next = NULL;
@@ -1771,15 +1876,14 @@ HandleDnsThread(ForkDesc *desc)
 
     if ((dres.dr_Flags & DF_QUIET) == 0) {
 	logit(LOG_INFO,
-	    "connect to %s from %s%s%s%s%s (%s:%d)", 
+	    "connect to %s from %s%s%s%s%s (%s)", 
 	    dres.dr_VServerDef->vs_Name,
 	    (dres.dr_AuthUser[0] ? dres.dr_AuthUser : ""),
 	    (dres.dr_AuthUser[0] ? "/" : ""),
 	    (dres.dr_IdentUser[0] ? dres.dr_IdentUser : ""),
 	    (dres.dr_IdentUser[0] ? "@" : ""),
 	    dres.dr_Host,
-	    inet_ntoa(desc->d_SaveRSin.sin_addr),
-	    ntohs(desc->d_SaveRSin.sin_port)
+	    NetAddrToSt(0, (struct sockaddr *)&desc->d_SaveRSin, 1, 1, 1)
 	);
     }
 }
@@ -1807,14 +1911,13 @@ deleteDnsResHash(DnsRes *dres, ForkDesc *desc)
 		if (dr->dr_ReaderPid == desc->d_Pid) {
 		    logit(
 			LOG_NOTICE,
-			"killed from %s%s%s%s%s (%s:%d)",
+			"killed from %s%s%s%s%s (%s)",
 			(dr->dr_AuthUser[0] ? dr->dr_AuthUser : ""),
 			(dr->dr_AuthUser[0] ? "/" : ""),
 			(dr->dr_IdentUser[0] ? dr->dr_IdentUser : ""),
 			(dr->dr_IdentUser[0] ? "@" : ""),
 			dr->dr_Host,
-			inet_ntoa(dr->dr_Addr),
-			ntohs(dr->dr_Port)
+			NetAddrToSt(0, (struct sockaddr *)&dr->dr_Addr, 1, 1, 1)
 		    );
 		    *pdr = dr->dr_HNext;
 		    zfree(&DnsResPool, dr, sizeof(DnsRes));
@@ -1824,30 +1927,28 @@ deleteDnsResHash(DnsRes *dres, ForkDesc *desc)
 	    }
 	}
     } else {
-	int hi = shash(inet_ntoa(dres->dr_Addr)) & DRHMASK;
+	int hi = shash(NetAddrToSt(0, (struct sockaddr *)&dres->dr_Addr, 1, 0, 0)) & DRHMASK;
 	DnsRes **pdr = &DnsResHash[hi];
 	DnsRes *dr;
 
 	while ((dr = *pdr) != NULL) {
 	    if (dr->dr_ReaderPid == desc->d_Pid &&
-		dr->dr_Addr.s_addr == dres->dr_Addr.s_addr &&
-		dr->dr_Port == dres->dr_Port
+		memcmp(&dr->dr_Addr, &dres->dr_Addr, sizeof(dr->dr_Addr)) == 0
 	    ) {
 		if ((dr->dr_Flags & DF_QUIET) == 0) {
 		    logit(
 			LOG_INFO,
 #ifdef	DREADER_CLIENT_TIMING
-			"closed from %s%s%s%s%s (%s:%d) groups=%u arts=%u bytes=%.0f posts=%d postbytes=%d postfail=%d secs=%d utime=%.3f stime=%.3f",
+			"closed from %s%s%s%s%s (%s) groups=%u arts=%u bytes=%.0f posts=%d postbytes=%d postfail=%d secs=%d utime=%.3f stime=%.3f",
 #else
-			"closed from %s%s%s%s%s (%s:%d) groups=%u arts=%u bytes=%.0f posts=%d postbytes=%d postfail=%d secs=%d",
+			"closed from %s%s%s%s%s (%s) groups=%u arts=%u bytes=%.0f posts=%d postbytes=%d postfail=%d secs=%d",
 #endif
 			(dr->dr_AuthUser[0] ? dr->dr_AuthUser : ""),
 			(dr->dr_AuthUser[0] ? "/" : ""),
 			(dr->dr_IdentUser[0] ? dr->dr_IdentUser : ""),
 			(dr->dr_IdentUser[0] ? "@" : ""),
 			dr->dr_Host,
-			inet_ntoa(dres->dr_Addr),
-			ntohs(dr->dr_Port),
+			NetAddrToSt(0, (struct sockaddr *)&dr->dr_Addr, 1, 1, 1),
 			dres->dr_GrpCount,
 			dres->dr_ArtCount,
 			dres->dr_ByteCount,
@@ -2019,7 +2120,7 @@ getVStats(FILE *fo)
 void
 getStats(FILE *fo, int raw)
 {
-    fprintf(fo, "UPTIME %ld\n", time(NULL) - TimeStart);
+    fprintf(fo, "UPTIME %d\n", time(NULL) - TimeStart);
     fprintf(fo, "SERVER Connect=%d Failed=%d Dns=%d/%d Act=%d/%d\n", 
 		ConnectCount, FailCount,
 		NumPending, DOpts.ReaderDns, 
