@@ -58,14 +58,21 @@ MBFlush(Connection *conn, MBufHead *mh)
 	     * timer to re-enable the select descriptor later when we
 	     * can write more.
 	     */
-	    if (desc != NULL && conn->co_Auth.dr_ReaderDef && conn->co_Auth.dr_CurrRateLimit) {
+	    if (desc != NULL && conn->co_Auth.dr_ReaderDef != NULL &&
+			conn->co_ByteCountType < DRBC_XXXX &&
+			conn->co_Auth.dr_ReaderDef->rd_RateLimit[conn->co_ByteCountType]) {
+		int rl;
 		int dt = CurTime.tv_sec - conn->co_RateTv.tv_sec;
 		if (dt != 0) {
 		    conn->co_RateCounter = 0;
 		    conn->co_RateTv.tv_sec = CurTime.tv_sec;
 		}
-		if (n > conn->co_Auth.dr_CurrRateLimit - conn->co_RateCounter){
-		    n = conn->co_Auth.dr_CurrRateLimit - conn->co_RateCounter;
+		rl = conn->co_Auth.dr_ReaderDef->rd_RateLimit[conn->co_ByteCountType];
+		if (conn->co_Auth.dr_ReaderDef->rd_RateLimitTax)
+		    rl -= conn->co_Auth.dr_ReaderDef->rd_RateLimitTax *
+			conn->co_Auth.dr_ConnCount;
+		if (n > rl - conn->co_RateCounter) {
+		    n = rl - conn->co_RateCounter;
 		    if (n < 0)
 			n = 0;	/* n shouldn't be < zero, but be check anyway */
 		    AddTimer(desc, (1000000 - CurTime.tv_usec) / 1000, TIF_WRITE);
@@ -91,6 +98,22 @@ MBFlush(Connection *conn, MBufHead *mh)
 		conn->co_RateCounter += n;
 		conn->co_ClientTotalByteCount += n;
 		conn->co_ClientGroupByteCount += n;
+		switch(conn->co_ByteCountType) {
+			case	DRBC_ARTICLE:	conn->co_Auth.dr_ByteCountArticle += n;
+						break;
+			case	DRBC_HEAD:	conn->co_Auth.dr_ByteCountHead += n;
+						break;
+			case	DRBC_BODY:	conn->co_Auth.dr_ByteCountBody += n;
+						break;
+			case	DRBC_LIST:	conn->co_Auth.dr_ByteCountList += n;
+						break;
+			case	DRBC_XOVER:	conn->co_Auth.dr_ByteCountXover += n;
+						break;
+			case	DRBC_XHDR:	conn->co_Auth.dr_ByteCountXhdr += n;
+						break;
+			case	DRBC_NONE:	conn->co_Auth.dr_ByteCountOther += n;
+						break;
+	        }
 		if (n > 0)
 		    conn->co_LastActiveTime = CurTime.tv_sec;
 	    }
