@@ -50,6 +50,7 @@ NNTPAuthInfo(Connection *conn, char **pptr)
 	if (strcasecmp(type, "user") == 0) {
 	    strncpy(conn->co_Auth.dr_AuthUser, args,
 				sizeof(conn->co_Auth.dr_AuthUser) - 1);
+	    conn->co_Auth.dr_AuthUser[sizeof(conn->co_Auth.dr_AuthUser) - 1] = '\0';
 	    ok = 1;
 	    conn->co_Auth.dr_Flags |= DF_AUTHREQUIRED;
 	    MBLogPrintf(conn, &conn->co_TMBuf, "381 PASS required\r\n");
@@ -57,6 +58,7 @@ NNTPAuthInfo(Connection *conn, char **pptr)
 				*conn->co_Auth.dr_AuthUser) {
             strncpy(conn->co_Auth.dr_AuthPass, args,
 				sizeof(conn->co_Auth.dr_AuthPass) - 1);
+	    conn->co_Auth.dr_AuthPass[sizeof(conn->co_Auth.dr_AuthPass) - 1] = '\0';
 	    conn->co_Auth.dr_Flags &= ~DF_AUTHREQUIRED;
 	    conn->co_Auth.dr_ResultFlags = DR_REQUIRE_DNS;
 	    return;
@@ -176,26 +178,6 @@ NNTPGroup(Connection *conn, char **pptr)
     zfreeStr(&conn->co_MemPool, &conn->co_GroupName);
     conn->co_GroupName = zallocStr(&conn->co_MemPool, group);
 
-    /*
-     * update artBeg, locate first valid article.  We don't hold the
-     * lock through this due to having to call NNTestOverview()
-     * in the middle.  XXX
-     */
-    {
-	while (artBeg <= artEnd) {
-	    conn->co_ArtNo = artBeg;
-	    if (NNTestOverview(conn) == 0)
-		break;
-	    ++artBeg;
-	    adjustedArtBeg = 1;
-	}
-	if (adjustedArtBeg) {
-	    char aabegbuf[16];
-	    sprintf(aabegbuf, "%010d", artBeg);
-	    KPDBWrite(KDBActive, group, "NB", aabegbuf, 0);
-	}
-    }
-
     MBLogPrintf(conn, &conn->co_TMBuf, "211 %d %d %d %s\r\n",
 	artEnd - artBeg + 1,
 	artBeg,
@@ -225,7 +207,7 @@ NNTPLast(Connection *conn, char **pptr)
 	int ovlen;
 
 	--conn->co_ArtNo;
-	if ((ovdata = NNRetrieveHead(conn, &ovlen, &msgid)) != NULL) {
+	if ((ovdata = NNRetrieveHead(conn, &ovlen, &msgid, NULL, NULL, NULL)) != NULL) {
 	    MBLogPrintf(conn, &conn->co_TMBuf, 
 		"223 %d %s Article retrieved; request text separately.\r\n", 
 		conn->co_ArtNo,
@@ -257,7 +239,7 @@ NNTPNext(Connection *conn, char **pptr)
 	int ovlen;
 
 	++conn->co_ArtNo;
-	if ((ovdata = NNRetrieveHead(conn, &ovlen, &msgid)) != NULL) {
+	if ((ovdata = NNRetrieveHead(conn, &ovlen, &msgid, NULL, NULL, NULL)) != NULL) {
 	    MBLogPrintf(conn, &conn->co_TMBuf, 
 		"223 %d %s Article retrieved; request text separately.\r\n", 
 		conn->co_ArtNo,
@@ -314,7 +296,7 @@ NNArticleRetrieve(Connection *conn, const char *artid, int mode)
 	     */
 	    if (mode == COM_ARTICLEWVF)
 		conn->co_ArtMode = COM_ARTICLE;
-	    NNArticleRetrieveByMessageId(conn, artid, 0);
+	    NNArticleRetrieveByMessageId(conn, artid, 0, 0, -1, 0);
 	}
     }
 }
@@ -324,14 +306,15 @@ NNArticleRetrieveByArtNo(Connection *conn, int artNo)
 {
     const char *ovdata;
     const char *msgid;
-    int ovlen;
+    int ovlen, timeRcvd=0, grpIter=-1, endNo=0;
 
     if (DebugOpt)
 	printf("ArtNo %d\n", artNo);
 
     conn->co_RequestFlags = ARTFETCH_ARTNO;
     conn->co_ArtNo = artNo;
-    if ((ovdata = NNRetrieveHead(conn, &ovlen, &msgid)) != NULL) {
+    if ((ovdata = NNRetrieveHead(conn, &ovlen, &msgid, &timeRcvd, &grpIter, &endNo)) != NULL) {
+	
 	/*
 	 * COM_ARTICLEWVF requires that we verify that we have a valid article
 	 * body prior to dumping a valid response code.  At this point we only
@@ -377,13 +360,13 @@ NNArticleRetrieveByArtNo(Connection *conn, int artNo)
 	if (conn->co_ArtMode == COM_ARTICLEWVF ||
 	    conn->co_ArtMode == COM_BODYWVF
 	) {
-	    NNArticleRetrieveByMessageId(conn, msgid, 1);
+	    NNArticleRetrieveByMessageId(conn, msgid, 1, timeRcvd, grpIter, endNo);
 	} else if (
 	    conn->co_ArtMode == COM_ARTICLE || 
 	    conn->co_ArtMode == COM_BODY
 	) {
 	    conn->co_ArtMode = COM_BODYNOSTAT;
-	    NNArticleRetrieveByMessageId(conn, msgid, 1);
+	    NNArticleRetrieveByMessageId(conn, msgid, 1, timeRcvd, grpIter, endNo);
 	} else {
 	    if (conn->co_ArtMode != COM_STAT)
 		MBPrintf(&conn->co_TMBuf, ".\r\n");
@@ -459,6 +442,8 @@ DumpOVHeaders(Connection *conn, const char *ovdata, int ovlen)
 
 	for (i = 0; i < ovlen && ovdata[i] != '\n'; ++i)
 	    ;
+	if (i == 1 && ovdata[0] == '\r' && ovdata[1] == '\n')
+	    break;
 	if (ovdata[0] == '.')
 	    MBPrintf(&conn->co_TMBuf, ".");
 	ch = tolower(ovdata[0]);
