@@ -61,7 +61,11 @@ char *NNPHMangle(char *vserv, char *node, char *cryptpw)
 	/* Safety: in the event we cant md5, then leave some useful
 	   data in md5buf */
 	snprintf(md5buf, sizeof(md5buf), "%s", node);
+#ifdef	__APPLE_CC__
+	MD5(buffer, strlen(buffer), md5buf);
+#else
 	MD5Data(buffer, strlen(buffer), md5buf);
+#endif
 	/* Clip the string.  8 digits is enough */
 	md5buf[8] = '\0';
 	snprintf(res, sizeof(res), "%s.%s", md5buf, vserv);
@@ -252,6 +256,11 @@ NNPostBuffer(Connection *conn)
 
     MBInit(&tmbuf, -1, &conn->co_MemPool, &conn->co_BufPool);
 
+    if (!errMsg && conn->co_Flags & COF_POSTTOOBIG) {
+	conn->co_Flags &= ~COF_POSTTOOBIG;
+	errMsg = "441 Article too big for this server\r\n";
+    }
+
 #ifdef	POST_CFILTERHOOK
     if (! errMsg) {
 	errMsg = PostCFilter(base, artLen, conn);
@@ -396,6 +405,7 @@ NNPostBuffer(Connection *conn)
 		errMsg = "441 Obsolete \"To\" header rejected\r\n";
 	} else if (strncasecmp(base, "Cc:", 3) == 0) {
 		errMsg = "441 Obsolete \"Cc\" header rejected\r\n";
+#endif
 	} else if (strncasecmp(base, "X-Complaints-To:", 16) == 0) {
 		errMsg = "441 Illegal \"X-Complaints-To\" header rejected\r\n";
 	} else if (strncasecmp(base, "NNTP-Posting-Date:", 18) == 0) {
@@ -404,14 +414,6 @@ NNPostBuffer(Connection *conn)
 		errMsg = "441 Illegal \"NNTP-Posting-Host\" header rejected\r\n";
 	} else if (strncasecmp(base, "X-Trace:", 8) == 0) {
 		errMsg = "441 Illegal \"X-Trace\" header rejected\r\n";
-#else
-	} else if (strncasecmp(base, "NNTP-Posting-Date:", 18) == 0) {
-	    keep = 0;	/* delete */
-	} else if (strncasecmp(base, "NNTP-Posting-Host:", 18) == 0) {
-	    keep = 0;	/* delete */
-	} else if (strncasecmp(base, "X-Trace:", 8) == 0) {
-	    keep = 0;	/* delete */
-#endif
 	} else if (strncasecmp(base, "Xref:", 5) == 0) {
 	    keep = 0;	/* delete */
 	} else if (strncasecmp(base, "Control:", 8) == 0) {
@@ -527,6 +529,27 @@ NNPostBuffer(Connection *conn)
 		MBPrintf(&tmbuf, "Organization: %s\r\n",
 			conn->co_Auth.dr_VServerDef->vs_Org);
 	}
+	if (*conn->co_Auth.dr_VServerDef->vs_Comments) {
+		if (! strstr(conn->co_Auth.dr_VServerDef->vs_Comments, "\\n")) {
+			MBPrintf(&tmbuf, "X-Comments: %s\r\n",
+				conn->co_Auth.dr_VServerDef->vs_Comments);
+		} else {
+			/* This is sinful even for me. JG20030118 */
+			char buf[512], *ptr, *head;
+			int commentnum = 1;
+
+			snprintf(buf, sizeof(buf), "%s", conn->co_Auth.dr_VServerDef->vs_Comments);
+			head = buf;
+			while ((ptr = strstr(head, "\\n"))) {
+				*ptr = '\0';
+				MBPrintf(&tmbuf, "X-Comments-%d: %s\r\n",
+					commentnum++, head);
+				head = ptr + 2;
+			}
+			MBPrintf(&tmbuf, "X-Comments-%d: %s\r\n",
+				commentnum++, head);
+		}
+	}
 
 #if defined  POST_NNTP_POSTING_DATE_GMT || defined POST_NNTP_POSTING_DATE_LOCALTIME
 	/*
@@ -573,7 +596,7 @@ NNPostBuffer(Connection *conn)
 		*conn->co_Auth.dr_AuthUser ? "/" : "",
 		*conn->co_Auth.dr_IdentUser ? conn->co_Auth.dr_IdentUser : "",
 		*conn->co_Auth.dr_IdentUser ? "@" : "",
-		inet_ntoa(conn->co_Auth.dr_Addr)
+		NetAddrToSt(0, (struct sockaddr *)&conn->co_Auth.dr_Addr, 1, 1, 1)
 	    );
 	    MBPrintf(&tmbuf, "X-Trace: DXC=%s\r\n",
 		XTRAMangle(xtrace, conn->co_Auth.dr_VServerDef->vs_CryptPw));
@@ -587,7 +610,7 @@ NNPostBuffer(Connection *conn)
 		*conn->co_Auth.dr_AuthUser ? "/" : "",
 		*conn->co_Auth.dr_IdentUser ? conn->co_Auth.dr_IdentUser : "",
 		*conn->co_Auth.dr_IdentUser ? "@" : "",
-		inet_ntoa(conn->co_Auth.dr_Addr)
+		NetAddrToSt(0, (struct sockaddr *)&conn->co_Auth.dr_Addr, 1, 1, 1)
 	    );
 	if (*conn->co_Auth.dr_VServerDef->vs_AbuseTo)
 		MBPrintf(&tmbuf, "X-Complaints-To: %s\r\n",
@@ -601,6 +624,19 @@ NNPostBuffer(Connection *conn)
     if (errMsg == NULL) {
 	MBWrite(&tmbuf, base, artLen);
     }
+    if (*conn->co_Auth.dr_VServerDef->vs_PostTrailer) {
+	/* This is sinful even for me. JG20030206 */
+	char buf[512], *ptr, *head;
+
+	snprintf(buf, sizeof(buf), "%s", conn->co_Auth.dr_VServerDef->vs_PostTrailer);
+	head = buf;
+	while ((ptr = strstr(head, "\\n"))) {
+		*ptr = '\0';
+		MBPrintf(&tmbuf, "%s\r\n", head);
+		head = ptr + 2;
+	}
+	MBPrintf(&tmbuf, "%s\r\n", head);
+    }
     MBFree(&conn->co_ArtBuf);
 
     /*
@@ -619,16 +655,15 @@ NNPostBuffer(Connection *conn)
 	MBCopy(&tmbuf, &conn->co_ArtBuf);
 	MBFree(&tmbuf);
 
-	syslog(
+	logit(
 	    LOG_NOTICE,
-	    "%s%s%s%s%s (%s:%d) post ok %s %d", 
+	    "%s%s%s%s%s (%s) post ok %s %d", 
 	    *conn->co_Auth.dr_AuthUser ? conn->co_Auth.dr_AuthUser : "",
 	    *conn->co_Auth.dr_AuthUser ? "/" : "",
 	    (conn->co_Auth.dr_IdentUser[0] ? conn->co_Auth.dr_IdentUser : ""),
 	    (conn->co_Auth.dr_IdentUser[0] ? "@" : ""),
 	    conn->co_Auth.dr_Host,
-	    inet_ntoa(conn->co_Auth.dr_Addr),
-	    ntohs(conn->co_Auth.dr_Port),
+	    NetAddrToSt(0, (struct sockaddr *)&conn->co_Auth.dr_Addr, 1, 1, 1),
 	    conn->co_IHaveMsgId,
 	    conn->co_ArtBuf.mh_Bytes
 	);
@@ -647,7 +682,7 @@ NNPostBuffer(Connection *conn)
 	    MBLogPrintf(conn, &conn->co_TMBuf, "240 Article mailed to moderator\r\n");
 	    NNCommand(conn);
 	} else {
-	    NNServerRequest(conn, NULL, conn->co_IHaveMsgId, NULL, SREQ_POST);
+	    NNServerRequest(conn, NULL, conn->co_IHaveMsgId, SREQ_POST, 0, -1, 0);
 	}
     } else {
 	MBLogPrintf(conn, &conn->co_TMBuf, "%s", errMsg);
@@ -813,6 +848,12 @@ BreakIfBadGroup(char *groups, Connection *conn)
 
 	for (j = i; groups[j] && groups[j] != ','; ++j)
 	    ;
+
+	if (j - i > MAXGNAME) {
+	    r = -1;
+	    break;
+	}
+
 	c = groups[j];
 	groups[j] = 0;
 
