@@ -620,7 +620,6 @@ NNTerminate(Connection *conn)
     conn->co_State = "term";
     conn->co_RMBuf.mh_REof = 1;
     conn->co_TMBuf.mh_WEof = 1;
-    FD_SET(conn->co_Desc->d_Fd, &WFds);
 
     /*
      * conn->co_SReq is only non-NULL for a client
@@ -655,6 +654,7 @@ NNTerminate(Connection *conn)
 	ActiveCacheWriteUnlock();
     }
     MBFlush(conn, &conn->co_TMBuf);
+    FD_SET(conn->co_Desc->d_Fd, &WFds);
 }
 
 void
@@ -664,7 +664,7 @@ NNAuthDone(Connection *conn)
     if (conn->co_Auth.dr_Code == 1 && (conn->co_Auth.dr_Flags & DF_AUTHREQUIRED) == 0) {
 	conn->co_Auth.dr_Flags &= ~DF_AUTHREQUIRED;
 	MBLogPrintf(conn, &conn->co_TMBuf, "281 Ok\r\n");
-	syslog(LOG_DEBUG,
+	logit(LOG_DEBUG,
 		"Access granted to %s via %s", conn->co_Auth.dr_Host,
 		conn->co_Auth.dr_ReaderName);
 	NNCommand(conn);
@@ -708,6 +708,7 @@ WelcomeString(char *base, Vserver *vs, const char *servertype)
 	    r[l++] = *base++;
 	}
     }
+    r[l] = '\0';
     return(r);
 }
 
@@ -778,6 +779,7 @@ typedef struct Command {
     int		cmd_Flags;
     void	(*cmd_Func)(Connection *conn, char **pptr);
     const char	*cmd_Help;
+    int		cmd_DRBC_Type;
 } Command;
 
 Command Cmds[] = {
@@ -785,175 +787,204 @@ Command Cmds[] = {
 	"article",
 	CMDF_AUTH|CMDF_SERVER|CMDF_READER|CMDF_NOTFEEDONLY,
 	NNTPArticle,
-	"[MessageID|Number]"
+	"[MessageID|Number]",
+	DRBC_ARTICLE
     },
     { 
 	"body",
 	CMDF_AUTH|CMDF_SERVER|CMDF_READER|CMDF_NOTFEEDONLY,	
 	NNTPBody,
-	"[MessageID|Number]"
+	"[MessageID|Number]",
+	DRBC_BODY
     },
     { 
 	"date",		
 	0	 |CMDF_SERVER|CMDF_READER,
 	NNTPDate,
-	""
+	"",
+	DRBC_NONE
     },
     { 
 	"head",	
 	CMDF_AUTH|CMDF_SERVER|CMDF_READER|CMDF_NOTFEEDONLY,
 	NNTPHead,
-	"[MessageID|Number]"
+	"[MessageID|Number]",
+	DRBC_HEAD
     },
     { 
 	"help",		
 	0	 |CMDF_SERVER|CMDF_READER,
 	NNTPHelp,
-	""
+	"",
+	DRBC_NONE
     },
     { 
 	"ihave",
 	CMDF_AUTH|CMDF_SERVER|CMDF_READER,
 	NNTPIHave,
-	""
+	"",
+	DRBC_NONE
     },
     { 
 	"takethis",	
 	CMDF_AUTH|CMDF_SERVER|0		 ,
 	NNTPTakeThis,
-	"MessageID"
+	"MessageID",
+	DRBC_NONE
     },
     { 
 	"check",
 	CMDF_AUTH|CMDF_SERVER|0		 ,
 	NNTPCheck,
-	"MessageID"
+	"MessageID",
+	DRBC_NONE
     },
     { 
 	"mode",
 	CMDF_AUTH|CMDF_READER,
 	NNTPMode,
-	"reader|stream"
+	"reader|stream",
+	DRBC_NONE
     },
     { 
 	"mode",
 	CMDF_AUTH|CMDF_SERVER,
 	NNTPMode ,
-	"reader|stream|headfeed"
+	"reader|stream|headfeed",
+	DRBC_NONE
     },
     { 
 	"slave",
 	CMDF_AUTH|CMDF_SERVER|CMDF_READER,
 	NNTPSlave,
-	""
+	"",
+	DRBC_NONE
     },
     { 
 	"quit",
 	0	 |CMDF_SERVER|CMDF_READER,
 	NNTPQuit,
-	""
+	"",
+	DRBC_NONE
     },
     { 
 	"group",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPGroup ,
-	"newsgroup"
+	"newsgroup",
+	DRBC_NONE
     },
     { 
 	"last",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPLast,
-	""
+	"",
+	DRBC_NONE
     },
     { 
 	"next",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPNext,
-	""
+	"",
+	DRBC_NONE
     },
     { 
 	"list",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPList,
-	"[active|active.times|newsgroups|extensions|distributions|distrib.pats|moderators|overview.fmt|subscriptions]"
+	"[active|active.times|newsgroups|extensions|distributions|distrib.pats|moderators|overview.fmt|subscriptions]",
+	DRBC_LIST
     },
     { 
 	"listgroup",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPListGroup,
-	"newsgroup"
+	"newsgroup",
+	DRBC_LIST
     },
     { 
 	"newgroups",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPNewgroups,
-	"yymmdd hhmmss [\"GMT\"] [<distributions>]"
+	"yymmdd hhmmss [\"GMT\"] [<distributions>]",
+	DRBC_LIST
     },
     { 
 	"newnews",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPNewNews,
-	"newsgroups yymmdd hhmmss [\"GMT\"] [<distribution>]"
+	"newsgroups yymmdd hhmmss [\"GMT\"] [<distribution>]",
+	DRBC_LIST
     },
     { 
 	"post",	
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPPost,
-	""
+	"",
+	DRBC_NONE
     },
     { 
 	"stat",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPStat,
-	"[MessageID|Number]"
+	"[MessageID|Number]",
+	DRBC_NONE
     },
     { 
 	"xgtitle",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPXGTitle,
-	"[group_pattern]"
+	"[group_pattern]",
+	DRBC_LIST
     },
     { 
 	"xhdr",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPXHdr,
-	"header [range|MessageID]"
+	"header [range|MessageID]",
+	DRBC_XHDR
     },
     { 
 	"hdr",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPXHdr,
-	"header [range|MessageID]"
+	"header [range|MessageID]",
+	DRBC_XHDR
     },
     { 
 	"over",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPXOver,
-	"[range]"
+	"[range]",
+	DRBC_XOVER
     },
     { 
 	"xover",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPXOver,
-	"[range]"
+	"[range]",
+	DRBC_XOVER
     },
     { 
 	"xpat",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPXPat,
-	"header range|MessageID pat"
+	"header range|MessageID pat",
+	DRBC_XHDR
     },
     { 
 	"xpath",
 	CMDF_AUTH|0	     |CMDF_READER,
 	NNTPXPath,
-	"MessageID"
+	"MessageID",
+	DRBC_XHDR
     },
     { 
 	"authinfo",
 	0	 |CMDF_SERVER|CMDF_READER,
 	NNTPAuthInfo,
-	"user Name|pass Password"
+	"user Name|pass Password",
+	DRBC_NONE
     }
 };
 
@@ -979,8 +1010,10 @@ NNCommand(Connection *conn)
     if (conn->co_Auth.dr_ReaderDef->rd_ByteLimit &&
 	    conn->co_Auth.dr_ReaderDef->rd_ByteLimit <
 	    conn->co_ClientTotalByteCount) {
-	logit(LOG_INFO, "bytelimit session %s",
-	    conn->co_Auth.dr_Host);
+	logit(LOG_INFO, "bytelimit session %s byte=%d/%d",
+		conn->co_Auth.dr_Host,
+		conn->co_ClientTotalByteCount,
+		conn->co_Auth.dr_ReaderDef->rd_ByteLimit);
 	MBLogPrintf(conn,
 	    &conn->co_TMBuf,
 	    "400 %s: Session byte limit reached.\r\n",
@@ -1027,6 +1060,8 @@ NNCommand2(Connection *conn)
 	return;
     }
 
+    conn->co_ByteCountType = DRBC_NONE;
+
     /*
      * check EOF
      */
@@ -1097,6 +1132,7 @@ NNCommand2(Connection *conn)
 	    MBLogPrintf(conn, &conn->co_TMBuf, "480 Authentication required for command\r\n");
 	    NNCommand(conn);
 	} else {
+	    conn->co_ByteCountType = scan->cmd_DRBC_Type;
 	    scan->cmd_Func(conn, &ptr);
 	}
     } else {
