@@ -11,11 +11,11 @@
 
 Prototype void CheckServerConfig(time_t t, int force);
 Prototype void LogServerInfo(Connection *conn, int fd);
-Prototype void NNArticleRetrieveByMessageId(Connection *conn, const char *msgid, int grouphint);
+Prototype void NNArticleRetrieveByMessageId(Connection *conn, const char *msgid, int grouphint, int TimeRcvd, int grpIter, int endNo);
 Prototype void NNServerIdle(Connection *conn);
 Prototype void NNServerTerminate(Connection *conn);
 Prototype void NNFinishSReq(Connection *conn, const char *ctl, int requeue);
-Prototype void NNServerRequest(Connection *conn, const char *grp, const char *msgid, FILE *cache, int req);
+Prototype void NNServerRequest(Connection *conn, const char *grp, const char *msgid, int req, int TimeRcvd, int grpIter, int endNo);
 
 Prototype int   ServersTerminated;
 Prototype int	NReadServers;
@@ -28,11 +28,13 @@ void QueueServerRequests(void);
 int queueServerRequest(ServReq *sreq, int type, int maxq);
 void queueDesc(ServReq *sreq, ForkDesc *desc);
 int requeueServerRequest(Connection *conn, ServReq *sreq, int type, int maxq);
-int AddServer(char *bindinfo, int txbufsize, int rxbufsize, const char *host, int type, int port, int flags, int pri, char *groups, const char *localspool, HashFeed *hashFeed);
+int AddServer(char *bindinfo, int txbufsize, int rxbufsize, const char *host, int type, int port, int flags, int pri, char *groups, const char *localspool, HashFeed *hashFeed, int cache, int cachemin, int cachemax, int cacheabletime, double rratio, double cratio, char *login, char *password);
 void NNServerPrime1(Connection *conn);
 void NNServerPrime2(Connection *conn);
 void NNServerPrime3(Connection *conn);
 void NNServerPrime4(Connection *conn);
+void NNServerPrimeAuth1(Connection *conn);
+void NNServerPrimeAuth2(Connection *conn);
 void NNServerConnect(Connection *conn);
 
 ServReq	*SReadBase;
@@ -127,7 +129,17 @@ CheckServerConfig(time_t t, int force)
 		HashFeed hash = { 0 };
 		int txbufsize = 0;
 		int rxbufsize = 0;
+		char *login = NULL;
+		char *password = NULL;
+		int cache = CACHE_OFF;
+		int cachemin = 0;
+		int cachemax = 0;
+		int cacheabletime = 0;
+		double rratio = 4;
+		double cratio = -1;
 
+		if (DOpts.ReaderCacheMode)
+		    cache = CACHE_ON;
 		if (host == NULL || host[0] == '#')
 		    continue;
 		if ((flags = strtok(NULL, " \t\n")) == NULL)
@@ -199,6 +211,66 @@ CheckServerConfig(time_t t, int force)
 			    if ((p = strchr(option, '/')) != NULL)
 				hash.hf_Mod = strtol(++p, NULL, 0);
 			}
+		    } else if (strncmp(option, "login=", 6) == 0) {
+			login = option + 6;
+			nflags |= COF_LOGIN;
+		    } else if (strncmp(option, "password=", 9) == 0) {
+			password = option + 9;
+		    } else if (strncmp(option, "cache=", 6) == 0) {
+			if (strncmp(option+6,"off", 3) == 0) {
+			    cache = CACHE_OFF;
+			} else if (strncmp(option+6,"on", 2) == 0) {
+			    if (DOpts.ReaderCacheMode) {
+				cache = CACHE_ON;
+			    } else {
+				logit(LOG_ERR, "Ignoring cache option as diablo.conf cache option is disabled");
+			    }
+			} else if (strncmp(option+6,"lazy", 4) == 0) {
+			    if (DOpts.ReaderCacheMode) {
+				cache = CACHE_LAZY;
+			    } else {
+				logit(LOG_ERR, "Ignoring cache option as diablo.conf cache option is disabled");
+			    }
+			} else if (strncmp(option+6,"scoring(", 8) == 0) {
+			    char *p;
+			    option += 14;
+			    if (strchr(option, ')') == NULL) {
+				logit(LOG_ERR, "Syntax error on scoring option");
+			    } else {
+				if (*option == ',') {
+				    rratio = -1;
+				    cratio = strtod(++option, NULL);
+				} else if ((p = strchr(option, ',')) != NULL) {
+				    rratio = strtod(option, NULL);
+				    cratio = strtod(++p, NULL);
+				} else {
+				    rratio = strtod(option, NULL);
+				    cratio = -1;
+				}
+
+				if (DOpts.ReaderCacheMode) {
+				    cache = CACHE_SCOREBOARD;
+				    OpenCacheHits();
+				} else {
+				    logit(LOG_ERR, "Ignoring cache option as diablo.conf cache option is disabled");
+				}
+			    }
+			} else if (strncmp(option+6,"scoring", 7) == 0) {
+			    if (DOpts.ReaderCacheMode) {
+				cache = CACHE_SCOREBOARD;
+				OpenCacheHits();
+			    } else {
+				logit(LOG_ERR, "Ignoring cache option as diablo.conf cache option is disabled");
+			    }
+			} else {
+			    logit(LOG_ERR, "Unknown option '%s' - ignoring", option);
+			}
+		    } else if (strncmp(option, "cachemin=", 9) == 0) {
+			cachemin = strtol(option + 9, NULL, 0);
+		    } else if (strncmp(option, "cachemax=", 9) == 0) {
+			cachemax = strtol(option + 9, NULL, 0);
+		    } else if (strncmp(option, "cacheabletime=", 14) == 0) {
+			cacheabletime = strtol(option + 14, NULL, 0);
 		    } else {
 			logit(LOG_ERR, "Unknown option '%s' - ignoring", option);
 		    }
@@ -206,7 +278,9 @@ CheckServerConfig(time_t t, int force)
 		if (addAsServer) {
 		    if (AddServer(bindinfo, txbufsize, rxbufsize, host,
 				THREAD_SPOOL, port, nflags, npri,
-				groups, localspool, &hash) == 0) {
+				groups, localspool, &hash, cache,
+				cachemin, cachemax, cacheabletime,
+				rratio, cratio, login, password) == 0) {
 			++NReadServers;
 			++NReadServAct;
 		    } else {
@@ -216,7 +290,9 @@ CheckServerConfig(time_t t, int force)
 		if (addAsPoster) {
 		    if (AddServer(bindinfo, txbufsize, rxbufsize, host,
 				THREAD_POST, port, nflags, npri,
-				groups, localspool, &hash) == 0) {
+				groups, localspool, &hash, cache,
+				cachemin, cachemax, cacheabletime,
+				rratio, cratio, login, password) == 0) {
 			++NWriteServers;
 			++NWriteServAct;
 		    } else {
@@ -244,10 +320,13 @@ LogServerInfo(Connection *conn, int fd)
     if (!conn->co_ServerByteCount ||
 			conn->co_LastServerLog + 300 > now)
 	return;
-    syslog(LOG_INFO, "info server %s articles=%ld bytes=%ld",
+    logit(LOG_INFO, "info server %s articles=%ld bytes=%ld req=%ld notfound=%ld err=%ld",
 		(conn->co_Desc->d_Id ? conn->co_Desc->d_Id : "UNKNOWN"),
 		conn->co_ServerArticleCount,
-		conn->co_ServerByteCount
+		conn->co_ServerByteCount,
+		conn->co_ServerArticleRequestedCount,
+		conn->co_ServerArticleNotFoundErrorCount,
+		conn->co_ServerArticleMiscErrorCount
     );
     {
 	DnsRes dr;
@@ -258,6 +337,9 @@ LogServerInfo(Connection *conn, int fd)
     }
     conn->co_ServerByteCount = 0;
     conn->co_ServerArticleCount = 0;
+    conn->co_ServerArticleRequestedCount = 0;
+    conn->co_ServerArticleNotFoundErrorCount = 0;
+    conn->co_ServerArticleMiscErrorCount = 0;
     conn->co_LastServerLog = now;
 }
 
@@ -284,7 +366,7 @@ makeGroupList(Connection *conn, char *groups)
 }
 
 int
-AddServer(char *bindinfo, int txbufsize, int rxbufsize, const char *host, int type, int port, int flags, int pri, char *groups, const char *localspool, HashFeed *hashFeed)
+AddServer(char *bindinfo, int txbufsize, int rxbufsize, const char *host, int type, int port, int flags, int pri, char *groups, const char *localspool, HashFeed *hashFeed, int cache, int cachemin, int cachemax, int cacheabletime, double rratio, double cratio, char *login, char *password)
 {
     ForkDesc *desc;
     int fd;
@@ -352,7 +434,8 @@ AddServer(char *bindinfo, int txbufsize, int rxbufsize, const char *host, int ty
     if (connect(fd, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
 	if (errno != EINPROGRESS) {
 	    close(fd);
-	    logit(LOG_ERR, "connect() call failed on host %s: %s\n", host, strerror(errno));
+	    logit(LOG_ERR, "connect() call failed on host %s:%d: %s\n",
+						host, port, strerror(errno));
 	    return(-1);
 	}
     }
@@ -366,6 +449,15 @@ AddServer(char *bindinfo, int txbufsize, int rxbufsize, const char *host, int ty
     desc = AddThread(host, fd, -1, type, -1, pri);
     FD_SET(desc->d_Fd, &RFds);
     conn = InitConnection(desc, NULL);
+    if (flags & COF_LOGIN) {
+	if (password) {
+	    snprintf(conn->co_Auth.dr_AuthUser, sizeof(conn->co_Auth.dr_AuthUser), "%s", login);
+	    snprintf(conn->co_Auth.dr_AuthPass, sizeof(conn->co_Auth.dr_AuthPass), "%s", password);
+	} else {
+	    logit(LOG_ERR, "no password specified with login: %s\n", login);
+	    flags &= ~COF_LOGIN;
+	}
+    }
     conn->co_Flags |= flags;
     conn->co_Flags |= COF_INPROGRESS | COF_ININIT;
     desc->d_Count = THREAD_LIMSIZE;
@@ -374,6 +466,12 @@ AddServer(char *bindinfo, int txbufsize, int rxbufsize, const char *host, int ty
     bcopy(hashFeed, &conn->co_RequestHash, sizeof(HashFeed));
     if (localspool != NULL)
 	desc->d_LocalSpool = zallocStr(&conn->co_MemPool, localspool);
+    desc->d_Cache = cache;
+    desc->d_CacheMin = cachemin;
+    desc->d_CacheMax = cachemax;
+    desc->d_CacheableTime = cacheabletime;
+    desc->d_ReadNewRatio = rratio;
+    desc->d_CacheReadRatio = cratio;
     NNServerConnect(conn);
     if (DebugOpt)
 	printf("Added Server (type=%d fd=%d pid=%d)\n", type, fd, (int)getpid());
@@ -597,7 +695,7 @@ FreeSReq(ServReq *sreq)
  */
 
 void
-NNServerRequest(Connection *conn, const char *grp, const char *msgid, FILE *cache, int req)
+NNServerRequest(Connection *conn, const char *grp, const char *msgid, int req, int TimeRcvd, int grpIter, int endNo)
 {
     ServReq *sreq = zalloc(&SysMemPool, sizeof(ServReq));
 
@@ -606,7 +704,10 @@ NNServerRequest(Connection *conn, const char *grp, const char *msgid, FILE *cach
     sreq->sr_Time = time(NULL);
     sreq->sr_Group = grp ? zallocStr(&SysMemPool, grp) : NULL;
     sreq->sr_MsgId = msgid ? zallocStr(&SysMemPool, msgid) : NULL;
-    sreq->sr_Cache = cache;	/* may be NULL 			*/
+    sreq->sr_Cache = NULL;
+    sreq->sr_TimeRcvd = TimeRcvd;
+    sreq->sr_GrpIter = grpIter;
+    sreq->sr_endNo = endNo;
 
     conn->co_SReq = sreq;	/* client has active sreq		*/
 
@@ -673,7 +774,7 @@ NNFinishSReq(Connection *conn, const char *ctl, int requeue)
 	     */
 	    if (sreq->sr_CConn) {
 		if (ctl) {
-		    if (ctl[1] == '4') {
+		    if (ctl[1] == '4' || ctl[0] == '4') {
 			/*
 			 * x4x codes are POST results of some sort -
 			 * this result only appears here via NNPostResponseX.
@@ -681,6 +782,9 @@ NNFinishSReq(Connection *conn, const char *ctl, int requeue)
 			 * client as it may be an informative error message
 			 * unlike the average spool server transaction
 			 * which we probably don't care about.
+			 *
+			 * 4xx codes may be spool failures - these should
+			 * also be of interest
 			 */
 		        MBLogPrintf(sreq->sr_CConn, &sreq->sr_CConn->co_TMBuf, "%s", ctl);
 		    } else {
@@ -716,7 +820,10 @@ NNFinishSReq(Connection *conn, const char *ctl, int requeue)
 void
 NNServerPrime1(Connection *conn)
 {
-    if (conn->co_Flags & COF_MODEREADER) {
+    if (conn->co_Flags & COF_LOGIN) {
+	MBPrintf(&conn->co_TMBuf, "authinfo user %s\r\n", conn->co_Auth.dr_AuthUser);
+	NNServerPrimeAuth1(conn);
+    } else if (conn->co_Flags & COF_MODEREADER) {
 	MBPrintf(&conn->co_TMBuf, "mode reader\r\n");
 	NNServerPrime2(conn);
     } else if (conn->co_Flags & COF_READONLY) {
@@ -787,6 +894,49 @@ NNServerPrime4(Connection *conn)
     }
 }
 
+void
+NNServerPrimeAuth1(Connection *conn)
+{
+    int len;
+    char *ptr;
+
+    conn->co_Func = NNServerPrimeAuth1;
+    conn->co_State = "primeauth1";
+
+    if ((len = MBReadLine(&conn->co_RMBuf, &ptr)) != 0) {
+	int code = strtol(ptr, NULL, 10);
+	if (code >= 300 && code <= 399) {
+	    MBPrintf(&conn->co_TMBuf, "authinfo pass %s\r\n", conn->co_Auth.dr_AuthPass);
+
+	    NNServerPrimeAuth2(conn);
+	} else {
+	    logit(LOG_ERR, "authinfo-user(%s) failed: %s", conn->co_Desc->d_Id, ptr);
+	    NNServerTerminate(conn);
+	}
+    }
+}
+
+void
+NNServerPrimeAuth2(Connection *conn)
+{
+    int len;
+    char *ptr;
+
+    conn->co_Func = NNServerPrimeAuth2;
+    conn->co_State = "primeauth2";
+
+    if ((len = MBReadLine(&conn->co_RMBuf, &ptr)) != 0) {
+	int code = strtol(ptr, NULL, 10);
+	if (code >= 200 && code <= 299) {
+	    logit(LOG_INFO, "authinfo-pass(%s) %s", conn->co_Desc->d_Id, ptr);
+	    NNServerStart(conn);
+	} else {
+	    logit(LOG_ERR, "authinfo-pass(%s) failed: %s", conn->co_Desc->d_Id, ptr);
+	    NNServerTerminate(conn);
+	}
+    }
+}
+
 /*
  * NNSERVERSTART() - start normal server operations.  Clean up from connect
  *		     code, make server available to clients.
@@ -951,6 +1101,10 @@ NNServerConnect(Connection *conn)
 	    logit(LOG_ERR, "connect(%s) failed", conn->co_Desc->d_Id);
 	    NNServerTerminate(conn);
 	}
+	if ( (conn->co_SessionStartTime + 60 ) < CurTime.tv_sec) {
+	    logit(LOG_ERR, "connect(%s) timed out",  conn->co_Desc->d_Id);
+	    NNServerTerminate(conn);
+	}
 	return;
     }
 
@@ -996,10 +1150,8 @@ NNServerConnect(Connection *conn)
  */
 
 void
-NNArticleRetrieveByMessageId(Connection *conn, const char *msgid, int grouphint)
+NNArticleRetrieveByMessageId(Connection *conn, const char *msgid, int grouphint, int TimeRcvd, int grpIter, int endNo)
 {   
-    FILE *cache = NULL;
-
     /*
      * (a) retrieve from cache if caching is enabled
      */
@@ -1019,7 +1171,7 @@ NNArticleRetrieveByMessageId(Connection *conn, const char *msgid, int grouphint)
 	    if (DebugOpt)
 		printf("good cache\n");
 	    if ((map = xmap(NULL, size, PROT_READ, MAP_SHARED, cfd, 0)) != NULL) {
-		xadvise(map, ((size < 65536) ? size : 65536), XADV_WILLNEED);
+		xadvise(map, size, XADV_WILLNEED);
 		if (conn->co_ArtMode != COM_BODYNOSTAT) {
 		    MBLogPrintf(conn, &conn->co_TMBuf, "%03d %d %s %s\r\n", 
 			GoodRC(conn),
@@ -1029,7 +1181,7 @@ NNArticleRetrieveByMessageId(Connection *conn, const char *msgid, int grouphint)
 		    );
 		}
 		if (conn->co_ArtMode != COM_STAT) {
-		    DumpArticleFromCache(conn, map, size);
+		    DumpArticleFromCache(conn, map, size, grpIter, endNo);
 		    MBPrintf(&conn->co_TMBuf, ".\r\n");
 		}
 		xunmap((void *)map, size);
@@ -1044,16 +1196,6 @@ NNArticleRetrieveByMessageId(Connection *conn, const char *msgid, int grouphint)
 	    close(cfd);
 	    NNCommand(conn);
 	    return;
-	} else if (valid == 0) {
-	    /*
-	     * not in cache, cache file created if cfd >= 0
-	     */
-	    if (DebugOpt)
-		printf("bad cache\n");
-	    if (cfd >= 0) {
-		cache = fdopen(cfd, "w");
-	    }
-	    /* fall through */
 	} else if (valid < 0) {
 	    /*
 	     * negatively cached
@@ -1075,8 +1217,10 @@ NNArticleRetrieveByMessageId(Connection *conn, const char *msgid, int grouphint)
     /*
      * (b)(c)(d)
      */
-    NNServerRequest(conn, grouphint ? conn->co_GroupName : NULL, msgid,
-							cache, SREQ_RETRIEVE);
+    if (DebugOpt)
+	printf("bad cache\n");
+
+    NNServerRequest(conn, grouphint ? conn->co_GroupName : NULL, msgid, SREQ_RETRIEVE, TimeRcvd, grpIter, endNo);
     NNWaitThread(conn);
 }
 
