@@ -33,6 +33,7 @@ NNSpoolCommand1(Connection *conn)
 	NNGetLocal1(conn) ;
     } else {
 	MBPrintf(&conn->co_TMBuf, "article %s\r\n", sreq->sr_MsgId);
+	conn->co_ServerArticleRequestedCount++;
 	NNSpoolResponse1(conn);
     }
 }
@@ -88,7 +89,7 @@ NNGetLocal1(Connection *conn)
 			const char *ovdata;
 			const char *msgid;
 			int ovlen;
-			if ((ovdata = NNRetrieveHead(sreq->sr_CConn, &ovlen, &msgid)) != NULL) {
+			if ((ovdata = NNRetrieveHead(sreq->sr_CConn, &ovlen, &msgid, NULL, NULL, NULL)) != NULL) {
 			    MBLogPrintf(sreq->sr_CConn, &sreq->sr_CConn->co_TMBuf, "%03d %d %s %s\r\n",
 						GoodRC(sreq->sr_CConn),
 						sreq->sr_CConn->co_ArtNo,
@@ -173,6 +174,7 @@ NNGetLocal1(Connection *conn)
 	     * whereis request failed, try the normal way
 	     */
 	    MBPrintf(&conn->co_TMBuf, "article %s\r\n", sreq->sr_MsgId);
+	    conn->co_ServerArticleRequestedCount++;
 	    NNSpoolResponse1(conn);
 	}
     } else if (len < 0) {
@@ -197,6 +199,10 @@ NNSpoolResponse1(Connection *conn)
     if ((len = MBReadLine(&conn->co_RMBuf, &buf)) > 0) {
 	conn->co_ServerByteCount += len;
 	if (strtol(buf, NULL, 10) == 220) {
+	    /* We have a positive answer, we may cache article */
+	    if (conn->co_Desc->d_Cache) {
+		CreateCache(conn);
+	    }
 	    /*
 	     * sr_CConn may be NULL if client was terminated while
 	     * server operation was still in progress.
@@ -231,7 +237,7 @@ NNSpoolResponse1(Connection *conn)
 			const char *msgid;
 			int ovlen;
 
-			if ((ovdata = NNRetrieveHead(sreq->sr_CConn, &ovlen, &msgid)) != NULL) {
+			if ((ovdata = NNRetrieveHead(sreq->sr_CConn, &ovlen, &msgid, NULL, NULL, NULL)) != NULL) {
 			    MBLogPrintf(sreq->sr_CConn, &sreq->sr_CConn->co_TMBuf, "%03d %d %s %s\r\n",
 				GoodRC(sreq->sr_CConn),
 				sreq->sr_CConn->co_ArtNo,
@@ -252,6 +258,13 @@ NNSpoolResponse1(Connection *conn)
 	    NNSpoolResponse2(conn);
 	    return;
 	}
+	else if (strtol(buf, NULL, 10) == 430) {
+	  conn->co_ServerArticleNotFoundErrorCount++;
+	}
+	else {
+	  conn->co_ServerArticleMiscErrorCount++;
+	}
+
 	if (sreq->sr_CConn == NULL)
 	    NNFinishSReq(conn, NULL, 0);
 	else if (sreq->sr_CConn->co_ArtMode == COM_BODYNOSTAT)
@@ -498,7 +511,7 @@ NNSpoolResponse3(Connection *conn)
 		if (ferror(sreq->sr_Cache) || error)
 		    AbortCache(fileno(sreq->sr_Cache), sreq->sr_MsgId, 0);
 		else
-		    CommitCache(fileno(sreq->sr_Cache), sreq->sr_MsgId, 0);
+		    CommitCache(conn, 0);
 		fclose(sreq->sr_Cache);
 		sreq->sr_Cache = NULL;
 	    }
@@ -578,7 +591,7 @@ NNSpoolResponseScrap(Connection *conn)
 		if (ferror(sreq->sr_Cache))
 		    AbortCache(fileno(sreq->sr_Cache), sreq->sr_MsgId, 0);
 		else
-		    CommitCache(fileno(sreq->sr_Cache), sreq->sr_MsgId, 0);
+		    CommitCache(conn, 0);
 		fclose(sreq->sr_Cache);
 		sreq->sr_Cache = NULL;
 	    }
