@@ -53,6 +53,8 @@ shash(const char *s)
 {
     unsigned int hv = 0xAFC32344;
 
+    if (s == NULL)
+	return(0);
     while (*s) {
 	hv = (hv << 5) ^ *s ^ (hv >> 23);
 	++s;
@@ -181,8 +183,8 @@ dumpConnectionInfo(Connection *conn)
     printf("         pri           : %d\n", conn->co_Desc->d_Pri);
     }
     printf(" Conn    state         : %s\n", conn->co_State);
-    printf("         starttime     : %ld\n", conn->co_SessionStartTime);
-    printf("         lastactivetime: %ld\n", conn->co_LastActiveTime);
+    printf("         starttime     : %d\n", conn->co_SessionStartTime);
+    printf("         lastactivetime: %d\n", conn->co_LastActiveTime);
     printf("         fcounter      : %d\n", conn->co_FCounter);
     printf("         bytecounter   : %.0f\n", conn->co_ByteCounter);
     printf("         bytesheader   : %d\n", conn->co_BytesHeader);
