@@ -133,7 +133,7 @@ NNTPXPat(Connection *conn, char **pptr)
 
     if ((hdr = parseword(pptr, " \t")) == NULL ||
 	(ran = parseword(pptr, " \t")) == NULL ||
-	(pat = parseword(pptr, " \t")) == NULL
+	*(pat = *pptr) == '\0'
     ) {
 	MBLogPrintf(conn, &conn->co_TMBuf, "501 header range|MessageID pat\r\n");
 	NNCommand(conn);
@@ -262,6 +262,7 @@ void
 NNListOverviewRange(Connection *conn)
 {
     OverInfo *ov;
+    int xpat_count = 0;
 
     conn->co_Func = NNListOverviewRange;
     conn->co_State = "listover";
@@ -271,6 +272,13 @@ NNListOverviewRange(Connection *conn)
 	NNCommand(conn);
 	return;
     }
+    if (conn->co_TMBuf.mh_WError) {
+	logit(LOG_ERR, "%s mh_WError (co_ArtMode = %d)",
+		conn->co_Auth.dr_Host, conn->co_ArtMode);
+	PutOverInfo(ov);
+	NNCommand(conn);
+	return;
+    }
     while (conn->co_TMBuf.mh_Bytes < 800 && 
 	    conn->co_ListBegNo <= conn->co_ListEndNo
     ) {
@@ -279,9 +287,13 @@ NNListOverviewRange(Connection *conn)
 	const char *res;
 	TimeRestrict *tr = NULL;
 
+	if (conn->co_ArtMode == COM_XPAT && ++xpat_count > 50) {
+	    FD_SET(conn->co_Desc->d_Fd, &WFds);
+	    break;
+	}
 	if (conn->co_ArtMode == COM_NEWNEWS)
 	    tr = &conn->co_TimeRestrict;
-	if ((res = GetOverRecord(ov, conn->co_ListBegNo, &resLen, &artSize, tr)) != NULL) {
+	if ((res = GetOverRecord(ov, conn->co_ListBegNo, &resLen, &artSize, tr, NULL)) != NULL) {
 	    OutputOverview(conn, res, resLen, artSize);
 	}
 	++conn->co_ListBegNo;
@@ -410,8 +422,12 @@ OutputOverview(Connection *conn, const char *res, int resLen, int artSize)
 			if (rlen >= sizeof(hdr))
 			    p = zalloc(&conn->co_MemPool, rlen + 1);
 			memcpy(p, rline, rlen);
+			/* Handle CR/LF in overview data */
+			if (rlen > 2 && p[rlen - 2] == '\r' && p[rlen - 1] == '\n') {
+			    p[rlen - 2] = 0;
+			}
 			p[rlen] = 0;
-			if (WildCmp(conn->co_ListPat, p + glen) == 0)
+			if (wildmat(p + glen, conn->co_ListPat))
 			    printHdr = ' ';
 			if (p != hdr)
 			    zfree(&conn->co_MemPool, p, rlen + 1);
