@@ -33,10 +33,10 @@
 
 Prototype int DiabFilter(char *fpath, char *loc, int wireformat);
 Prototype void DiabFilter_freeMem(void);
+Prototype void DiabFilterClose(void);
 
 static ssize_t writeAll(int fd, char *buf, ssize_t len);
 void filter_failed(char *reason, char *fpath);
-void close_filter_program(void);
 void open_filter_program(char *fpath);
 int sizeit(int len);
 
@@ -102,11 +102,11 @@ filter_failed(char *reason, char *fpath)
 	delay = filter_fail_count * filter_fail_count;
 	delay = (delay > 900) ? 900 : delay;
 	filter_try_again = time(NULL) + delay;
-	syslog(LOG_ERR, "diab-filter(%s): filter failed, %s, sleeping for %d seconds\n", fpath, reason, delay);
+	logit(LOG_ERR, "diab-filter(%s): filter failed, %s, sleeping for %d seconds\n", fpath, reason, delay);
 }
 
 void 
-close_filter_program(void)
+DiabFilterClose(void)
 {
 	int status, rval, loop;
 
@@ -121,27 +121,27 @@ close_filter_program(void)
 	if (filter_pid) {
 		for (loop = 0; loop < 10; loop++) {
 			if ((rval = waitpid(filter_pid, &status, WNOHANG)) < 0) {
-				syslog(LOG_ERR, "waitpid for %d failed: %m", filter_pid);
+				logit(LOG_ERR, "External filter waitpid for %d failed: %m", filter_pid);
 				filter_pid = 0;
 				return;
 			}
 			if (WIFEXITED(status)) {
 				if (WEXITSTATUS(status)) {
-					syslog(LOG_ERR, "filter returned exit %d", WEXITSTATUS(status));
+					logit(LOG_ERR, "External filter returned exit %d", WEXITSTATUS(status));
 				} else {
-					syslog(LOG_NOTICE, "filter exited normally");
+					logit(LOG_NOTICE, "External filter exited normally");
 				}
 				filter_pid = 0;
 				return;
 			}
 			if (WIFSIGNALED(status)) {
-				syslog(LOG_ERR, "filter exited on signal %d", WTERMSIG(status));
+				logit(LOG_ERR, "External filter exited on signal %d", WTERMSIG(status));
 				filter_pid = 0;
 				return;
 			}
 			sleep(1);
 		}
-		syslog(LOG_ERR, "filter failed to exit");
+		logit(LOG_ERR, "filter failed to exit");
 		filter_pid = 0;
 		return;
 	}
@@ -165,7 +165,7 @@ open_filter_program(char *fpath)
 	if (! (*fpath == '/')) {
 		/* Not a path name!  Guess that it is a TCP connection */
 		if ((nfd = connect_tcp_socket(fpath, 0, 0)) < 0) {
-			syslog(LOG_ERR, "couldnt connect to remote filter (%s): %m", fpath);
+			logit(LOG_ERR, "couldnt connect to remote filter (%s): %m", fpath);
 			filter_failed("couldnt connect to remote filter", fpath);
 			return;
 		}
@@ -173,16 +173,16 @@ open_filter_program(char *fpath)
 		filter_fd_stdout = nfd;
 
 		if (fcntl(filter_fd_stdin, F_SETFD, 1) < 0) {
-			syslog(LOG_ERR, "fcntl filter stdin: %m");
+			logit(LOG_ERR, "fcntl filter stdin: %m");
 		}
 		if (fcntl(filter_fd_stdout, F_SETFD, 1) < 0) {
-			syslog(LOG_ERR, "fcntl filter stdout: %m");
+			logit(LOG_ERR, "fcntl filter stdout: %m");
 		}
 
 		filter_pid = 0;
 
 		/* "Woohoo!" */
-		syslog(LOG_NOTICE, "filter connected to remote filter");
+		logit(LOG_NOTICE, "filter connected to remote filter");
 		return;
 	}
 
@@ -249,16 +249,16 @@ open_filter_program(char *fpath)
 	filter_fd_stdout = stdoutfds[0];
 
 	if (fcntl(filter_fd_stdin, F_SETFD, 1) < 0) {
-		syslog(LOG_ERR, "fcntl filter stdin: %m");
+		logit(LOG_ERR, "fcntl filter stdin: %m");
 	}
 	if (fcntl(filter_fd_stdout, F_SETFD, 1) < 0) {
-		syslog(LOG_ERR, "fcntl filter stdout: %m");
+		logit(LOG_ERR, "fcntl filter stdout: %m");
 	}
 
 	filter_pid = newpid;
 
 	/* "Woohoo!" */
-	syslog(LOG_NOTICE, "External filter launched");
+	logit(LOG_NOTICE, "External filter launched");
 	return;
 }
 
@@ -281,7 +281,7 @@ int
 DiabFilter(char *fpath, char *loc, int wireformat)
 {
 	int rval, count, eoln, nbytes, llen;
-	char *aptr, *nptr;
+	char *aptr, *nptr, *pptr;
 
 	if (! loc || ! (aptr = strrchr(loc, ','))) {
 		return(-1);
@@ -298,7 +298,7 @@ DiabFilter(char *fpath, char *loc, int wireformat)
 		}
 		aptr = nzalloc(&filter_abufPool, count);
 		if (aptr == NULL) {
-			syslog(LOG_ERR, "zalloc %d failed", count);
+			logit(LOG_ERR, "zalloc %d failed", count);
 			return(-1);
 		}
 		filter_abufsiz = count;
@@ -312,7 +312,7 @@ DiabFilter(char *fpath, char *loc, int wireformat)
 		}
 		nptr = nzalloc(&filter_nbufPool, count);
 		if (nptr == NULL) {
-			syslog(LOG_ERR, "zalloc %d failed", count);
+			logit(LOG_ERR, "zalloc %d failed", count);
 			return(-1);
 		}
 		filter_nbufsiz = count;
@@ -328,7 +328,7 @@ DiabFilter(char *fpath, char *loc, int wireformat)
 
 	if ((rval = diab_read(loc, filter_abuf, filter_abufsiz)) < 0) {
 #ifdef DEBUG
-		syslog(LOG_ERR, "diab_read failed: %s", loc);
+		logit(LOG_ERR, "diab_read failed: %s", loc);
 #endif
 		return(-1);
 	}
@@ -336,11 +336,13 @@ DiabFilter(char *fpath, char *loc, int wireformat)
 	count = rval;
 	aptr = filter_abuf + sizeof(SpoolArtHdr);
 	nptr = filter_nbuf;
+	pptr = aptr;
 	eoln = 0;
 	while (count--) {
 		if (wireformat) {
-			if (*aptr == '\n')
+			if (*pptr == '\r' && *aptr == '\n')
 			    eoln = 1;
+			pptr = aptr;
 			*nptr++ = *aptr++;
 		} else if (*aptr == '\n') {
 			*nptr++ = '\r';
@@ -357,11 +359,11 @@ DiabFilter(char *fpath, char *loc, int wireformat)
 		}
 	}
 	if (! eoln) {
-		syslog(LOG_ERR, "article did not end with a return: %s", loc);
+		logit(LOG_ERR, "article did not end with a return: %s", loc);
 		*nptr++ = '\r';
 		*nptr++ = '\n';
 	}
-	if (!wireformat) {
+	if (!wireformat || rval < 5 || strncmp(nptr - 5, "\r\n.\r\n", 5) != 0) {
 		*nptr++ = '.';
 		*nptr++ = '\r';
 		*nptr++ = '\n';
@@ -371,22 +373,27 @@ DiabFilter(char *fpath, char *loc, int wireformat)
 
 	/* Send the article to the filter ... */
 	if ((rval = writeAll(filter_fd_stdin, filter_nbuf, nbytes)) != nbytes) {
-		syslog(LOG_ERR, "filter write failure: wanted to write %d, wrote %d: %m", nbytes, rval);
+		logit(LOG_ERR, "filter write failure: wanted to write %d, wrote %d: %m", nbytes, rval);
 		filter_failed("write", fpath);
-		close_filter_program();
+		DiabFilterClose();
 		return(-1);
 	}
 
 	/* Get the response of the filter ... */
 	/* XXX this is Pure Evil(tm) because the response isn't going to 
 	 * have to be atomic */
+	filter_abuf[0] = '\0';
 	if ((rval = read(filter_fd_stdout, filter_abuf, filter_abufsiz)) <= 0) {
-		syslog(LOG_ERR, "filter read failure: got %d: %m", rval);
+		logit(LOG_ERR, "filter read failure: got %d: %m", rval);
 		filter_failed("read", fpath);
-		close_filter_program();
+		DiabFilterClose();
 		return(-1);
 	}
 	filter_abuf[rval] = '\0';
+	if (DebugOpt > 1)
+	    printf("External filter response: %s\n", filter_abuf);
+
+	filter_abuf[rval] = '\0';
 
 	if (*filter_abuf == '3') {
 		return(0);
@@ -394,7 +401,7 @@ DiabFilter(char *fpath, char *loc, int wireformat)
 	if (*filter_abuf == '4') {
 		return(1);
 	}
-	syslog(LOG_ERR, "filter read failure: got unknown response: %s",
+	logit(LOG_ERR, "filter read failure: got unknown response: %s",
 			filter_abuf);
 	return(-1);
 }
