@@ -32,8 +32,8 @@ Prototype int diab_read(char *namepos, char *buf, int szbuf);
 int
 diab_read(char *namepos, char *buf, int szbuf)
 {
-	char filename[FILENAMELEN];
-	char name[NAMELEN], *ptr, *nptr;
+	static char filename[FILENAMELEN];
+	static char name[NAMELEN], *ptr, *nptr;
 	static char lastname[NAMELEN] = { 0 };
 	static int fd = -1;
 	int offset, size, nsize;
@@ -66,8 +66,11 @@ diab_read(char *namepos, char *buf, int szbuf)
 		}
 	}
 	if (fd < 0) {
-		snprintf(filename, sizeof(filename), "%s", PatExpand(SpoolHomePat));
-		snprintf(filename + strlen(filename), sizeof(filename) - strlen(filename), "/%s", name);
+		if (name[0] == '/')
+			strncpy(filename, name, sizeof(filename) -1);
+		else
+			snprintf(filename, sizeof(filename), "%s/%s",
+						PatExpand(SpoolHomePat), name);
 
 		if ((fd = open(filename, O_RDONLY, 0)) < 0) {
 #ifdef	DEBUG
