@@ -23,7 +23,7 @@ InitDActive(const char *fileNamePat)
 {
     DActiveDB = KPDBOpen(PatDbExpand(fileNamePat), O_RDWR);
     if (DActiveDB == NULL) {
-	syslog(LOG_CRIT, "InitDActive, open failed: %s", PatDbExpand(fileNamePat));
+	logit(LOG_CRIT, "InitDActive, open failed: %s", PatDbExpand(fileNamePat));
 	exit(1);
     }
 }
@@ -106,7 +106,7 @@ GenerateXRef(Buffer *b, const char *nglist, const char *npName, const char *ctl)
 
 	    if (! rec && DOpts.FeederAutoAddToActive) {
 		if (ValidGroupName(group) < 0) {
-		    /* syslog(LOG_ERR, "group %s illegal", group); */
+		    /* logit(LOG_ERR, "group %s illegal", group); */
 		} else {
 		    char tsBuf[64];
 
@@ -244,6 +244,7 @@ UpdateActiveNX(char *xrefdata)
     p=strtok(NULL, " \t\n");
     while (p != NULL) {
 	strncpy(group, p, sizeof(group) - 1);
+	group[sizeof(group) - 1] = '\0';
 	artnum=strchr(group, ':');
 	if (artnum != NULL) {
 	    const char *rec;
