@@ -66,7 +66,7 @@ pagealloc(int *psize, int bytes)
     *psize = bytes;
     b = (void *)mmap((caddr_t)0, bytes, PROT_READ|PROT_WRITE, MAP_ANON|MAP_PRIVATE, -1, 0);
     if (b == NULL || b == (void *)-1) {
-	syslog(LOG_EMERG, "mmap/allocation/1 failed: %s", strerror(errno));
+	logit(LOG_EMERG, "mmap/allocation/1 failed: %s", strerror(errno));
 	exit(1);
     }
     return(b);
@@ -109,7 +109,7 @@ pagealloc(int *psize, int bytes)
     if (ZoneFd < 0) {
 	ZoneFd=open("/dev/zero", O_RDWR);
 	if (ZoneFd < 0) {
-	    syslog(LOG_EMERG, "unable to open %dK zone file /dev/zero: %s", 
+	    logit(LOG_EMERG, "unable to open %dK zone file /dev/zero: %s", 
 		bytes / 1024,
 		strerror(errno)
 	    );
@@ -121,7 +121,7 @@ pagealloc(int *psize, int bytes)
     *psize = bytes;
     b = (void *)mmap((caddr_t)0, bytes, PROT_READ|PROT_WRITE, MAP_PRIVATE, ZoneFd, 0);
     if (b == NULL || b == (void *)-1) {
-	syslog(LOG_EMERG, "mmap/allocation/3 failed: %s", strerror(errno));
+	logit(LOG_EMERG, "mmap/allocation/3 failed: %s", strerror(errno));
 	exit(1);
     }
     return(b);
@@ -175,7 +175,7 @@ pagealloc(int *psize, int bytes)
 	    }
 	}
 	if (ZoneFd < 0) {
-	    syslog(LOG_EMERG, "unable to create %dK zone file in /tmp: %s", 
+	    logit(LOG_EMERG, "unable to create %dK zone file in /tmp: %s", 
 		bytes / 1024,
 		strerror(errno)
 	    );
@@ -186,7 +186,7 @@ pagealloc(int *psize, int bytes)
 
     if (bytes > ZoneBytes) {
 	if (ftruncate(ZoneFd, bytes) < 0) {
-	    syslog(LOG_EMERG, "unable to extend zone file in /tmp: %s", 
+	    logit(LOG_EMERG, "unable to extend zone file in /tmp: %s", 
 		strerror(errno)
 	    );
 	    exit(1);
@@ -197,7 +197,7 @@ pagealloc(int *psize, int bytes)
     *psize = bytes;
     b = (void *)mmap((caddr_t)0, bytes, PROT_READ|PROT_WRITE, MAP_PRIVATE, ZoneFd, 0);
     if (b == NULL || b == (void *)-1) {
-	syslog(LOG_EMERG, "mmap/allocation/2 failed: %s", strerror(errno));
+	logit(LOG_EMERG, "mmap/allocation/2 failed: %s", strerror(errno));
 	exit(1);
     }
     return(b);
@@ -232,7 +232,7 @@ pagealloc(int *psize, int bytes)
     *psize = bytes;
     b = malloc(bytes);
     if (b == NULL) {
-	syslog(LOG_EMERG, "mmap/allocation/4 failed: %s", strerror(errno));
+	logit(LOG_EMERG, "mmap/allocation/4 failed: %s", strerror(errno));
 	exit(1);
     }
     bzero(b, bytes);
