@@ -361,6 +361,8 @@ int classifyLineAsTypes(char *buf, int len)
 
 	if (ptr[len - 1] == '\n')
 	    len--;
+	if (ptr[len - 1] == '\r')
+	    len--;
 
 	/* UUENCODE-specific checks:  UUENCODE data is well-behaved */
 	if (*ptr != 'M' || len != 61) {
@@ -368,7 +370,7 @@ int classifyLineAsTypes(char *buf, int len)
 	}
 
 	/* BASE64-specific checks: BASE64 data isn't well-behaved */
-	if (len < 64 || len > 77) {
+	if (len < 60 || len > 77) {
 		/*
 		 * I can't actually find anything that specifies what the
 		 * line length of base64 data is supposed to be.  RFC1113
@@ -376,7 +378,7 @@ int classifyLineAsTypes(char *buf, int len)
 		 * more than 76 characters".  Typical.  Examination of
 		 * actual base64 content shows lines to always be 74, for
 		 * a relatively large sample size.  But better safe than
-		 * sorry...  allow 64-76.
+		 * sorry...  allow 60-76.
 		 */
 
 		isnttype &= ~B64;
@@ -456,6 +458,12 @@ ArtTypeConv(char *t)
 	return(ARTTYPE_UUENCODE);
     else if (strcasecmp(t, "base64") == 0)
 	return(ARTTYPE_BASE64);
+    else if (strcasecmp(t, "yenc") == 0)
+	return(ARTTYPE_YENC);
+    else if (strcasecmp(t, "bommanews") == 0)
+	return(ARTTYPE_BOMMANEWS);
+    else if (strcasecmp(t, "unidata") == 0)
+	return(ARTTYPE_UNIDATA);
     else if (strcasecmp(t, "multipart") == 0)
 	return(ARTTYPE_MULTIPART);
     else if (strcasecmp(t, "html") == 0)
@@ -471,7 +479,7 @@ ArtTypeConv(char *t)
     else if (strcasecmp(t, "all") == 0)
 	return(ARTTYPE_ALL);
     else {
-	syslog(LOG_ERR, "Unknown article type: %s", t);
+	logit(LOG_ERR, "Unknown article type: %s", t);
 	return(ARTTYPE_NONE);
     }
 }
@@ -529,16 +537,27 @@ ArticleType(char *line, int len, int inheader)
 	    Type |= ARTTYPE_MULTIPART;
 	if (! strncasecmp(ptr, "Content-transfer-encoding: base64", 33))
 	    Type |= ARTTYPE_BASE64 | ARTTYPE_BINARY;
+	if (! strncasecmp(ptr, "Content-transfer-encoding: X-Bommanews", 38))
+	    Type |= ARTTYPE_BOMMANEWS | ARTTYPE_BINARY;
+	if (! strncasecmp(ptr, "Content-transfer-encoding: X-UnidataEncoding", 44))
+	    Type |= ARTTYPE_UNIDATA | ARTTYPE_BINARY;
 	if (! strncasecmp(ptr, "Content-Type: application/postscript", 36))
 	    Type |= ARTTYPE_PS;
 	if (! strncasecmp(ptr, "Content-Type: application/mac-binhex40", 38))
 	    BinHex = 1;
+	if (! strncasecmp(ptr, "Content-Type: application/octet-stream", 38))
+	    Type |= ARTTYPE_BINARY;
 	if (! strncasecmp(ptr, "Content-Type: message/partial", 29))
 	    Type |= ARTTYPE_PARTIAL;
     }
     if (first == '(' && strncasecmp(ptr, "(This file must be converted with BinHex 4.0)", 33) == 0)
 	BinHex = 1;
 
+    if (first == '=' && strncasecmp(ptr, "=ybegin part=", 13) == 0)
+	Type |= ARTTYPE_BINARY|ARTTYPE_PARTIAL|ARTTYPE_YENC;
+    if (first == '=' && strncasecmp(ptr, "=ybegin line=", 13) == 0)
+	Type |= ARTTYPE_BINARY|ARTTYPE_YENC;
+
     if (inheader) {
 	if (first == 'c' && tolower(*(ptr + 1) == 'o')) {
 	    if (! strncasecmp(ptr, "Control: ", 9))
