@@ -177,7 +177,7 @@ bgets(Buffer *b, int *pbytes)
 	    char *p = b->bu_Data + b->bu_Beg;
 
 	    if (pbytes == NULL) {
-		syslog(LOG_ERR, "Line buffer overflow fd %d", b->bu_Fd);
+		logit(LOG_ERR, "Line buffer overflow fd %d", b->bu_Fd);
 		p = (void *)-1;
 	    } else {
 		*pbytes = b->bu_End - b->bu_Beg;
@@ -219,13 +219,13 @@ bgets(Buffer *b, int *pbytes)
 			rc = COMPRESS_BUFFER_LENGTH - b->bu_CBuf->z_str.avail_in;
 			/* check for decompressor buffer overflow */
 			if (rc == 0) {
-			    syslog(LOG_ERR, "decompressor buffer overflow fd %d",
+			    logit(LOG_ERR, "decompressor buffer overflow fd %d",
 								b->bu_Fd);
 			    return((void *)-1);
 			}
 			nc = read(b->bu_Fd, &b->bu_CBuf->cb_Buf[b->bu_CBuf->z_str.avail_in], rc);
 			if (nc < 1) { 
-			    syslog(LOG_ERR,"eof (%i) on read",nc);
+			    logit(LOG_ERR,"eof (%i) on read",nc);
 			    n = nc;
 			    goto err_or_eof;
 			}   
@@ -251,7 +251,7 @@ bgets(Buffer *b, int *pbytes)
 
 		    if (code == Z_BUF_ERROR) {
 #ifdef COMPDEBUG 
-			syslog(LOG_ERR, "code=%i,read=%u,decomp=%u,avail_in=%u,avail_out=%u",
+			logit(LOG_ERR, "code=%i,read=%u,decomp=%u,avail_in=%u,avail_out=%u",
 				code, nc, decompcount,
 				b->bu_CBuf->z_str.avail_in,
 				b->bu_CBuf->z_str.avail_out
@@ -270,7 +270,7 @@ bgets(Buffer *b, int *pbytes)
 		     */
 
 		    if (code == Z_STREAM_ERROR) {
-			syslog(LOG_ERR, "code=%i,read=%u,decomp=%u,avail_in=%u,avail_out=%u",
+			logit(LOG_ERR, "code=%i,read=%u,decomp=%u,avail_in=%u,avail_out=%u",
 				code, nc, decompcount,
 				b->bu_CBuf->z_str.avail_in,
 				b->bu_CBuf->z_str.avail_out
@@ -287,7 +287,7 @@ bgets(Buffer *b, int *pbytes)
 		     */
 
 		    if (code != Z_OK) {
-			syslog(LOG_ERR, "inflate error (%i) on fd %d: %s",
+			logit(LOG_ERR, "inflate error (%i) on fd %d: %s",
 					code, b->bu_Fd, b->bu_CBuf->z_str.msg);
 			b->bu_CBuf->dataError = 1;
 			return((void *)-1);
