@@ -14,6 +14,7 @@ Prototype void LoadDiabloConfig(int ac, char **av);
 Prototype void PrintVersion(void);
 Prototype void SetMyHostName(char **hostname);
 Prototype void SetNewsAdmin(char **admin, char *host);
+Prototype void AddBindList(BindList **blist, char *interface, char *port, int append);
 Prototype int SetGroupHashMethod(const char *opt, GroupHashType *pmethod);
 Prototype int SetCommand(FILE *fo, char *cmd, char *opt, int *cmdErr);
 Prototype int CheckConfigOptions(int which);
@@ -53,9 +54,42 @@ SetNewsAdmin(char **admin, char *host)
     *admin = strdup(buf);
 }
 
+void
+AddBindList(BindList **blist, char *interface, char *port, int append)
+{
+    BindList *bl;
+    bl = (BindList *)malloc(sizeof(BindList));
+    bl->bl_Host = strdup(interface);
+    bl->bl_Port = strdup(port);
+    bl->bl_Next = NULL;
+    if (*blist == NULL) {
+	*blist = bl;
+	return;
+    }
+    if (!append && *blist != NULL) {
+	BindList *tbl;
+	while (*blist != NULL) {
+	    tbl = *blist;
+	    *blist = (*blist)->bl_Next;
+	    if (tbl->bl_Host != NULL)
+		free(tbl->bl_Host);
+	    if (tbl->bl_Port != NULL)
+		free(tbl->bl_Port);
+	    free(tbl);
+	}
+	*blist = bl;
+    } else {
+	BindList *tbl = *blist;
+	while (tbl->bl_Next != NULL)
+	    tbl = tbl->bl_Next;
+	tbl->bl_Next = bl;
+    }
+}
+
 void
 initDOpts(void)
 {
+    int i;
     /*
      * Set all the default options for the global config
      */
@@ -69,17 +103,23 @@ initDOpts(void)
     DOpts.ReaderGroupHashMethod.gh_dirinfo[0] = 2;
     DOpts.HashSize = 16 * 1024 * 1024;
     DOpts.FeederBufferSize = 1;
-    DOpts.FeederMaxArtSize = 0;
+    DOpts.FeederMaxArtSize = 10000000;
+    DOpts.FeederArtTypes = 1;
+    DOpts.FeederPreloadArt = 1;
+    DOpts.SpoolPreloadArt = 1;
+    DOpts.ReaderMaxArtSize = 10000000;
     DOpts.ReaderForks = 10;
     DOpts.ReaderThreads = 40;
     DOpts.ReaderFeedForks = 2;
     DOpts.ReaderDns = 5;
     DOpts.ReaderCacheMode = 1;
+    DOpts.ReaderCacheHashSize = 4096;
     DOpts.ReaderXOverMode = 1;
     DOpts.ReaderAutoAddToActive = 0;
     DOpts.FeederAutoAddToActive = 0;
     DOpts.ReaderDetailLog = 1;
-    DOpts.RememberDays = 7;
+    DOpts.RememberSecs = 14 * 24 * 60 * 60;
+    DOpts.FeederMaxAcceptAge = DOpts.RememberSecs;
     DOpts.HostCacheRebuildTime = 60 * 60;
     DOpts.DisplayAdminVersion = 1;
     DOpts.FeederRTStats = RTSTATS_NONE;
@@ -103,6 +143,8 @@ initDOpts(void)
     DOpts.ReaderPort = strdup("nntp");
     DOpts.ReaderCacheDirs.dt_dirlvl = 1;
     DOpts.ReaderCacheDirs.dt_dirinfo[0] = 512;
+    for (i = 0; i < 10; i++)
+	DOpts.PgpVerifyArgs[i] = NULL;
 }
 
 void
@@ -185,7 +227,7 @@ LoadDiabloConfig(int ac, char **av)
 		    if (opt) {
 			optErr = SetGroupHashMethod(opt, &DOpts.ReaderGroupHashMethod);
 			if (optErr != 0)
-			    syslog(LOG_CRIT, "Illegal reader group hash: %s", opt);
+			    logit(LOG_CRIT, "Illegal reader group hash: %s", opt);
 		    }
 		} else if (strcasecmp(cmd, "hsize") == 0) {
 		    if (opt) {
@@ -195,12 +237,12 @@ LoadDiabloConfig(int ac, char **av)
 			if (n < 256 * 1024 || n > 128 * 1024 * 1024) {
 			    optErr = 1;
 			    fprintf(stderr, "Illegal Hash size: %d\n", n);
-			    syslog(LOG_CRIT, "Illegal Hash size: %d", n);
+			    logit(LOG_CRIT, "Illegal Hash size: %d", n);
 			}
 			if ((n ^ (n - 1)) != (n << 1) - 1) {
 			    optErr = 1;
 			    fprintf(stderr, "Hash size not a power of 2: %d\n", n);
-			    syslog(LOG_CRIT, "Hash size not a power of 2: %d", n);
+			    logit(LOG_CRIT, "Hash size not a power of 2: %d", n);
 			}
 			if (optErr == 0)
 			    DOpts.HashSize = n;
@@ -310,8 +352,10 @@ LoadDiabloConfig(int ac, char **av)
 			pptr = &DServerHostsPat;
 		    else if (strcasecmp(cmd + 5, "moderators") == 0) 
 			pptr = &ModeratorsPat;
-		    else if (strcasecmp(cmd + 5, "spam_cache") == 0) 
-			pptr = &SpamCachePat;
+		    else if (strcasecmp(cmd + 5, "spam_body_cache") == 0) 
+			pptr = &SpamBodyCachePat;
+		    else if (strcasecmp(cmd + 5, "spam_nph_cache") == 0) 
+			pptr = &SpamNphCachePat;
 		    else if (strcasecmp(cmd + 5, "pcommit_cache") == 0) 
 			pptr = &PCommitCachePat;
 		    else if (strcasecmp(cmd + 5, "dexpover_list") == 0)
@@ -326,6 +370,10 @@ LoadDiabloConfig(int ac, char **av)
 			pptr = &DistribDotPatsPat;
 		    else if (strcasecmp(cmd + 5, "distributions") == 0) 
 			pptr = &DistributionsPat;
+		    else if (strcasecmp(cmd + 5, "dspoolctl") == 0) 
+			pptr = &DSpoolCtlPat;
+		    else if (strcasecmp(cmd + 5, "generallog") == 0) 
+			pptr = &GeneralLogPat;
 		    else if (strcasecmp(cmd + 5, "incominglog") == 0) 
 			pptr = &IncomingLogPat;
 		    else if (strcasecmp(cmd + 5, "drincominglog") == 0) 
@@ -334,8 +382,12 @@ LoadDiabloConfig(int ac, char **av)
 			pptr = &DFeedStatsPat;
 		    else if (strcasecmp(cmd + 5, "pathlog") == 0) 
 			pptr = &FPathLogPat;
+		    else if (strcasecmp(cmd + 5, "artlog") == 0) 
+			pptr = &FArtLogPat;
 		    else if (strcasecmp(cmd + 5, "shutdown_cleanup") == 0) 
 			pptr = &ShutdownCleanup;
+		    else if (strcasecmp(cmd + 5, "cachehits") == 0) 
+			pptr = &CacheHitsPat;
 		    else
 			cmdErr = 1;
 
@@ -361,7 +413,7 @@ LoadDiabloConfig(int ac, char **av)
 		    }
 		} else if (strcasecmp(cmd, "feederbindhost") == 0) {
 		    if (opt) {
-			strdupfree(&DOpts.FeederBindHost, opt, NULL);
+			strdupfree(&DOpts.FeederBindHost, SanitiseAddr(opt), NULL);
 			optErr = 0;
 		    }
 		} else if (strcasecmp(cmd, "readerbindport") == 0) {
@@ -371,7 +423,7 @@ LoadDiabloConfig(int ac, char **av)
 		    }
 		} else if (strcasecmp(cmd, "readerbindhost") == 0) {
 		    if (opt) {
-			strdupfree(&DOpts.ReaderBindHost, opt, NULL);
+			strdupfree(&DOpts.ReaderBindHost, SanitiseAddr(opt), NULL);
 			optErr = 0;
 		    }
 		} else if (strcasecmp(cmd, "readerpath") == 0) {
@@ -422,7 +474,7 @@ LoadDiabloConfig(int ac, char **av)
 		    exitMe = 1;
 		    fprintf(stderr, "Unknown diablo.config command: %s %s\n",
 						cmd, ((opt) ? opt : ""));
-		    syslog(LOG_CRIT, "Unknown diablo.config command: %s %s",
+		    logit(LOG_CRIT, "Unknown diablo.config command: %s %s",
 						cmd, ((opt) ? opt : ""));
 		}
 	    }
@@ -430,15 +482,16 @@ LoadDiabloConfig(int ac, char **av)
 		exitMe = 1;
 		fprintf(stderr, "Unknown diablo.config command: %s %s\n",
 						cmd, ((opt) ? opt : ""));
-		syslog(LOG_CRIT, "Unknown diablo.config command: %s %s",
+		logit(LOG_CRIT, "Unknown diablo.config command: %s %s",
 						cmd, ((opt) ? opt : ""));
 	    }
 	}
 	fclose(fi);
+	CheckConfigOptions(0);
     } else {
 	fprintf(stderr, "Unable to open diablo.config using pattern %s\n",
 							diabloConfig);
-	syslog(LOG_CRIT, "Unable to open diablo.config using pattern %s",
+	logit(LOG_CRIT, "Unable to open diablo.config using pattern %s",
 							diabloConfig);
 	exitMe = 1;
     }
@@ -460,7 +513,13 @@ SetCommand(FILE *fo, char *cmd, char *opt, int *cmdErr)
 
     if (strcasecmp(cmd, "remember") == 0) {
 	    if (opt) {
-		DOpts.RememberDays = strtol(opt, NULL, 0);
+		DOpts.RememberSecs = TimeSpec(opt, "d");
+		DOpts.FeederMaxAcceptAge = DOpts.RememberSecs;
+		optErr = 0;
+	    }
+    } else if (strcasecmp(cmd, "feedermaxacceptage") == 0) {
+	    if (opt) {
+		DOpts.FeederMaxAcceptAge = TimeSpec(opt, "d");
 		optErr = 0;
 	    }
     } else if (strcasecmp(cmd, "maxconnect") == 0) {
@@ -535,6 +594,18 @@ SetCommand(FILE *fo, char *cmd, char *opt, int *cmdErr)
 		DOpts.ReaderCacheMode = enabled(opt);
 		optErr = 0;
 	    }
+	} else if (strcasecmp(cmd, "readercachehashsize") == 0) {
+	    if (opt) {
+		int n = bsizetol(opt);
+
+		if (n<=0) {
+		    fprintf(stderr, "Hash size should be greater than 0 (%d)\n", n);
+		    logit(LOG_CRIT, "Hash size should be greater than 0 (%d)", n);
+		} else {
+		    DOpts.ReaderCacheHashSize = n;
+		    optErr = 0;
+		}
+	    }
 	} else if (strcasecmp(cmd, "readerxover") == 0) {
 	    if (opt) {
 		if (strcasecmp(opt, "trackonly") == 0) {
@@ -556,7 +627,7 @@ SetCommand(FILE *fo, char *cmd, char *opt, int *cmdErr)
 		    DOpts.FeederRTStats = RTSTATS_HOST;
 		else {
 		    fprintf(fo, "Unknown feedertstats option: '%s'\n", opt);
-		    syslog(LOG_CRIT, "Unknown feederrtstats option: '%s'\n",
+		    logit(LOG_CRIT, "Unknown feederrtstats option: '%s'\n",
 								opt);
 		}
 		optErr = 0;
@@ -618,7 +689,7 @@ SetCommand(FILE *fo, char *cmd, char *opt, int *cmdErr)
 		optErr = SetCacheDirs(opt, &DOpts.ReaderCacheDirs);
 		if (optErr != 0) {
 		    fprintf(fo, "Illegal reader cache dir format\n");
-		    syslog(LOG_CRIT, "Illegal reader cache dir format");
+		    logit(LOG_CRIT, "Illegal reader cache dir format");
 		}
 	    }
 	} else if (strcasecmp(cmd, "newsadmin") == 0) {
@@ -654,7 +725,7 @@ SetCommand(FILE *fo, char *cmd, char *opt, int *cmdErr)
 		optErr = 0;
 		if ((n ^ (n - 1)) != (n << 1) - 1) {
 		    fprintf(stderr, "Buffer size not a power of 2: %d\n", n);
-		    syslog(LOG_CRIT, "Buffer size not a power of 2: %d", n);
+		    logit(LOG_CRIT, "Buffer size not a power of 2: %d", n);
 		    optErr = 1;
 		}
 		if (optErr == 0)
@@ -665,6 +736,42 @@ SetCommand(FILE *fo, char *cmd, char *opt, int *cmdErr)
 		DOpts.FeederMaxArtSize = bsizetol(opt);
 		optErr = 0;
 	    }
+	} else if (strcasecmp(cmd, "readermaxartsize") == 0) {
+	    if (opt) {
+		DOpts.ReaderMaxArtSize = bsizetol(opt);
+		optErr = 0;
+	    }
+	} else if (strcasecmp(cmd, "feederarttypes") == 0) {
+	    if (opt) {
+		DOpts.FeederArtTypes = enabled(opt);
+		optErr = 0;
+	    }
+	} else if (strcasecmp(cmd, "feederpreloadart") == 0) {
+	    if (opt) {
+		DOpts.FeederPreloadArt = enabled(opt);
+		optErr = 0;
+	    }
+	} else if (strcasecmp(cmd, "spoolpreloadart") == 0) {
+	    if (opt) {
+		DOpts.SpoolPreloadArt = enabled(opt);
+		optErr = 0;
+	    }
+	} else if (strcasecmp(cmd, "pgpverifyargs") == 0) {
+	    if (opt) {
+		char *p = opt;
+		int i = 0;
+		char path[PATH_MAX];
+
+		snprintf(path, sizeof(path), PGP_VERIFY_PATH, NewsHome);
+		strdupfree(&DOpts.PgpVerifyArgs[i++], path, NULL);
+		strdupfree(&DOpts.PgpVerifyArgs[i++], PGP_VERIFY_ARG0, NULL);
+		for (p = strtok(p, " \t"); p != NULL; p = strtok(NULL, " \t")) {
+		    if (i < 6)
+			strdupfree(&DOpts.PgpVerifyArgs[i++], p, NULL);
+		}
+		DOpts.PgpVerifyArgs[i] = NULL;
+		optErr = 0;
+	    }
     } else {
 	optErr = 0;
 	if (cmdErr != NULL)
@@ -717,14 +824,16 @@ dumpConfigFeeder(FILE *fo, char *cmd)
 	fprintf(fo, "feederxrefhost: %s\n",
 					safestr(DOpts.FeederXRefHost, NULL));
     if (cmd == NULL || strcasecmp(cmd, "internalfilter") == 0)
-	fprintf(fo, "internalfilter: %s\n",
+	fprintf(fo, "*internalfilter: %s\n",
 					safestr(DOpts.SpamFilterOpt, NULL));
     if (cmd == NULL || strcasecmp(cmd, "feederfilter") == 0)
 	fprintf(fo, "feederfilter: %s\n", safestr(DOpts.FeederFilter, NULL));
     if (cmd == NULL || strcasecmp(cmd, "rejectartswithnul") == 0)
 	fprintf(fo, "rejectartswithnul: %d\n", DOpts.RejectArtsWithNul);
     if (cmd == NULL || strcasecmp(cmd, "remember") == 0)
-	fprintf(fo, "remember: %d\n", DOpts.RememberDays);
+	fprintf(fo, "remember: %d\n", DOpts.RememberSecs / 24 / 60 / 60);
+    if (cmd == NULL || strcasecmp(cmd, "feedermaxacceptage") == 0)
+	fprintf(fo, "feedermaxacceptage: %d\n", DOpts.FeederMaxAcceptAge);
     if (cmd == NULL || strcasecmp(cmd, "maxconnect") == 0)
 	fprintf(fo, "maxconnect: %d\n", DOpts.MaxPerRemote);
     if (cmd == NULL || strcasecmp(cmd, "hostcacherebuildtime") == 0)
@@ -757,6 +866,8 @@ dumpConfigFeeder(FILE *fo, char *cmd)
 	fprintf(fo, "feederbuffersize: %d\n", DOpts.FeederBufferSize);
     if (cmd == NULL || strcasecmp(cmd, "feedermaxartsize") == 0)
 	fprintf(fo, "feedermaxartsize: %d\n", DOpts.FeederMaxArtSize);
+    if (cmd == NULL || strcasecmp(cmd, "feederarttypes") == 0)
+	fprintf(fo, "feederarttypes: %d\n", DOpts.FeederArtTypes);
 }
 
 void
@@ -804,6 +915,9 @@ dumpConfigReader(FILE *fo, char *cmd)
 	fprintf(fo, "readerthreads: %d\n", DOpts.ReaderThreads);
     if (cmd == NULL || strcasecmp(cmd, "readercache") == 0)
 	fprintf(fo, "readercache: %d\n", DOpts.ReaderCacheMode);
+    if (cmd == NULL || strcasecmp(cmd, "readercachehashsize") == 0) {
+	fprintf(fo, "readercachehashsize: %d\n", DOpts.ReaderCacheHashSize);
+    }
     if (cmd == NULL || strcasecmp(cmd, "readerxover") == 0) {
 	switch (DOpts.ReaderXOverMode) {
 	    case 0: fprintf(fo, "readerxover: off\n");
@@ -817,6 +931,8 @@ dumpConfigReader(FILE *fo, char *cmd)
     if (cmd == NULL || strcasecmp(cmd, "readercrash") == 0)
 	fprintf(fo, "readercrash: %s\n",
 				safestr(DOpts.ReaderCrashHandler, NULL));
+    if (cmd == NULL || strcasecmp(cmd, "readermaxartsize") == 0)
+	fprintf(fo, "readermaxartsize: %d\n", DOpts.ReaderMaxArtSize);
 #ifdef READER_BAN_LISTS
     if (cmd == NULL || strcasecmp(cmd, "readerban") == 0)
 	fprintf(fo, "readerban: %s\n",
@@ -862,6 +978,14 @@ CheckConfigOptions(int which)
 		SetMyHostName(&DOpts.ReaderHostName);
 	    break;
     }
+    if (DOpts.PgpVerifyArgs[0] == NULL) {
+	char path[PATH_MAX];
+
+	snprintf(path, sizeof(path), PGP_VERIFY_PATH, NewsHome);
+	strdupfree(&DOpts.PgpVerifyArgs[0], path, NULL);
+	strdupfree(&DOpts.PgpVerifyArgs[1], PGP_VERIFY_ARG0, NULL);
+	DOpts.PgpVerifyArgs[2] = NULL;
+    }
     return(0);
 }
 
