@@ -80,6 +80,9 @@
  *
  *	DIABLO_FILTER		Enable/Disable Joe Greco's diablo-filter 
  *				support (it is off by default).
+ *
+ *	MMAP_DOES_NOT_UPDATE_ATIME Enable a single read to update cache atime
+ *
  */
 
 /*
@@ -110,6 +113,8 @@
 #define HAS_UTIMES		1	/* human readability only	 */
 #define HAS_USLEEP		1	/* < 1 second sleeps		 */
 #define HAS_PROC_TITLE		1
+#define HAS_SA_LEN		1
+#define MMAP_DOES_NOT_UPDATE_ATIME 1	/* use read to update atime	 */
 #if __FreeBSD_version >= 300003
 #define USE_POLL		1	/* poll() syscall		 */
 #endif
@@ -189,8 +194,6 @@
 #define USE_STRERROR            1
 #define USE_MEMMOVE             1
 #define HAVE_SNPRINTF           0
-#define	USE_MMAP_BCOPY		1	/* Can only access 4 bytes on a
-					   4 byte boundary */
 
 #else
 
@@ -216,8 +219,6 @@
 #define NEED_TERMIOS		1	/* for TIOCNOTTY */
 #define USE_POLL		1	/* poll() syscall		 */
 #define HAS_USLEEP		1	/* < 1 second sleeps		 */
-#define	USE_MMAP_BCOPY		1	/* Can only access 4 bytes on a
-					   4 byte boundary */
 
 #define USE_ANON_MMAP		0
 #define USE_ZERO_MMAP		1
@@ -234,7 +235,17 @@
  */
 
 #ifdef __linux__
+#include <linux/version.h>
+#ifdef LINUX_VERSION_CODE
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
+#define USE_MADVISE           1       /* new (test) - premap article pte's */
+#endif  /* _KERNEL_VERSION >= 2.4.0 */
+
+#else
+#warning "linux/versionh was not found, perhaps you will have to set options"
+#warning "by hand in lib/config.h"
+#endif
 #define USE_SYSV_SETPGRP	1
 #define USE_SYS_VFS		1
 #define USE_PCOMMIT_SHM		1	/* use sharedmem r/w map vs file map */
@@ -301,6 +312,36 @@
 
 #endif
 
+/********************************************************
+ * 			OSF/Tru64			*
+ ********************************************************
+ *
+ */
+
+#ifdef __osf__
+
+#include <sys/bitypes.h>
+
+uint64_t		i;
+
+#define FDPASS_USES_CMSG	0	/* solaris uses a different	*/
+#define FDPASS_USES_ACC		1	/* structural field names	*/
+#define USE_MADVISE		1	/* new (test) - premap article pte's */
+#define USE_SYSV_DIR		1	/* use more portable dirent.h (1)*/
+#define USE_PCOMMIT_SHM		1	/* use sharedmem r/w map vs file map */
+#define USE_PCOMMIT_RW_MAP	1	/* use r/w map for pcommit cache */
+#define USE_SPAM_SHM		1	/* use sharedmem r/w map vs file map */
+#define USE_SPAM_RW_MAP		1	/* use r/w map for spam cache	 */
+#define USE_KP_RW_MAP		1	/* use r/w map for KPDB		 */
+#define USE_ANON_MMAP		1	/* use anonymous mmap for alloc  */
+#define	USE_INTERNAL_VSYSLOG	1
+#define DO_PCOMMIT_POSTCACHE	1	/* do precommit post caching	 */
+#define HAS_UTIMES		1	/* human readability only	 */
+#define HAS_USLEEP		1	/* < 1 second sleeps		 */
+#define HAVE_STRSEP		0
+#define USE_POLL		1	/* poll() syscall		 */
+#endif
+
 #include "lib/vendor.h"
 
 /************************************************************************
@@ -371,6 +412,9 @@
 #ifndef HAS_PROC_TITLE
 #define HAS_PROC_TITLE		0
 #endif
+#ifndef HAS_SA_LEN
+#define HAS_SA_LEN		0
+#endif
 #ifndef USE_SPAM_SHM
 #define USE_SPAM_SHM		1	/* required for lib/seq.c */
 #endif
