@@ -83,8 +83,19 @@ typedef struct direct den_t;
 #include	<md5.h>
 #endif
 #ifdef	POST_CRYPTXTRACE
+#if __FreeBSD_version > 470100
+#include        <openssl/des.h>
+#else
 #include	<des.h>
 #endif
+#endif
+#ifdef	NETREMOTE_ENABLED
+#if __FreeBSD_version > 470100
+#include        <openssl/des.h>
+#else
+#include	<des.h>
+#endif
+#endif
 #ifdef	RADIUS_ENABLED
 #include	<radlib.h>
 #endif
@@ -154,6 +165,12 @@ typedef struct CacheDirType {
 	int dt_dirinfo[10];	/* Number of dirs per level */
 } CacheDirType;
 
+typedef struct BindList {
+    struct BindList *bl_Next;
+    char        *bl_Port;
+    char        *bl_Host;
+} BindList;
+
 struct DiabloOpts {
     int HashMethod;
     int HashSize;
@@ -169,17 +186,23 @@ struct DiabloOpts {
     int ReaderFeedForks;
     int ReaderDns;
     int ReaderCacheMode;
+    int ReaderCacheHashSize;
     int ReaderXOverMode;
     int ReaderAutoAddToActive;
     int ReaderDetailLog;
-    int RememberDays;
+    int RememberSecs;
+    int FeederMaxAcceptAge;
     int MaxPerRemote;
     int HostCacheRebuildTime;
     int DisplayAdminVersion;
     int RejectArtsWithNul;
     int FeederBufferSize;
     int FeederMaxArtSize;
+    int ReaderMaxArtSize;
     int WireFormat;
+    int FeederArtTypes;
+    int FeederPreloadArt;
+    int SpoolPreloadArt;
     char *SpamFilterOpt;
     char *FeederPathHost;
     char *FeederXRefHost;
@@ -200,6 +223,7 @@ struct DiabloOpts {
     char *ReaderBan;
     CacheDirType ReaderCacheDirs;
     GroupHashType ReaderGroupHashMethod;
+    char *PgpVerifyArgs[10];
 } DiabloOpts;
 
 /*
@@ -225,7 +249,44 @@ typedef struct HistHead {
 } HistHead;
 
 #define HMAGIC		((uint32)0xA1B2C3D4)
-#define HVERSION	1
+#define HDEADMAGIC	((uint32)0xDEADF5E6)
+#define HVERSION	2
+
+/*
+ * Dreaderd cache scoreboard
+ *
+ * cache consist of :
+ * - an header (CacheHitHead)
+ * - an hash (uint32[chh_hashSize]) pointing to a list of CacheHitEntry
+ * - entries (CacheHitEntry)
+ */
+
+#define CHMAGIC		((uint32)0xD1C2B3A4)
+#define CHVERSION	1
+
+typedef struct CacheHitHead {
+    uint32	chh_magic;	/* 0xD1C2B3A4			*/
+    uint32	chh_version;	/* version of scoreboard	*/
+    uint32	chh_hashSize;	/* entries in hash table	*/
+    uint32	chh_newEntry;	/* next entry to add		*/
+    uint32	chh_end;	/* number of byte in mmap	*/
+    time_t	chh_lastExpired; /* last expiration date	*/
+} CacheHitHead;
+
+typedef struct CacheHash_t {		/* hash key for scoreboad cache	*/
+    uint64_t    h1;
+    uint64_t    h2;
+    int		iter;
+} CacheHash_t;
+
+typedef struct CacheHitEntry {
+    struct CacheHitHead	che_hash; /* group hash			*/
+    uint32	che_Next;	/* linking entries		*/
+    int		che_ReadArt;	/* number of read articles	*/
+    uint32	che_Hits;	/* number of cache hits		*/
+    uint32	che_LastHi;	/* to compute number of new articles */
+    uint32	che_NewArt;	/* number of new articles */
+} CacheHitEntry;
 
 /*
  * History file entry.  The filename is calculated from the gmt start
@@ -244,14 +305,16 @@ typedef struct HistHead {
  *  h.iter == -1 for rejected articles
  */
 
+typedef uint32  HistIndex;
+
 typedef struct History {
-    uint32 	next;	/* next link		*/
-    uint32	gmt;	/* expire slot number	*/
-    hash_t	hv;	/* hash value		*/
-    uint16 	iter;	/* file id 		*/
-    uint16	exp;	/* hours relative to gmt minutes */
-    uint32	boffset;/* starting offset in file	 */
-    int32	bsize;	/* size of file		*/
+    HistIndex 	next;		/* next link			*/
+    uint32	gmt;		/* expire slot number		*/
+    hash_t	hv;		/* hash value			*/
+    uint16 	iter;		/* file id 			*/
+    uint16	exp;		/* hours relative to gmt minutes */
+    uint32	boffset;	/* starting offset in file	*/
+    int32	bsize;		/* size of article		*/
 } History;
 
 #define EXPF_HEADONLY	0x8000	/* header-only flag (in exp field) */
@@ -262,6 +325,20 @@ typedef struct History {
 #define H_SPOOL(n)      ((uint16)(H_NOFLAGS(n) > 100 && H_NOFLAGS(n) < 500 ? \
 				H_NOFLAGS(n) - 100 : 0))
 
+
+/*
+ * Internal spamfilter
+ */
+
+typedef struct SpamInfo {
+    md5hash_t	BodyHash;
+    char	*PostingHost;
+    md5hash_t	PostingHostHash;
+    int		Lines;
+    char	*MsgId;
+    hash_t	MsgIdHash;
+} SpamInfo;
+
 /*
  * Pre/PostCommit cache
  */
@@ -271,9 +348,16 @@ typedef struct History {
 #define PC_DELCOMM		0x004
 
 /*
- * Article header - appended to all articles written to the spool.
- * The "header" is appended because we only know the size of the article
- * after it has been written to disk
+ * Article header - prepended to all articles written to the spool.
+ *
+ * All articles on spool consist of:
+ *	SpoolArtHdr
+ *	Article
+ *	Nul
+ *
+ *	SpoolArtHdr.StoreLen = SpoolArtHdr + Article + Nul
+ *	SpoolArtHdr.HeadLen  = SpoolArtHdr
+ *	SpoolArtHdr.ArtLen   = Article
  */
 
 #define	STORE_MAGIC1		0xff
@@ -294,7 +378,7 @@ typedef struct SpoolArtHdr {
     uint32 ArtHdrLen;		/* Size of stored article header */
     uint32 ArtLen;		/* Total article size (including \r\n) */
     uint32 StoreLen;		/* Size of article on disk */
-    uint8 HdrEnd;		/* Marks start of the article */
+    uint8 HdrEnd;		/* Nul */
 } SpoolArtHdr;
 
 /*
@@ -320,17 +404,19 @@ typedef struct NewslinkInfo {
     int			li_StartDelay;
     int			li_TransmitBuf;
     int			li_ReceiveBuf;
+    int			li_QoS;			/* 0 = default, don't set it */
     char		*li_BindAddress;
     int			li_MaxParallel;
     int			li_NoStream;
     int			li_DelayFeed;
+    int			li_DelayInFeed;
     int			li_RealTime;
     int			li_Notify;
     int			li_MaxQueueFile;
     int			li_HeadFeed;
     int			li_PreserveBytes;
     int			li_GenLines;
-    int			li_NoCheck;
+    int			li_Check;
     int			li_MaxStream;
     int			li_Priority;
     int			li_Compress;
@@ -377,6 +463,7 @@ typedef struct HashFeed {
 #define HGF_NOSEARCH	0x02
 #define HGF_MLOCK	0x03
 #define HGF_READONLY	0x04
+#define HGF_EXCHECK	0x08
 
 typedef struct Node {
     struct Node		*no_Next;
@@ -387,7 +474,7 @@ typedef struct Node {
 } Node;
 
 #define MAXLINE		16384
-#define MAXMSGIDLEN	512
+#define MAXMSGIDLEN	250
 #define MAXGNAME	256
 #define MAXFORKS	512
 #define MAXFEEDS	128
@@ -418,6 +505,9 @@ typedef struct GroupList {
 #define SPOOL_ALLOC_NONE	0x00
 #define SPOOL_ALLOC_SEQ		0x01	/* Default sequential alloc */
 #define SPOOL_ALLOC_SPACE	0x02	/* Choose spool with most space free */
+#define SPOOL_ALLOC_SINGLE	0x03	/* Write all feeds to a single spool */
+
+#define	SPOOL_MAX_FILE_SIZE	1073741824	/* Max size of spool files */
 
 /*
  * Expire methods
@@ -446,7 +536,7 @@ typedef struct MetaSpool {
     long		ms_KeepTime;		/* secs */
     ArtTypeList		*ms_ArtTypes;
     HashFeed		ms_HashFeed;
-    char		*ms_Label;
+    LabelList		*ms_Label;
     int			ms_RejectArts;
     int			ms_DontStore;
     int			ms_AllocationStrategy;
@@ -646,6 +736,18 @@ typedef struct Buffer {
     char		bu_Buf[1024];	/* included buffer		*/
 } Buffer;
 
+typedef struct LogInfo {
+    const char		**Pat;
+    const char		*Ident;
+    FILE		*Fd;
+    time_t		NextCheck;
+    int			UseSyslog;
+    int			Disabled;
+    int			LastInode;
+    int			Pid;
+    char		Fname[PATH_MAX];
+} LogInfo;
+
 
 #define XADV_WILLNEED	1
 #define XADV_SEQUENTIAL	2
@@ -675,6 +777,9 @@ typedef struct Buffer {
 #define		ARTTYPE_BINHEX		0x008000
 #define		ARTTYPE_PARTIAL		0x010000
 #define		ARTTYPE_PGPMESSAGE	0x020000
+#define		ARTTYPE_YENC		0x040000
+#define		ARTTYPE_BOMMANEWS	0x080000
+#define		ARTTYPE_UNIDATA		0x100000
 
 #define		ARTTYPE_ALL		0xFFFFFF
 
@@ -702,6 +807,34 @@ IPList *IPHash;
 #define INADDR_NONE	((unsigned long)-1)
 #endif
 
+#ifndef NI_MAXHOST
+#ifdef INET6
+#define	NI_MAXHOST	1025
+#else
+#define	NI_MAXHOST	256
+#endif
+#endif
+
+/*
+ * This is very messy because sa_len doesn't exist as part of the sockaddr
+ * structure on some OS's. So much for getting it right with IPv6.
+ */
+#ifdef INET6
+#if HAS_SA_LEN
+#define SA_LEN(x) ( (x)->sa_len )
+#else
+#define SA_LEN(x) ( \
+	((x)->sa_family == AF_INET) ? sizeof(struct sockaddr_in) : \
+	((x)->sa_family == AF_INET6) ? sizeof(struct sockaddr_in6) : \
+	0)
+#endif	/* HAS_SA_LEN */
+#else
+#define SA_LEN(x) ( \
+	((x)->sa_family == AF_INET) ? sizeof(struct sockaddr_in) : \
+	0)
+#endif	/* INET6 */
+
+
 #ifndef TEST
 #include "obj/lib-protos.h"
 #endif
