@@ -154,7 +154,7 @@ LoadExpireCtl(int force)
 	    fi = fopen(PatLibExpand(DExpireCtlPat), "r");
 
 	    if (fi == NULL) {
-		syslog(LOG_EMERG, "%s file not found",
+		logit(LOG_CRIT, "%s file not found",
 					PatLibExpand(DExpireCtlPat));
 		exit(1);
 	    }
