@@ -21,10 +21,7 @@ fatal(const char *ctl, ...)
     vsnprintf(buf, sizeof(buf), ctl, va);
     va_end(va);
     fprintf(stderr, "%s\n", buf);
-
-    va_start(va, ctl);
-    vsyslog(LOG_CRIT, ctl, va);
-    va_end(va);
+    logit(LOG_ERR, "%s", buf);
 
     exit(1);
 }
