@@ -43,12 +43,14 @@ Prototype const char *ServerDActivePat;		/* db relative	*/
 Prototype const char *ReaderDActivePat;		/* db relative	*/
 Prototype const char *DHistoryPat;		/* db relative	*/
 Prototype const char *DumpHistPat;		/* db relative	*/
-Prototype const char *SpamCachePat;		/* db relative	*/
+Prototype const char *SpamBodyCachePat;		/* db relative	*/
+Prototype const char *SpamNphCachePat;		/* db relative	*/
 Prototype const char *PCommitCachePat;		/* db relative	*/
 Prototype const char *DExpireOverListPat;	/* db relative  */
 Prototype const char *DHostsCachePat;		/* db relative  */
 Prototype const char *DHostsLockPat;		/* db relative  */
 Prototype const char *DFeedStatsPat;		/* db relative  */
+Prototype const char *CacheHitsPat;		/* db relative  */
 
 Prototype const char *DRVserverCachePat;	/* db relative  */
 Prototype const char *DRGroupCachePat;		/* db relative  */
@@ -58,9 +60,11 @@ Prototype const char *DRReaderCachePat;		/* db relative  */
 Prototype const char *DRAccessCachePat;		/* db relative  */
 Prototype const char *DRAccessLockPat;		/* db relative  */
 
+Prototype const char *GeneralLogPat;		/* log relative */
 Prototype const char *IncomingLogPat;		/* log relative */
 Prototype const char *DRIncomingLogPat;		/* log relative */
 Prototype const	char *FPathLogPat;		/* log relative */
+Prototype const	char *FArtLogPat;		/* log relative */
 
 Prototype const char *ShutdownCleanup;
 
@@ -95,10 +99,12 @@ const char *DReaderAccessPat = "%s/dreader.access";
 const char *VServerPat = "%s/dreader.virtuals"; 
 const char *DServerHostsPat = "%s/dserver.hosts";
 const char *ModeratorsPat = "%s/moderators";
-const char *SpamCachePat = "%s/spam.cache";
+const char *SpamBodyCachePat = "%s/spam.body.cache";
+const char *SpamNphCachePat = "%s/spam.nph.cache";
 const char *PCommitCachePat = "%s/pcommit.cache";
 const char *DExpireOverListPat = "%s/dexpover.dat";
 const char *DHostsCachePat = "%s/dhosts.cache";
+const char *CacheHitsPat = "%s/cache.hits";
 const char *DHostsLockPat = "%s/.hostslock";
 const char *DFeedStatsPat = "%s/feedstats";
 const char *DNewsfeedsPat = "%s/dnewsfeeds";
@@ -114,9 +120,11 @@ const char *DRReaderCachePat = "%s/cache.readerdef";
 const char *DRAccessCachePat = "%s/cache.readeraccess";
 const char *DRAccessLockPat = "%s/.accesslock";
 
+const char *GeneralLogPat = "SYSLOG";
 const char *IncomingLogPat = "%s/incoming.log";
 const char *DRIncomingLogPat = "NONE";
 const char *FPathLogPat = "NONE";
+const char *FArtLogPat = "NONE";
 
 const char *ShutdownCleanup = "NONE";
 
