@@ -35,6 +35,7 @@ Prototype const char *GFName(const char *group, int ftype, int artBase, int want
 void FindGroupDir(const char *group, char *path);
 Prototype int ExtractGroupHashInfo(const char *name, char *Hash, long *artBase, int *iter);
 Prototype char *GetHash(GroupHashType *Hash, char *hashStr);
+Prototype void SetCacheHash(struct CacheHash_t* ch, const char *group, int iter, GroupHashType *method);
 
 #define P1	7834891
 #define P2	6017489
@@ -60,9 +61,10 @@ int
 quickhash(const char *st)
 {
     register int rval = 0;
+    register unsigned char *p = (unsigned char *)st;
 
-    while (*st)
-	rval += *st++;
+    while (*p)
+	rval += *p++;
     return(rval);
 }
 
@@ -326,7 +328,7 @@ GFHash(const char *group, GroupHashType *hashtype)
     switch (hashtype->gh_type) {
 	case HASHGRP_CRC: {
 	    hash_t hv = hhash(group);
-	    snprintf(GFBuf, hashtype->gh_sigbytes, "%08lx.%08lx", (long)hv.h1, (long)hv.h2 );
+	    snprintf(GFBuf, hashtype->gh_sigbytes, "%08x.%08x", (uint32)hv.h1, (uint32)hv.h2 );
 	    break;
 	}
 	case HASHGRP_32MD5: case HASHGRP_64MD5: {
@@ -385,8 +387,8 @@ GFName(const char *group, int gftype, int artBase, int wantdir, int iter, GroupH
 	else
 	    itbuf[0] = 0;
 	    if (wantdir)
-		snprintf(GFBuf, sizeof(GFBuf), "%02lx/%s.%d.%s%s",
-						(long)(uint8)hv.h1,
+		snprintf(GFBuf, sizeof(GFBuf), "%02x/%s.%d.%s%s",
+						(uint8)hv.h1,
 						ftype,
 						artBase,
 						hashstr,
@@ -499,6 +501,28 @@ GFName(const char *group, int gftype, int artBase, int wantdir, int iter, GroupH
     return(GFBuf);
 }
 
+void
+SetCacheHash(struct CacheHash_t* ch, const char *group, int iter, GroupHashType *method)
+{
+    ch->iter = iter;
+    switch (method->gh_type) {
+	case HASHGRP_CRC: {
+	    hash_t hv = hhash(group);
+	    ch->h1 = (uint64_t) hv.h1;
+	    ch->h2 = (uint64_t) hv.h2;
+	    break;
+	}
+	case HASHGRP_32MD5:
+	case HASHGRP_64MD5:
+	case HASHGRP_HIER: { /* we just hope there will be no collision */
+	    md5hash_t *hv = md5hash(group);
+	    ch->h1 = hv->h1;
+	    ch->h2 = hv->h2;
+	    break;
+	}
+    }
+}
+
 void
 FindGroupDir(const char *group, char *path)
 {
@@ -631,7 +655,7 @@ main(int ac, char **av)
 
     for (i = 1; i < ac; ++i) {
 	hash_t hv = hhash(av[i]);
-	printf("%08lx.%08lx\t%s\n", hv.h1, hv.h2, av[i]);
+	printf("%08x.%08x\t%s\n", hv.h1, hv.h2, av[i]);
     }
     return(0);
 }
