@@ -30,12 +30,16 @@
 
 #include "defs.h"
 
-Prototype void HistoryOpen(const char *fileName, int fastMode);
-Prototype int  HistoryClose(void);
+Prototype int HistoryOpen(const char *fileName, int fastMode);
+Prototype int HistoryClose(void);
 Prototype int HistoryLookup(const char *msgid, History *h);
 Prototype int HistoryLookupByHash(hash_t hv, History *h);
-Prototype int32 HistoryPosLookupByHash(hash_t hv, History *h);
+Prototype HistIndex HistoryPosLookupByHash(hash_t hv, History *h);
 Prototype int HistoryAdd(const char *msgid, History *h);
+Prototype int HistoryStore(History *h);
+Prototype void HistoryStoreExp(History *h, HistIndex index);
+Prototype int HistoryExpire(const char *msgid, History *h, int unexp);
+Prototype void PrintHistory(History *h);
 
 Prototype int NewHSize;
 
@@ -43,24 +47,32 @@ Prototype int NewHSize;
 #define HBLKSIZE	256
 
 HistHead	HHead;
-uint32 		*HAry;
+HistHead	*HHeadMap;
+HistIndex 	*HAry;
 int		HFd = -1;
 int		LoggedDHistCorrupt;
 int		HFlags;
 int		HSize;
 int		HMask;
-off_t		HScanOff;
+off_t		HEntryOff;		/* Start of actual history records */
 int		NewHSize = 0;
 int		HBlkIncr = HBLKINCR;
 int		HBlkGood;
+char		HistoryFileName[PATH_MAX];
+int		DoingReOpen = 0;
 
-void
+int
 HistoryOpen(const char *fileName, int hflags)
 {
     int fd;
     struct stat st;
     int openflags;
 
+    if (fileName == NULL)
+	fileName = strdup(PatDbExpand(DHistoryPat));
+
+    strcpy(HistoryFileName, fileName);
+
     HFlags = hflags;
 
     if (NewHSize == 0)
@@ -75,21 +87,25 @@ HistoryOpen(const char *fileName, int hflags)
     else
 	openflags = O_RDWR|O_CREAT;
 
-    if (fileName)
-	fd = open(fileName, openflags, 0644);
-    else
-	fd = open(PatDbExpand(DHistoryPat), openflags, 0644);
+    if (HFlags & HGF_EXCHECK && stat(fileName, &st) == 0) {
+	fprintf(stderr, "%s in fast mode already exists - exiting\n",
+						fileName);
+	exit(1);
+    }
+    fd = open(fileName, openflags, 0644);
 
     if (fd < 0) {
-	syslog(LOG_ERR, "open %s failed (%s)", PatDbExpand(DHistoryPat),
+	if (DoingReOpen)
+	    return(-2);
+	logit(LOG_ERR, "open %s failed (%s)", fileName, 
 							strerror(errno));
-	fprintf(stderr, "open %s failed (%s)", PatDbExpand(DHistoryPat),
+	fprintf(stderr, "open %s failed (%s)", fileName,
 							strerror(errno));
 	exit(1);
     }
 
     if (fstat(fd, &st) < 0) {
-	syslog(LOG_ERR, "fstat %s failed", PatDbExpand(DHistoryPat));
+	logit(LOG_ERR, "fstat %s failed", fileName);
 	exit(1);
     }
 
@@ -103,6 +119,14 @@ HistoryOpen(const char *fileName, int hflags)
 	read(fd, &HHead, sizeof(HHead)) != sizeof(HHead) ||
 	HHead.hmagic != HMAGIC
     ) {
+	/*
+	 * Is this history an old one and we need to wait for a new one?
+	 */
+	if (DoingReOpen && HHead.hmagic == HDEADMAGIC) {
+	    close(fd);
+	    return(-2);
+	}
+
 	/*
 	 * lock after finding the history file to be invalid and recheck.
 	 */
@@ -125,15 +149,15 @@ HistoryOpen(const char *fileName, int hflags)
 	     */
 
 	    if (st.st_size) {
-		syslog(LOG_ERR, "Incompatible history file version or corrupted history file\n");
+		logit(LOG_ERR, "Incompatible history file version or corrupted history file\n");
 		exit(1);
 	    }
 
 	    if (HFlags & HGF_READONLY) {
-		syslog(LOG_ERR, "Unable to create %s - read-only mode\n",
-						PatDbExpand(DHistoryPat));
+		logit(LOG_ERR, "Unable to create %s - read-only mode\n",
+							fileName);
 		fprintf(stderr, "Unable to create %s - read-only mode\n",
-						PatDbExpand(DHistoryPat));
+							fileName);
 		exit(1);
 	    }
 
@@ -141,7 +165,7 @@ HistoryOpen(const char *fileName, int hflags)
 	     * create new history file
 	     */
 
-	    syslog(LOG_INFO, "Creating history file");
+	    logit(LOG_INFO, "Creating history file");
 
 	    lseek(fd, 0L, 0);
 	    ftruncate(fd, 0);
@@ -159,7 +183,7 @@ HistoryOpen(const char *fileName, int hflags)
 	     */
 
 	    n = 0;
-	    b = HHead.hashSize * sizeof(int32);
+	    b = HHead.hashSize * sizeof(HistIndex);
 
 	    while (n < b) {
 		int r = (b - n > 4096) ? 4096 : b - n;
@@ -167,6 +191,14 @@ HistoryOpen(const char *fileName, int hflags)
 		write(fd, z, r);
 		n += r;
 	    }
+	    /*
+	     * Write out a dummy history entry so we don't use zero offset
+	     */
+	    if (HHead.version > 1) {
+		History h = { 0 };
+		write(fd, &h, sizeof(h));
+	    }
+
 	    fsync(fd);
 
 	    /*
@@ -178,14 +210,14 @@ HistoryOpen(const char *fileName, int hflags)
 	    write(fd, &HHead, sizeof(HHead));
 
 	    free(z);
-	    syslog(LOG_INFO, "History file creation complete");
+	    logit(LOG_INFO, "History file creation complete");
 	}
 
 	hflock(fd, 0, XLOCK_UN);
     }
 
-    if (HHead.version < HVERSION) {
-	syslog(LOG_ERR, "DHistory version %d, expected %d, use the biweekly adm script to regenerate the history file",
+    if (HHead.version < 1) {
+	logit(LOG_ERR, "DHistory version %d, expected %d, use the biweekly adm script to regenerate the history file",
 	    HHead.version,
 	    HVERSION
 	);
@@ -196,12 +228,28 @@ HistoryOpen(const char *fileName, int hflags)
 	exit(1);
     }
     if (HHead.version > HVERSION) {
-	syslog(LOG_ERR, "DHistory version %d, expected %d, YOU ARE RUNNING AN OLD DIABLO ON A NEW HISTORY FILE!",
+	logit(LOG_ERR, "DHistory version %d, expected %d, YOU ARE RUNNING AN OLD DIABLO ON A NEW HISTORY FILE!",
 	    HHead.version,
 	    HVERSION
 	);
 	exit(1);
     }
+    if (HHead.hmagic != HMAGIC) {
+	logit(LOG_NOTICE, "DHistory magic mismatch");
+	fprintf(stderr, "DHistory magic mismatch\n");
+    }
+
+    /*
+     * Map the history header so that we can check it quickly
+     */
+
+    HHeadMap = xmap(NULL, HHead.headSize, PROT_READ, MAP_SHARED, fd, 0);
+    if (HHeadMap == 0) {
+	if (fd >= 0)
+	    close(fd);
+	logit(LOG_CRIT, "dhistory header mmap error: %s", strerror(errno));
+	exit(1);
+    }
 
     /*
      * Map history file
@@ -221,34 +269,46 @@ HistoryOpen(const char *fileName, int hflags)
      * portion of the history file.
      */
 
-    if (HFlags & HGF_FAST)
-	hflock(fd, 0, XLOCK_EX);
+    if (HFlags & HGF_FAST) {
+	if (HFlags & HGF_READONLY) {
+	    fprintf(stderr, "Ignoring fast mode for readonly history open\n");
+	} else if (hflock(fd, 0, XLOCK_EX|XLOCK_NB) == -1) {
+	    fprintf(stderr, "Unable to lock history file in fast mode (%s)\n",
+							strerror(errno));
+	    exit(1);
+	}
+    }
 
     if (HFlags & HGF_FAST) {
-	HAry = calloc(HSize, sizeof(int32));
+	HAry = calloc(HSize, sizeof(HistIndex));
 	if (HAry == NULL) {
 	    perror("calloc");
 	    exit(1);
 	}
 	lseek(fd, HHead.headSize, 0);
-	if (read(fd, HAry, HSize * sizeof(int32)) != HSize * sizeof(int32)) {
+	if (read(fd, HAry, HSize * sizeof(HistIndex)) != HSize * sizeof(HistIndex)) {
 	    perror("read");
 	    exit(1);
 	}
     } else {
-	HAry = xmap(NULL, HSize * sizeof(int32), PROT_READ, MAP_SHARED, fd, HHead.headSize);
+	int mapflags = PROT_READ;
+
+	if ((HFlags & HGF_READONLY) == 0)
+	    mapflags |= PROT_WRITE;
+	HAry = xmap(NULL, HSize * sizeof(HistIndex), mapflags, MAP_SHARED, fd, HHead.headSize);
 	if (HFlags & HGF_MLOCK)
-	    mlock(HAry, HSize * sizeof(int32));
+	    mlock(HAry, HSize * sizeof(HistIndex));
     }
 
-    if (HAry == NULL || HAry == (uint32 *)-1) {
+    if (HAry == NULL || HAry == (HistIndex *)-1) {
 	if (fd >= 0)
 	    close(fd);
-	syslog(LOG_CRIT, "dhistory mmap error: %s", strerror(errno));
+	logit(LOG_CRIT, "dhistory mmap error: %s", strerror(errno));
 	exit(1);
     }
     HFd = fd;
-    HScanOff = HHead.headSize + HHead.hashSize * sizeof(int32);
+    HEntryOff = HHead.headSize + HHead.hashSize * sizeof(HistIndex);
+    return(0);
 }
 
 /*
@@ -265,21 +325,24 @@ HistoryClose(void)
     if (HFd >= 0 && !(HFlags & HGF_READONLY)) {
 	if (HFlags & HGF_FAST) {
 	    lseek(HFd, HHead.headSize, 0);
-	    if (write(HFd, HAry, HSize * sizeof(int32)) != HSize * sizeof(int32))
+	    if (write(HFd, HAry, HSize * sizeof(HistIndex)) != HSize * sizeof(HistIndex))
 		r = RCTRYAGAIN;
 	    else
 		free(HAry);
 	} else {
-	    if (HAry && HAry != (uint32 *)-1) {
+	    if (HAry && HAry != (HistIndex *)-1) {
 		if (HFlags & HGF_MLOCK)
-		    munlock(HAry, HSize * sizeof(int32));
-		xunmap((void *)HAry, HSize * sizeof(int32));
+		    munlock(HAry, HSize * sizeof(HistIndex));
+		xunmap((void *)HAry, HSize * sizeof(HistIndex));
 	    }
 	}
     }
     if (r == RCOK) {
-	if (HFd >= 0)
+	if (HFd >= 0) {
+	    if (HFlags & HGF_FAST)
+		hflock(HFd, 0, XLOCK_UN);
 	    close(HFd);
+	}
 	HFd = -1;
 	HAry = NULL;
     }
@@ -287,7 +350,24 @@ HistoryClose(void)
 }
 
 void
-printHistory(History *h)
+historyReOpen(void)
+{
+    int count = 0;
+
+    DoingReOpen = 1;
+    HistoryClose();
+    while (HistoryOpen(HistoryFileName, HFlags) == -2 && count++ < 360)
+	sleep(1);
+    if (count++ >= 360) {
+	fprintf(stderr, "Unable to reopen history file due to old magic\n");
+	logit(LOG_ERR, "Unable to reopen history file due to old magic");
+	exit(1);
+    }
+    DoingReOpen = 0;
+}
+
+void
+PrintHistory(History *h)
 {
     printf(" [hv=%08x.%08x gm=%d ex=%d off=%d len=%d F=%s]\n",
 		h->hv.h1,
@@ -303,42 +383,59 @@ printHistory(History *h)
 int
 HistoryLookup(const char *msgid, History *nh)
 {
-    hash_t hv = hhash(msgid);
-    int32 poff = HHead.headSize + ((hv.h1 ^ hv.h2) & HMask) * sizeof(int32);
-    int32 off = HAry[(hv.h1 ^ hv.h2) & HMask];
+    hash_t hv;
+    HistIndex hi;
+    HistIndex pindex;
+    HistIndex index;
+    off_t off;
     History h = { 0 };
-    static int HLAlt;
+    static int HLAlt = 0;
     int r = -1;
     int counter = 0;
     int statfailed = 0;
 
-    while (off) {
-	lseek(HFd, (off_t)(uint32)off, 0);
+    if (HHeadMap->hmagic != HMAGIC)
+	historyReOpen();
+
+    hv = hhash(msgid);
+    hi = (hv.h1 ^ hv.h2) & HMask;
+    pindex = HHead.headSize + hi * sizeof(HistIndex);
+    index = HAry[hi];
+
+    while (index) {
+	if (HHead.version > 1)
+	    off = (off_t)HEntryOff + (off_t)index * sizeof(History);
+	else 
+	    off = index;
+	lseek(HFd, off, 0);
 	if (read(HFd, &h, sizeof(h)) != sizeof(h)) {
 	    if ((LoggedDHistCorrupt & 1) == 0 || DebugOpt) {
 		LoggedDHistCorrupt |= 1;
-		syslog(LOG_ERR, "dhistory file corrupted on lookup @ %d->%d chain %d", (int)poff, (int)off, (int)((hv.h1 ^ hv.h2) & HMask));
+		logit(LOG_ERR, "dhistory file corrupted on lookup @ %d->%d chain %d  offset %ld  msgid %s  counter=%d",
+					pindex, index,
+					(int)((hv.h1 ^ hv.h2) & HMask),
+					off, msgid, counter);
 		sleep(1);
 	    }
 	    break;
 	}
 	if (h.hv.h1 == hv.h1 && h.hv.h2 == hv.h2)
 	    break;
-	poff = off;
-	off = h.next;
+	pindex = index;
+	index = h.next;
 	if (counter++ > 5000) {
-	    syslog(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d (%s)",
-			(int)poff, (int)off, (int)((hv.h1 ^ hv.h2) & HMask), msgid);
-	    off = 0;
+	    logit(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d (%s)",
+					(int)pindex, (int)index,
+					(int)((hv.h1 ^ hv.h2) & HMask),
+					msgid);
+	    index = 0;
 	}
     }
-    if (off != 0)
+    if (index != 0)
 	r = 0;
-
     /*
      * On failure, try alternate hash method (for lookup only)
      */
-
     if (r < 0 && DOpts.CompatHashMethod >= 0 && HLAlt == 0 && !statfailed) {
 	int save = DOpts.HashMethod;
 
@@ -358,309 +455,92 @@ HistoryLookup(const char *msgid, History *nh)
 int
 HistoryLookupByHash(hash_t hv, History *h)
 {
-    int32 poff = HHead.headSize + ((hv.h1 ^ hv.h2) & HMask) * sizeof(int32);
-    int32 off = HAry[(hv.h1 ^ hv.h2) & HMask];
+    HistIndex hi;
+    HistIndex pindex;
+    HistIndex index;
+    off_t off;
     int counter = 0;
 
-    while (off) {
-	lseek(HFd, (off_t)(uint32)off, 0);
+    if (HHeadMap->hmagic != HMAGIC)
+	historyReOpen();
+
+    hi = (hv.h1 ^ hv.h2) & HMask;
+    pindex = HHead.headSize + hi * sizeof(HistIndex);
+    index = HAry[hi];
+
+    while (index) {
+	if (HHead.version > 1)
+	    off = (off_t)HEntryOff + (off_t)index * sizeof(History);
+	else 
+	    off = index;
+	lseek(HFd, off, 0);
 	if (read(HFd, h, sizeof(*h)) != sizeof(*h)) {
 	    if ((LoggedDHistCorrupt & 1) == 0 || DebugOpt) {
 		LoggedDHistCorrupt |= 1;
-		syslog(LOG_ERR, "dhistory file corrupted on lookup");
+		logit(LOG_ERR, "dhistory file corrupted on lookup");
 		sleep(1);
 	    }
 	    break;
 	}
 	if (h->hv.h1 == hv.h1 && h->hv.h2 == hv.h2)
 	    break;
-	poff = off;
-	off = h->next;
+	pindex = index;
+	index = h->next;
 	if (counter++ > 5000) {
-	    syslog(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d",
-			(int)poff, (int)off, (int)((hv.h1 ^ hv.h2) & HMask));
-	    off = 0;
+	    logit(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d",
+			(int)pindex, (int)index, (int)((hv.h1 ^ hv.h2) & HMask));
+	    index = 0;
 	}
     }
-    if (off != 0) {
+    if (index != 0) {
 	return(0);
     }
     return(-1);
 }
 
-int32
+HistIndex
 HistoryPosLookupByHash(hash_t hv, History *h)
 {
-    int32 poff = HHead.headSize + ((hv.h1 ^ hv.h2) & HMask) * sizeof(int32);
-    int32 off = HAry[(hv.h1 ^ hv.h2) & HMask];
+    HistIndex hi;
+    HistIndex pindex;
+    HistIndex index;
+    off_t off;
     int counter = 0;
 
-    while (off) {
-	lseek(HFd, (off_t)(uint32)off, 0);
+    hi = (hv.h1 ^ hv.h2) & HMask;
+    pindex = HHead.headSize + hi * sizeof(HistIndex);
+    index = HAry[hi];
+
+    while (index) {
+	if (HHead.version > 1)
+	    off = (off_t)HEntryOff + (off_t)index * sizeof(History);
+	else 
+	    off = index;
+	lseek(HFd, off, 0);
 	if (read(HFd, h, sizeof(*h)) != sizeof(*h)) {
 	    if ((LoggedDHistCorrupt & 1) == 0 || DebugOpt) {
 		LoggedDHistCorrupt |= 1;
-		syslog(LOG_ERR, "dhistory file corrupted on lookup");
+		logit(LOG_ERR, "dhistory file corrupted on lookup");
 		sleep(1);
 	    }
 	    break;
 	}
 	if (h->hv.h1 == hv.h1 && h->hv.h2 == hv.h2)
 	    break;
-	poff = off;
-	off = h->next;
+	pindex = index;
+	index = h->next;
 	if (counter++ > 5000) {
-	    syslog(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d",
-			(int)poff, (int)off, (int)((hv.h1 ^ hv.h2) & HMask));
-	    off = 0;
+	    logit(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d",
+			(int)pindex, (int)index, (int)((hv.h1 ^ hv.h2) & HMask));
+	    index = 0;
 	}
     }
-    if (off != 0) {
-	return(off);
+    if (index != 0) {
+	return(index);
     }
     return(-1);
 }
 
-int
-HistoryAddOld(const char *msgid, History *h)
-{
-    int32 hi = (h->hv.h1 ^ h->hv.h2) & HMask;
-    int32 poff = HHead.headSize + hi * sizeof(int32);
-    int32 boff = poff;
-    int r = RCOK;
-
-    /*
-     * This should not happen, but if it does, log it and pretend we did it
-     */
-    if (h->gmt == 0) {
-	syslog(LOG_ERR, "Not adding history entry with gmt=0");
-	return(RCOK);
-    }
-
-    if (HFlags & HGF_READONLY)
-	return(RCOK);
-
-    /*
-     * record lock, search for message id
-     *
-     */
-
-    if ((HFlags & HGF_FAST) == 0)	/* lock hash chain */
-	hflock(HFd, boff, XLOCK_EX);
-
-    /*
-     * make sure message is not already in hash table
-     */
-
-    {
-	int32 off;
-
-	off = HAry[hi];
-
-	if ((HFlags & HGF_NOSEARCH) == 0) {
-	    int counter = 0;
-
-	    while (off) {
-		History ht;
-
-		lseek(HFd, (off_t)(uint32)off, 0);
-		if (read(HFd, &ht, sizeof(ht)) != sizeof(ht)) {
-		    if ((LoggedDHistCorrupt & 2) == 0 || DebugOpt) {
-			LoggedDHistCorrupt |= 2;
-			syslog(LOG_ERR, "dhistory file corrupted @ %d", off);
-			sleep(1);
-		    }
-		    break;
-		}
-		if (ht.hv.h1 == h->hv.h1 && ht.hv.h2 == h->hv.h2) {
-		    r = RCALREADY;
-		    break;
-		}
-		poff = off;
-		off = ht.next;
-		if (counter++ > 5000) {
-		    syslog(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d (%s)",
-				(int)poff, (int)off,
-				(int)((ht.hv.h1 ^ ht.hv.h2) & HMask), msgid);
-		    off = 0;
-		}
-	    }
-	}
-    }
-
-    /*
-     * If we are ok, reserve space in the history file for the record
-     * and insert the record.  This may loop if intermediate scans determine
-     * that we must append a new block to the file.
-     */
-
-    while (r == RCOK) {
-	off_t offBeg;
-	off_t offEnd;
-	History hary[HBLKSIZE];
-
-	/*
-	 * append/scan lock
-	 */
-
-	if ((HFlags & HGF_FAST) == 0)	/* append/scan lock */
-	    hflock(HFd, 4, XLOCK_EX);
-
-	/*
-	 * Figure out scan start & end point.  We only look at the last
-	 * HBLKSIZE*2 records worth of entries in the worst case.
-	 */
-
-	offEnd = lseek(HFd, 0L, 2);
-	{
-	    off_t n;
-
-	    n = offEnd - HHead.headSize - HHead.hashSize * sizeof(int32);
-	    n = n % HHead.henSize;
-	    offEnd -= n;
-	}
-
-	offBeg = HScanOff;
-
-	if (offBeg > offEnd)
-	    offBeg = offEnd;
-
-	if (offBeg < offEnd - HBLKSIZE * sizeof(History)) {
-	    off_t n;
-
-	    offBeg = offEnd - HBLKSIZE * sizeof(History);
-	    n = offBeg - HHead.headSize - HHead.hashSize * sizeof(int32);
-	    n = n % HHead.henSize;
-	    offBeg -= n;
-	}
-
-	/*
-	 * Look for a blank record (gmt value is 0)
-	 */
-
-	lseek(HFd, offBeg, 0);
-
-	while (offBeg < offEnd) {
-	    int q = (offEnd - offBeg) / sizeof(History);
-	    int i;
-	    int n;
-
-	    if (q > arysize(hary))
-		q = arysize(hary);
-	    if (q > HBlkIncr)
-		q = HBlkIncr;
-
-	    n = read(HFd, hary, q * sizeof(History));
-
-	    if (n != q * sizeof(History)) {
-		r = RCTRYAGAIN;
-		break;
-	    }
-
-	    for (i = 0; r == RCOK && i < q; ++i) {
-		if (hary[i].gmt == 0 && 
-		    ((HFlags & HGF_FAST) ||
-		    hflock(HFd, offBeg, XLOCK_NB|XLOCK_EX) >= 0)
-		) {
-		    break;
-		}
-		offBeg += sizeof(History);
-	    }
-
-	    /*
-	     * Found, break, or not found, continue.  Adjust HBlkIncr
-	     * appropriately.
-	     */
-	    if (i != q) {
-		if (++HBlkGood == 4) {
-		    HBlkGood = 0;
-		    if ((HBlkIncr -= HBLKINCR) < HBLKINCR)
-			HBlkIncr = HBLKINCR;
-		}
-		break;
-	    }
-	    if (offBeg != offEnd) {
-		HBlkGood = 0;
-		if ((HBlkIncr += HBLKINCR) > HBLKSIZE) {
-		    HBlkIncr = HBLKSIZE;
-		}
-	    }
-	}
-
-	HScanOff = offBeg;
-
-	/*
-	 * break if something went wrong or if we found the record.  With
-	 * the proper record explicitly locked, we can release the append
-	 * lock.
-	 */
-
-	if (r != RCOK || offBeg != offEnd) {
-	    if ((HFlags & HGF_FAST) == 0)	/* append/scan unlock */
-		hflock(HFd, 4, XLOCK_UN);
-	    break;
-	}
-
-	/*
-	 * Otherwise append a new block and retry.  XXX put on filesystem
-	 * block boundry.
-	 */
-
-	lseek(HFd, 0L, 2);
-	bzero(hary, sizeof(hary));
-	write(HFd, hary, sizeof(hary));
-	if ((HFlags & HGF_FAST) == 0)
-	    hflock(HFd, 4, XLOCK_UN);
-    } /* WHILE */
-
-    /*
-     * If we did find a blank record, HScanOff is set to it.  The record
-     * is locked at this point.
-     *
-     * We insert the record into the chain at the beginning of the
-     * chain, even though we are appending the record.  This is for
-     * time-locality of reference and also reduces 'append-to-history'
-     * file overhead by localizing disk writes a bit better.
-     *
-     * XXX next/linkages are 32 bits.  At the moment we can go to 4G even
-     * though we are using a signed 32 bit int because we cast seeks to
-     * unsigned 32 bits.
-     */
-
-    if (r == RCOK) {
-	int n;
-
-	h->next = HAry[hi];
-
-	lseek(HFd, HScanOff, 0);
-
-	n = write(HFd, h, sizeof(History));
-
-	if ((HFlags & HGF_FAST) == 0)
-	    hflock(HFd, HScanOff, XLOCK_UN);
-
-	if (n == sizeof(History)) {
-	    if ((HFlags & HGF_FAST) == 0) {
-		int32 off = (int32)HScanOff;	/* for write XXX */
-
-		lseek(HFd, HHead.headSize + hi * sizeof(int32), 0);
-		if (write(HFd, &off, sizeof(int32)) != sizeof(int32)) {
-		    r = RCTRYAGAIN;
-		}
-	    } else {
-		HAry[hi] = (int)HScanOff;
-	    }
-	} else {
-	    r = RCTRYAGAIN;
-	}
-    }
-
-    if ((HFlags & HGF_FAST) == 0)	/* unlock hash chain */
-	hflock(HFd, boff, XLOCK_UN);
-
-    return(r);
-}
-
 /*
  * This is a new and simplifed HistoryAdd that locks an append lock
  * (pos 4) and just appends the single history record and unlocks.
@@ -670,16 +550,21 @@ HistoryAddOld(const char *msgid, History *h)
 int
 HistoryAdd(const char *msgid, History *h)
 {
-    int32 hi = (h->hv.h1 ^ h->hv.h2) & HMask;
-    int32 poff = HHead.headSize + hi * sizeof(int32);
-    int32 boff = poff;
+    HistIndex hi;
+    HistIndex pindex;
+    HistIndex index;
+    off_t off;
+    off_t chainlock;
     int r = RCOK;
 
+    if (HHeadMap->hmagic != HMAGIC)
+	historyReOpen();
+
     /*
      * This should not happen, but if it does, log it and pretend we did it
      */
     if (h->gmt == 0) {
-	syslog(LOG_ERR, "Not adding history entry with gmt=0");
+	logit(LOG_ERR, "Not adding history entry with gmt=0");
 	return(RCOK);
     }
     /*
@@ -687,46 +572,49 @@ HistoryAdd(const char *msgid, History *h)
      *
      */
 
+    hi = (h->hv.h1 ^ h->hv.h2) & HMask;
+    pindex = HHead.headSize + hi * sizeof(HistIndex);
+    chainlock = (off_t)pindex;
+
     if ((HFlags & HGF_FAST) == 0)	/* lock hash chain */
-	hflock(HFd, boff, XLOCK_EX);
+	hflock(HFd, chainlock, XLOCK_EX);
 
     /*
-     * make sure message is not already in hash table
+     * make sure message-id is not already in hash table
      */
+    if ((HFlags & HGF_NOSEARCH) == 0) {
+	int counter = 0;
 
-    {
-	int32 off;
+	index = HAry[hi];
+	while (index) {
+	    static History ht;
 
-	off = HAry[hi];
-
-	if ((HFlags & HGF_NOSEARCH) == 0) {
-	    int counter = 0;
-
-	    while (off) {
-		History ht;
-
-		lseek(HFd, (off_t)(uint32)off, 0);
-		if (read(HFd, &ht, sizeof(ht)) != sizeof(ht)) {
-		    if ((LoggedDHistCorrupt & 2) == 0 || DebugOpt) {
-			LoggedDHistCorrupt |= 2;
-			syslog(LOG_ERR, "dhistory file corrupted @ %d", off);
-			sleep(1);
-		    }
-		    break;
-		}
-		if (ht.hv.h1 == h->hv.h1 && ht.hv.h2 == h->hv.h2) {
-		    r = RCALREADY;
-		    break;
-		}
-		poff = off;
-		off = ht.next;
-		if (counter++ > 5000) {
-		    syslog(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d (%s)",
-				(int)poff, (int)off,
-				(int)((ht.hv.h1 ^ ht.hv.h2) & HMask),
-				msgid ? msgid : "no message-id");
-		    off = 0;
+	    if (HHead.version > 1)
+		off = (off_t)HEntryOff + (off_t)index * sizeof(History);
+	    else 
+		off = index;
+	    lseek(HFd, off, 0);
+	    if (read(HFd, &ht, sizeof(ht)) != sizeof(ht)) {
+		if ((LoggedDHistCorrupt & 2) == 0 || DebugOpt) {
+		    LoggedDHistCorrupt |= 2;
+		    logit(LOG_ERR, "dhistory file corrupted @ %u (%s)",
+						index, strerror(errno));
+		    sleep(1);
 		}
+		break;
+	    }
+	    if (ht.hv.h1 == h->hv.h1 && ht.hv.h2 == h->hv.h2) {
+		r = RCALREADY;
+		break;
+	    }
+	    pindex = index;
+	    index = ht.next;
+	    if (counter++ > 5000) {
+		logit(LOG_ERR, "dhistory file chain loop @ %d->%d chain %d (%s)",
+					(int)pindex, (int)index,
+					(int)((ht.hv.h1 ^ ht.hv.h2) & HMask),
+					msgid ? msgid : "no message-id");
+		index = 0;
 	    }
 	}
     }
@@ -745,11 +633,10 @@ HistoryAdd(const char *msgid, History *h)
 	    break;
 	}
 
-	HScanOff = writePos;
-
 	n = write(HFd, h, sizeof(History));
 
 	if (n != sizeof(History)) {
+	    logit(LOG_ERR, "Error writing to history: %s", strerror(errno));
 	    lseek(HFd, writePos, 0);
 	    ftruncate(HFd, writePos);
 	}
@@ -757,16 +644,19 @@ HistoryAdd(const char *msgid, History *h)
 	if ((HFlags & HGF_FAST) == 0)	/* append/scan lock */
 	    hflock(HFd, 4, XLOCK_UN);
 
+	if (HHead.version > 1)
+	    index = (HistIndex)((writePos - HEntryOff) / sizeof(History));
+	else
+	    index = (HistIndex)writePos;
 	if (n == sizeof(History)) {
 	    if ((HFlags & HGF_FAST) == 0) {
-		int32 off = (int32)HScanOff;	/* for write XXX */
-
-		lseek(HFd, HHead.headSize + hi * sizeof(int32), 0);
-		if (write(HFd, &off, sizeof(int32)) != sizeof(int32)) {
+		lseek(HFd, HHead.headSize + hi * sizeof(HistIndex), 0);
+		if (write(HFd, &index, sizeof(index)) != sizeof(index)) {
+		    logit(LOG_ERR, "Error writing to history: %s", strerror(errno));
 		    r = RCTRYAGAIN;
 		}
 	    } else {
-		HAry[hi] = (int)HScanOff;
+		HAry[hi] = index;
 	    }
 	} else {
 	    r = RCTRYAGAIN;
@@ -775,8 +665,62 @@ HistoryAdd(const char *msgid, History *h)
     }
 
     if ((HFlags & HGF_FAST) == 0)	/* unlock hash chain */
-	hflock(HFd, boff, XLOCK_UN);
+	hflock(HFd, chainlock, XLOCK_UN);
 
     return(r);
 }
 
+int
+HistoryStore(History *h)
+{
+    HistIndex index;
+    History th;
+
+    index = HistoryPosLookupByHash(h->hv, &th);
+    if (index == (HistIndex)-1)
+	return(0);
+    if (HHead.version > 1)
+	lseek(HFd, (off_t)HEntryOff + (off_t)index * sizeof(History), 0);
+    else
+	lseek(HFd, index, 0);
+    write(HFd, h, sizeof(History));
+    return(1);
+}
+
+void
+HistoryStoreExp(History *h, HistIndex index)
+{
+    if (HHead.version > 1)
+	lseek(HFd, (off_t)HEntryOff + (off_t)index * sizeof(History) +
+						offsetof(History, exp), 0);
+    else
+	lseek(HFd, index + offsetof(History, exp), 0);
+    write(HFd, &h->exp, sizeof(h->exp));
+}
+
+/*
+ * Cancel/expire article in history
+ */
+int
+HistoryExpire(const char *msgid, History *h, int unexp)
+{
+    HistIndex index;
+    hash_t hv;
+
+    if (msgid == NULL)
+	hv = h->hv;
+    else
+	hv = hhash(msgid);
+    index = HistoryPosLookupByHash(hv, h);
+    if (index == (HistIndex)-1)
+	return(0);
+    if (unexp && H_EXPIRED(h->exp)) {
+	h->exp &= ~EXPF_EXPIRED;
+	HistoryStoreExp(h, index);
+    } else if (!H_EXPIRED(h->exp)) {
+	h->exp |= EXPF_EXPIRED;
+	HistoryStoreExp(h, index);
+    }
+    return(1);
+}
+
