@@ -11,7 +11,7 @@
 #include "defs.h"
 
 Prototype int LoadHostAccess(time_t t, int force, int rebuildtime);
-Prototype char *Authenticate(int fd, struct sockaddr_in *asin, char *label);
+Prototype char *Authenticate(int fd, struct sockaddr *res, char *ipst, char *label);
 Prototype void DumpHostCache(const char *fname);
 
 #define	TESTTYPE_NONE	0
@@ -22,8 +22,7 @@ Prototype void DumpHostCache(const char *fname);
 typedef struct HostAccess {
     char		entry[256];
     int			testtype;
-    struct sockaddr_in 	ipaddr;
-    int			iplen;
+    char 		addrst[NI_MAXHOST];
     char		label[256];
 } HostAccess;
 
@@ -116,7 +115,7 @@ LoadHostAccess(time_t t, int force, int rebuildtime)
 
 	if ((lockfd = open(PatDbExpand(DHostsLockPat), O_RDWR|O_CREAT, 0644)) < 0) {
 	    if (t != 0)
-		syslog(LOG_INFO, "%s : Unable to create lockfile (%s)",
+		logit(LOG_INFO, "%s : Unable to create lockfile (%s)",
 				PatDbExpand(DHostsLockPat), strerror(errno));
 	    else
 		printf("%s : Unable to create lockfile (%s)\n",
@@ -126,7 +125,7 @@ LoadHostAccess(time_t t, int force, int rebuildtime)
 	}
 	if (xflock(lockfd, XLOCK_EX|XLOCK_NB) == -1) {
 	    if (t != 0)
-		syslog(LOG_INFO, "%s : Rebuild already in progress",
+		logit(LOG_INFO, "%s : Rebuild already in progress",
 						PatDbExpand(DHostsCachePat));
 	    else
 		printf("%s : Rebuild already in progress\n",
@@ -142,7 +141,7 @@ LoadHostAccess(time_t t, int force, int rebuildtime)
 	fo = fopen(buf, "w");
 	if (fo == NULL) {
 	    if (t != 0)
-		syslog(LOG_EMERG, "%s: %s", buf, strerror(errno));
+		logit(LOG_CRIT, "%s: %s", buf, strerror(errno));
 	    else
 		fprintf(stderr, "%s: %s\n", buf, strerror(errno));
 	    xflock(lockfd, XLOCK_UN);
@@ -150,7 +149,7 @@ LoadHostAccess(time_t t, int force, int rebuildtime)
 	}
 
 	if (t != 0)
-	    syslog(LOG_INFO, "Rebuilding host cache");
+	    logit(LOG_INFO, "Rebuilding host cache");
 	else
 	    printf("Rebuilding host cache\n");
 
@@ -184,9 +183,7 @@ LoadHostAccess(time_t t, int force, int rebuildtime)
 int
 convHost(char *host, HostAccess *HA, FILE *fo)
 {
-    struct hostent *he;
-    int i;
-    int found;
+    int found = 0;
 
     /*
      * Handle the forcing of entry type
@@ -212,8 +209,7 @@ convHost(char *host, HostAccess *HA, FILE *fo)
     }
 
     strcpy(HA->entry, host);
-    memset(&HA->ipaddr, 0, sizeof(HA->ipaddr));
-    HA->iplen = 0;
+    HA->addrst[0] = 0;
 
     /*
      * Don't lookup IP for wildcards/CIDR
@@ -224,20 +220,39 @@ convHost(char *host, HostAccess *HA, FILE *fo)
 	return(1);
     }
 
-    found = 0;
-    if ((he = gethostbyname(host)) != NULL) {
-	for (i = 0; he->h_addr_list && he->h_addr_list[i] != NULL; ++i) {
-	    const struct in_addr *haddr = (const void *)he->h_addr_list[i];
-	    memcpy(&HA->ipaddr.sin_addr, haddr, he->h_length);
-	    HA->iplen = he->h_length;
-	    HA->testtype |= TESTTYPE_IP;
-	    fwrite(HA, sizeof(*HA), 1, fo);
-	    found = 1;
+#ifdef INET6
+    {
+	struct addrinfo *ai0;
+
+	if (getaddrinfo(host, NULL, NULL, &ai0) == 0) {
+	    struct addrinfo *ai;
+
+	    for (ai = ai0; ai; ai = ai->ai_next) {
+		if (getnameinfo(ai->ai_addr, ai->ai_addrlen, HA->addrst,
+					sizeof(HA->addrst), NULL,
+					0, NI_NUMERICHOST) == 0)
+		    HA->testtype |= TESTTYPE_IP;
+		    fwrite(HA, sizeof(*HA), 1, fo);
+		    found++;
+	    }
+	    freeaddrinfo(ai0);
 	}
-    } else {
-	logit(LOG_NOTICE, "Unable to gethostbyname(%s): %s\n",
-						host, strerror(errno));
     }
+#else
+    {
+	struct hostent *he;
+	int i;
+
+	if ((he = gethostbyname(host)) != NULL)
+	    for (i = 0; he->h_addr_list && he->h_addr_list[i] != NULL; ++i) {
+		struct in_addr *haddr = (void *)he->h_addr_list[i];
+		strcpy(HA->addrst, inet_ntoa(*haddr));
+		HA->testtype |= TESTTYPE_IP;
+		fwrite(HA, sizeof(*HA), 1, fo);
+		found++;
+	    }
+    }
+#endif
     return(found);
 }
 
@@ -303,7 +318,7 @@ loadDnewsFeeds(FILE *fo, int t, int *count)
     fi = fopen(PatLibExpand(DNewsfeedsPat), "r");
     if (fi == NULL) {
 	if (t != 0)
-	    syslog(LOG_CRIT, "%s: %s", PatLibExpand(DNewsfeedsPat),
+	    logit(LOG_CRIT, "%s: %s", PatLibExpand(DNewsfeedsPat),
 							strerror(errno));
 	else
 	    fprintf(stderr, "%s: %s\n", PatLibExpand(DNewsfeedsPat),
@@ -354,7 +369,7 @@ loadDnewsFeeds(FILE *fo, int t, int *count)
  */
 
 char *
-Authenticate(int fd, struct sockaddr_in *asin, char *label)
+Authenticate(int fd, struct sockaddr *res, char *ipst, char *label)
 {
     char *hname = NULL;		/* authenticated hostname or NULL */
     char *pname = NULL;		/* IP address (as string)	  */
@@ -365,28 +380,55 @@ Authenticate(int fd, struct sockaddr_in *asin, char *label)
      * or neither, but never both.
      */
 
+#ifdef INET6
+    {
+	char *st;
+	char hostst[NI_MAXHOST];
+	int error;
+
+	st = NetAddrToSt(0, res, 1, 0, 0);
+	if (st != NULL) {
+	    pname = zallocStr(&SysMemPool, st);
+	} else {
+	    logit(LOG_ERR, "getnameinfo: Unable to get numeric address");
+	    return(NULL);
+	}
+	st[0]= 0;
+	if ((error = getnameinfo(res, SA_LEN(res), hostst, sizeof(hostst),
+					NULL, 0, NI_NAMEREQD)) == 0) {
+	    if (TestForwardLookup(hostst, (const char *)ipst, res) >= 0)
+		hname = zallocStr(&SysMemPool, hostst);
+	} else {
+	    fprintf(stderr, "getnameinfo: %d", error);
+	}
+    }
+#else
+    struct sockaddr_in *sin = (struct sockaddr_in *)res;
+
+    /*
+     * set pname
+     */
+    pname = zallocStr(&SysMemPool, inet_ntoa(sin->sin_addr));
+
     {
 	struct hostent *he;
 
 	he = gethostbyaddr(
-	    (char *)&asin->sin_addr,
-	    sizeof(asin->sin_addr),
+	    (char *)&sin->sin_addr,
+	    sizeof(sin->sin_addr),
 	    AF_INET
 	);
 
 	if (he != NULL) {
 	    hname = zallocStr(&SysMemPool, he->h_name);
-	    if (TestForwardLookup(hname, asin) < 0) {
+	    if (TestForwardLookup(hname, (const char *)ipst, res) < 0) {
 		zfreeStr(&SysMemPool, &hname);
 		hname = NULL;
 	    }
 	}
     }
 
-    /*
-     * set pname
-     */
-    pname = zallocStr(&SysMemPool, inet_ntoa(asin->sin_addr));
+#endif /* INET6 */
 
     /*
      * Check IP against cache file of diablo.hosts. 
@@ -398,7 +440,7 @@ Authenticate(int fd, struct sockaddr_in *asin, char *label)
 	    HostAccess HA;
 
 	    while (fread(&HA, sizeof(HA), 1, fi) == 1) {
-		if (HA.iplen && (HA.testtype & TESTTYPE_IP) && memcmp(&asin->sin_addr, &HA.ipaddr.sin_addr, HA.iplen) == 0) {
+		if (HA.addrst[0] && (HA.testtype & TESTTYPE_IP) && strcmp(ipst, HA.addrst) == 0) {
 		    isValid = 1;
 		    break;
 		} else if ((strchr(HA.entry, '/') != NULL) && (HA.testtype & TESTTYPE_IP) && CidrMatch(HA.entry, pname)) {
@@ -427,9 +469,10 @@ Authenticate(int fd, struct sockaddr_in *asin, char *label)
 		    method = "Exact Hostname match";
 		if (isValid == 5)
 		    method = "Wildcmp hostname match";
-		syslog(LOG_DEBUG, "Matched %s (%s) to %s (label:%s) using %s",
-					hname ? hname : "NOTFOUND",
-					pname, HA.entry, HA.label, method);
+		logit(LOG_DEBUG, "Matched %s (%s) to %s (label:%s) using %s",
+					hname ? hname : "NOTFOUND", pname,
+					HA.entry, HA.label,
+					method);
 		if (HA.label[0])
 		    strcpy(label, HA.label);
 		else
@@ -437,7 +480,7 @@ Authenticate(int fd, struct sockaddr_in *asin, char *label)
 	    }
 	    fclose(fi);
 	} else {
-	    syslog(LOG_ERR, "%s file not found", PatDbExpand(DHostsCachePat));
+	    logit(LOG_ERR, "%s file not found", PatDbExpand(DHostsCachePat));
 	}
     }
 
@@ -478,9 +521,8 @@ DumpHostCache(const char *fname)
 		strcat(tt, "H:");
 	    if (HA.testtype & TESTTYPE_ANY)
 		strcat(tt, "ANY:");
-	    tt[strlen(tt) - 1] = '\0';
 	    printf("%s label=%s type=%s ip=%s\n", HA.entry, HA.label, tt,
-				(HA.testtype & TESTTYPE_IP) ? inet_ntoa(HA.ipaddr.sin_addr) : "NONE");
+				(HA.testtype & TESTTYPE_IP) ? HA.addrst : "NONE");
 	}
     } else {
 	fprintf(stderr, "Unable to open %s\n", fname);
