@@ -145,7 +145,7 @@ KPDBReSize(KPDB *kpdb)
 {
     kpdb->kp_ReSize = 1;
     if (KPDBReOpen(kpdb) == NULL) {
-	syslog(LOG_ERR, "resize %s failed", kpdb->kp_FileName);
+	logit(LOG_ERR, "resize %s failed", kpdb->kp_FileName);
 	exit(1);
     }
     kpdb->kp_ReSize = 0;
@@ -736,7 +736,7 @@ KPDBWrite(KPDB *kpdb, const char *key, const char *tok, const char *data, int lo
 
 	nlockoff = lseek(kpdb->kp_Fd, 0L, 2);
 	if (hflock(kpdb->kp_Fd, nlockoff, XLOCK_EX|XLOCK_NB) < 0) {
-	    syslog(LOG_ERR, "unable to lock new %s record offset %d", kpdb->kp_FileName, nlockoff);
+	    logit(LOG_ERR, "unable to lock new %s record offset %d", kpdb->kp_FileName, nlockoff);
 	}
 
 	if (copyOld) {
