@@ -24,7 +24,7 @@ const char
     struct tm *t;
 
     gettimeofday(&tv, NULL);
-    t = localtime(&tv.tv_sec);
+    t = localtime((const time_t *)&tv.tv_sec);
 
     sprintf(result, "%04d-%02d-%02d %02d:%02d:%02d.%03d",
 	t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
