@@ -66,6 +66,15 @@ MsgId(const char *s, const char **badmsgid)
 	return(badId);
     }
 
+    /*
+     * Last character cannot be a '.'
+     */
+    if (i > 2 && s[i - 1] == '.') {
+	if (badmsgid)
+	    *badmsgid = s;
+	return(badId);
+    }
+
     /*
      * check length, check for garbage (note: whitespace after msgid 
      * is ok and may indicate a tagged-on comment).
