@@ -22,9 +22,11 @@ Prototype int IsDelayed(const char *hlabel);
 Prototype int FeedQuerySpam(const char *hlabel, const char *hostInfo);
 Prototype int PathElmMatches(const char *hlabel, const char *p, int bytes, int *pidx);
 Prototype int CommonElmMatches(const char *common, const char *p, int bytes);
+Prototype int FeedInDelay(const char *hlabel);
 Prototype void FeedGetThrottle(const char *hlabel, int *delay, int *lines);
 Prototype int FeedReadOnly(const char *hlabel);
 Prototype int FeedWhereIs(const char *hlabel);
+Prototype int FeedPriority(const char *hlabel);
 Prototype int FeedPrecommitReject(const char *hlabel);
 Prototype LabelList *FeedLinkLabelList(void);
 Prototype NewslinkInfo *FeedLinkInfo(const char *hlabel);
@@ -33,6 +35,10 @@ Prototype int FeedSpam(int which, const char *nglist, const char *npath, const c
 Prototype void DumpAllFeedInfo(FILE *fo);
 Prototype void DumpFeedInfo(FILE *fo, char *label);
 Prototype void FeedGetMaxInboundRate(const char *hlabel, int *bps);
+Prototype int ArtHashIsFiltered(const char *msgid);
+#if USE_OFFER_FILTER
+Prototype int OfferIsFiltered(const char *hlabel, const char *msgid);
+#endif  /* USE_OFFER_FILTER */
 
 Prototype FILE *FeedFo;
 Prototype int FeedDebug;
@@ -54,6 +60,9 @@ typedef struct NewsFeed {
     struct Node		*nf_FilterBase;
     struct Node		*nf_SpamBase;
     struct Node		*nf_RequireGroupsBase;
+#if USE_OFFER_FILTER
+    struct Node		*nf_OfferFilterBase;
+#endif  /* USE_OFFER_FILTER */
     int			nf_MaxCrossPost;	/* 0=unlimited 		*/
     int			nf_MinCrossPost;	/* 0=unlimited 		*/
     int			nf_MaxArtSize;		/* 0=unlimited 		*/
@@ -72,6 +81,7 @@ typedef struct NewsFeed {
     int			nf_ThrottleLines;
     int			nf_ReadOnly;
     int			nf_WhereIs;
+    int			nf_IncomingPriority;	/* nice() value of process */
     int			nf_PrecommitReject;	/* reject precommit hits */
     HashFeed		nf_HashFeed;		/* hashfeed */
     ArtTypeList		*nf_ArtTypes;		/* What type of articles */
@@ -117,6 +127,9 @@ int feedPathQuery(NewsFeed *feed, const char *path);
 int feedSpamPathQuery(NewsFeed *feed, const char *path);
 int feedDistQuery(const char *item, int len, const char *list);
 int feedSpamFeedOK(int feed, int article);
+#if USE_OFFER_FILTER
+int feedOfferFilterQuery(NewsFeed *feed, const char *msgid);
+#endif  /* USE_OFFER_FILTER */
 
 void resolveGroupList(const char *label, Node *scan);
 void AltFeed(NewsFeed *nf, FILE *fi);
@@ -155,13 +168,13 @@ FeedAdd(const char *msgid, time_t t, History *h, const char *nglist, const char
 
 	ArticleFileName(path, sizeof(path), h, ARTFILE_FILE_REL);
 
-	fprintf(FeedFo, "SOUT\t%s\t%ld,%ld\t%s\t%s\t%s\t%s\t%d\t%s\t%s\n",
-	    path, (long)h->boffset, (long)h->bsize, msgid, nglist, dist, npath,
+	fprintf(FeedFo, "SOUT\t%s\t%lld,%ld\t%s\t%s\t%s\t%s\t%d\t%s\t%s\n",
+	    path, (off_t)h->boffset, (long)h->bsize, msgid, nglist, dist, npath,
 	    headerOnly, artType, cSize
 	);
 	fflush(FeedFo);	/* simplify the parent reader on the pipe */
 	if (ferror(FeedFo)) {
-	    syslog(LOG_CRIT, "lost backchannel to master server");
+	    logit(LOG_CRIT, "lost backchannel to master server");
 	    r = -1;
 	}
     }
@@ -254,9 +267,9 @@ FeedWrite(int logIt,
 
 	}
     }
-    /* syslog(LOG_INFO, "%s", linebuf); */
+    /* logit(LOG_INFO, "%s", linebuf); */
     if (logIt)
-	LogIncoming("%s%s%s\n", linebuf, "", "");
+	LogIncoming("%s%s%s", linebuf, "", "");
     return(r);
 }
 
@@ -300,6 +313,16 @@ feedHashFeedOK(HashFeed *hashfeed, int article)
     return(!HashFeedMatch(hashfeed, article));
 }
 
+int
+ArtHashIsFiltered(const char *msgid)
+{
+    if(IFBase && IFBase->nf_HashFeed.hf_Mod) {
+      return(!HashFeedMatch(&(IFBase->nf_HashFeed),quickhash(msgid)));
+    }
+    return(0);
+}
+
+
 /*
  * IsFiltered() - return 0 if the article should be scrapped because
  *		  one of the gropus is filtered out, -1 otherwise.
@@ -338,12 +361,9 @@ IsDelayed(const char *hlabel)
 {
     NewsFeed *nf;
 
-    if ((nf = NFCache) == NULL) {
-	for (nf = NFBase; nf; nf = nf->nf_Next) {
-	    if (strcmp(hlabel, nf->nf_Label) == 0)
-		break;
-	}
-	NFCache = nf;
+    for (nf = NFBase; nf; nf = nf->nf_Next) {
+	if (strcmp(hlabel, nf->nf_Label) == 0)
+	    break;
     }
 
     if (nf == NULL)
@@ -446,7 +466,7 @@ feedQuerySpamPaths(NewsFeed *feed, const char *npath, int size, const char *msgi
 	r = pat;
 
     if (DebugOpt > 4)
-	syslog(LOG_DEBUG, "%s: Spam test, '%s' '%s' '%s'", 
+	logit(LOG_DEBUG, "%s: Spam test, '%s' '%s' '%s'", 
 		PatLibExpand(DNewsfeedsPat), npath, pat, r);
 
     /*
@@ -455,11 +475,11 @@ feedQuerySpamPaths(NewsFeed *feed, const char *npath, int size, const char *msgi
        assuming the Path: syntax was not bogus
      */
     if (feedSpamPathQuery(feed, pat) == 0) {
-	syslog(LOG_DEBUG, "Spam Path A detect, %s %s", msgid, pat);
+	logit(LOG_DEBUG, "Spam Path A detect, %s %s", msgid, pat);
 	return(-1);
     }
     if (feedSpamPathQuery(feed, r) == 0) {
-	syslog(LOG_DEBUG, "Spam Path B detect, %s %s", msgid, r);
+	logit(LOG_DEBUG, "Spam Path B detect, %s %s", msgid, r);
 	return(-1);
     }
     return(0);
@@ -728,7 +748,7 @@ PathElmMatches(const char *hlabel, const char *p, int bytes, int *pidx)
 	}
 	NFCache = feed;
     }
-    for (i = 0; i < bytes && p[i] != '!' && p[i] != '\n'; ++i)
+    for (i = 0; i < bytes && p[i] != '!' && p[i] != '\n' && p[i] != '\r'; ++i)
 	;
     *pidx = i;
 
@@ -763,11 +783,11 @@ CommonElmMatches(const char *common, const char *p, int bytes)
 
     while (bytes >= l) {
 	if (strncmp(common, p, l) == 0 &&
-	    (bytes == l || p[l] == '!' || p[l] == '\n')
+	    (bytes == l || p[l] == '!' || p[l] == '\n' || p[l] == '\r')
 	) {
 	    return(0);
 	}
-	while (bytes && *p != '!' && *p != '\n') {
+	while (bytes && *p != '!' && *p != '\n' && *p != '\r') {
 	    --bytes;
 	    ++p;
 	}
@@ -779,6 +799,30 @@ CommonElmMatches(const char *common, const char *p, int bytes)
     return(-1);
 }
 
+/*
+ * Returns how long the feed in quesiton should be delayed before allowing
+ * a connection.
+ */
+
+int
+FeedInDelay(const char *hlabel)
+{
+    NewsFeed *feed;
+
+    if ((feed = NFCache) == NULL) {
+	for (feed = NFBase; feed; feed = feed->nf_Next) {
+	    if (strcmp(hlabel, feed->nf_Label) == 0)
+		break;
+	}
+	NFCache = feed;
+    }
+
+    if (feed)
+	return feed->nf_LinkInfo.li_DelayInFeed;
+    else
+	return 0;
+}
+
 /*
  * Write the throttling delay and line count into the appropriate
  * memory locations.
@@ -852,6 +896,29 @@ FeedWhereIs(const char *hlabel)
 	return 0;
 }
 
+/*
+ * Return the incoming priority setting
+ */
+
+int
+FeedPriority(const char *hlabel)
+{
+    NewsFeed *feed;
+
+    if ((feed = NFCache) == NULL) {
+	for (feed = NFBase; feed; feed = feed->nf_Next) {
+	    if (strcmp(hlabel, feed->nf_Label) == 0)
+		break;
+	}
+	NFCache = feed;
+    }
+
+    if (feed)
+	return feed->nf_IncomingPriority;
+    else
+	return 0;
+}
+
 /*
  * Return 1 if we need to reject precommit hits for this incoming feed
  * 0 otherwise.
@@ -993,6 +1060,58 @@ feedPathQuery(NewsFeed *feed, const char *path)
     return(r);
 }
 
+#if USE_OFFER_FILTER
+/*
+ * OfferIsFiltered() - return 0 if the article should be refused because
+ *		  the Message-ID is filtered out, -1 otherwise.
+ */
+
+int 
+OfferIsFiltered(const char *hlabel, const char *msgid)
+{
+    NewsFeed *nf;
+    int r;
+
+    if ((nf = NFCache) == NULL) {
+	for (nf = NFBase; nf; nf = nf->nf_Next) {
+	    if (strcmp(hlabel, nf->nf_Label) == 0)
+		break;
+	}
+	NFCache = nf;
+    }
+
+    if (nf == NULL) {
+	r = 0;
+    } else {
+	r = feedOfferFilterQuery(nf, msgid);
+    }
+    if (DebugOpt > 1)
+	printf("OfferIsFiltered: %d (%s,%s)\n", r, hlabel, msgid);
+    return(r);
+}
+
+/*
+ * Return non-0 if we get a match
+ */
+
+int
+feedOfferFilterQuery(NewsFeed *feed, const char *msgid)
+{
+    int r = 0;
+
+    if (feed) {
+	r = recursiveScan(
+	    feed, 
+	    offsetof(NewsFeed, nf_OfferFilterBase),
+	    msgid, 
+	    cbWildCmpNoStop,
+	    r 
+	);
+    }
+    return(r);
+}
+#endif  /* USE_OFFER_FILTER */
+
 time_t NFGmtMin = (time_t)-1;
 time_t NFMTime = 0;
 
@@ -1024,27 +1143,26 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 
     if (force || t == 0 || t / 60 != NFGmtMin) {
 	struct stat st = { 0 };
-	FILE *fi;
+	FILE *fi = NULL;
 
 	if (t)
 	    NFGmtMin = t / 60;
 
 	errno = 0;
 
-	if (DebugOpt > 1)
-	    printf("Opening dnewsfeeds file: %s\n", PatLibExpand(DNewsfeedsPat));
-	fi = fopen(PatLibExpand(DNewsfeedsPat), "r");
-
-	if (fi == NULL)
-	    syslog(LOG_EMERG, "%s: %s", PatLibExpand(DNewsfeedsPat), strerror(errno));
-
-	if (fi == NULL || 
-	    (fstat(fileno(fi), &st) == 0 && st.st_mtime != NFMTime && !TooNear(st.st_mtime)) ||
+	if ((istat(0, &st) == 0 && st.st_mtime != NFMTime && !TooNear(st.st_mtime)) ||
 	    force ||
 	    NFMTime == 0
 	) {
 	    char buf[MAXGNAME+256];
 
+	    if (DebugOpt > 1)
+	        printf("Opening dnewsfeeds file: %s\n", PatLibExpand(DNewsfeedsPat));
+	    fi = iopen(PatLibExpand(DNewsfeedsPat), "r");
+
+	    if (fi == NULL)
+	        logit(LOG_CRIT, "%s: %s", PatLibExpand(DNewsfeedsPat), strerror(errno));
+
 	    NFMTime = st.st_mtime;	/* may be 0 if file failed to open */
 
 	    /*
@@ -1096,11 +1214,14 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 		Node **psNode = NULL;
 		Node **pfNode = NULL;
 		Node **prNode = NULL;
+#if USE_OFFER_FILTER
+		Node **pofNode = NULL;
+#endif  /* USE_OFFER_FILTER */
 		int lineNo = 0;
 		int inGroupDef = 0;
 		FILE *altFi = NULL;
 
-		while (fi && fgets(buf, sizeof(buf), fi) != NULL) {
+		while (fi && igets(buf, sizeof(buf), fi) != NULL) {
 		    char *s1 = strtok(buf, " \t\n");
 		    char *s2 = (s1) ? strtok(NULL, " \t\n") : NULL;
 		    int err = 1;
@@ -1112,7 +1233,7 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 
 		    if (strcmp(s1, "label") == 0) {
 			if (nf) {
-			    syslog(LOG_CRIT, 
+			    logit(LOG_CRIT, 
 				"Newsfeed config line %d, no end before new label!\n",
 				lineNo
 			    );
@@ -1141,6 +1262,12 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 				*prNode = NULL;
 				prNode = NULL;
 			    }
+#if USE_OFFER_FILTER
+			    if (pofNode) {
+				*pofNode = NULL;
+				pofNode = NULL;
+			    }
+#endif  /* USE_OFFER_FILTER */
 
 			    if (nf) {
 				if (inGroupDef) {
@@ -1187,6 +1314,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    pgNode = &nf->nf_GroupAcceptBase;
 			    psNode = &nf->nf_SpamBase;
 			    prNode = &nf->nf_RequireGroupsBase;
+#if USE_OFFER_FILTER
+			    pofNode = &nf->nf_OfferFilterBase;
+#endif  /* USE_OFFER_FILTER */
 
 			    setLinkStart(&nf->nf_LinkInfo);
 
@@ -1211,6 +1341,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 				MakeNodeAppList(&psNode, &NFMemPool, gl, 2);
 				MakeNodeAppList(&prNode, &NFMemPool, gl, 2);
 				MakeNodeAppList(&pfNode, &NFMemPool, gl, 2);
+#if USE_OFFER_FILTER
+				MakeNodeAppList(&pofNode, &NFMemPool, gl, 2);
+#endif  /* USE_OFFER_FILTER */
 			    }
 			}
 		    } else if (strcmp(s1, "alias") == 0) {
@@ -1274,6 +1407,18 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    (void)MakeNodeAppList(&pfNode, &NFMemPool, s2, -1);
 			    err = 0;
 			}
+#if USE_OFFER_FILTER
+		    } else if (strcmp(s1, "offerfilter") == 0) {
+			if (nf && s2) {
+			    (void)MakeNodeAppList(&pofNode, &NFMemPool, s2, 1);
+			    err = 0;
+			}
+		    } else if (strcmp(s1, "noofferfilter") == 0) {
+			if (nf && s2) {
+			    (void)MakeNodeAppList(&pofNode, &NFMemPool, s2, 0);
+			    err = 0;
+			}
+#endif  /* USE_OFFER_FILTER */
 		    } else if (strcmp(s1, "maxconnect") == 0) {
 			if (nf && s2) {
 			    nf->nf_MaxConnect = strtol(s2, NULL, 0);
@@ -1318,7 +1463,7 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			}
 		    } else if (strcmp(s1, "groupdef") == 0) {
 			if (nf) {
-			    syslog(LOG_CRIT,
+			    logit(LOG_CRIT,
 			       "Newsfeed config line %d, no end before new groupdef!\n", 
 				lineNo
 			    );
@@ -1346,6 +1491,12 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 				*prNode = NULL;
 				prNode = NULL;
 			    }
+#if USE_OFFER_FILTER
+			    if (pofNode) {
+				*pofNode = NULL;
+				pofNode = NULL;
+			    }
+#endif  /* USE_OFFER_FILTER */
 
 			    if (inGroupDef) {
 				nf->nf_Next = GRBase;
@@ -1377,6 +1528,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    pgNode = &nf->nf_GroupAcceptBase;
 			    psNode = &nf->nf_SpamBase;
 			    prNode = &nf->nf_RequireGroupsBase;
+#if USE_OFFER_FILTER
+			    pofNode = &nf->nf_OfferFilterBase;
+#endif  /* USE_OFFER_FILTER */
 			    err = 0;
 			}
 		    } else if (strcmp(s1, "groupref") == 0) {
@@ -1387,6 +1541,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    (void)MakeNodeAppList(&psNode, &NFMemPool, s2, 2);
 			    (void)MakeNodeAppList(&prNode, &NFMemPool, s2, 2);
 			    (void)MakeNodeAppList(&pfNode, &NFMemPool, s2, 2);
+#if USE_OFFER_FILTER
+			    (void)MakeNodeAppList(&pofNode, &NFMemPool, s2, 2);
+#endif  /* USE_OFFER_FILTER */
 			    err = 0;
 			}
 		    } else if (strcmp(s1, "delgroupany") == 0) {
@@ -1436,7 +1593,7 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			}
 		    } else if (strcmp(s1, "allow_readonly") == 0) {
 			if (nf) {
-			    syslog(LOG_CRIT, 
+			    logit(LOG_CRIT, 
 				"Newsfeed config line %d, allow_readonly ignored, use R flag in dserver.hosts\n",
 				lineNo
 			    );
@@ -1447,6 +1604,11 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    nf->nf_WhereIs = enabled(s2);
 			    err = 0;
 			}
+		    } else if (strcmp(s1, "incomingpriority") == 0) {
+			if (nf) {
+			    nf->nf_IncomingPriority = enabled(s2);
+			    err = 0;
+			}
 		    } else if (strcmp(s1, "precomreject") == 0) {
 			if (nf) {
 			    nf->nf_PrecommitReject = enabled(s2);
@@ -1467,6 +1629,11 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    nf->nf_LinkInfo.li_DelayFeed = atoi(s2);
 			    err = 0;
 			}
+		    } else if (strcmp(s1, "delayinfeed") == 0) {
+			if (nf) {
+			    nf->nf_LinkInfo.li_DelayInFeed = atoi(s2);
+			    err = 0;
+			}
 		    } else if (strcmp(s1, "transmitbuf") == 0) {
 			if (nf) {
 			    nf->nf_LinkInfo.li_TransmitBuf = atoi(s2);
@@ -1477,6 +1644,11 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    nf->nf_LinkInfo.li_ReceiveBuf = atoi(s2);
 			    err = 0;
 			}
+		    } else if (strcmp(s1, "setqos") == 0) {
+			if (nf) {
+			    nf->nf_LinkInfo.li_QoS = strtol(s2,NULL,0); /* both hex & decimal */
+			    err = 0;
+			}
 		    } else if (strcmp(s1, "maxparallel") == 0) {
 			if (nf) {
 			    nf->nf_LinkInfo.li_MaxParallel = atoi(s2);
@@ -1529,8 +1701,12 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    err = 0;
 			}
 		    } else if (strcmp(s1, "check") == 0) {
-			if (nf) {
-			    nf->nf_LinkInfo.li_NoCheck = !enabled(s2);
+			if (nf && s2) {
+			    if (strcmp(s2, "nortcheck") == 0) {
+			    	nf->nf_LinkInfo.li_Check = 2;
+			    } else {
+				nf->nf_LinkInfo.li_Check = enabled(s2);
+			    }
 			    err = 0;
 			}
 		    } else if (strcmp(s1, "compress") == 0) {
@@ -1574,7 +1750,7 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			}
 		    } else if (strcmp(s1, "bindaddress") == 0) {
 			if (nf && s2) {
-			    nf->nf_LinkInfo.li_BindAddress = zallocStr(&NFMemPool, s2);
+			    nf->nf_LinkInfo.li_BindAddress = zallocStr(&NFMemPool, SanitiseAddr(s2));
 			    err = 0;
 			}
 		    } else if (strcmp(s1, "logarts") == 0) {
@@ -1643,6 +1819,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    *psNode = NULL;
 			    *prNode = NULL;
 			    *pfNode = NULL;
+#if USE_OFFER_FILTER
+			    *pofNode = NULL;
+#endif  /* USE_OFFER_FILTER */
 
 			    paNode = NULL;
 			    psaNode = NULL;
@@ -1650,6 +1829,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    psNode = NULL;
 			    prNode = NULL;
 			    pfNode = NULL;
+#if USE_OFFER_FILTER
+			    pofNode = NULL;
+#endif  /* USE_OFFER_FILTER */
 
 			    if (inGroupDef ||
 					strcmp(nf->nf_Label, "GLOBAL") == 0) {
@@ -1670,7 +1852,7 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 			    err = 0;
 			}
 		    } else {
-			syslog(LOG_CRIT, 
+			logit(LOG_CRIT, 
 			    "Newsfeed config line %d, unknown command\n", 
 			    lineNo
 			);
@@ -1683,7 +1865,7 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 		     */
 
 		    if (err && (nf != NULL || SaveHLabel == NULL)) {
-			syslog(LOG_ERR, "Newsfeed config line %d, command in unexpected position or command requires argument", lineNo);
+			logit(LOG_ERR, "Newsfeed config line %d, command in unexpected position or command requires argument", lineNo);
 		    }
 		}
 		if (nf) {
@@ -1693,6 +1875,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 		    *psNode = NULL;
 		    *prNode = NULL;
 		    *pfNode = NULL;
+#if USE_OFFER_FILTER
+		    *pofNode = NULL;
+#endif  /* USE_OFFER_FILTER */
 
 		    paNode = NULL;
 		    psaNode = NULL;
@@ -1700,6 +1885,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 		    psNode = NULL;
 		    prNode = NULL;
 		    pfNode = NULL;
+#if USE_OFFER_FILTER
+		    pofNode = NULL;
+#endif  /* USE_OFFER_FILTER */
 
 		    if (inGroupDef) {
 			nf->nf_Next = GRBase;
@@ -1719,7 +1907,7 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 	    }
 	}
 	if (fi)
-	    fclose(fi);
+	    iclose(fi);
 
 	/*
 	 * Resolve NewsFeed references after the fact.  This allows
@@ -1735,6 +1923,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 		resolveGroupList(nf->nf_Label, nf->nf_FilterBase);
 		resolveGroupList(nf->nf_Label, nf->nf_SpamBase);
 		resolveGroupList(nf->nf_Label, nf->nf_RequireGroupsBase);
+#if USE_OFFER_FILTER
+		resolveGroupList(nf->nf_Label, nf->nf_OfferFilterBase);
+#endif  /* USE_OFFER_FILTER */
 	    }
 	    for (nf = GRBase; nf; nf = nf->nf_Next) {
 		resolveGroupList(nf->nf_Label, nf->nf_PathAliasBase);
@@ -1743,6 +1934,9 @@ LoadNewsFeed(time_t t, int force, const char *hlabel)
 		resolveGroupList(nf->nf_Label, nf->nf_FilterBase);
 		resolveGroupList(nf->nf_Label, nf->nf_SpamBase);
 		resolveGroupList(nf->nf_Label, nf->nf_RequireGroupsBase);
+#if USE_OFFER_FILTER
+		resolveGroupList(nf->nf_Label, nf->nf_OfferFilterBase);
+#endif  /* USE_OFFER_FILTER */
 	    }
 
 	    /*
@@ -1786,7 +1980,7 @@ resolveGroupList(const char *label, Node *scan)
 		}
 	    }
 	    if (gl == NULL) {
-		syslog(LOG_CRIT, "Error: grouplist %s does not exist (from %s)\n", scan->no_Name, label);
+		logit(LOG_CRIT, "Error: grouplist %s does not exist (from %s)\n", scan->no_Name, label);
 	    }
 	}
     }
@@ -1878,16 +2072,18 @@ setLinkGLOBAL(NewslinkInfo *nf)
     nf->li_StartDelay = 0;
     nf->li_TransmitBuf = 0;
     nf->li_ReceiveBuf = 0;
+    nf->li_QoS = 0;
     nf->li_MaxParallel = 2;
     nf->li_NoStream = 0;
     nf->li_DelayFeed = 0;
+    nf->li_DelayInFeed = 0;
     nf->li_RealTime = 0;
     nf->li_Notify = 0;
     nf->li_MaxQueueFile = 0;
     nf->li_HeadFeed = 0;
     nf->li_PreserveBytes = 0;
     nf->li_GenLines = 0;
-    nf->li_NoCheck = 0;
+    nf->li_Check = 1;
     nf->li_Compress = 0;
     nf->li_QueueSkip = 0;
     nf->li_ArticleStat = 0;
@@ -1910,17 +2106,19 @@ setLinkStart(NewslinkInfo *nf)
     nf->li_StartDelay = -1;
     nf->li_TransmitBuf = -1;
     nf->li_ReceiveBuf = -1;
+    nf->li_QoS = -1;
     nf->li_BindAddress = NULL;
     nf->li_MaxParallel = -1;
     nf->li_NoStream = -1;
     nf->li_DelayFeed = -1;
+    nf->li_DelayInFeed = -1;
     nf->li_RealTime = -1;
     nf->li_Notify = -1;
     nf->li_MaxQueueFile = -1;
     nf->li_HeadFeed = -1;
     nf->li_PreserveBytes = -1;
     nf->li_GenLines = -1;
-    nf->li_NoCheck = -1;
+    nf->li_Check = -1;
     nf->li_Compress = -1;
     nf->li_QueueSkip = -1;
     nf->li_ArticleStat = -1;
@@ -1941,6 +2139,8 @@ setLinkDefaults(NewslinkInfo *nf, NewslinkInfo *gl)
 	nf->li_TransmitBuf = gl->li_TransmitBuf;
     if (nf->li_ReceiveBuf < 0 && gl->li_ReceiveBuf >= 0)
 	nf->li_ReceiveBuf = gl->li_ReceiveBuf;
+    if (nf->li_QoS < 0 && gl->li_QoS >= 0)
+	nf->li_QoS = gl->li_QoS;
     if (!nf->li_BindAddress && gl->li_BindAddress)
 	nf->li_BindAddress = gl->li_BindAddress;
     if (nf->li_MaxParallel < 0 && gl->li_MaxParallel >= 0)
@@ -1949,6 +2149,8 @@ setLinkDefaults(NewslinkInfo *nf, NewslinkInfo *gl)
 	nf->li_NoStream = gl->li_NoStream;
     if (nf->li_DelayFeed < 0 && gl->li_DelayFeed >= 0)
 	nf->li_DelayFeed = gl->li_DelayFeed;
+    if (nf->li_DelayInFeed < 0 && gl->li_DelayInFeed >= 0)
+	nf->li_DelayInFeed = gl->li_DelayInFeed;
     if (nf->li_RealTime < 0 && gl->li_RealTime >= 0)
 	nf->li_RealTime = gl->li_RealTime;
     if (nf->li_Notify < 0 && gl->li_Notify >= 0)
@@ -1961,8 +2163,8 @@ setLinkDefaults(NewslinkInfo *nf, NewslinkInfo *gl)
 	nf->li_PreserveBytes = gl->li_PreserveBytes;
     if (nf->li_GenLines < 0 && gl->li_GenLines >= 0)
 	nf->li_GenLines = gl->li_GenLines;
-    if (nf->li_NoCheck < 0 && gl->li_NoCheck >= 0)
-	nf->li_NoCheck = gl->li_NoCheck;
+    if (nf->li_Check < 0 && gl->li_Check >= 0)
+	nf->li_Check = gl->li_Check;
     if (nf->li_Compress < 0 && gl->li_Compress >= 0)
 	nf->li_Compress = gl->li_Compress;
     if (nf->li_QueueSkip < 0 && gl->li_QueueSkip >= 0)
@@ -2036,6 +2238,8 @@ resolveDefaults(NewsFeed *nf, Node *no)
 		nf->nf_ReadOnly = gl->nf_ReadOnly;
 	    if (!nf->nf_WhereIs && gl->nf_WhereIs)
 		nf->nf_WhereIs = gl->nf_WhereIs;
+	    if (!nf->nf_IncomingPriority && gl->nf_IncomingPriority)
+		nf->nf_IncomingPriority = gl->nf_IncomingPriority;
 	    if (!nf->nf_PrecommitReject && gl->nf_PrecommitReject)
 		nf->nf_PrecommitReject = gl->nf_PrecommitReject;
 
@@ -2061,7 +2265,7 @@ recursiveScan(NewsFeed *feed, int off, const void *data, int (*callback)(Node *n
 	    if (++RecurCount == MAXRECURSION) {
 		if (RecurWarn == 0) {
 		    RecurWarn = 1;
-		    syslog(LOG_EMERG, "Infinite recursion in dnewsfeeds file!");
+		    logit(LOG_EMERG, "Infinite recursion in dnewsfeeds file!");
 		}
 	    } else {
 		if (node->no_Data != NULL)
@@ -2241,12 +2445,12 @@ FeedGetMaxInboundRate(const char *hlabel, int *bps)
 		        ptr++;
 		    }
 		    if (ptr == p2) {
-		        syslog(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (A%02x)", hlabel, feed->nf_MaxInboundRate, *ptr);
+		        logit(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (A%02x)", hlabel, feed->nf_MaxInboundRate, *ptr);
 		    }
 		    hourtmp = atoi(p2);
 		    if (*ptr == ',' || *ptr == ';' || ! *ptr) {
 			if (hourtmp < 0 || hourtmp > 23) {
-		            syslog(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (B%d)", hlabel, feed->nf_MaxInboundRate, hourtmp);
+		            logit(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (B%d)", hlabel, feed->nf_MaxInboundRate, hourtmp);
 			} else {
 			    hours[hourtmp] = bpstmp;
 			}
@@ -2262,11 +2466,11 @@ FeedGetMaxInboundRate(const char *hlabel, int *bps)
 		            ptr++;
 		        }
 		        if (ptr == p2) {
-		            syslog(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (C%02x)", hlabel, feed->nf_MaxInboundRate, *ptr);
+		            logit(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (C%02x)", hlabel, feed->nf_MaxInboundRate, *ptr);
 		        }
 		        hourendtmp = atoi(p2);
 			if (hourtmp < 0 || hourtmp > 23 || hourendtmp < 0 || hourendtmp > 23 || hourtmp >= hourendtmp) {
-		            syslog(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (D%d/%d)", hlabel, feed->nf_MaxInboundRate, hourtmp, hourendtmp);
+		            logit(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (D%d/%d)", hlabel, feed->nf_MaxInboundRate, hourtmp, hourendtmp);
 			} else {
 			    for (i = hourtmp; i <= hourendtmp; i++) {
 			        hours[i] = bpstmp;
@@ -2276,7 +2480,7 @@ FeedGetMaxInboundRate(const char *hlabel, int *bps)
 			    ptr++;
 			}
 		    } else {
-		        syslog(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (E%02x)", hlabel, feed->nf_MaxInboundRate, *ptr);
+		        logit(LOG_ERR, "Newsfeed entry %s having trouble making sense of MaxInboundRate expression %s (E%02x)", hlabel, feed->nf_MaxInboundRate, *ptr);
 			ptr++;
 		    }
 		}
@@ -2360,7 +2564,20 @@ DumpFeedInfo(FILE *fo, char *label)
     fprintf(fo, "  HashFeed       : %d-%d/%d\n", nf->nf_HashFeed.hf_Begin,
 						 nf->nf_HashFeed.hf_End,
 						 nf->nf_HashFeed.hf_Mod);
+    fprintf(fo, "  IncomingPriority: %d\n", nf->nf_IncomingPriority);
     fprintf(fo, "  PrecommitReject: %d\n", nf->nf_PrecommitReject);
+#if USE_OFFER_FILTER
+    fprintf(fo, "  OfferFilter    : ");
+    {
+	Node *n;
+	for (n = nf->nf_OfferFilterBase; n != NULL; n = n->no_Next)
+	    fprintf(fo, "%s%s,",
+	    			(n->no_Value == 1) ? "" : /* not filtered */
+	    			(n->no_Value == 0) ? "!" : "?",
+	    			n->no_Name);
+	fprintf(fo, "\n");
+    }
+#endif  /* USE_OFFER_FILTER */
     fprintf(fo, "  ArtTypes       : ");
     {
 	ArtTypeList *at;
@@ -2389,17 +2606,21 @@ DumpFeedInfo(FILE *fo, char *label)
     fprintf(fo, "  Headfeed       : %d\n", nf->nf_LinkInfo.li_HeadFeed);
     fprintf(fo, "  TransmitBuf    : %d\n", nf->nf_LinkInfo.li_TransmitBuf);
     fprintf(fo, "  ReceiveBuf     : %d\n", nf->nf_LinkInfo.li_ReceiveBuf);
+#ifdef IP_TOS
+    fprintf(fo, "  QoS            : %d\n", nf->nf_LinkInfo.li_QoS);
+#endif
     fprintf(fo, "  BindAddress    : %s\n", nf->nf_LinkInfo.li_BindAddress ?
 					nf->nf_LinkInfo.li_BindAddress : "ALL");
     fprintf(fo, "  MaxParallel    : %d\n", nf->nf_LinkInfo.li_MaxParallel);
     fprintf(fo, "  NoStream       : %d\n", nf->nf_LinkInfo.li_NoStream);
     fprintf(fo, "  DelayFeed      : %d\n", nf->nf_LinkInfo.li_DelayFeed);
+    fprintf(fo, "  DelayInFeed    : %d\n", nf->nf_LinkInfo.li_DelayInFeed);
     fprintf(fo, "  RealTime       : %d\n", nf->nf_LinkInfo.li_RealTime);
     fprintf(fo, "  Notify         : %d\n", nf->nf_LinkInfo.li_Notify);
     fprintf(fo, "  MaxQueueFile   : %d\n", nf->nf_LinkInfo.li_MaxQueueFile);
     fprintf(fo, "  PreserveBytes  : %d\n", nf->nf_LinkInfo.li_PreserveBytes);
     fprintf(fo, "  GenLines       : %d\n", nf->nf_LinkInfo.li_GenLines);
-    fprintf(fo, "  NoCheck        : %d\n", nf->nf_LinkInfo.li_NoCheck);
+    fprintf(fo, "  Check          : %d\n", nf->nf_LinkInfo.li_Check);
     fprintf(fo, "  MaxStream      : %d\n", nf->nf_LinkInfo.li_MaxStream);
     fprintf(fo, "  Priority       : %d\n", nf->nf_LinkInfo.li_Priority);
     fprintf(fo, "  Compress       : %d\n", nf->nf_LinkInfo.li_Compress);
