@@ -50,6 +50,9 @@ parsedate(char *d)
     } else if (ti.tm_year >= 1900) {
 	ti.tm_year -= 1900;
     }
+    ti.tm_hour = strtol(d, &d, 10);
+    d++;
+    ti.tm_min = strtol(d, &d, 10);
     if (ti.tm_mday && ti.tm_mon >= 0 && ti.tm_year)
 	return(mktime(&ti));
     else
