@@ -32,16 +32,17 @@ PGPVerify(Control *ctl, const char *art, int artLen)
     const char *badCmd = "Unable to exec pgp verification program\n";
     int fds[2];
     pid_t pid;
-    char *argv[3] = { NULL };
-    char path[256];
 
-    snprintf(path, sizeof(path), PGP_VERIFY_PATH, NewsHome);
+    if (DebugOpt) {
+	int i = 0;
+	while (i < 10 && DOpts.PgpVerifyArgs[i] != NULL) {
+	    printf("PgpverifyArg[%d]=%s\n", i, DOpts.PgpVerifyArgs[i]);
+	    i++;
+	}
+    }
 
-    argv[0] = path;
-    argv[1] = PGP_VERIFY_ARG0;
-    argv[2] = NULL;
-
-    if ((pid = RunProgramPipe(fds, RPF_STDIN|RPF_STDOUT, argv, NULL)) > 0) {
+    if ((pid = RunProgramPipe(fds, RPF_STDIN|RPF_STDOUT,
+					DOpts.PgpVerifyArgs, NULL)) > 0) {
 	/*
 	 * write article to pgpverify, result answer
 	 */
@@ -90,6 +91,8 @@ PGPVerify(Control *ctl, const char *art, int artLen)
 		if (strcmp(buf, ctl->ct_Verify) == 0) {
 		    badCmd = NULL;
 		} else {
+		    if (DebugOpt)
+			printf("pgp label mismatch:%s:%s:\n", buf, ctl->ct_Verify);
 		    badCmd = "pgp parsed ok, but label did not match\n";
 		}
 	    } else {
@@ -99,7 +102,7 @@ PGPVerify(Control *ctl, const char *art, int artLen)
 	close(fds[0]);
 	waitpid(pid, NULL, 0);
     } else {
-	logit(LOG_ERR, "Unable to run %s", argv[0]);
+	logit(LOG_ERR, "Unable to run %s", DOpts.PgpVerifyArgs[0]);
     }
     return(badCmd);
 }
