@@ -75,7 +75,7 @@ InitPreCommit(void)
     struct shmid_ds ds;
 
     if (sid < 0) {
-	syslog(LOG_CRIT, "sysv shared memory alloc of %d failed, is your machine configured with a high enough maximum segment size?",
+	logit(LOG_CRIT, "sysv shared memory alloc of %d failed, is your machine configured with a high enough maximum segment size?",
 	    PCHSize * sizeof(pchash_t)
 	);
 	exit(1);
@@ -84,12 +84,12 @@ InitPreCommit(void)
     PCHAry = (pchash_t *)shmat(sid, NULL, SHM_R|SHM_W);
 
     if (shmctl(sid, IPC_STAT, &ds) < 0 || shmctl(sid, IPC_RMID, &ds) < 0) {
-	syslog(LOG_CRIT, "sysv shmctl stat/rmid failed");
+	logit(LOG_CRIT, "sysv shmctl stat/rmid failed");
 	exit(1);
     }
     if (PCHAry == (pchash_t *)-1) {
 	PCHAry = NULL;
-	syslog(LOG_CRIT, "sysv shared memory map failed");
+	logit(LOG_CRIT, "sysv shared memory map failed");
 	exit(1);
     }
 #endif
@@ -118,7 +118,7 @@ PreCommit(const char *msgid, int flags)
 
     if (PCHAry == NULL) {
 #if USE_PCOMMIT_SHM
-	syslog(LOG_CRIT, "unable to initialize precommit cache");
+	logit(LOG_CRIT, "unable to initialize precommit cache");
 	exit(1);
 #else
 	struct stat st;
