@@ -53,14 +53,13 @@ SpoolObject *CurrentSpoolObject = NULL;
 SpoolObject *SpoolObjectMap[MAX_SPOOL_OBJECTS];
 MetaSpool *MetaSpools = NULL;
 GroupExpire *ExBase = NULL;
-MemPool *EXMemPool = NULL;
+MemPool *SPMemPool = NULL;
 MemPool *GRMemPool = NULL;
 time_t DirTime = 0;
 
-SpoolObject	*SpoolAllocations[MAX_SPOOL_OBJECTS];
-
-void createSpoolDir(SpoolObject *so, uint32 gmt);
+int createSpoolDir(SpoolObject *so, uint32 gmt);
 uint16 findSpoolGrp(const char *msgid, GroupList *groups, int size, int ngcount, int arttype, char *label, int *t, int *complvl);
+int findLabel(LabelList *ll, char *label);
 void loadSpoolCtl(FILE *fi);
 double spaceFreeOn(char *part);
 void dumpSpoolConfig(void);
@@ -123,6 +122,8 @@ ArticleFileName(char *path, int pathSize, History *h, int opt)
 int
 SpoolCompressed(uint16 spool)
 {
+    if (spool > MAX_SPOOL_OBJECTS)
+	return(0);
     if (SpoolObjectMap[spool] != NULL &&
 				SpoolObjectMap[spool]->so_CompressLvl != -1)
 	return(SpoolObjectMap[spool]->so_CompressLvl);
@@ -144,6 +145,8 @@ GetSpoolPath(uint16 spool, int gmt, int opt)
 {
     static char path[PATH_MAX];
 
+    if (spool > MAX_SPOOL_OBJECTS)
+	return(NULL);
     if (SpoolObjectMap[spool] && SpoolObjectMap[spool]->so_Path != NULL) {
 	if (*SpoolObjectMap[spool]->so_Path == '/' ||
 			opt == ARTFILE_DIR_REL || opt == ARTFILE_FILE_REL) {
@@ -233,7 +236,14 @@ SpoolDirTime(void)
     return(DirTime);
 }
 
-void
+/*
+ * Create the spool article directory on a spool object
+ *
+ * Returns:
+ * 	-1	error
+ * 	0	success
+ */
+int
 createSpoolDir(SpoolObject *so, uint32 gmt)
 {
     char path[PATH_MAX];
@@ -246,12 +256,15 @@ createSpoolDir(SpoolObject *so, uint32 gmt)
     if (stat(path, &st) != 0 || !S_ISDIR(st.st_mode)) {
 	if (DebugOpt > 1)
 	    printf("Creating spool article dir: %s\n", path);
-	if (mkdir(path, 0755) == -1 && errno != EEXIST)
-	    syslog(LOG_CRIT, "%s: Unable to create article dir %s: %s",   
+	if (mkdir(path, 0755) == -1 && errno != EEXIST) {
+	    logit(LOG_CRIT, "%s: Unable to create article dir %s: %s",   
 					PatLibExpand(DSpoolCtlPat), path,
 					strerror(errno));
+	    return(-1);
+	}
     }
     so->so_DirTime = gmt;
+    return(0);
 }
 
 /*
@@ -270,9 +283,11 @@ AllocateSpools(time_t t)
     double space;
     double maxspace;
 
-    DirTime = gmt - gmt % 10;
-
     for (ms = MetaSpools; ms; ms = ms->ms_Next) {
+	if (ms->ms_ReAllocInterval / 60 == 0)
+	    DirTime = gmt;
+	else
+	    DirTime = gmt - gmt % (ms->ms_ReAllocInterval / 60);
 	switch (ms->ms_AllocationStrategy) {
 	    case SPOOL_ALLOC_NONE:
 		ms->ms_AllocationStrategy = SPOOL_ALLOC_SEQ;
@@ -304,13 +319,23 @@ AllocateSpools(time_t t)
 		    }
 		}
 		break;
+	    case SPOOL_ALLOC_SINGLE:
+		/*
+		 * Allocate the same spool until a timelimit is up
+		 */
+		ms->ms_NextAllocation = (t / ms->ms_ReAllocInterval) %
+							ms->ms_NumSpoolObjects;
+		ms->ms_AllocatedSpool = ms->ms_SpoolObjects[ms->ms_NextAllocation];
+		break;
 	}
 
 	if (ms->ms_AllocatedSpool == NULL)
 	    continue;
+
 	if (ms->ms_AllocatedSpool->so_DirTime == 0 ||
 				DirTime != ms->ms_AllocatedSpool->so_DirTime)
-	    createSpoolDir(ms->ms_AllocatedSpool, DirTime);
+	    if (createSpoolDir(ms->ms_AllocatedSpool, DirTime) == -1)
+		return(-1);
     }
     return(1);
 }
@@ -341,8 +366,8 @@ findSpoolGrp(const char *msgid, GroupList *groups, int size, int ngcount, int ar
 	    if (ngcount && ex->ex_MetaSpool->ms_MaxCross && ngcount >
 					ex->ex_MetaSpool->ms_MaxCross)
 		continue;
-	    if (ex->ex_MetaSpool->ms_Label &&
-				strcmp(ex->ex_MetaSpool->ms_Label, label) != 0)
+	    if (ex->ex_MetaSpool->ms_Label != NULL &&
+			findLabel(ex->ex_MetaSpool->ms_Label, label) == 0)
 		continue;
 	    if (arttype && ArtTypeMatch(arttype,
 					ex->ex_MetaSpool->ms_ArtTypes) == 0)
@@ -377,9 +402,9 @@ addSpool(SpoolObject *so)
 	for (i = 0; i < so->so_SpoolDirs; i++) {
 	    path = GetSpoolPath(so->so_SpoolNum, 10 * i, ARTFILE_DIR);
 	    if (stat(path, &st) != 0) {
-		syslog(LOG_INFO, "Creating spooldir N.%02x for %s", i, path);
+		logit(LOG_INFO, "Creating spooldir N.%02x for %s", i, path);
 		if (mkdir(path, 0755) == -1)  {
-		    syslog(LOG_CRIT, "%s: Unable to create spooldir N.%02x on %s: %s",
+		    logit(LOG_CRIT, "%s: Unable to create spooldir N.%02x on %s: %s",
 				PatLibExpand(DSpoolCtlPat), i, path, strerror(errno));
 		    exit(1);
 		}
@@ -388,7 +413,7 @@ addSpool(SpoolObject *so)
     } else {
 	path = GetSpoolPath(so->so_SpoolNum, 0, ARTFILE_DIR);
 	if (stat(path, &st) != 0) {
-	    syslog(LOG_CRIT, "%s: Missing spool partition %s",
+	    logit(LOG_CRIT, "%s: Missing spool partition %s",
 				PatLibExpand(DSpoolCtlPat), path);
 	    exit(1);
 	}
@@ -408,9 +433,9 @@ addMeta(MetaSpool *ms)
 {
     if (!ms->ms_DontStore && !ms->ms_RejectArts &&
 						ms->ms_NumSpoolObjects == 0) {
-	syslog(LOG_ERR, "%s: No spools defined for metaspool '%s'",
+	logit(LOG_ERR, "%s: No spools defined for metaspool '%s'",
 				PatLibExpand(DSpoolCtlPat), ms->ms_Name);
-	zfree(&EXMemPool, ms, sizeof(MetaSpool));
+	zfree(&SPMemPool, ms, sizeof(MetaSpool));
 	exit(1);
     }
     ms->ms_Next = NULL;
@@ -427,9 +452,9 @@ void
 addExpire(GroupExpire *ex)
 {
     if (ex->ex_MetaSpool == NULL) {
-	syslog(LOG_ERR, "%s: No metaspool defined for expire '%s'",
+	logit(LOG_ERR, "%s: No metaspool defined for expire '%s'",
 				PatLibExpand(DSpoolCtlPat), ex->ex_Wild);
-	zfree(&EXMemPool, ex, sizeof(GroupExpire));
+	zfree(&SPMemPool, ex, sizeof(GroupExpire));
 	exit(1);
     }
     ex->ex_Next = NULL;
@@ -453,6 +478,17 @@ findMeta(char *meta)
     return(NULL);
 }
 
+int
+findLabel(LabelList *ll, char *label)
+{
+    while (ll != NULL) {
+	if (strcmp(ll->label, label) == 0)
+	    return(1);
+	ll = ll->next;
+    }
+    return(0);
+}
+
 uint32 SpoolGmtMin = (uint32)-1;
 time_t SpoolMTime = 0;
 
@@ -473,7 +509,7 @@ loadSpoolCtl(FILE *fi)
     MetaSpool *metaSpool = NULL;
     GroupExpire *expire = NULL;
 
-    freePool(&EXMemPool);
+    freePool(&SPMemPool);
     SpoolObjects = NULL;
     MetaSpools = NULL;
     ExBase = NULL;
@@ -484,7 +520,7 @@ loadSpoolCtl(FILE *fi)
      * It can be modified later
      */
     {
-	spoolObj = zalloc(&EXMemPool, sizeof(SpoolObject));
+	spoolObj = zalloc(&SPMemPool, sizeof(SpoolObject));
 	spoolObj->so_SpoolNum = 0;
 	spoolObj->so_CompressLvl = -1;
 	spoolObj->so_Next = NULL;
@@ -523,7 +559,7 @@ loadSpoolCtl(FILE *fi)
 		     expire = NULL;
 		     break;
 		default:
-		     syslog(LOG_ERR, "%s: Extra 'end' in line %d",
+		     logit(LOG_ERR, "%s: Extra 'end' in line %d",
 					PatLibExpand(DSpoolCtlPat), line);
 		     exit(1);
 	    }
@@ -539,11 +575,11 @@ loadSpoolCtl(FILE *fi)
 	    if (spoolnum == 0) {
 		spoolObj = SpoolObjects;
 	    } else {
-		spoolObj = zalloc(&EXMemPool, sizeof(SpoolObject));
+		spoolObj = zalloc(&SPMemPool, sizeof(SpoolObject));
 		spoolObj->so_SpoolNum = spoolnum;
 	    }
 	    if (spoolObj->so_SpoolNum > 99) {
-		syslog(LOG_ERR, "%s: Invalid spool object number %d in line %d",
+		logit(LOG_ERR, "%s: Invalid spool object number %d in line %d",
 					PatLibExpand(DSpoolCtlPat),
 					spoolObj->so_SpoolNum, line);
 		exit(1);
@@ -553,7 +589,7 @@ loadSpoolCtl(FILE *fi)
 	    continue;
 	}
 	if (!status && strcmp(cmd, "metaspool") == 0) {
-	    metaSpool = zalloc(&EXMemPool, sizeof(MetaSpool));
+	    metaSpool = zalloc(&SPMemPool, sizeof(MetaSpool));
 	    snprintf(metaSpool->ms_Name, sizeof(metaSpool->ms_Name), "%s", arg);
 	    status = EXSTAT_META;
 	    metaSpool->ms_ArtTypes = NULL;
@@ -563,20 +599,20 @@ loadSpoolCtl(FILE *fi)
 	}
 	if (!status && strcmp(cmd, "expire") == 0) {
 	    if (arg == NULL) {
-		syslog(LOG_ERR, "%s: Missing metaspool for expire in line %d",
+		logit(LOG_ERR, "%s: Missing metaspool for expire in line %d",
 					PatLibExpand(DSpoolCtlPat), line);
 		exit(1);
 	    }
 	    cmd = strtok(NULL, " \t\n\r");
 	    if (cmd == NULL) {
-		syslog(LOG_ERR, "%s: Missing metaspool for expire in line %d",
+		logit(LOG_ERR, "%s: Missing metaspool for expire in line %d",
 					PatLibExpand(DSpoolCtlPat), line);
 		exit(1);
 	    }
-	    expire = zalloc(&EXMemPool, sizeof(GroupExpire));
+	    expire = zalloc(&SPMemPool, sizeof(GroupExpire));
 	    snprintf(expire->ex_Wild, sizeof(expire->ex_Wild), "%s", arg);
 	    if ((expire->ex_MetaSpool = findMeta(cmd)) == NULL) {
-		syslog(LOG_ERR, "%s: Unknown metaspool '%s' in line %d",
+		logit(LOG_ERR, "%s: Unknown metaspool '%s' in line %d",
 					PatLibExpand(DSpoolCtlPat), cmd, line);
 		exit(1);
 	    }
@@ -587,7 +623,7 @@ loadSpoolCtl(FILE *fi)
 	}
 
 	if (status == EXSTAT_NONE) {
-	    syslog(LOG_ERR, "%s: Unknown definition '%s' in line %d",
+	    logit(LOG_ERR, "%s: Unknown definition '%s' in line %d",
 				PatLibExpand(DSpoolCtlPat), cmd, line);
 	    exit(1);
 	}
@@ -611,7 +647,7 @@ loadSpoolCtl(FILE *fi)
 		else if (strcmp(arg, "dirsize") == 0)
 		    spoolObj->so_ExpireMethod = 1;
 		else  {
-		    syslog(LOG_ERR, "%s: Unknown expiremethod '%s' in line %d",
+		    logit(LOG_ERR, "%s: Unknown expiremethod '%s' in line %d",
 				PatLibExpand(DSpoolCtlPat), arg, line);
 		    exit(1);
 		}                   
@@ -620,15 +656,18 @@ loadSpoolCtl(FILE *fi)
 		const char *shome = PatExpand(SpoolHomePat);
 		if (strncmp(arg, "%s/", 3) == 0) {
 		    arg += 3;
-		    strncpy(spoolObj->so_Path, arg, sizeof(spoolObj->so_Path));
+		    strncpy(spoolObj->so_Path, arg, sizeof(spoolObj->so_Path) -1);
+		    spoolObj->so_Path[sizeof(spoolObj->so_Path) - 1] = '\0';
 		} else if (strcmp(arg, shome) != 0 &&
 				strncmp(arg, shome, strlen(shome)) == 0 &&
 				strlen(arg) > strlen(shome) &&
 				arg[strlen(shome)] == '/') {
 		    strncpy(spoolObj->so_Path, arg + strlen(shome) + 1,
-						sizeof(spoolObj->so_Path));
+						sizeof(spoolObj->so_Path) - 1);
+		    spoolObj->so_Path[sizeof(spoolObj->so_Path) - 1] = '\0';
 		} else {
-		    strncpy(spoolObj->so_Path, arg, sizeof(spoolObj->so_Path));
+		    strncpy(spoolObj->so_Path, arg, sizeof(spoolObj->so_Path) - 1);
+		    spoolObj->so_Path[sizeof(spoolObj->so_Path) - 1] = '\0';
 		}
 		continue;
 	    } else if (strcmp(cmd, "spooldirs") == 0) {
@@ -638,7 +677,7 @@ loadSpoolCtl(FILE *fi)
 		spoolObj->so_CompressLvl = strtol(arg, NULL, 0);
 		continue;
 	    } else {
-		syslog(LOG_ERR, "%s: Unknown spool option '%s' in line %d",
+		logit(LOG_ERR, "%s: Unknown spool option '%s' in line %d",
 					PatLibExpand(DSpoolCtlPat), cmd, line);
 	    }
 	}
@@ -654,13 +693,15 @@ loadSpoolCtl(FILE *fi)
 		continue;
 	    } else if (strcmp(cmd, "reallocint") == 0) {
 		metaSpool->ms_ReAllocInterval = btimetol(arg);
+		if (metaSpool->ms_ReAllocInterval < 60)
+		    metaSpool->ms_ReAllocInterval = 60;
 		continue;
 	    } else if (strcmp(cmd, "arttypes") == 0) {
 		char *p = arg;
 		ArtTypeList *atp = NULL;
 		ArtTypeList *tatp;
 		while ((p = strsep(&arg, " \t:,")) != NULL) {
-		    tatp = zalloc(&EXMemPool, sizeof(ArtTypeList));
+		    tatp = zalloc(&SPMemPool, sizeof(ArtTypeList));
 		    tatp->negate = 0;
 		    if (*p == '!') {
 			tatp->negate = 1;
@@ -683,7 +724,7 @@ loadSpoolCtl(FILE *fi)
 		    if ((p = strchr(arg, '/')) != NULL)
 			metaSpool->ms_HashFeed.hf_Mod = strtol(++p, NULL, 0);
 		    else
-			syslog(LOG_ERR, "%s: Unknown hashfeed in line %d\n",
+			logit(LOG_ERR, "%s: Unknown hashfeed in line %d\n",
 				PatLibExpand(DSpoolCtlPat), line);
 		} else {
 		    metaSpool->ms_HashFeed.hf_Begin = strtol(arg, NULL, 0);
@@ -691,7 +732,7 @@ loadSpoolCtl(FILE *fi)
 		    if ((p = strchr(arg, '/')) != NULL)
 			metaSpool->ms_HashFeed.hf_Mod = strtol(++p, NULL, 0);
 		    else
-			syslog(LOG_ERR, "%s: Unknown hashfeed in line %d\n",
+			logit(LOG_ERR, "%s: Unknown hashfeed in line %d\n",
 				PatLibExpand(DSpoolCtlPat), line);
 		}
 		continue;
@@ -704,16 +745,23 @@ loadSpoolCtl(FILE *fi)
 		    metaSpool->ms_DontStore = 1;
 		continue;
 	    } else if (strcmp(cmd, "label") == 0) {
-		metaSpool->ms_Label = zalloc(&EXMemPool, strlen(arg) + 1);
-		strcpy(metaSpool->ms_Label, arg);
+		LabelList *l;
+
+		l = zalloc(&SPMemPool, sizeof(LabelList));
+		l->label = zalloc(&SPMemPool, strlen(arg) + 1);
+		l->next = metaSpool->ms_Label;
+		metaSpool->ms_Label = l;
+		strcpy(metaSpool->ms_Label->label, arg);
 		continue;
 	    } else if (strcmp(cmd, "allocstrat") == 0) {
 		if (strcasecmp(arg, "sequential") == 0)
 		    metaSpool->ms_AllocationStrategy = SPOOL_ALLOC_SEQ;
 		else if (strcasecmp(arg, "space") == 0)
 		    metaSpool->ms_AllocationStrategy = SPOOL_ALLOC_SPACE;
+		else if (strcasecmp(arg, "single") == 0)
+		    metaSpool->ms_AllocationStrategy = SPOOL_ALLOC_SINGLE;
 		else
-		    syslog(LOG_ERR, "%s: Unknown alloc strategy in line %d\n",
+		    logit(LOG_ERR, "%s: Unknown alloc strategy in line %d\n",
 				PatLibExpand(DSpoolCtlPat), line);
 		continue;
 	    } else if (strcmp(cmd, "spool") == 0) {
@@ -726,7 +774,7 @@ loadSpoolCtl(FILE *fi)
 		    }
 		}
 		if (ts == NULL) {
-		    syslog(LOG_ERR, "%s: Unknown spool '%d' in line %d",
+		    logit(LOG_ERR, "%s: Unknown spool '%d' in line %d",
                                 PatLibExpand(DSpoolCtlPat), sn, line);
 		    exit(1);
 
@@ -735,18 +783,18 @@ loadSpoolCtl(FILE *fi)
 	    } else if (strcmp(cmd, "addgroup") == 0) {
 		cmd = strtok(cmd, " \t\n\r");
 		if (cmd == NULL) {
-		    syslog(LOG_ERR, "%s: Missing group for addgroup in line %d",
+		    logit(LOG_ERR, "%s: Missing group for addgroup in line %d",
 					PatLibExpand(DSpoolCtlPat), line);
 		    exit(1);
 		}
-		expire = zalloc(&EXMemPool, sizeof(GroupExpire));
+		expire = zalloc(&SPMemPool, sizeof(GroupExpire));
 		snprintf(expire->ex_Wild, sizeof(expire->ex_Wild), "%s", arg);
 		expire->ex_MetaSpool = metaSpool;
 		addExpire(expire);
 		expire = NULL;
 		continue;
 	    } else {
-		syslog(LOG_ERR, "%s: Unknown metaspool option '%s' in line %d",
+		logit(LOG_ERR, "%s: Unknown metaspool option '%s' in line %d",
 					PatLibExpand(DSpoolCtlPat), cmd, line);
 	    }
 	}
@@ -781,7 +829,7 @@ LoadSpoolCtl(time_t gmt, int force)
 	    fi = fopen(PatLibExpand(DSpoolCtlPat), "r");
 
 	    if (fi == NULL) {
-		syslog(LOG_EMERG, "%s file not found",
+		logit(LOG_EMERG, "%s file not found",
 					PatLibExpand(DSpoolCtlPat));
 		exit(1);
 	    }
@@ -864,7 +912,7 @@ spaceFreeOn(char *part)
 #else
     if (statfs(part, &stmp) != 0) {
 #endif
-        syslog(LOG_ERR, "unable to statfs %s (%s)", part, strerror(errno));   
+        logit(LOG_ERR, "unable to statfs %s (%s)", part, strerror(errno));   
         return(0.0);
     }
     return(stmp.f_bavail * 1.0 * stmp.f_bsize);
