@@ -334,7 +334,7 @@ dumpInStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
     if (fs->RecStats.TimeStart == 0)
 	return;
     if (raw)
-	fprintf(fo, "INFEED %-20s secs=%lu con=%u off=%u rec=%u acc=%u ref=%u rej=%u recbytes=%.0f accbytes=%.0f rejbytes=%.0f\n",
+	fprintf(fo, "INFEED %-20s secs=%u con=%u off=%u rec=%u acc=%u ref=%u rej=%u recbytes=%.0f accbytes=%.0f rejbytes=%.0f\n",
 				hostname,
 				time(NULL) - fs->RecStats.TimeStart,
 				fs->RecStats.ConnectCnt,
@@ -348,7 +348,7 @@ dumpInStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
 				fs->RecStats.RejectedBytes
 	);
     else
-	fprintf(fo, "INFEED %-20s secs=%lu con=%u off=%u rec=%u acc=%u ref=%u rej=%u recbytes=%s accbytes=%s rejbytes=%s\n",
+	fprintf(fo, "INFEED %-20s secs=%u con=%u off=%u rec=%u acc=%u ref=%u rej=%u recbytes=%s accbytes=%s rejbytes=%s\n",
 				hostname,
 				time(NULL) - fs->RecStats.TimeStart,
 				fs->RecStats.ConnectCnt,
@@ -369,7 +369,7 @@ dumpInDetStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
     if (fs->RecStats.TimeStart == 0)
 	return;
     if (raw)
-	fprintf(fo, "INFEEDDETAIL %-20s secs=%ld ihave=%d chk=%d takethis=%d rec=%d acc=%d ref=%d precom=%d postcom=%d his=%d badmsgid=%d rej=%d ctl=%d failsafe=%d misshdrs=%d tooold=%d grpfilt=%d intspamfilt=%d extspamfilt=%d incfilter=%d nospool=%d ioerr=%d notinactv=%d pathtab=%d ngtab=%d posdup=%d hdrerr=%d toosmall=%d incompl=%d nul=%d nobytes=%d proto=%d msgidmis=%d err=%d toobig=%d recbytes=%.0f accbytes=%.0f rejbytes=%.0f\n",
+	fprintf(fo, "INFEEDDETAIL %-20s secs=%u ihave=%d chk=%d takethis=%d rec=%d acc=%d ref=%d precom=%d postcom=%d his=%d badmsgid=%d rej=%d ctl=%d failsafe=%d misshdrs=%d tooold=%d grpfilt=%d intspamfilt=%d extspamfilt=%d incfilter=%d nospool=%d ioerr=%d notinactv=%d pathtab=%d ngtab=%d posdup=%d hdrerr=%d toosmall=%d incompl=%d nul=%d nobytes=%d proto=%d msgidmis=%d err=%d toobig=%d recbytes=%.0f accbytes=%.0f rejbytes=%.0f\n",
 				hostname,
 				time(NULL) - fs->RecStats.TimeStart,
 				fs->RecStats.Stats[STATS_IHAVE],
@@ -411,7 +411,7 @@ dumpInDetStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
 				fs->RecStats.RejectedBytes
 	);
     else
-	fprintf(fo, "INFEEDDETAIL %-20s secs=%ld ihave=%d chk=%d takethis=%d rec=%d acc=%d ref=%d precom=%d postcom=%d his=%d badmsgid=%d rej=%d ctl=%d failsafe=%d misshdrs=%d tooold=%d grpfilt=%d intspamfilt=%d extspamfilt=%d incfilter=%d nospool=%d ioerr=%d notinactv=%d pathtab=%d ngtab=%d posdup=%d hdrerr=%d toosmall=%d incompl=%d nul=%d nobytes=%d proto=%d msgidmis=%d err=%d toobig=%d recbytes=%s accbytes=%s rejbytes=%s\n",
+	fprintf(fo, "INFEEDDETAIL %-20s secs=%u ihave=%d chk=%d takethis=%d rec=%d acc=%d ref=%d precom=%d postcom=%d his=%d badmsgid=%d rej=%d ctl=%d failsafe=%d misshdrs=%d tooold=%d grpfilt=%d intspamfilt=%d extspamfilt=%d incfilter=%d nospool=%d ioerr=%d notinactv=%d pathtab=%d ngtab=%d posdup=%d hdrerr=%d toosmall=%d incompl=%d nul=%d nobytes=%d proto=%d msgidmis=%d err=%d toobig=%d recbytes=%s accbytes=%s rejbytes=%s\n",
 				hostname,
 				time(NULL) - fs->RecStats.TimeStart,
 				fs->RecStats.Stats[STATS_IHAVE],
@@ -460,7 +460,7 @@ dumpOutStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
     if (fs->SentStats.TimeStart == 0)
 	return;
     if (raw)
-	fprintf(fo, "OUTFEED %-20s secs=%lu con=%u off=%u acc=%u ref=%u rej=%u deffail=%u accbytes=%.0f rejbytes=%.0f\n",
+	fprintf(fo, "OUTFEED %-20s secs=%u con=%u off=%u acc=%u ref=%u rej=%u deffail=%u accbytes=%.0f rejbytes=%.0f\n",
 				hostname,
 				time(NULL) - fs->SentStats.TimeStart,
 				fs->SentStats.ConnectCnt,
@@ -473,7 +473,7 @@ dumpOutStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
 				fs->SentStats.RejectedBytes
 		);
     else
-	fprintf(fo, "OUTFEED %-20s secs=%lu con=%u off=%u acc=%u ref=%u rej=%u deffail=%u accbytes=%s rejbytes=%s\n",
+	fprintf(fo, "OUTFEED %-20s secs=%u con=%u off=%u acc=%u ref=%u rej=%u deffail=%u accbytes=%s rejbytes=%s\n",
 				hostname,
 				time(NULL) - fs->SentStats.TimeStart,
 				fs->SentStats.ConnectCnt,
@@ -493,7 +493,7 @@ dumpSpoolStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
     if (fs->SpoolStats.TimeStart == 0)
 	return;
     if (raw)
-	fprintf(fo, "SPOOL %-20s secs=%lu con=%u stat=%u article=%u head=%u body=%u bytes=%.0f\n",
+	fprintf(fo, "SPOOL %-20s secs=%u con=%u stat=%u article=%u head=%u body=%u bytes=%.0f\n",
 				hostname,
 				time(NULL) - fs->SpoolStats.TimeStart,
 				fs->SpoolStats.ConnectCnt,
@@ -504,7 +504,7 @@ dumpSpoolStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
 				fs->SpoolStats.ArtsBytesSent
 		);
     else
-	fprintf(fo, "SPOOL %-20s secs=%lu con=%u stat=%u article=%u head=%u body=%u bytes=%s\n",
+	fprintf(fo, "SPOOL %-20s secs=%u con=%u stat=%u article=%u head=%u body=%u bytes=%s\n",
 				hostname,
 				time(NULL) - fs->SpoolStats.TimeStart,
 				fs->SpoolStats.ConnectCnt,
@@ -522,7 +522,7 @@ dumpSpoolDetStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
     if (fs->SpoolStats.TimeStart == 0)
 	return;
     if (raw)
-	fprintf(fo, "SPOOLDETAIL %-20s secs=%lu con=%u stat=%u statmiss=%d statexp=%d staterr=%d article=%u articlemiss=%d articleexp=%d articleerr=%d head=%u headmiss=%d headexp=%d headerr=%d body=%u bodymiss=%d bodyexp=%d bodyerr=%d bytes=%.0f\n",
+	fprintf(fo, "SPOOLDETAIL %-20s secs=%u con=%u stat=%u statmiss=%d statexp=%d staterr=%d article=%u articlemiss=%d articleexp=%d articleerr=%d head=%u headmiss=%d headexp=%d headerr=%d body=%u bodymiss=%d bodyexp=%d bodyerr=%d bytes=%.0f\n",
 				hostname,
 				time(NULL) - fs->SpoolStats.TimeStart,
 				fs->SpoolStats.ConnectCnt,
@@ -545,7 +545,7 @@ dumpSpoolDetStats(FILE *fo, char *hostname, FeedStats *fs, int raw)
 				fs->SpoolStats.ArtsBytesSent
 		);
     else
-	fprintf(fo, "SPOOLDETAIL %-20s secs=%lu con=%u stat=%u statmiss=%d statexp=%d staterr=%d article=%u articlemiss=%d articleexp=%d articleerr=%d head=%u headmiss=%d headexp=%d headerr=%d body=%u bodymiss=%d bodyexp=%d bodyerr=%d bytes=%s\n",
+	fprintf(fo, "SPOOLDETAIL %-20s secs=%u con=%u stat=%u statmiss=%d statexp=%d staterr=%d article=%u articlemiss=%d articleexp=%d articleerr=%d head=%u headmiss=%d headexp=%d headerr=%d body=%u bodymiss=%d bodyexp=%d bodyerr=%d bytes=%s\n",
 				hostname,
 				time(NULL) - fs->SpoolStats.TimeStart,
 				fs->SpoolStats.ConnectCnt,
