@@ -21,7 +21,10 @@ Prototype const char *PatSpoolExpand(const char *pat);
 Prototype const char *PatRunExpand(const char *pat);
 Prototype int CidrMatch(const char *cidr, const char *ip);
 Prototype int IsIpAddr(char *a);
-Prototype int TestForwardLookup(const char *hostName, const struct sockaddr_in *rsin);
+Prototype char *SanitiseAddr(char *addr);
+Prototype void GetIPHost(struct sockaddr *sa, char *hname, int hnamelen, char *ipname, int iplen);
+Prototype int TestForwardLookup(const char *hostName, const char *ipst, const struct sockaddr *res);
+Prototype char *NetAddrToSt(int fd, struct sockaddr *sa, int ip4conv, int port, int v6spec);
 Prototype int ValidGroupName(const char *name);
 Prototype void SanitizeString(char *name);
 Prototype void SanitizeDescString(char *name);
@@ -36,6 +39,8 @@ Prototype char *strdupfree(char **st, char *val, char *empty);
 Prototype char *safestr(char *st, char *noval);
 Prototype int enabled(char *st);
 Prototype int HashFeedMatch(HashFeed *hashfeed, int article);
+Prototype int MoveFile(char *from, char *to);
+Prototype int TimeSpec(char *t, char *def);
 
 u_long ascToAddr(const char *str);
 u_long ascToMask(const char *str);
@@ -85,19 +90,6 @@ diablo_strlcpynl(char *d, const char *s, int ssize, int dsize)
     }
 }
 
-#if USE_INTERNAL_VSYSLOG
-
-void
-vsyslog(int priority, const char *ctl, va_list va)
-{
-    char buf[1024];
-
-    vsnprintf(buf, sizeof(buf), ctl, va);
-    syslog(priority, "%s", buf);
-}
-
-#endif
-
 static char PatPath1[128];
 static char PatPath2[128];
 
@@ -149,13 +141,42 @@ PatRunExpand(const char *pat)
 int 
 IsIpAddr(char *a)
 {
-    while (*a) {
-        if (! (isdigit((int)*a) || *a == '.' || *a == '/' ||
-                *a == '*' || *a == '?'))  
-            return(0);
-        a++;
-    }
-    return(1);
+    const char *spanstr;
+
+#ifdef INET6
+    if (strchr(a, ':') != NULL)
+	spanstr = "0123456789abcdefgh.:/*?[]";
+    else
+#endif
+	spanstr = "0123456789./*?";
+    if (strspn(a, spanstr) == strlen(a))
+	return(1);
+    else
+	return(0);
+}
+
+/*
+ * Sanitise an IP[v6] address by removing the '[]'
+ */
+char *
+SanitiseAddr(char *addr)
+{
+    static char st[64];
+    char *p;
+    int len;
+
+    if (addr == NULL)
+	return(NULL);
+    if (*addr == '[')
+	addr++;
+    len = strlen(addr);
+    if ((p = strchr(addr, ']')) != NULL)
+	len = p - addr;
+    if (len >= sizeof(st))
+	len = sizeof(st) - 1;
+    strncpy(st, addr, len);
+    st[len] = '\0';
+    return(st);
 }
 
 u_long
@@ -216,21 +237,99 @@ ascToMask(const char *str)
 
 /*
  * Match CIDR address - return 1 for sucess else 0
+ *
+ * This is really messy
  */
 int
 CidrMatch(const char *cidr, const char *ip)
 {
-    u_long net;
-    u_long mask;
-    u_long addr;
+#ifdef INET6
+    char addr[256];
+    char mask[256];
+    uint8 maskbits = 128;
+    char maskst[NI_MAXHOST];
+    char *p;
+    int i;
 
-    addr = ascToAddr(ip);
-    net = ascToAddr(cidr);
-    mask = ascToMask(cidr);
-    if ((addr & mask) != (net & mask))
-	return(0);
-    else
+    if (strchr(ip, ':') != NULL) {
+	uint8 b8;
+	uint8 b;
+
+	if (strchr(cidr, ':') == NULL)	/* Don't compare against non v6 CIDR */
+	    return(0);
+	strcpy(maskst, cidr);
+	if ((p = strchr(maskst, '/')) != NULL) {
+	    *p = '\0';
+	    maskbits = atoi(++p);
+	}
+	if (inet_pton(AF_INET6, ip, addr) != 1) {
+	    logit(LOG_ERR, "Invalid IPv6 address: %s", ip);
+	    return(0);
+	}
+	if (inet_pton(AF_INET6, maskst, mask) != 1) {
+	    logit(LOG_ERR, "Invalid IPv6 address: %s", cidr);
+	    return(0);
+	}
+	
+	for (i = 0; i * 8 < maskbits; i++) {
+	    if (i < maskbits / 8)
+		b8 = 255;
+	    else
+		for (b8 = 0, b = 0; b < maskbits % 8; b++)
+		    b8 |= (1 << (7 - b));
+	    if ((addr[i] & b8) != (mask[i] & b8))
+		return(0);
+	}
 	return(1);
+    } else
+#endif
+    {
+	u_long net;
+	u_long mask;
+	u_long addr;
+
+	if (strchr(cidr, ':') != NULL)	/* Don't compare against v6 CIDR */
+	    return(0);
+	addr = ascToAddr(ip);
+	net = ascToAddr(cidr);
+	mask = ascToMask(cidr);
+	if ((addr & mask) != (net & mask))
+	    return(0);
+	return(1);
+    }
+}
+
+void
+GetIPHost(struct sockaddr *sa, char *hname, int hnamelen, char *ipname, int iplen)
+{
+    strncpy(ipname, NetAddrToSt(0, sa, 1, 0, 0), iplen - 1);
+    ipname[iplen - 1] = '\0';
+
+    *hname = '\0';
+    {
+#ifdef INET6
+	int error;
+
+	if ((error = getnameinfo(sa, SA_LEN(sa), hname, hnamelen,
+					NULL, 0, NI_NAMEREQD)) != 0)
+		*hname = '\0';
+#else
+	struct hostent *he;
+	struct sockaddr_in *sin = (struct sockaddr_in *)sa;
+
+	he = gethostbyaddr(
+	    (char *)&sin->sin_addr,
+	    sizeof(sin->sin_addr),
+	    AF_INET
+	);
+
+	if (he != NULL) {
+	    strncpy(hname, he->h_name, hnamelen - 1);
+	    hname[hnamelen - 1] = '\0';
+	}
+#endif /* INET6 */
+    }
+
 }
 
 /*
@@ -240,9 +339,33 @@ CidrMatch(const char *cidr, const char *ip)
  */
 
 int
-TestForwardLookup(const char *hostName, const struct sockaddr_in *rsin)
+TestForwardLookup(const char *hostName, const char *ipst, const struct sockaddr *sa)
 {
+#ifdef INET6
+    struct addrinfo *ai0;
+    int r = -1;
+
+    stprintf("forward auth %s", hostName);
+    if (getaddrinfo(hostName, NULL, NULL, &ai0) == 0) {
+	struct addrinfo *ai;
+
+	for (ai = ai0; ai; ai = ai->ai_next) {
+	    char *st = NetAddrToSt(0, ai->ai_addr, 1, 0, 0);
+	    if (st != NULL && strcmp(ipst, st) == 0) {
+		r = 0;
+		break;
+	    }
+	}
+	if (ai == NULL)
+	    logit(LOG_NOTICE, "DNS Fwd/Rev mismatch: %s/%s", hostName, ipst);
+	freeaddrinfo(ai0);
+    } else {
+	logit(LOG_NOTICE, "DNS Fwd/Rev mismatch: lookup of %s failed", hostName);
+    }
+    return(r);
+#else
     struct hostent *he;
+    struct sockaddr_in *sin = (struct sockaddr_in *)sa;
     int r = -1;
 
     if ((he = gethostbyname(hostName)) != NULL) {
@@ -250,17 +373,18 @@ TestForwardLookup(const char *hostName, const struct sockaddr_in *rsin)
 
 	for (i = 0; he->h_addr_list && he->h_addr_list[i]; ++i) {
 	    const struct in_addr *haddr = (const void *)he->h_addr_list[i];
-	    if (rsin->sin_addr.s_addr == haddr->s_addr) {
+	    if (sin->sin_addr.s_addr == haddr->s_addr) {
 		r = 0;
 		break;
 	    }
 	}
 	if (he->h_addr_list[i] == NULL)
-	    logit(LOG_NOTICE, "DNS Fwd/Rev mismatch: %s/%s", hostName, inet_ntoa(rsin->sin_addr));
+	    logit(LOG_NOTICE, "DNS Fwd/Rev mismatch: %s/%s", hostName, ipst);
     } else {
 	logit(LOG_NOTICE, "DNS Fwd/Rev mismatch: lookup of %s failed", hostName);
     }
     return(r);
+#endif
 }
 
 int 
@@ -327,28 +451,10 @@ void
 SanitizeDescString(char *name)
 {
     for (; *name; ++name) {
-	if (*name >= 'a' && *name <= 'z')
-	    continue;
-	if (*name >= 'A' && *name <= 'Z')
-	    continue;
-	if (*name >= '0' && *name <= '9')
-	    continue;
 	if (*name == '\t')
 	    *name = ' ';
-	if (*name == '@' ||
-	    *name == '-' ||
-	    *name == '+' ||
-	    *name == '_' ||
-	    *name == '.' ||
-	    *name == ',' ||
-	    *name == '(' ||
-	    *name == ')' ||
-	    *name == '<' ||
-	    *name == '>' ||
-	    *name == ' '
-	) {
+	if (!iscntrl((int)*name))  
 	    continue;
-	}
 	*name = '?';
     }
 }
@@ -365,8 +471,8 @@ StrnCpyNull(char *dst, const char *src, size_t maxlen)
     if (dst) {
     	if (src) {
 	    if (strlen(src) >= maxlen) {
-		strncpy(dst, src, maxlen-1);
-		dst[maxlen] = '\0';
+		strncpy(dst, src, maxlen - 1);
+		dst[maxlen - 1] = '\0';
 	    }
 	    else {
 		strcpy(dst, src);
@@ -528,6 +634,77 @@ dtlenstr(time_t t)
     return(tb);
 }
 
+#ifdef INET6
+/*
+ * A protocol independent conversion of network address to string
+ *
+ * Strips the initial portion of a IPv6 encoded IPv4 address if
+ * requested
+ *
+ * If sa == NULL then the address is obtained from the fd passed
+ * otherwise the fd is ignored.
+ */
+char *
+NetAddrToSt(int fd, struct sockaddr *sa, int ip4conv, int port, int v6spec)
+{
+    static char st[NI_MAXHOST + 8];
+    char addrst[NI_MAXHOST];
+    char serv[16];
+    int error;
+
+    if (sa == NULL) {
+	static struct sockaddr_storage res;
+	int salen = sizeof(res);
+
+	sa = (struct sockaddr *)&res;
+	if (getpeername(fd, sa, &salen) != 0) {
+	    logit(LOG_ERR, "%s", strerror(errno));
+	    return(NULL);
+	}
+    }
+    if (SA_LEN(sa) == 0)
+	return(NULL);
+    if ((error = getnameinfo(sa, SA_LEN(sa), addrst, sizeof(addrst),
+					serv, sizeof(serv),
+					NI_NUMERICHOST|NI_NUMERICSERV)) == 0) {
+	if (v6spec && strchr(addrst, ':')) {
+	    snprintf(st, sizeof(st), "[%s]", addrst);
+	} else {
+	    strncpy(st, addrst, sizeof(st) - 1);
+	    st[sizeof(st) - 1] = '\0';
+	}
+	if (port) {
+	    strncat(st, ":", sizeof(st) - 1);
+	    strncat(st, serv, sizeof(st) - 1);
+	}
+	if (ip4conv && strncmp(st, "::ffff:", 7) == 0)
+            return(st + 7);
+	else
+            return(st);
+    } else {
+            logit(LOG_ERR, "getnameinfo: %s\n", gai_strerror(error));
+    }
+    return(NULL);
+}
+#else
+char *
+NetAddrToSt(int fd, struct sockaddr *sa, int ip4conv, int port, int v6spec)
+{
+    static char st[128];
+    struct sockaddr_in *sin = (struct sockaddr_in *)sa;
+
+    strncpy(st, inet_ntoa(sin->sin_addr), sizeof(st) - 1);
+    st[sizeof(st) - 1] = '\0';
+    if (port) {
+	char port[16];
+	snprintf(port, sizeof(port), ":%d", ntohs(sin->sin_port));
+	strncat(st, port, sizeof(st) - 1);
+    }
+    return(st);
+}
+
+#endif
+
 /*
  * Create the directories required for the file pathname specified in path
  *   path = the full path/filename which gets clobbered.
@@ -607,3 +784,75 @@ HashFeedMatch(HashFeed *hashfeed, int hash)
 
 }
 
+int
+MoveFile(char *from, char *to)
+{
+    int from_fd;
+    int to_fd;
+    static char buffer[32768];
+    int rlen;
+
+    if ((from_fd = open(from, O_RDONLY, 0)) < 0) {
+	printf("ERROR: Unable to open input file: %s\n", from);
+	return(-1);
+    }
+    if ((to_fd = open(to, O_CREAT | O_TRUNC | O_WRONLY, 0644)) < 0) {
+	printf("ERROR: Unable to create output file: %s\n", to);
+	return(-1);
+    }
+    while ((rlen = read(from_fd, buffer, 32768)) > 0)
+	if (write(to_fd, buffer, rlen) != rlen) {
+	    printf("ERROR: Unable to writing to file: %s\n", to);
+	    close(from_fd);
+	    close(to_fd);
+	    unlink(to);
+	    return(-1);
+	}
+    if (rlen < 0) {
+	printf("ERROR: Unable to read from file: %s\n", from);
+	close(from_fd);
+	close(to_fd);
+	unlink(to);
+	return(-1);
+    }
+    close(from_fd);
+    if (close(to_fd) != 0) {
+	printf("ERROR: Unable to close file: %s\n", to);
+	unlink(to);
+	return(-1);
+    }
+    unlink(from);
+    return(0);
+}
+
+/*
+ * Convert a string time specification in days, hours, minutes or seconds
+ * into a integer of seconds.
+ */
+int
+TimeSpec(char *t, char *def)
+{
+    char *endp;
+    int tt;
+
+    tt = strtol(t, &endp, 0);
+    if (endp == NULL || !*endp)
+	endp = def;
+    switch (*endp) {
+	case 's':
+	    break;
+	case 'm':
+	    tt *= 60;
+	    break;
+	case 'h':
+	    tt *= 60 * 60;
+	    break;
+	case 'd':
+	    tt *= 24 * 60 * 60;
+	    break;
+	default:
+	    fprintf(stderr, "Invalid time specification: %s \n", t);
+	    tt = -1;
+    }
+    return(tt);
+}
