@@ -100,9 +100,9 @@ typedef struct SpoolEnt {
 struct FileSystem *FileSystems = NULL;
 struct Partition *Partitions = NULL;
 struct SpoolEnt **SpoolEntries;
-int		entryIdx;
+int		entryIdx = 0;
 int		entryMax = 64;
-long		FreeSpaceTargetList[100];
+double		FreeSpaceTargetList[100];
 uint16		*SinglePart = NULL;
 time_t		TimeNow;
 
@@ -113,6 +113,7 @@ int UseDirSize = 0;
 int MaxPass = 0;
 int UnexpireOpt = 0;
 int DoCleanup = 1;
+int TestOpt = 0;
 int UpdateHistory = 0;		/* Do we run the history update */
 int HistoryUpdateOpt = -1;	/* Do we actually update the history entry 
 				 *	0 = No
@@ -121,6 +122,9 @@ int HistoryUpdateOpt = -1;	/* Do we actually update the history entry
 				 *	-1 = if changes
 				 */
 int WriteHashesToFileOpt = 0;
+const char *HistoryFile = NULL;
+off_t HistoryEnd;
+int HistoryFd = -1;
 
 int CleanAllSpools(void);
 int spoolEntSort(const void *s1, const void *s2);
@@ -133,17 +137,20 @@ void printPartition(Partition *p);
 int updateHistory(void);
 int findNode(const char *path, int createMe);
 void freeNodes(void);
+void dumpNodeTable(void);
 time_t getAge(char *dirname);
 double freeSpaceOn(char *path, int logit, long *freefiles);
 FileSystem *findFileSys(char *path);
+void DumpSpoolEntries(void);
 
 void
 Usage(void)
 {
 	printf("This program performs an expire run on a diablo spool\n");
-	printf("dexpire -a|n [-c0] [-h0|1] [-k] [-O nn] [-o] [-q] [-R s:n] [-S nn] [-s n] [-u] [-v] [-z] [-C diablo.config] [-d[n]] [-V]\n");
+	printf("dexpire -a|n [-c0] [-f historyfile] [-h0|1] [-k] [-O nn] [-o] [-q] [-R s:n] [-S nn] [-s n] [-u] [-v] [-z] [-C diablo.config] [-d[n]] [-V]\n");
 	printf("\t-a\tactually make changes (nothing is done without this option)\n");
 	printf("\t-c0\tdon't perform file removal pass\n");
+	printf("\t-f\tspecify the history file to use\n");
 	printf("\t-h0\tdon't scan history file at all\n");
 	printf("\t-h1\tforce scan of history file\n");
 	printf("\t-k\tdon't update history entries\n");
@@ -154,6 +161,7 @@ Usage(void)
 	printf("\t-R\tset a freespacetarget for a spool (spoolnum:target)\n");
 	printf("\t-S\tonly expire the specified spool number\n");
 	printf("\t-s\tsync before disk free space checks\n");
+	printf("\t-t\ttest expired status to spool status only\n");
 	printf("\t-u\t'unexpire' all articles marked as expired in dhistory\n");
 	printf("\t-v\tbe more verbose\n");
 	printf("\t-z\texpire based on directory size, rather than disk space\n");
@@ -185,6 +193,9 @@ main(int ac, char **av)
 	    case 'c':
 		DoCleanup = strtol(ptr, NULL, 0);;
 		break;
+	    case 'f':
+		HistoryFile = *ptr ? ptr : av[++n];
+		break;
 	    case 'h':
 		if (*ptr == '0')
 		    HistoryUpdateOpt = 0;
@@ -222,6 +233,10 @@ main(int ac, char **av)
 		else
 		    SoftUpdates = 1;
 		break;
+	    case 't':
+		TestOpt = 1;
+		VerboseOpt = 1;
+		break;
 	    case 'u':
 		UnexpireOpt = 1;
 		break;
@@ -278,6 +293,8 @@ main(int ac, char **av)
     if (NotForReal && HistoryUpdateOpt == 1)
 	HistoryUpdateOpt = 2;
     count = 1;
+    if (HistoryFile == NULL)
+	HistoryFile = strdup(PatDbExpand(DHistoryPat));
     TimeNow = time(NULL);
     while (!allOk) {
 	if (VerboseOpt)
@@ -310,6 +327,27 @@ CleanAllSpools(void)
      
     LoadSpoolCtl(0, 1);
 
+    /*
+     * Open and get a copy of the history file size before we scan the spools
+     * because we shouldn't check history entries added beyond this point
+     */
+    if (HistoryUpdateOpt != 0 || UnexpireOpt) {
+	struct stat sb;
+
+	HistoryFd = open(HistoryFile, O_RDWR, 0644);
+	if (HistoryFd == -1) {
+	    fprintf(stderr, "Unable to open history(%s): %s\n",
+						HistoryFile, strerror(errno));
+	    exit(1);
+	}
+	if (fstat(HistoryFd, &sb) != 0) {
+	    fprintf(stderr, "Unable to stat history(%s): %s\n",
+						HistoryFile, strerror(errno));
+	    exit(1);
+	}
+	HistoryEnd = sb.st_size;
+    }
+
     /*
      * Get into spool directory
      */
@@ -358,7 +396,7 @@ CleanAllSpools(void)
 		p->filesys = findFileSys(p->partname);
 		p->next = Partitions;
 		Partitions = p;
-		if (FreeSpaceTargetList[spoolnum])
+		if (FreeSpaceTargetList[spoolnum] > 0.0)
 		    p->minfree = FreeSpaceTargetList[spoolnum];
 		scanDirectory(p);
 	    }
@@ -395,11 +433,17 @@ CleanAllSpools(void)
 	int n;
 	printf("DExpire updating history file\n");
 	n = updateHistory();
-	printf("DExpire history file update complete, %d articles marked expired\n", n);
+	printf("DExpire history file update complete, %d articles %smarked %sexpired\n",
+					n,
+					NotForReal ? "would be " : "",
+					UnexpireOpt ? "un" : "");
     } else {
 	printf("DExpire history file will not be updated\n");
     }
 
+    if (HistoryUpdateOpt != 0 || UnexpireOpt)
+	close(HistoryFd);
+
     /*
      * Cleanup all the allocated space
      */
@@ -453,8 +497,8 @@ addFreeSpaceTarget(char *which)
 	fprintf(stderr, "Invalid spool number for -R (%s)\n", which);
 	exit(1);
     }
-    FreeSpaceTargetList[part] = strtol(++p, NULL, 0);
-    if (FreeSpaceTargetList[part] < 0) {
+    FreeSpaceTargetList[part] = strtod(++p, NULL);
+    if (FreeSpaceTargetList[part] < 0.0) {
 	fprintf(stderr, "Invalid free space target in -R (%d)\n", part);
 	exit(1);
     }
@@ -601,7 +645,7 @@ cleanupDirectories(void)
 		printf("\tsize=%s/%s\n", ftos(part->partsize),
 						ftos(part->maxsize));
 	    if (part->keeptime)
-		printf("\tkeeptime=%ld/%ld secs\n", part->age, part->keeptime);
+		printf("\tkeeptime=%u/%lu secs\n", part->age, part->keeptime);
 	    if (part->minfree)
 		printf("\tfreespace=%s/%s\n", ftos(spacefree),
 						ftos(part->minfree));
@@ -810,7 +854,7 @@ scanDirectory(Partition *scanpart)
 			scanpart->age = TimeNow - t;
 		}
 		if (DebugOpt > 2)
-		    printf("ADD DIR %s%s dirsize=%s age=%ld partsize=%s\n",
+		    printf("ADD DIR %s%s dirsize=%s age=%u partsize=%s\n",
 				scanpart->partname,
 				den->d_name,
 				ftos(SpoolEntries[entryIdx]->dirsize),
@@ -867,16 +911,22 @@ findSize(char *pathname, char *dirname)
 int
 updateHistory(void)
 {
-    uint32 startGmt = TimeNow / 60;	/* time_t in minutes */
     uint32 countExp = 0;
     uint32 numEnt = 0;
-    uint32 countEnt = 0;
+    off_t countEnt = 0;
+    off_t countTestSpool = 0;
+    off_t countTestHistory = 0;
+    off_t countTestExpired = 0;
+    off_t countTestValid = 0;
     int lastPerc = 0;
     FILE *DExpOverList = NULL;
-    char path[512];
+    char path[PATH_MAX];
     int i;
     char spoolHome[512];
     int spoolHomeLen;
+    off_t npos = -1;
+    off_t bpos = -1;
+    History *h;
 
     /*
      * Write expired article msgid hashes to a file if requested.
@@ -936,25 +986,33 @@ updateHistory(void)
     }
 
     /*
-     * scan history file and update the expiration
+     * Scan history file and update the expiration
+     *
+     * The history file was opened before the spool scan to make sure
+     * we get the right history file at this point and that we know
+     * where the end of the file is before the spool entry hash is built
      */
 
     {
-	uint32 nowGmt = time(NULL) / 60;
-	int fd = open(PatDbExpand(DHistoryPat), O_RDWR, 0644);
 	int n;
 	HistHead hh;
-	History hist[4096];
+	History hist[65536];
+
+	if ((n = read(HistoryFd, &hh, sizeof(hh))) != sizeof(hh)) {
+	    if (n == -1)
+		fprintf(stderr, "Unable to read history header from %s (%s)\n",
+						HistoryFile, strerror(errno));
+	    else 
+		fprintf(stderr, "Read %d bytes from history %s, expected %d\n",
+						n, HistoryFile, sizeof(hh));
 
-	if (fd < 0 || read(fd, &hh, sizeof(hh)) != sizeof(hh)) {
-	    fprintf(stderr, "Unable to open file %s\n", PatDbExpand(DHistoryPat));
 	    exit(1);
 	}
 	if (hh.hmagic != HMAGIC) {
-	    fprintf(stderr, "corrupted history file or old version of history file\n");
+	    fprintf(stderr, "corrupted history file or old version of history file: %x : %x\n", hh.hmagic, HMAGIC);
 	    exit(1);
 	}
-	if (hh.version != HVERSION) {
+	if (hh.version > HVERSION) {
 	    fprintf(stderr, "wrong dhistory file version (%d), expected %d\n",
 		hh.version,
 		HVERSION
@@ -962,40 +1020,76 @@ updateHistory(void)
 	    exit(1);
 	}
 
-	lseek(fd, hh.headSize + sizeof(int32) * hh.hashSize, 0);
+	lseek(HistoryFd, hh.headSize + sizeof(HistIndex) * hh.hashSize, 0);
 
-	{
-	    struct stat st;
-	    if (fstat(fd, &st) == 0)
-		numEnt = (st.st_size -
-			(hh.headSize + sizeof(int32) * hh.hashSize)) /
+	numEnt = (HistoryEnd -
+			(hh.headSize + sizeof(HistIndex) * hh.hashSize)) /
 					sizeof(History);
-	}
-	
 	if (VerboseOpt) {
 	    printf("History contains %d entries ....\n", numEnt);
 	    fflush(stdout);
 	}
 
-	while ((n = read(fd, hist, sizeof(hist))) > 0) {
+	npos = lseek(HistoryFd, 0L, 1);
+	while ((n = read(HistoryFd, hist, sizeof(hist))) > 0) {
 	    int i;
 	    int changed = 0;
-	    int lastreadcount = n;
-	    int32 npos = -1, bpos = -1; /* set only when changed is modified */
 
+	    bpos = npos;
+	    npos += n;
 	    n /= sizeof(History);
 
 	    for (i = 0; i < n; ++i) {
-		History *h = &hist[i];
-		char path[256];
+		/*
+		 * Don't scan beyond the stored history position
+		 */
+		if (bpos + i * sizeof(History) >= HistoryEnd)
+		    break;
 
+		countEnt++;
 		if (VerboseOpt && numEnt > 0 &&
-				++countEnt * 100 / numEnt >= lastPerc + 10) {
+				countEnt * 100 / numEnt >= lastPerc + 10) {
 		    lastPerc = countEnt * 100 / numEnt;
-		    printf("\t%-10d of %-10d (%d%%) complete\n",
-						countEnt, numEnt, lastPerc);
+		    printf("\t%-10ld of %-10d (%d%%) complete  %llu  %u   \n",
+					(long)countEnt, numEnt, lastPerc,
+					bpos, countExp);
 		    fflush(stdout);
 		}
+
+		h = &hist[i];
+		path[0] = 0;
+
+		if (TestOpt) {
+		    int res;
+
+		    ArticleFileName(path, sizeof(path), h, ARTFILE_DIR_REL);
+		    res = findNode(path, 0);
+
+		    if (res == 0) {
+			if (H_EXPIRED(h->exp)) {
+			    countTestSpool++;
+			    printf("%08x.%08x expired @%llu (index=%llu) but on spool (%s)\n",
+					h->hv.h1, h->hv.h2,
+					npos + i * sizeof(History),
+					countEnt, path);
+			    findNode(path, 0);
+			} else {
+			    countTestValid++;
+			}
+		    } else {
+			if (H_EXPIRED(h->exp)) {
+			    countTestExpired++;
+			} else {
+			    countTestHistory++;
+			    printf("%08x.%08x not expired @%llu (index=%llu) and not on spool (%s)\n",
+					h->hv.h1, h->hv.h2,
+					npos + i * sizeof(History),
+					countEnt, path);
+			}
+		    }
+		    continue;
+		}
+
 		/*
 		 * skip if the article has already expired or if it
 		 * is a new article that we may not have scanned, or
@@ -1006,12 +1100,13 @@ updateHistory(void)
 		    continue;
 		if (!UnexpireOpt && H_EXPIRED(h->exp))
 		    continue;
+		if (UnexpireOpt && !H_EXPIRED(h->exp))
+		    continue;
 		if (h->hv.h1 == 0 && h->hv.h2 == 0)
 		    continue;
-		if (!UnexpireOpt && h->gmt >= startGmt - 11 && h->gmt < nowGmt + 24 * 60)
-		    continue;
 
-		ArticleFileName(path, sizeof(path), h, ARTFILE_DIR_REL);
+		if (!UnexpireOpt)
+		    ArticleFileName(path, sizeof(path), h, ARTFILE_DIR_REL);
 
 		if (UnexpireOpt || findNode(path, 0) < 0) {
 		    if (!UnexpireOpt && VerboseOpt > 1) {
@@ -1022,40 +1117,40 @@ updateHistory(void)
 			);
 		    }
 		    if (UnexpireOpt || HistoryUpdateOpt != 2) {
-			if ( !changed ) {
-			    npos = lseek(fd, 0L, 1);
-			    bpos = npos - lastreadcount;
-			    changed = 1;
-			}
+			changed = 1;
 			if (UnexpireOpt)
 			    h->exp &= ~EXPF_EXPIRED;
 			else
 			    h->exp |= EXPF_EXPIRED;
 			lseek(
-			    fd,
+			    HistoryFd,
 			    bpos + sizeof(History) * i + offsetof(History, exp),
 			    0
 			);
 			if (!NotForReal)
-			    write(fd, &h->exp, sizeof(h->exp));
+			    write(HistoryFd, &h->exp, sizeof(h->exp));
 
 			if (WriteHashesToFileOpt == 1)
 			    fwrite(&h->hv, sizeof(hash_t), 1, DExpOverList);
-
 		    }
 		    ++countExp;
 		}
 	    }
-
 	    if (changed)
-		lseek(fd, npos, 0);
+		lseek(HistoryFd, npos, 0);
 	}
-	close(fd);
     }
 
     if (WriteHashesToFileOpt == 1)
 	fclose(DExpOverList);
 
+    if (TestOpt) {
+	printf("Total entries scanned      : %12llu\n", countEnt);
+	printf("Total !expired + on spool  : %12llu\n", countTestValid);
+	printf("Total expired + ! on spool : %12llu\n", countTestExpired);
+	printf("Total expired + on spool   : %12llu\n", countTestSpool);
+	printf("Total !expired + ! on spool: %12llu\n", countTestHistory);
+    }
     return(countExp);
 }
 
@@ -1069,6 +1164,12 @@ typedef struct ENode {
 
 ENode	*NodeAry[EHSIZE];
 
+/*
+ * Find a path in the node array
+ * returns:	0 for found
+ *		1 for created
+ *		-1 for not found
+ */
 int
 findNode(const char *path, int createMe)
 {
@@ -1076,11 +1177,13 @@ findNode(const char *path, int createMe)
     int i;
     ENode **pnode;
     ENode *node;
+    unsigned int index;
 
     for (i = 0; path[i]; ++i)
 	hv = (hv << 5) ^ path[i] ^ (hv >> 23);
 
-    for (pnode = &NodeAry[(hv ^ (hv >> 16)) & EHMASK]; 
+    index = (hv ^ (hv >> 16)) & EHMASK;
+    for (pnode = &NodeAry[index]; 
 	(node = *pnode) != NULL; 
 	pnode = &node->no_Next
     ) {
@@ -1117,6 +1220,22 @@ freeNodes(void)
     }
 }
 
+void
+dumpNodeTable(void)
+{
+    int i;
+
+    for (i = 0; i < EHSIZE; ++i) {
+	ENode *node;
+
+	node = NodeAry[i];
+	while (node != NULL) {
+	    printf("NODEent: %d : %s\n", i, node->no_Path);
+	    node = node->no_Next;
+	}
+    }
+}
+
 FileSystem *
 findFileSys(char *path)
 {
@@ -1156,7 +1275,7 @@ getAge(char *dirname)
 {
     time_t t = 0;
 
-    sscanf(dirname, "D.%lx", &t);
+    sscanf(dirname, "D.%x", &t);
     if (t)
 	t = t * 60;
     return(t);
@@ -1204,7 +1323,51 @@ freeSpaceOn(char *path, int syncit, long *freefiles)
     if (freefiles != NULL)
 	*freefiles = stmp.f_ffree;
     if (DebugOpt)
-	printf("Tested fs avail: %s (%ld inodes)\n", ftos(avail), stmp.f_ffree);
+	printf("Tested fs avail: %s (%ld inodes)\n", ftos(avail), (long)stmp.f_ffree);
     return(avail);
 }
 
+void
+DumpSpoolEntries(void)
+{
+    char path[PATH_MAX];
+    int i;
+    char spoolHome[PATH_MAX];
+    int spoolHomeLen;
+
+    snprintf(spoolHome, sizeof(spoolHome), "%s", PatExpand(SpoolHomePat));
+    spoolHomeLen = strlen(spoolHome);
+    printf("List of directories on spool:\n");
+    for (i = 0; i < entryIdx; ++i) {
+	int partnameLen;
+
+	if (SpoolEntries[i]->removed)
+	    continue;
+
+	partnameLen = strlen(SpoolEntries[i]->partn->partname);
+
+	/* If partname has a "/news/spool/news/" prefix, strip it off */
+	if (partnameLen > spoolHomeLen &&
+	    strncmp(SpoolEntries[i]->partn->partname, spoolHome,
+		    spoolHomeLen) == 0 &&
+	    SpoolEntries[i]->partn->partname[spoolHomeLen] == '/'
+	    ) {
+	    snprintf(path, sizeof(path), "%s%s",
+		     SpoolEntries[i]->partn->partname + spoolHomeLen + 1,
+		     SpoolEntries[i]->dirname);
+	} else {
+	    /* No "/news/spool/news/" prefix; go for absolute path */
+	    snprintf(path, sizeof(path), "%s%s",
+					SpoolEntries[i]->partn->partname,
+					SpoolEntries[i]->dirname);
+	}
+
+	printf("path= %s  dirname=%s  partition=%s  removed=%d  found=%d\n",
+		path,
+                SpoolEntries[i]->dirname,
+                SpoolEntries[i]->partn->partname,
+                SpoolEntries[i]->removed,
+		findNode(path, 0));
+    }
+}
+
