@@ -750,6 +750,13 @@ ProcessOverviewFile(const char *dirPath, const char *name, int type)
 		KPDBUnlock(KDBActive, rec);
 
 		if (group->gr_EndNo - group->gr_StartNo >= oh.oh_MaxArts) {
+		    if (VerboseOpt)
+			printf("adjust %s startno %d to %d (endno=%d  maxarts=%d)\n",
+					group->gr_GroupName,
+					group->gr_StartNo,
+					group->gr_EndNo - oh.oh_MaxArts + 1,
+					group->gr_EndNo,
+					oh.oh_MaxArts);
 		    group->gr_StartNo = group->gr_EndNo - oh.oh_MaxArts + 1;
 		    group->gr_State |= GRF_EDITEDBEG | GRF_MODIFIED;
 		}
@@ -883,12 +890,11 @@ ProcessOverviewFile(const char *dirPath, const char *name, int type)
 	if (artBase + OD_HARTS <= group->gr_StartNo ||
 	    artBase >= group->gr_EndNo + OD_HARTS
 	) {
-	    printf("Deleting stale overview data %s: %s\n",
-		group->gr_GroupName,
-		path
-	    );
-	    printf("  artBase=%ld  StartNo=%d  EndNo=%d\n",
-			artBase, group->gr_StartNo, group->gr_EndNo);
+	    if (VerboseOpt)
+		printf("Deleting stale overview data for %s: %s (artBase=%ld  StartNo=%d  EndNo=%d\n",
+				group->gr_GroupName, path,
+				artBase, group->gr_StartNo, group->gr_EndNo
+		);
 	    if (ForReal)
 		remove(path);
 	} 
@@ -985,7 +991,11 @@ EnterGroup(const char *groupName, int begNo, int endNo, int lmts, int cts, int i
     }
     if (endNo >= 0) {
 	group->gr_State |= GRF_ENDNO;
-	if (group->gr_EndNo != endNo) {
+	if (endNo < group->gr_EndNo) {
+	    printf("*** Would be adjusting NE down - not doing it ***\n");
+	    printf("adjust %s NE from %d to %d\n", group->gr_GroupName,
+				endNo, group->gr_EndNo);
+	} else if (group->gr_EndNo != endNo) {
 	    group->gr_EndNo = endNo;
 	    group->gr_State |= GRF_MODIFIED;
 	}
@@ -1527,7 +1537,9 @@ RewriteData(Group *group, int fd, OverHead *oh, const char *dirPath)
 						oh->oh_Version, OH_VERSION,
 						group->gr_GroupName);
 	    oh->oh_Version = OH_VERSION;
-	    strncpy(oh->oh_Gname, group->gr_GroupName, sizeof(oh->oh_Gname));
+	    strncpy(oh->oh_Gname, group->gr_GroupName,
+						sizeof(oh->oh_Gname) - 1);
+	    oh->oh_Gname[sizeof(oh->oh_Gname) - 1] = '\0';
 	    oh->oh_HeadSize = sizeof(OverHead);
 	}
 	ovFd = open(path1, O_RDWR|O_CREAT|O_TRUNC, 0644);
@@ -1752,6 +1764,13 @@ ResizeGroup(Group *group, int fd, OverHead *oh, int newSize)
 	KPDBUnlock(KDBActive, rec);
 
 	if (group->gr_EndNo - group->gr_StartNo >= newSize) {
+	    if (VerboseOpt)
+		printf("adjust %s startno %d to %d (endno=%d  newsize=%d)\n",
+				group->gr_GroupName,
+				group->gr_StartNo,
+				group->gr_EndNo - newSize + 1,
+				group->gr_EndNo,
+				newSize);
 	    group->gr_StartNo = group->gr_EndNo - newSize + 1;
 	    group->gr_State |= GRF_EDITEDBEG | GRF_MODIFIED;
 	}
@@ -1779,7 +1798,8 @@ ResizeGroup(Group *group, int fd, OverHead *oh, int newSize)
 						oh->oh_Version, OH_VERSION);
 		oh->oh_Version = OH_VERSION;
 		strncpy(oh->oh_Gname, group->gr_GroupName,
-							sizeof(oh->oh_Gname));
+						sizeof(oh->oh_Gname) - 1);
+		oh->oh_Gname[sizeof(oh->oh_Gname) - 1] = '\0';
 		oh->oh_HeadSize = sizeof(OverHead);
 	    }
 	    lseek(fd, 0L, 0);
