@@ -455,7 +455,7 @@ GrpCtlNewGroup(int ac, char **av, int pos)
 	KPDBWriteEncode(KDBActive, group, "M", moderator, KP_LOCK_CONTINUE);
     }
     if (description) {
-	SanitizeString(description);
+	SanitizeDescString(description);
 	KPDBWriteEncode(KDBActive, group, "GD", description, KP_LOCK_CONTINUE);
     }
 
@@ -536,6 +536,10 @@ GrpCtlAdjust(int ac, char **av, int pos)
 	printf(">> adjust: missing field to adjust\n");
 	return(NULL);
     }
+    if ((av[pos]) == NULL) {
+	printf(">> adjust: missing adjust value\n");
+	return(NULL);
+    }
     adjust = strtol(av[pos++], NULL, 0);
 
     if (ValidGroupName(group) < 0) {
