@@ -37,10 +37,12 @@ main(int ac, char **av)
 	}
 	soun.sun_family = AF_UNIX;
 	sprintf(soun.sun_path, "%s", PatRunExpand(DiabloSocketPat));
+	stprintf("connecting to %s", soun.sun_path);
 	if (connect(ufd, (struct sockaddr *)&soun, offsetof(struct sockaddr_un, sun_path[strlen(soun.sun_path)+1])) < 0) {
 	    perror("udom-connect");
 	    exit(1);
 	}
+	stprintf("connected to %s", soun.sun_path);
 	fo = fdopen(dup(ufd), "w");
 	fi = fdopen(ufd, "r");
     }
@@ -48,9 +50,11 @@ main(int ac, char **av)
     if (ac == 1) {
 	while(fgets(buf, sizeof(buf), stdin) != NULL) {
 	    r = 1;
+	    stprintf("sending %s", buf);
 	    fprintf(fo, "%s", buf);
 	    fflush(fo);
 	    while (fgets(buf, sizeof(buf), fi) != NULL) {
+	        stprintf("received %s", buf);
 		printf("%s", buf);
 		if (strcmp(buf, ".\n") == 0) {
 		    r = 0;
@@ -63,14 +67,17 @@ main(int ac, char **av)
 
 	for (i = 1; i < ac; ++i) {
 	    char *ptr = av[i];
-	    if (*ptr != '-' || i > 1)
+	    if (*ptr != '-' || i > 1) {
+	        stprintf("sending %s%s", ptr, (i + 1 < ac) ? " " : "");
 		fprintf(fo, "%s%s", ptr, (i + 1 < ac) ? " " : "");
+	    }
 	}
 	fprintf(fo, "\n");
 	fprintf(fo, "quit\n");
 	fflush(fo);
 	r = 1;
 	while (fgets(buf, sizeof(buf), fi) != NULL) {
+	    stprintf("received %s", buf);
 	    if (strcmp(buf, ".\n") == 0) {
 		r = 0;
 		break;
