@@ -30,12 +30,24 @@ int OldOpt = 0;
 int StartOff = 0;
 int LineModeOpt = 0;
 int VerboseOpt = 0;
+int QuietOpt = 0;
 int VerifyHistory = 0;
 int NextOpt = 0;
 int EntriesOpt = 0;
+int EmptyOpt = 0;
 int ShowProgress = 0;
+int DumpHashOnly = 0;
 char *FindHash = NULL;
 time_t MaxAge = -1;
+int HistoryVersion = 0;
+uint32 HOffset = 0;
+
+uint32 ExpireDropCount = 0;
+uint32 ExpireKeepCount = 0;
+uint32 MaxAgeCount = 0;
+uint32 ZeroGmtCount = 0;
+uint32 LookupCount = 0;
+uint32 DumpedCount = 0;
 
 void DumpTrace(int fd, int hsize, int rsize);
 void DumpQuick(int fd, int hsize, int rsize);
@@ -54,14 +66,17 @@ Usage(void)
     printf("\t-H\t- dump hash trace for specified Message-ID or hash\n");
     printf("\t-h\t- display entries not found with a HistoryLookup\n");
     printf("\t-l\t- line mode - flush output after every line\n");
+    printf("\t-m\t- dump msgid hashes only\n");
     printf("\t-n\t- also dump the value of the 'next' pointer\n");
     printf("\t-o\t- dump old-style (diablo V <= 1.07) history file\n");
-    printf("\t-p\t- show progress on stdout\n");
+    printf("\t-p\t- show progress on stderr\n");
+    printf("\t-q\t- quiet - don't show stats\n");
     printf("\t-rN\t- set rememberdays\n");
     printf("\t-t\t- hash table trace (slow)\n");
     printf("\t-TN\t- don't dump articles older than N seconds\n");
     printf("\t-v\t- Include additional (synthesized) info\n");
     printf("\t-x\t- do not dump records older than rememberdays old\n");
+    printf("\t-z\t- include entries with a gmt of zero in the dump\n");
     printf("\t-Cfile\t- specify diablo.config to use\n");
     printf("\t-d[n]\t- set debug [with optional level]\n");
     printf("\t-V\t- print version and exit\n");
@@ -115,6 +130,9 @@ main(int ac, char **av)
 	case 'l':
 	    LineModeOpt = 1;
 	    break;
+	case 'm':
+	    DumpHashOnly = 1;
+	    break;
 	case 'n':
 	    NextOpt = 1;
 	    break;
@@ -124,8 +142,15 @@ main(int ac, char **av)
 	case 'p':
 	    ShowProgress = 1;
 	    break;
+	case 'q':
+	    QuietOpt = 1;
+	    break;
 	case 'r':
-	    DOpts.RememberDays = strtol(*ptr ? ptr : av[++i], NULL, 0);
+	    if (!*ptr)
+		ptr = av[++i];
+	    DOpts.RememberSecs = TimeSpec(ptr, "d");
+	    if (DOpts.RememberSecs == -1)
+		Usage();
 	    break;
 	case 'T':
 	    MaxAge = btimetol(*ptr ? ptr : av[++i]);
@@ -142,6 +167,9 @@ main(int ac, char **av)
 	case 'x':
 	    DNPOpt = 1;
 	    break;
+	case 'z':
+	    EmptyOpt = 1;
+	    break;
 	/* Common options */
 	case 'C':		/* parsed by LoadDiabloConfig */
 	    if (*ptr == 0)
@@ -223,20 +251,21 @@ main(int ac, char **av)
 		fprintf(stderr, "corrupted history file\n");
 		exit(1);
 	    }
-	    if (hh.version != HVERSION) {
+	    if (hh.version > HVERSION) {
 		fprintf(stderr, "WARNING! Version mismatch file V%d, expecting V%d\n", hh.version, HVERSION);
 		fprintf(stderr, "dump may be invalid\n");
 	    }
+	    HistoryVersion = hh.version;
 	    rsize = hh.henSize;
 	    hsize = hh.hashSize;
+	    HOffset = hh.headSize + hsize * sizeof(HistIndex);
 
 	    lseek(fd, hh.headSize, 0);
 	}
 
-	fprintf(stderr, "Dumping, hash table %d entries, record size %d\n",
-	    hsize,
-	    rsize
-	);
+	if (!QuietOpt)
+	    fprintf(stderr, "Dumping, hash table %d entries, record size %d\n",
+							hsize, rsize);
 
 	if (TraceMode) {
 	    DumpTrace(fd, hsize, rsize);
@@ -259,20 +288,25 @@ main(int ac, char **av)
 }
 
 void
-PrintTrace(int fd, int32 off, int rsize)
+PrintTrace(int fd, HistIndex index, int rsize)
 {
+    off_t off;
     int maxChainLen = 1000;
 
-    while (off) {
+    while (index) {
 	History h = { 0 };
 
-	lseek(fd, (off_t)(uint32)off, 0);
+	if (HistoryVersion > 1)
+	    off = (off_t)HOffset + (off_t)index * sizeof(History);
+	else
+	    off = index;
+	lseek(fd, off, 0);
 	if (read(fd, &h, rsize) != rsize) {
-	    printf("read error @ %d", off);
+	    fprintf(stderr, "read error @ %d (%llu)", index, off);
 	    break;
 	} else {
 	    printf(" [%u,%u %08x.%08x.%04x gm=%d ex=%d boff=%d bsize=%d F=%s part=%d]",
-		(unsigned int)off, (unsigned int)h.next,
+		index, (unsigned int)h.next,
 		h.hv.h1,
 		h.hv.h2,
 		(int)h.iter,
@@ -300,14 +334,14 @@ PrintTrace(int fd, int32 off, int rsize)
 		printf("\ttime=%s\n", tbuf);
 	    }
 	}
-	off = h.next;
+	index = h.next;
 	if (--maxChainLen == 0) {
 	    printf(" MAXIMUM CHAIN LENGTH EXCEEDED!");
 	    break;
 	}
     }
-    if (off)
-	printf(" offset error: %d", off);
+    if (index)
+	printf(" offset error: %d", index);
     printf("\n");
 }
 
@@ -315,19 +349,17 @@ void
 DumpTrace(int fd, int hsize, int rsize)
 {
     int i;
-    int32 *Ary = calloc(hsize, sizeof(int32));
+    HistIndex *Ary = calloc(hsize, sizeof(HistIndex));
 
-    if (read(fd, Ary, hsize * sizeof(int32)) != hsize * sizeof(int32)) {
-	printf("Unable to read hash table array\n");
+    if (read(fd, Ary, hsize * sizeof(HistIndex)) != hsize * sizeof(HistIndex)) {
+	fprintf(stderr, "Unable to read hash table array\n");
 	exit(1);
     }
 
     for (i = 0; i < hsize; ++i) {
-	int32 off;
-
-	if ((off = Ary[i]) != 0) {
+	if (Ary[i] != 0) {
 	    printf("Index %d: ", i);
-	    PrintTrace(fd, off, rsize);
+	    PrintTrace(fd, Ary[i], rsize);
 	}
     }
 }
@@ -335,11 +367,11 @@ DumpTrace(int fd, int hsize, int rsize)
 void
 DumpChain(int fd, int hsize, int rsize, hash_t *hv)
 {
-    int32 *Ary = calloc(hsize, sizeof(int32));
-    int32 off;
+    uint32 *Ary = calloc(hsize, sizeof(uint32));
+    uint32 off;
 
-    if (read(fd, Ary, hsize * sizeof(int32)) != hsize * sizeof(int32)) {
-	printf("Unable to read hash table array\n");
+    if (read(fd, Ary, hsize * sizeof(uint32)) != hsize * sizeof(uint32)) {
+	fprintf(stderr, "Unable to read hash table array\n");
 	exit(1);
     }
 
@@ -350,74 +382,104 @@ DumpChain(int fd, int hsize, int rsize, hash_t *hv)
 void
 DumpQuick(int fd, int hsize, int rsize)
 {
-    char hbuf[32768];
+    char *hbuf;
     int n;
-    int hlen = (sizeof(hbuf) / rsize) * rsize;
-    int rememberMins = (DOpts.RememberDays + 1) * 24 * 60;
+    int hlen;
+    int rememberMins = DOpts.RememberSecs / 60;
     uint32 gmt = time(NULL) / 60;
     History th;
     off_t seekpos = 0;
-    int totalentries = 0;
-    int count = 0;
+    uint32 totalentries = 0;
+    uint32 count = 0;
+    int i;
+    History *h;
 
+
+    hlen = rsize * 4096;;
+    if ((hbuf = (char *)malloc(hlen)) == NULL) {
+	fprintf(stderr, "ERROR: Unable to malloc %dB for history cache (%s)\n",
+				hlen, strerror(errno));
+	return;
+    }
     if (StartOff)
 	seekpos = lseek(fd, StartOff, 0);
     else if (FOpt || FCount)
 	seekpos = lseek(fd, -FCount * rsize, 2);
     else
-	seekpos = lseek(fd, hsize * sizeof(int32), 1);
+	if (HistoryVersion > 1)
+	    seekpos = lseek(fd, (off_t)hsize * sizeof(HistIndex) + (off_t)rsize, 1);
+	else
+	    seekpos = lseek(fd, (off_t)hsize * sizeof(HistIndex), 1);
 
-    if (EntriesOpt || ShowProgress) {
+    {
 	struct stat st;
 
 	if (fstat(fd, &st) == -1) {
 	    fprintf(stderr, "Unable to fstat history: %s\n", strerror(errno));
 	    exit(1);
 	}
-	totalentries = (int)(st.st_size - seekpos) / rsize;
+	totalentries = (int)(((double)st.st_size - (double)seekpos) / (double)rsize);
 	if (EntriesOpt)
-	    printf("ENTRIES %d\n", totalentries);
+	    printf("ENTRIES %u\n", totalentries);
     }
 
-    fprintf(stderr, "@%d  %d records\n", (int)lseek(fd, 0L, 1), totalentries);
+    if (!QuietOpt)
+	fprintf(stderr, "@%llu  %u records (%d bytes per record)\n",
+				lseek(fd, 0L, 1), totalentries, rsize);
 
 top:
-    while ((n = read(fd, &hbuf, hlen)) > 0) {
-	int i;
+    while ((n = read(fd, hbuf, hlen)) > 0) {
 
 	n /= rsize;
+	gmt = time(NULL) / 60;
 
 	for (i = 0; i < n; ++i) {
-	    History *h = (History *)(hbuf + i * rsize);
+	    h = (History *)(hbuf + i * rsize);
 
-	    if ((i & 255) == 0)
-		gmt = time(NULL) / 60;
+
+	    if (ShowProgress && (count++ % 32768) == 0)
+		fprintf(stderr, "%u/%u  (%d%%)    \r", count, totalentries,
+			(int)((double)count * 100.0 / (double)totalentries));
 
 	    /*
 	     * A gmt of zero is an empty entry - don't print it out unless
 	     * we are verifying history
 	     */
-	    if (h->gmt == 0 && !VerifyHistory)
+	    if (h->gmt == 0 && !VerifyHistory && !EmptyOpt) {
+		ZeroGmtCount++;
 		continue;
+	    }
 
 	    /*
 	     * If we specified a maximum age, then deal with it
 	     */
 	    if (MaxAge != -1) {
 		int32 dgmt = (gmt - h->gmt) * 60;	/* Delta seconds */
-		if (dgmt > MaxAge)
+		if (dgmt > MaxAge) {
+		    MaxAgeCount++;
 		    continue;
+		}
 	    }
 
 	    if (DNPOpt && H_EXPIRED(h->exp)) {
 		int32 dgmt = gmt - h->gmt;	/* DELTA MINUTES */
-		if (dgmt < -rememberMins || dgmt > rememberMins)
+		if (dgmt < -rememberMins || dgmt > rememberMins) {
+		    ExpireDropCount++;
 		    continue;
+		} else {
+		    ExpireKeepCount++;
+		}
 	    }
 
-	    if (VerifyHistory && HistoryLookupByHash(h->hv, &th) == 0)
+	    if (VerifyHistory && HistoryLookupByHash(h->hv, &th) == 0) {
+		LookupCount++;
 		continue;
+	    }
 
+	    if (DumpHashOnly) {
+		printf("%08x.%08x\n", h->hv.h1, h->hv.h2);
+		continue;
+	    }
 	    printf("DUMP %08x.%08x.%04x gm=%d ex=%-2d",
 		h->hv.h1,
 		h->hv.h2,
@@ -439,19 +501,33 @@ top:
 		    printf(" next=%d", h->next);
 	    }
 	    printf("\n");
+	    DumpedCount++;
 	    if (LineModeOpt)
 		fflush(stdout);
-	    if (totalentries > 0 && (count++ & 8192) == 0) {
-		fprintf(stderr, "%d/%d\r", count, totalentries);
-	    }
 	}
     }
+    if (n < 0)
+	fprintf(stderr, "Error reading file: %s\n", strerror(errno));
+
     if (FOpt) {
-	sleep(1);
+	usleep(100000);
 	goto top;
     }
-    if (totalentries > 0)
-	fprintf(stderr, "%d/%d\n", count, totalentries);
-    printf(".\n");
+    if (!QuietOpt && count > 0)
+	fprintf(stderr, "%u/%u  (%d%%)    \n", count, totalentries,
+			(int)((double)count * 100.0 / (double)totalentries));
+    if (!QuietOpt)
+	fprintf(stderr, "%u entries dumped\n", DumpedCount);
+    if (VerifyHistory) {
+	fprintf(stderr, "%u entries found\n", LookupCount);
+    } else if (!QuietOpt) {
+	fprintf(stderr, "%u expired entries kept\n", ExpireKeepCount);
+	fprintf(stderr, "%u expired entries dropped\n", ExpireDropCount);
+	fprintf(stderr, "%u dropped as beyond max age\n", MaxAgeCount);
+	fprintf(stderr, "%u dropped with gmt=0\n", ZeroGmtCount);
+    }
+    if (!QuietOpt)
+	printf(".\n");
+    free(hbuf);
 }
 
