@@ -35,6 +35,8 @@ char **FileAry;
 int hisfd = -1;
 char msgId[MAXMSGIDLEN];
 char newsgroups[16384];
+uint32 GmtStart = 0;
+uint32 GmtEnd = 0;
 
 void ScanSpoolObject(uint16 spoolobj);
 void ScanSpool(uint16 spoolobj);
@@ -42,7 +44,8 @@ void ScanSpoolDirectory(char *dpath, int gmt, uint16 spoolobj);
 void ScanSpoolFile(char *fpath, int gmt, int iter, uint16 spoolobj);
 void ScanSpoolFileMap(const char *base, int bytes, int gmt, int iter, char *dpath, uint16 spoolobj, int fd);
 void ScanSpoolFileMapOld(const char *base, int bytes, int gmt, int iter, char *dpath, uint16 spoolobj);
-void DoArticle(History *h, const char *id, char *newsgroups);
+void DoArticle(History *h, const char *id, char *nglist, char *dist,
+		char *npath, int headOnly, char *artType, char *cSize);
 int strSort(const void *s1, const void *s2);
 
 void
@@ -51,7 +54,8 @@ Usage(void)
     printf("This program scans the diablo spool and performs various tasks\n");
     printf("based on the articles found.\n\n");
     printf("diloadfromspool [-a] [-F dhistory-file] [-f] [-h hashtablesize]\n");
-    printf("                [-n] [-Q] [-q] [-S nn] [-u] [-v] [spooldir/spoolfile]\n");
+    printf("		    [-n] [-Q] [-q] [-S nn] [-tb TT] [-te TT]\n");
+    printf("		    [-u] [-v] [spooldir/spoolfile]\n");
     printf("\t-a scan all the spool objects found in dspool.ctl\n");
     printf("\t-e unexpire all entries marked expired in dhistory\n");
     printf("\t-F specify the history file to update\n");
@@ -61,11 +65,46 @@ Usage(void)
     printf("\t-Q print articles in format suitable for drequeue\n");
     printf("\t-q quiet mode\n");
     printf("\t-S specify the spool object to scan\n");
+    printf("\t-tb only scan spool directories since this time\n");
+    printf("\t-te only scan spool directories until this time\n");
     printf("\t-u check for duplicates in history before adding\n");
     printf("\t-v verbose mode\n");
+    printf("\n");
+    printf("\t the TT value is specified as YYYYMMDDHHMM or D.NNNNNNNN\n");
     exit(1);
 }
 
+uint32
+timeConv(char *tstr)
+{
+    if (tstr == NULL || (strlen(tstr) != 12 && strlen(tstr) != 10)) {
+	fprintf(stderr, "Invalid time specification for -t option\n");
+	fprintf(stderr, "Must be yyyymmddhhmm or D.nnnnnnnn\n");
+	Usage();
+    }
+    if (strlen(tstr) == 10) {
+	uint32 t;
+	if (sscanf(tstr, "D.%08x", &t) != 1) {
+	    fprintf(stderr, "Invalid time specification for -t option\n");
+	    fprintf(stderr, "Must be yyyymmddhhmm or D.nnnnnnnn\n");
+	    Usage();
+	}
+	return(t);
+    } else {
+	struct tm tm;
+	time_t t;
+
+	bzero(&tm, sizeof(tm));
+	if (strptime(tstr, "%Y%m%d%H%M", &tm) == NULL) {
+	    fprintf(stderr, "Invalid time specification for -t option\n");
+	    fprintf(stderr, "Must be yyyymmddhhmm or D.nnnnnnnn\n");
+	    Usage();
+	}
+	t = mktime(&tm);
+	return((uint32)(t / 60));
+    }
+}
+
 int
 main(int ac, char **av)
 {
@@ -133,6 +172,16 @@ main(int ac, char **av)
 	    case 'S':
 		spoolObj = (*ptr) ? strtol(ptr, NULL, 10) : strtol(av[++i], NULL, 10);
 		break;
+	    case 't':
+		    if (*ptr++ == 'b')
+			GmtStart = timeConv(*ptr ? ptr : av[++i]);
+		    else if (*ptr == 'e')
+			GmtEnd = timeConv(*ptr ? ptr : av[++i]);
+		    else {
+			fprintf(stderr, "Invalid option: %s\n", &ptr[-2]);
+			Usage();
+		    }
+		break;
 	    case 'u':
 		uflag = 1;
 		break;
@@ -166,6 +215,14 @@ main(int ac, char **av)
 	    flags &= ~HGF_NOSEARCH;
     }
 
+    if (VerboseOpt) {
+	if (GmtStart != 0 && GmtEnd != 0)
+	    printf("Scanning directories from D.%08x to D.%08x\n", GmtStart, GmtEnd);
+	else if (GmtStart == 0 && GmtEnd != 0)
+	    printf("Scanning directories from earliest to D.%08x\n", GmtEnd);
+	else if (GmtStart != 0 && GmtEnd == 0)
+	    printf("Scanning directories from D.%08x to latest\n", GmtStart);
+    }
     if (UnExpireOpt) {
 	hisfd = open(historyFileName, O_RDWR);
 	if (hisfd == -1) {
@@ -308,6 +365,9 @@ ScanSpool(uint16 spoolobj)
 	    int sd;
 
 	    if (sscanf(den->d_name, "D.%x", &gmt) == 1) {
+		if ((GmtStart != 0 && gmt < GmtStart) ||
+					(GmtEnd != 0 && gmt > GmtEnd))
+		    continue;
 		if (FileIdx == FileMax) {
 		    FileMax = FileMax * 2;
 		    FileAry = realloc(FileAry, FileMax * sizeof(char *));
@@ -328,6 +388,9 @@ ScanSpoolDirectory(char *dpath, int gmt, uint16 spoolobj)
 {
     DIR *dir;
 
+    if ((GmtStart != 0 && gmt < GmtStart) || (GmtEnd != 0 && gmt > GmtEnd))
+	return;
+
     if (VerboseOpt)
 	printf(" Scanning directory: %s\n", dpath);
 
@@ -400,34 +463,28 @@ ScanSpoolFileMap(const char *base, int bytes, int gmt, int iter, char *dpath, ui
     int arthdrlen;
     char *artbase = NULL;
     char *artpos;
-    SpoolArtHdr *ah;
+    SpoolArtHdr ah;
+    char cSize[64];
+    int headOnly;
 
     while (b < bytes) {
-#if	USE_MMAP_BCOPY
-	{
-	    static SpoolArtHdr tah;
-	    bcopy(base + b, &tah, sizeof(tah));
-	    ah = &tah;
-	}
-#else
-	ah = (SpoolArtHdr *)(base + b);
-#endif
-	if ((uint8)ah->Magic1 != STORE_MAGIC1 ||
-					(uint8)ah->Magic2 != STORE_MAGIC2) {
+	bcopy(base + b, &ah, sizeof(ah));
+	if ((uint8)ah.Magic1 != STORE_MAGIC1 ||
+					(uint8)ah.Magic2 != STORE_MAGIC2) {
 	    printf("\tFailed at offset %d: invalid header magic (%d:%d)\n", b,
-						ah->Magic1, ah->Magic2);
+						ah.Magic1, ah.Magic2);
 	    ScanSpoolFileMapOld(base + b, bytes - b, gmt, iter, dpath, spoolobj);
 	    return;
 	}
-	arthdrlen = ah->ArtHdrLen;
-	if (ah->StoreType & STORETYPE_GZIP) {
+	arthdrlen = ah.ArtHdrLen;
+	if (ah.StoreType & STORETYPE_GZIP) {
 #ifdef USE_ZLIB
 	    gzFile *gzf;
-	    long len = ah->ArtLen;
+	    long len = ah.ArtLen;
 
-	    artbase = (char *)malloc(ah->ArtLen + 2);
-	    bzero(artbase, ah->ArtLen + 2);
-	    lseek(fd, b + ah->HeadLen, 0);
+	    artbase = (char *)malloc(ah.ArtLen + 2);
+	    bzero(artbase, ah.ArtLen + 2);
+	    lseek(fd, b + ah.HeadLen, 0);
 	    if ((gzf = gzdopen(dup(fd), "r")) != NULL) {
 		if (gzread(gzf, artbase, len) != len)
 		    arthdrlen = 0;
@@ -440,7 +497,7 @@ ScanSpoolFileMap(const char *base, int bytes, int gmt, int iter, char *dpath, ui
 	    arthdrlen = 0;
 #endif
 	} else {
-	    artbase = (char *)base + b + ah->HeadLen;
+	    artbase = (char *)base + b + ah.HeadLen;
 	}
 	artpos = artbase;
 	msgId[0] = 0;
@@ -452,7 +509,7 @@ ScanSpoolFileMap(const char *base, int bytes, int gmt, int iter, char *dpath, ui
 	     * Scan line
 	     */
 
-	    for (l = 0; l < arthdrlen && artpos[l] != '\n'; ++l)
+	    for (l = 0; l < arthdrlen && artpos[l] != '\n' && artpos[l] != '\r'; ++l)
 		;
 	    if (l < arthdrlen)
 		++l;
@@ -473,17 +530,25 @@ ScanSpoolFileMap(const char *base, int bytes, int gmt, int iter, char *dpath, ui
 	    h.gmt = gmt;
 	    h.exp = 100 + spoolobj;
 	    h.boffset = b;
-	    h.bsize = ah->StoreLen - 1;
-	    if (ah->ArtHdrLen == ah->ArtLen)
+	    h.bsize = ah.StoreLen - 1;
+	    headOnly = 0;
+	    if (ah.ArtHdrLen == ah.ArtLen) {
 		h.exp |= EXPF_HEADONLY;
-	    DoArticle(&h, id, newsgroups);
+		headOnly = 1;
+	    }
+	    cSize[0] = 0;
+	    if (ah.StoreType & STORETYPE_GZIP) {
+		h.bsize = ah.ArtLen + ah.HeadLen;
+		sprintf(cSize, "%d", ah.StoreLen);
+	    }
+	    DoArticle(&h, id, newsgroups, " ", " ", headOnly, "0", cSize);
 	    count++;
 	} else {
 	    if (VerboseOpt)
-		printf("No Message-ID for %d,%d\n", b, ah->StoreLen - 1);
+		printf("No Message-ID for %d,%d\n", b, ah.StoreLen - 1);
 	}
-	b += ah->StoreLen;
-	if (ah->StoreType & STORETYPE_GZIP) {
+	b += ah.StoreLen;
+	if (ah.StoreType & STORETYPE_GZIP) {
 	    free(artbase);
 	    b++;
 	}
@@ -555,7 +620,7 @@ ScanSpoolFileMapOld(const char *base, int bytes, int gmt, int iter, char *dpath,
 	    h.bsize = i - b;
 	    if (numLines == 0)
 		h.exp |= EXPF_HEADONLY;
-	    DoArticle(&h, id, newsgroups);
+	    DoArticle(&h, id, newsgroups, " ", " ", 0, " ", " ");
 	    count++;
 	    ++i;
 	} else {
@@ -581,17 +646,16 @@ ScanSpoolFileMapOld(const char *base, int bytes, int gmt, int iter, char *dpath,
 }
 
 void
-DoArticle(History *h, const char *id, char *newsgroups)
+DoArticle(History *h, const char *id, char *nglist, char *dist,
+		char *npath, int headOnly, char *artType, char *cSize)
 {
     int r = 0;
     if (RequeueOpt) {
 	char path[PATH_MAX];
 	ArticleFileName(path, sizeof(path), h, ARTFILE_FILE_REL);
-	printf("SOUT %s %s %d,%d %s\n", 
-	    path, id,
-	    h->boffset, 
-	    h->bsize, 
-	    newsgroups
+	printf("SOUT\t%s\t%lld,%ld\t%s\t%s\t%s\t%s\t%d\t%s\t%s\n",
+	    path, (off_t)h->boffset, (long)h->bsize, id, nglist, dist, npath,
+	    headOnly, artType, cSize
 	);
 	++LoadCount;
     } else {
@@ -602,12 +666,8 @@ DoArticle(History *h, const char *id, char *newsgroups)
 		uint32 pos = HistoryPosLookupByHash(h->hv, &htmp);
 		htmp.exp &= ~EXPF_EXPIRED;
 		if (pos != -1 && ForReal && htmp.iter == h->iter &&
-						htmp.boffset == h->boffset) {
-		    lseek(hisfd, (off_t)(uint32)pos +
-					offsetof(History, exp), 0);
-		    write(hisfd, &htmp.exp, sizeof(htmp.exp));
-		    
-		}
+						htmp.boffset == h->boffset)
+		    HistoryStoreExp(&htmp, (HistIndex)pos);
 	    }
 	    ++LoadDupCount;
 	} else {
