@@ -44,6 +44,10 @@
  */
 
 #include "defs.h"
+#include <sys/uio.h>
+
+#define NDEBUG
+#include <assert.h>
 
 #define T_ACCEPTED	1
 #define T_REFUSED	2
@@ -144,20 +148,23 @@
 
 #define MAXREASON       100     /* maximal length of a peer response logged */
 
+#define CMDBUFFSIZE	32768	/* Output buffer for NNTP traffic */
+
 typedef struct Stream {
     int	st_State;		/* state	*/
     int st_DumpRCode;
     char *st_RelPath;		/* path		*/
     char *st_MsgId;		/* message id	*/
-    int32 st_Off;		/* file offset	*/
+    off_t st_Off;		/* file offset	*/
     int32 st_Size;		/* file size	*/
     int32 st_CompSize;		/* compressed file size	*/
 } Stream;
 
 int connectTo(const char *hostName, const char *serviceName, int defPort);
-int Transact(int cfd, const char *relPath, char *msgId, int o, int s, int cSize, int defers, char *stage, char *reason, char *buf, int *sentSize);
-int DumpArticle(int cfd, const char *relPath, int o, int s, int cSize);
-int StreamTransact(int cfd, const char *relPath, char *msgId, int o, int s, int cSize, int defers, char *stage, char *reason, char *buf, int *sentSize);
+int Transact(int cfd, const char *relPath, char *msgId, off_t off, int size, int cSize, int defers, char *stage, char *reason, char *buf, int *sentSize);
+int DumpArticle(int cfd, const char *relPath, off_t off, int size, int cSize);
+int writeLarge(int cfd, char *buffer, size_t size);
+int StreamTransact(int cfd, const char *relPath, char *msgId, off_t off, int size, int cSize, int defers, char *stage, char *reason, char *buf, int *sentSize);
 void StreamReload(int cfd);
 Stream *LocateStream(const char *msgId, int state);
 int RefilePendingStreams(FILE *fo);
@@ -172,9 +179,9 @@ void logStats(const char *description);
 int ValidMsgId(char *msgid);
 void AttemptRemoveRenamedFile(struct stat *st, const char *spoolFile);
 void cdinit(void);
-char *cdmap(const char *path, int off, int *psize, int cSize, int *multiArtFile);
+char *cdmap(const char *path, off_t off, int *psize, int cSize, int *multiArtFile);
 void cdunmap(char *ptr, int bytes, int multiArtFile, int compressed);
-int extractFeedLine(const char *buf, char *relPath, char *msgId, int *poff, int *psize, int *pheadOnly, time_t *queuedTime, int *cSize);
+int extractFeedLine(const char *buf, char *relPath, char *msgId, off_t *poff, int *psize, int *pheadOnly, time_t *queuedTime, int *cSize);
 void cdflush(void);
 const char *dtstamp(void);
 void clearCommandBuffer(void);
@@ -198,6 +205,7 @@ int CloseReopenAfter = 1000;
 int Timeout = 600;
 int DeleteDetectOpt = 0;
 int WaitTime = 10;
+int DelayTime = 0;
 int Port = 119;
 int TailOpt = 0;
 int TxBufSize = 0;
@@ -256,9 +264,9 @@ int CompressNextFlush = 0;
 
 #endif	/* USE_ZLIB */
 
-char *HostName;
-char *OutboundIpName;
-char *BatchFileCtl;
+char *HostName = NULL;
+char *OutboundIpName = NULL;
+char *BatchFileCtl = NULL;
 char CurrentBatchFile[1024];
 char LastErrBuf[256];
 int CBFIndex;
@@ -267,6 +275,7 @@ int NumBatches = 1;
 int HeaderSize = 0;
 int NotifyFd = -1;
 int NotifyLockFd = -1;
+int IPQoS=0;
 time_t LastNotify = 0;
 time_t LastFeedData = 0;
 
@@ -311,6 +320,9 @@ Usage(char *progname)
 	    "-o label        - enable feed notification from diablo\n"
 	    "-P #            - specify remote port (default: 119)\n"
 	    "-p              - input on stdin rather then file\n"
+#ifdef IP_TOS
+	    "-Q              - set QoS using IP_TOS\n"
+#endif
 	    "-n[op]          - NOP"
 	    "-P #            - specify destination tcp port\n"
 	    "-R #            - set receive buffer size\n"
@@ -319,6 +331,7 @@ Usage(char *progname)
 	    "-s \"<24chars>\"  - /bin/ps argv space for status\n"
 	    "-T #            - set transmit buffer size\n"
 	    "-t #            - specify timeout, default 600s\n"
+	    "-W #            - delay sending articles from the queue time\n"
 	    "-w #            - reconnect-after-failure delay\n"
     );
     exit(1);
@@ -330,7 +343,8 @@ main(int ac, char **av)
     int i;
     int eNoBat = 0;
 
-    openlog("newslink", (DebugOpt > 0? LOG_PERROR: 0) | LOG_NDELAY | LOG_PID, LOG_NEWS);
+    TimeNow = time(NULL);
+    OpenLog("newslink", (DebugOpt > 0? LOG_PERROR: 0) | LOG_NDELAY | LOG_PID);
     LoadDiabloConfig(ac, av);
 
     for (i = 1; i < ac; ++i) {
@@ -344,13 +358,7 @@ main(int ac, char **av)
 	case 'B':
 	    if (*ptr == 0)
 		ptr = av[++i];
-	    if (strrchr(ptr, ':')) {
-		Port = strtol(strrchr(ptr, ':') + 1, NULL, 0);
-		OutboundIpName = ptr;
-		*strrchr(OutboundIpName, ':') = 0;
-	    } else {
-		OutboundIpName = ptr;
-	    }
+	    OutboundIpName = strdup(SanitiseAddr(ptr));
 	    break;
 	case 'b':
 	    BatchFileCtl = (*ptr) ? ptr : av[++i];
@@ -412,6 +420,9 @@ main(int ac, char **av)
 	case 'p':
 	    PipeOpt = 1;	/* input from pipe 	*/
 	    break;
+	case 'Q':
+	    IPQoS = strtol(((*ptr) ? ptr : av[++i]), NULL, 0);
+	    break;
 	case 'R':
 	    /*
 	     * note: must be large enough to hold check responses that
@@ -443,6 +454,9 @@ main(int ac, char **av)
 	case 'V':
 	    PrintVersion();
 	    break;
+	case 'W':
+	    DelayTime = strtol(((*ptr) ? ptr : av[++i]), NULL, 0);
+	    break;
 	case 'w':
 	    WaitTime = strtol(((*ptr) ? ptr : av[++i]), NULL, 0);
 	    break;
@@ -487,7 +501,8 @@ main(int ac, char **av)
 	    {
 		char buf[255];
 		char *p;
-		strncpy(buf, BatchFileCtl, sizeof(buf));
+		strncpy(buf, BatchFileCtl, sizeof(buf) - 1);
+		buf[sizeof(buf) - 1] = '\0';
 		if ((p = strstr(buf, ".S%05d")) != NULL)
 		    *p = 0;
 		HostStats = FeedStatsFindSlot(buf);
@@ -505,6 +520,7 @@ main(int ac, char **av)
 
     while (--NumBatches >= 0) {
 	time_t ddTime;
+	time_t ddCount = 0;
 	time_t cdflushTime;
 	int fd;
 	int cfd = -1;
@@ -635,11 +651,11 @@ main(int ac, char **av)
 	    char relPath[1024];
 	    char msgId[1024];
 	    char path[1024];
-	    int fileOff;
-	    int fileSize;
-	    int sentFileSize;
-	    int headOnly;
-	    time_t queuedTime;
+	    off_t fileOff = 0;
+	    int fileSize = 0;
+	    int sentFileSize = 0;
+	    int headOnly = 0;
+	    time_t queuedTime = 0;
 	    FILE *fo = NULL;
 	    int connectCount = 0;
 	    int bufInval = 1;
@@ -695,7 +711,8 @@ main(int ac, char **av)
 			 * several 431 answers from the peer before reaching
 			 * this point again.
 			 */
-			strncpy(buf, deferbuf[deferbegin], sizeof(buf));
+			strncpy(buf, deferbuf[deferbegin], sizeof(buf) - 1);
+			buf[sizeof(buf) - 1] = '\0';
 			zfreeStr(&SysMemPool, &deferbuf[deferbegin++]);
 			bufInval = 0;
 		    } else {
@@ -713,6 +730,10 @@ main(int ac, char **av)
 		    }
 		}
 
+		if (TimeCounter++ >= 100)
+		    TimeNow = time(NULL);
+
+
 		if (bufInval && StreamPend == 0 && deferbegin == 0 && deferend > 0) {
 		    /* Start to begin spooling deferred articles at the end of 
 		     * input file. Subsequent articles are retrieved by the 
@@ -730,7 +751,8 @@ main(int ac, char **av)
 			 */
 			sleep(5);
 		    }
-		    strncpy(buf, deferbuf[deferbegin], sizeof(buf));
+		    strncpy(buf, deferbuf[deferbegin], sizeof(buf) - 1);
+		    buf[sizeof(buf) - 1] = '\0';
 		    zfreeStr(&SysMemPool, &deferbuf[deferbegin++]);
 		    bufInval = 0;
 		}
@@ -751,7 +773,7 @@ main(int ac, char **av)
 					bufInval, StreamPend, NumStream,
 					(bufInval) ? "?" : buf);
 
-		if (DeleteDetectOpt && !PipeOpt && (TimeNow - ddTime) > DDTIME) {
+		if (!PipeOpt && (TimeNow - ddTime) > DDTIME) {
 		    /*
 		     * Check if queue file removed out from under us.  Stop
 		     * processing if it has been.
@@ -759,13 +781,16 @@ main(int ac, char **av)
 		    struct stat st;
 
 		    if (fstat(fd, &st) == 0 && st.st_nlink == 0) {
-			NumBatches = 0;
-			break;
-		    }
+			if (DeleteDetectOpt || ddCount++ > 5) {
+			    NumBatches = 0;
+			    break;
+			}
+		    } else
+			ddCount = 0;
 		    ddTime = TimeNow;
 		}
 
-		if ((TimeNow = cdflushTime) > CACHEFLUSHTIME) {
+		if ((TimeNow - cdflushTime) > (CACHEFLUSHTIME / 3)) {
 		    /*
 		     * Flush old file handles
 		     */
@@ -791,6 +816,20 @@ main(int ac, char **av)
 			continue;
 		    }
 
+		    /*
+		     * If this is a delayed feed, then wait until we are
+		     * allowed to send it.
+		     */
+		    if (DelayTime && queuedTime) {
+			int sleepTime = (queuedTime + DelayTime) - TimeNow;
+			if (DebugOpt)
+			    printf("sleep %u (%u,%u,%u)\n",
+					sleepTime,
+					DelayTime, queuedTime, TimeNow);
+			if (sleepTime > 0)
+			    sleep(sleepTime);
+		    }
+
 		    /*
 		     * State processing.
 		     */
@@ -1057,7 +1096,8 @@ main(int ac, char **av)
 			CloseReopenCount += CloseReopenAfter;
 		    }
 		} else {
-		    dl_logit("Buffer syntax error: %s", buf);
+		    if (buf[0] != '#')
+			dl_logit("Buffer syntax error: %s", buf);
 		    bufInval = 1;
 		}
 
@@ -1204,19 +1244,223 @@ main(int ac, char **av)
     exit(0);
 }
 
+void
+setSocketOptions(int fd)
+{
+    if (fd >= 0) {
+	int on = 1;
+
+	/*
+	 * Make sure keepalive is turned on to prevent infinite hangs
+	 */
+	setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void *)&on, sizeof(on));
+
+	/*
+	 * Set the transmit and receive buffer size
+	 */
+	if (TxBufSize)
+	    setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (void *)&TxBufSize,
+							sizeof(TxBufSize));
+	if (RxBufSize)
+	    setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (void *)&RxBufSize,
+							sizeof(RxBufSize));
+
+#ifdef IP_TOS
+	/* Set TOS */
+	if (IPQoS > 0)
+	    setsockopt(fd, IPPROTO_IP, IP_TOS, (char *) &IPQoS, sizeof(IPQoS));
+#endif
+    }
+
+}
+
+/*
+ * Figure out the ip address and port.  Try to use our cached lookup
+ * info because a host might have multiple IN A's and we want to connect
+ * to the same one we succesfully connected to before.
+ */
+
+void
+getHostAddr(int *fd, const char *hostName, const char *serviceName, int defPort, struct sockaddr **addr, int *addrLen, int useSaved)
+{
+#ifdef INET6
+    static struct addrinfo *res0 = NULL;
+    static struct addrinfo *res = NULL;
+    if (useSaved && res != NULL) {
+	*addr = res->ai_addr;
+	*addrLen = res->ai_addrlen;
+	*fd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+    } else {
+	struct addrinfo hints;
+	char p[10];
+	int error;
+
+	/*
+	 * Open a wildcard listening socket
+	 */
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_family = PF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+	snprintf(p, sizeof(p), "%d", defPort);
+	if (res0 == NULL) {
+	    error = getaddrinfo(HostName, p, &hints, &res0);
+	    if (error == 0) {
+		res = res0;
+	    } else {
+		dl_logit("hostname lookup failure: %s:%s: %s\n",
+				HostName, serviceName, gai_strerror(error));
+		*fd = -1;
+		*addr = NULL;
+		return;
+	    }
+	} else if (res != NULL) {
+	    res = res->ai_next;
+	}
+	if (res == NULL) {
+	    *fd = -1;
+	    *addr = NULL;
+	} else {
+	    *fd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+	    *addr = res->ai_addr;
+	    *addrLen = res->ai_addrlen;
+	}
+    }
+
+#else
+    static struct sockaddr_in sin;
+
+    if (useSaved) {
+	*addr = (struct sockaddr *)&sin;
+	*addrLen = sizeof(sin);
+    } else {
+	struct hostent *hent0 = NULL;
+	static int hindex = 0;
+	char *hent;
+
+	memset(&sin, 0, sizeof(sin));
+
+	if (hent0 == NULL) {
+	    hent0 = gethostbyname(hostName);
+	    if (hent0 == NULL) {
+		dl_logit("hostname lookup failure: %s", strerror(errno));
+		*fd = -1;
+		*addr = NULL;
+		return;
+	    }
+	    hindex = 0;
+	} else {
+	    hindex++;
+	}
+	hent = hent0->h_addr_list[hindex];
+	if (hent == NULL) {
+	    dl_logit("hostname lookup failure: %s", strerror(errno));
+	    *fd = -1;
+	    *addr = NULL;
+	    hent0 = NULL;
+	    return;
+	} else {
+	    sin.sin_family = hent0->h_addrtype;
+	    memmove(&sin.sin_addr, hent, hent0->h_length);
+	}
+	{
+	    struct servent *serv = serviceName ?
+				getservbyname(serviceName, "tcp") : NULL;
+	    if (serv != NULL) {
+		sin.sin_port = serv->s_port;
+	    } else {
+		sin.sin_port = htons(defPort);
+		if (serviceName) {
+		    dl_logit("unable to lookup service '%s', using port %d", 
+			serviceName, 
+			defPort
+		    );
+		}
+	    }
+	}
+	*addr = (struct sockaddr *)&sin;
+	*addrLen = sizeof(sin);
+    }
+
+#endif	/* INET6 */
+
+}
+
+void
+bindOutboundInterface(int *fd)
+{
+    if (OutboundIpName != NULL) {
+#ifdef INET6
+	int error;
+	struct addrinfo hints;
+	struct addrinfo *bres;
+
+	/*
+	* Open a wildcard listening socket
+	*/
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_flags = AI_PASSIVE;
+	hints.ai_family = PF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+	error = getaddrinfo(OutboundIpName, 0, &hints, &bres);
+	if (error != 0) {
+	    dl_logit("local bind address lookup failure: %s",
+							gai_strerror(error));
+	    close(*fd);
+	    *fd = -1;
+	}
+
+	if (bind(*fd, bres->ai_addr, bres->ai_addrlen) < 0) {
+	    dl_logit("local bind address failed: %s", strerror(errno));
+	    close(*fd);
+	    *fd = -1;
+	}
+	freeaddrinfo(bres);
+#else	/* INET4 */
+	struct hostent *host = gethostbyname(OutboundIpName);
+	struct sockaddr_in bsin;
+
+	bzero(&bsin, sizeof(bsin));
+
+	if (host != NULL) {
+	    bsin.sin_family = host->h_addrtype;
+	    memmove(&bsin.sin_addr, host->h_addr, host->h_length);
+	} else if (IsIpAddr(OutboundIpName)) {
+	    bsin.sin_family = AF_INET;
+	    bsin.sin_addr.s_addr = inet_addr(OutboundIpName);
+	} else {
+	    dl_logit("local bind address lookup failure: %s", strerror(errno));
+	    close(*fd);
+	    *fd = -1;
+	}
+	if (bind(*fd, (struct sockaddr *)&bsin, sizeof(bsin)) < 0) {
+	    dl_logit("local bind address failed: %s", strerror(errno));
+	    close(*fd);
+	    *fd = -1;
+	}
+#endif	/* INET6 */
+    }
+}
+
 int
 connectTo(const char *hostName, const char *serviceName, int defPort)
 {
-    static struct sockaddr_in Save;
     static int UseSaved = 0;
-    struct sockaddr_in sin;
-    int fd = socket(AF_INET, SOCK_STREAM, 0);
+    int fd = -1;
     int cfd = -1;
+    static int ConnectCount;
+    struct sockaddr *addr = NULL;
+    int addrLen = 0;
+    int connected = 0;
+    int connectattempts = 0;
 
     WritesLosing = 0;
 
     clearResponseBuf();
 
+    ++ConnectCount;
+
+    stprintf("%s %s connect: %d", HostName, CurrentBatchFile + CBFIndex, ConnectCount);
+
 #ifdef	USE_ZLIB
     if (CompressOn > 0) {
 	int r;
@@ -1224,10 +1468,8 @@ connectTo(const char *hostName, const char *serviceName, int defPort)
 	    logit(LOG_ERR, "deflateEnd error: %s (at connect)", Z_Strm.msg);
     }
 #endif
-
     CompressOn = 0;
 
-    memset(&sin, 0, sizeof(sin));
     ++Stats->SentStats.ConnectCnt;
     ++Stats->SentStats.ConnectTotal;
     if (HostStats != NULL) {
@@ -1240,102 +1482,67 @@ connectTo(const char *hostName, const char *serviceName, int defPort)
     stprintf("%s %s connect: %d", HostName, CurrentBatchFile + CBFIndex,
 							Stats->SentStats.ConnectCnt);
 
-    /*
-     * Make sure keepalive is turned on to prevent infinite hangs
-     */
+/*
+ * This gets a little messy.
+ *
+ * The INET6 based host lookups allow us to keep track of the result
+ * of an address lookup across multiple getaddrinfo() calls. The INET4
+ * lookups don't without making a copy of the result.
+ *
+ * This means we need to do a socket() and local bind() before we lookup
+ * the host for INET4. In the INET6 case, we only want to do the bind()
+ * after we have done the lookup, so that we can do a local bind() using
+ * the correct protocol.
+ *
+ * Not sure if this is the right way, but that's how it is done for now.
+ *
+ */
+    while (!TermFlag && !connected && ++connectattempts <= 5) {
 
-    {
-	int on = 1;
-	setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void *)&on, sizeof(on));
-    }
-
-    /*
-     * Set the transmit and receive buffer size
-     */
-
-    if (TxBufSize) {
-	setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (void *)&TxBufSize, sizeof(TxBufSize));
-    }
-    if (RxBufSize) {
-	setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (void *)&RxBufSize, sizeof(RxBufSize));
-    }
-
-    /*
-     * figure out the ip address and port.  Try to use our cached lookup
-     * info because a host might have multiple IN A's and we want to connect
-     * to the same one we succesfully connected to before.
-     */
-
-    if (UseSaved) {
-	sin = Save;
-    } else {
-	struct hostent *host = gethostbyname(hostName);
-
-	if (host) {
-	    sin.sin_family = host->h_addrtype;
-	    memmove(&sin.sin_addr, host->h_addr, host->h_length);
-	} else if (strtol(hostName, NULL, 0) != 0) {
-	    sin.sin_family = AF_INET;
-	    sin.sin_addr.s_addr = inet_addr(hostName);
-	} else {
-	    dl_logit("hostname lookup failure: %s", strerror(errno));
-	    fd = -1;
+#ifndef INET6
+	fd = socket(AF_INET, SOCK_STREAM, 0);
+	if (addr == NULL && fd != -1) {
+	    setSocketOptions(fd);
+	    bindOutboundInterface(&fd);
 	}
-	{
-	    struct servent *serv = serviceName ? getservbyname(serviceName, "tcp") : NULL;
-	    if (serv) {
-		sin.sin_port = serv->s_port;
-	    } else {
-		sin.sin_port = htons(defPort);
-		if (serviceName) {
-		    dl_logit("unable to lookup service '%s', using port %d", 
-			serviceName, 
-			defPort
-		    );
-		}
-	    }
+#endif
+
+	getHostAddr(&fd, hostName, serviceName, defPort, &addr, &addrLen, UseSaved);
+
+#ifdef INET6
+	if (addr != NULL && fd != -1) {
+	    setSocketOptions(fd);
+	    bindOutboundInterface(&fd);
 	}
-	Save = sin;
-    }
-
-    /*
-     * bind outbound socket if requested
-     */
-
-    if (OutboundIpName != NULL) {
-	struct hostent *host = gethostbyname(OutboundIpName);
-	struct sockaddr_in bsin;
-
-	bzero(&bsin, sizeof(bsin));
-
-	if (host) {
-	    bsin.sin_family = host->h_addrtype;
-	    memmove(&bsin.sin_addr, host->h_addr, host->h_length);
-	} else if (strtol(OutboundIpName, NULL, 0) != 0) {
-	    bsin.sin_family = AF_INET;
-	    bsin.sin_addr.s_addr = inet_addr(OutboundIpName);
-	} else {
-	    dl_logit("outbound -B address lookup failure: %s", strerror(errno));
+#endif
+	if (addr == NULL) {
 	    close(fd);
 	    fd = -1;
-	}
-	if (bind(fd, (struct sockaddr *)&bsin, sizeof(bsin)) < 0) {
-	    dl_logit("bind() failed on -B address: %s", strerror(errno));
+	    connected = -1;
+	    UseSaved = 0;
+	} else if (fd == -1) {
+	    connected = 0;
+	    UseSaved = 0;
+	} else if (connect(fd, addr, addrLen) < 0) {
+	    dl_logit("%s %s connect: %s", NetAddrToSt(0, addr, 1, 1, 1),
+					CurrentBatchFile + CBFIndex,
+					strerror(errno));
 	    close(fd);
 	    fd = -1;
+	    stprintf("%s %s connect: fail", HostName,
+					CurrentBatchFile + CBFIndex);
+	    connected = 0;
+	    UseSaved = 0;
+	} else {
+	    connected = 1;
+	    connectattempts = 0;
+	}
+	if (connected == -1) {
+	    if (RealTimeOpt)
+		sleep(WaitTime);
+	    else
+		connectattempts = 999;
 	}
-    }
-
-    /*
-     * connect to the destination
-     */
-
-    if (fd >= 0 && connect(fd, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
-	dl_logit("connect: %s", strerror(errno));
-	close(fd);
-	fd = -1;
-	stprintf("%s %s connect: fail", HostName, CurrentBatchFile + CBFIndex);
-	sleep(5);
     }
 
     /*
@@ -1344,12 +1551,7 @@ connectTo(const char *hostName, const char *serviceName, int defPort)
      * parameters based on the destination route.  So, we do it here as well.
      */
 
-    if (TxBufSize && fd >= 0) {
-	setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (void *)&TxBufSize, sizeof(TxBufSize));
-    }
-    if (RxBufSize && fd >= 0) {
-	setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (void *)&RxBufSize, sizeof(RxBufSize));
-    }
+    setSocketOptions(fd);
 
 #ifdef TCP_NODELAY
     /*
@@ -1372,8 +1574,6 @@ connectTo(const char *hostName, const char *serviceName, int defPort)
     if (fd >= 0) {
 	char *ptr;
 
-	UseSaved = 1;
-
 	clearCommandBuffer();
 	cfd = fd;
 
@@ -1493,8 +1693,22 @@ connectTo(const char *hostName, const char *serviceName, int defPort)
     if (cfd < 0) {
 	if (LastErrBuf[0] == 0)
 	    sprintf(LastErrBuf, "connect-timeout");
-	sleep(WaitTime);
+	sleep(WaitTime / 3);
+    } else {
+	int n;
+
+	n = fcntl(cfd, F_GETFL);
+	if (n < 0) {
+	    dl_logit("getting socket flags: %s", strerror(errno));
+	} else {
+	    if (fcntl(cfd, F_SETFL, n | O_NONBLOCK) < 0) {
+		dl_logit("setting O_NONBLOCK: %s", strerror(errno));
+	    }
+	}
+
+	UseSaved = 1;
     }
+
     KillFd = fd;
     return(cfd);
 }
@@ -1505,7 +1719,7 @@ connectTo(const char *hostName, const char *serviceName, int defPort)
  */
 
 int
-Transact(int cfd, const char *relPath, char *msgId, int off, int size, int cSize, int defers, char *stage, char *reason, char *buf, int *sentSize)
+Transact(int cfd, const char *relPath, char *msgId, off_t off, int size, int cSize, int defers, char *stage, char *reason, char *buf, int *sentSize)
 {
     int r = 0;
     static char *ptr = NULL;
@@ -1608,7 +1822,7 @@ Transact(int cfd, const char *relPath, char *msgId, int off, int size, int cSize
 }
 
 int
-DumpArticle(int cfd, const char *relPath, int off, int size, int cSize)
+DumpArticle(int cfd, const char *relPath, off_t off, int size, int cSize)
 {
     static char pathBase[PATH_MAX];
     char *path;
@@ -1621,7 +1835,7 @@ DumpArticle(int cfd, const char *relPath, int off, int size, int cSize)
     int doBytes = 1;
     int wireFormat = 0;
     int artSize = 0;
-    SpoolArtHdr *ah;
+    SpoolArtHdr ah;
 
     HeaderSize = 0;
 
@@ -1655,27 +1869,35 @@ DumpArticle(int cfd, const char *relPath, int off, int size, int cSize)
 
 	artSize = size;
 
-#if USE_MMAP_BCOPY
-	{
-	    static SpoolArtHdr tah;
-	    bcopy(base, &tah, sizeof(tah));
-	    ah = &tah;
-	}
-#else
-	ah = (SpoolArtHdr *)base;
-#endif
+	bcopy(base, &ah, sizeof(ah));
 
-	if (artSize > 24 && (uint8)ah->Magic1 == (uint8)STORE_MAGIC1 &&
-				(uint8)ah->Magic2 == (uint8)STORE_MAGIC2) {
-	    artSize -= ah->HeadLen;
-	    ptr += ah->HeadLen;
+	if (artSize > 24 && (uint8)ah.Magic1 == (uint8)STORE_MAGIC1 &&
+				(uint8)ah.Magic2 == (uint8)STORE_MAGIC2) {
+	    artSize -= ah.HeadLen;
+	    ptr += ah.HeadLen;
 	    wireFormat = 0;
-	    if (ah->StoreType & STORETYPE_WIRE)
+	    if (ah.StoreType & STORETYPE_WIRE)
 		wireFormat = 1; 
 	} else {
 	    wireFormat = 0;
 	}
 
+	if (wireFormat && !HeaderOnlyFeed && !GenLinesHeader &&
+#ifdef USE_ZLIB
+			!CompressOn &&
+#endif
+			artSize > CMDBUFFSIZE ) { 
+	    if (writeLarge(cfd, ptr, artSize) < 0) {
+		if (base != NULL)
+		    cdunmap(base, size, multiArtFile, cSize > 0);
+		return(T_FAILED);
+	    } else {
+		if (base != NULL)
+		    cdunmap(base, size, multiArtFile, cSize > 0);
+		return(0);
+	    }
+	}
+	    
 	for (i = b = 0; i < artSize; b = i) {
 	    /*
 	     * if first character is a '.', escape it
@@ -1822,7 +2044,7 @@ StreamReload(int cfd)
 #endif
 
 int
-StreamTransact(int cfd, const char *relPath, char *msgId, int off, int size, int cSize, int defers, char *stage, char *reason, char *buf, int *sentSize)
+StreamTransact(int cfd, const char *relPath, char *msgId, off_t off, int size, int cSize, int defers, char *stage, char *reason, char *buf, int *sentSize)
 {
     Stream *s;
     int r = T_STREAMING;
@@ -1916,7 +2138,7 @@ StreamTransact(int cfd, const char *relPath, char *msgId, int off, int size, int
 		    sprintf(cst, " C%d", s->st_CompSize);
 		else
 		    cst[0] = 0;
-		sprintf(buf,"%s %s %d,%d %c%s", s->st_RelPath, s->st_MsgId,
+		sprintf(buf,"%s %s %lld,%d %c%s", s->st_RelPath, s->st_MsgId,
 			s->st_Off, s->st_Size, HeaderOnlyFeed ? 'H' : ' ',
 			cst);
 		break;
@@ -2182,7 +2404,7 @@ dl_logit(const char *ctl, ...)
     va_start(va, ctl);
     vsprintf(buf + strlen(buf), ctl, va);
     va_end(va);
-    syslog(LOG_NOTICE, "%s", buf);
+    logit(LOG_NOTICE, "%s", buf);
     if (DebugOpt > 0)
 	fprintf(stderr, "%s newslink[%ld] %s\n", LogTime(), (long)getpid(), buf);
     if (DebugOpt > 1)
@@ -2279,7 +2501,7 @@ logStats(const char *description)
 static char Buf[16384];
 static int Bufb;
 static int Bufe;
-static char Cmd[32768];
+static char Cmd[CMDBUFFSIZE];
 static int Cmde;
 
 void
@@ -2344,8 +2566,20 @@ flushCommandBuffer(int cfd)
 
 	credtime(OUR_DELAY);
 
+	errno = 0;
+	r = write(cfd, Cmd + b, Cmde - b);
+	if (r < 0) {
+	    if (errno != EAGAIN && errno != EINTR &&
+		errno != EWOULDBLOCK && errno != EINPROGRESS) {
+		    Cmde = 0;
+		    return(-1);
+	    }
+	    r = 0;
+	}
+	b += r;
+
 #if USE_POLL
-	{
+	if (b != Cmde) {
 	    struct pollfd pfd = { 0 };
 	    struct linger l;
 
@@ -2363,16 +2597,6 @@ flushCommandBuffer(int cfd)
 	}
 #endif
 
-	errno = 0;
-	if ((r = write(cfd, Cmd + b, Cmde - b)) < 0) {
-	    if (errno != EAGAIN && errno != EINTR &&
-		errno != EWOULDBLOCK && errno != EINPROGRESS) {
-		    Cmde = 0;
-		    return(-1);
-	    }
-	    r = 0;
-	}
-	b += r;
 	credtime(THEIR_DELAY);
     }
 
@@ -2380,6 +2604,87 @@ flushCommandBuffer(int cfd)
     return(0);
 }
 
+int
+writeLarge(int cfd, char *buffer, size_t size)
+{
+    struct iovec tmpvec[3]; /* structures to store writev info */
+
+    int status;	
+    size_t offset, bytes=0;
+    int iovleft=0,	/* iov's to handle */
+	i;		/* first iov in buffer that has data left */
+
+    if (WritesLosing) {
+	Cmde = 0;
+	return(-1);
+    }
+
+    if (Cmde > 0) { /* if there is something to write out in Cmd buffer */ 
+        tmpvec[iovleft].iov_base=Cmd;
+        tmpvec[iovleft].iov_len=Cmde;
+        iovleft++;
+        bytes += Cmde;
+	Cmde=0;
+    }
+    tmpvec[iovleft].iov_base=buffer;
+    tmpvec[iovleft].iov_len=size;
+    iovleft++;
+
+    bytes += size;
+
+    i = 0;
+    do {
+        /* Write out what's left and return success if it's all written. */
+	credtime(OUR_DELAY);
+	errno = 0;
+	status = writev(cfd, tmpvec + i, iovleft);
+	if (status < 0) {
+	    if (errno != EAGAIN && errno != EINTR &&
+		errno != EWOULDBLOCK && errno != EINPROGRESS) {
+		    return(-1);
+	    }
+	    status = 0; offset=0;
+	} else {
+            offset = status;
+            bytes -= offset;
+        }
+#if USE_POLL
+	if (bytes > 0) {
+	    struct pollfd pfd = { 0 };
+	    struct linger l;
+
+	    pfd.fd = cfd;
+	    pfd.events = POLLOUT;
+	    if (poll(&pfd, 1, Timeout * 1000) == 0) {
+		dl_logit("write timeout");
+		WritesLosing = 1;
+		l.l_onoff = 1;
+		l.l_linger = 0;
+		setsockopt(cfd, SOL_SOCKET, SO_LINGER, (void*)&l, sizeof(l));
+		TimeCounter = 100000;
+		break;
+	    }
+	}
+#endif
+	credtime(THEIR_DELAY);
+
+        /* Skip full iovecs */
+        for (; offset >= (size_t) tmpvec[i].iov_len && iovleft > 0; i++) {
+            offset -= tmpvec[i].iov_len;
+            iovleft--;
+        }
+	if (offset > 0) {
+	    tmpvec[i].iov_base = (char *) tmpvec[i].iov_base + offset;
+	    tmpvec[i].iov_len -= offset;
+	}
+
+    } while (iovleft > 0 && (status >= 0 || errno == EINTR));
+
+    assert(bytes == 0);
+
+    return status;
+}
+
 int
 commandWrite(int cfd, const void *buf, int bytes, int artdata)
 {
@@ -2509,27 +2814,29 @@ commandResponse(int cfd, char **rptr, const char *ctl, ...)
 	    if (WritesLosing)
 		break;
 
-	    if (Timeout > 0) {
+	    do {
+		if (Timeout > 0) {
 #if USE_POLL
-		struct pollfd pfd = { 0 };
+		    struct pollfd pfd = { 0 };
 
-		pfd.fd = cfd;
-		pfd.events = POLLIN;
-		if (poll(&pfd, 1, Timeout * 1000) == 0) {
-		    TimeCounter = 100000;
-		    break;
-		}
+		    pfd.fd = cfd;
+		    pfd.events = POLLIN;
+		    if (poll(&pfd, 1, Timeout * 1000) == 0) {
+			TimeCounter = 100000;
+			break;
+		    }
 #else
-		/*
-		 * set an alarm.  alarm() is really inefficient
-		 * syscallwise, so we only do it when we really need
-		 * to read().
-		 */
-		alarm(Timeout);
-		alarmSet = 1;
+		    /*
+		     * set an alarm.  alarm() is really inefficient
+		     * syscallwise, so we only do it when we really need
+		     * to read().
+		     */
+		    alarm(Timeout);
+		    alarmSet = 1;
 #endif
-	    }
-	    n = read(cfd, Buf + Bufe, sizeof(Buf) - Bufe - 1);
+		}
+		n = read(cfd, Buf + Bufe, sizeof(Buf) - Bufe - 1);
+	    } while (n < 0 && (errno == EAGAIN || errno == EWOULDBLOCK));
 	} while (n > 0);
 
 	credtime(THEIR_DELAY);
@@ -2647,7 +2954,7 @@ readretry(char *buf, int size)
 	    if (s->st_State == STATE_RETRY) {
 		if (s->st_Off || s->st_Size) {
 		    sprintf(buf, 
-			"%s\t%s\t%d,%d",
+			"%s\t%s\t%lld,%d",
 			s->st_RelPath, 
 			s->st_MsgId, 
 			s->st_Off,
@@ -2678,6 +2985,7 @@ readline(int fd, char *buf, int size, time_t t)
 {
     RBlock *rbs;
     int statCounter = 0;
+    int loopCounter = 0;
     static int usleeptime = 100000;
 
     if (fd < 0 || fd >= MAXFILEDES) {
@@ -2751,6 +3059,15 @@ readline(int fd, char *buf, int size, time_t t)
 
 	    if (n < 0 || RealTimeOpt == 0 || StreamPend != 0 || TermFlag)
 		break;
+
+	    /*
+	     * If we don't have any articles in realtime mode, flush the
+	     * article file cache occasionally to make sure that deleted
+	     * files are removed.
+	     */
+	    if (loopCounter++ > 60)
+		cdflush();
+
 	    /*
 	     * sleep and retry the read.  Effectively a tail -f.  When the file
 	     * is renamed AND a new realtime batchfile exists, we switch to the
@@ -2860,6 +3177,8 @@ ValidMsgId(char *msgid)
     int i;
     int l = strlen(msgid);
 
+    if (l >= MAXMSGIDLEN)
+	return(0);
     if (msgid[0] == '<') {
 	for (i = 1; i < l; ++i) {
 	    if (msgid[i] == ' ' || msgid[i] == '\t')
@@ -2940,7 +3259,7 @@ cdinit(void)
 }
 
 char *
-cdmap(const char *path, int off, int *psize, int cSize, int *multiArtFile)
+cdmap(const char *path, off_t off, int *psize, int cSize, int *multiArtFile)
 {
     int i;
     XCache *mc;
@@ -3017,7 +3336,7 @@ cdmap(const char *path, int off, int *psize, int cSize, int *multiArtFile)
 
 	    ptr = (char *)malloc(tah.ArtLen + tah.HeadLen + 2);
 	    if (ptr == NULL) {
-		syslog(LOG_CRIT, "Unable to malloc %d bytes for article (%s)\n",
+		logit(LOG_CRIT, "Unable to malloc %d bytes for article (%s)\n",
 				tah.ArtLen + tah.HeadLen + 2, strerror(errno));
 		gzclose(gzf);
 		return(NULL);
@@ -3033,17 +3352,19 @@ cdmap(const char *path, int off, int *psize, int cSize, int *multiArtFile)
 	    *psize = tah.ArtLen + tah.HeadLen;
 	    gzclose(gzf);
 #else
-	    syslog(LOG_CRIT, "Queue batch file indicates compressed file and compression not enabled");
+	    logit(LOG_CRIT, "Queue batch file indicates compressed file and compression not enabled");
 #endif
 	} else {
 	    ptr = xmap(NULL, *psize + *multiArtFile, PROT_READ, MAP_SHARED, mc->mc_Fd, off);
 	    if (ptr == NULL)
+		return(ptr);
 	    if (ptr) {
-		xadvise(ptr, *psize, XADV_WILLNEED);
+		if (DOpts.FeederPreloadArt)
+		    xadvise(ptr, *psize, XADV_WILLNEED);
 		xadvise(ptr, *psize, XADV_SEQUENTIAL);
 	    }
 	    if (*multiArtFile && ptr && ptr[*psize] != 0) {
-		syslog(LOG_CRIT, "article batch corrupted: %s @ %d,%d", path, off, *psize);
+		logit(LOG_CRIT, "article batch corrupted: %s @ %lld,%ld", path, off, *psize);
 		xunmap(ptr, *psize + *multiArtFile);
 		ptr = NULL;
 	    }
@@ -3071,7 +3392,8 @@ cdflush(void)
     int i;
     time_t t = time(NULL);
     for (i = 0; i < MAXMCACHE; ++i) {
-	if (XCacheAry[i].mc_Fd >= 0 && (t - XCacheAry[i].mc_OpenTime) > 600) {
+	if (XCacheAry[i].mc_Fd >= 0 &&
+			(t - XCacheAry[i].mc_OpenTime) > CACHEFLUSHTIME) {
 	close(XCacheAry[i].mc_Fd);
 	zfreeStr(&SysMemPool, &XCacheAry[i].mc_Path);
 	XCacheAry[i].mc_Fd = -1;
@@ -3104,14 +3426,21 @@ doArtLog(const char *class, char *msgid, char *message, const char *stage, const
 
 #define ISWHITE(c)	((c)=='\n' || (c) == '\r' || (c) == ' ' || (c) == '\t')
 
+/*
+ * Extract a line from the dqueue file
+ * Return 0 for success and -1 for failure
+ */
 int
-extractFeedLine(const char *buf, char *relPath, char *msgId, int *poff, int *psize, int *pheadOnly, time_t *queuedTime, int *cSize)
+extractFeedLine(const char *buf, char *relPath, char *msgId, off_t *poff, int *psize, int *pheadOnly, time_t *queuedTime, int *cSize)
 {
     const char *s = buf;
     const char *b;
     int l1 = 0;
     int l2 = 0;
 
+    if (*buf == '#')
+	return(-1);
+
     *pheadOnly = 0;
 
     /*
@@ -3149,7 +3478,7 @@ extractFeedLine(const char *buf, char *relPath, char *msgId, int *poff, int *psi
 
     b = s;
     if (isdigit((int)*s))
-	sscanf(s, "%d,%d", poff, psize);
+	sscanf(s, "%lld,%d", poff, psize);
     while ((s = strchr(s, ' ')) != NULL) {
 	while (ISWHITE(*s))
 	    ++s;
