@@ -110,7 +110,8 @@ PathList PList[] = {
     { "server_dactive",	&ServerDActivePat, PatDbExpand },
     { "reader_dactive",	&ReaderDActivePat, PatDbExpand },
     { "dhistory",	&DHistoryPat, PatDbExpand },
-    { "spam_cache",	&SpamCachePat, PatDbExpand },
+    { "spam.body.cache",	&SpamBodyCachePat, PatDbExpand },
+    { "spam.nph.cache",	&SpamNphCachePat, PatDbExpand },
     { "pcommit_cache",	&PCommitCachePat, PatDbExpand }
 };
 
