@@ -138,12 +138,14 @@ main(int ac, char **av)
 		    *++p = 0;
 		    hv = hhash(m);
 		    r = LookupHash(hv, m, NULL);
+		} else if (sscanf(buf, "%x.%x", &hv.h1, &hv.h2) == 2) {
+		    r = LookupHash(hv, NULL, NULL);
 		}
 	    }
 	    if (fi != stdin)
 		fclose(fi);
 	} else {
-	    fprintf(stderr, "Unable to open %s\n", file);
+	    fprintf(stderr, "Unable to open %s (%s)\n", file, strerror(errno));
 	}
     } else {
 	hash_t hv;
@@ -184,7 +186,7 @@ main(int ac, char **av)
 }
 
 int
-DumpFileHeader(int fd, int offset)
+DumpFileHeader(int fd, off_t offset)
 {
     SpoolArtHdr artHdr;
 
@@ -200,6 +202,7 @@ DumpFileHeader(int fd, int offset)
 	return(1);
     }
     printf("OK\n");
+    printf("Offset   : %llu\n", offset);
     printf("Version  : %d\n", artHdr.Version);
     printf("HeadLen  : %d\n", artHdr.HeadLen);
     printf("StoreType:%s%s%s\n",
@@ -217,18 +220,19 @@ MapArticle(int fd, char *fname, char **base, History *h, int *extra, int *artSiz
 {
     if (SpoolCompressed(H_SPOOL(h->exp))) {
 #ifdef USE_ZLIB
-        gzFile *gzf;
-        SpoolArtHdr tah = { 0 };
+	gzFile *gzf;
+	SpoolArtHdr tah = { 0 };
+	char *p;
 
-        lseek(fd, h->boffset, 0);
-        if (read(fd, &tah, sizeof(SpoolArtHdr)) != sizeof(SpoolArtHdr)) {
+	lseek(fd, h->boffset, 0);
+	if (read(fd, &tah, sizeof(SpoolArtHdr)) != sizeof(SpoolArtHdr)) {
 	    close(fd);
 	    fprintf(LogFo, "Unable to read article header (%s)\n",
 							strerror(errno));
 	    return(-1);
-        }
-        if ((uint8)tah.Magic1 != STORE_MAGIC1 &&
-				(uint8)tah.Magic2 != STORE_MAGIC2) {
+	}
+	if ((uint8)tah.Magic1 != STORE_MAGIC1 &&
+					(uint8)tah.Magic2 != STORE_MAGIC2) {
 	    lseek(fd, h->boffset, 0);
 	    tah.Magic1 = STORE_MAGIC1;
 	    tah.Magic2 = STORE_MAGIC2;
@@ -236,13 +240,13 @@ MapArticle(int fd, char *fname, char **base, History *h, int *extra, int *artSiz
 	    tah.ArtLen = h->bsize;
 	    tah.ArtHdrLen = h->bsize;
 	    tah.StoreLen = h->bsize;
-        }
-        gzf = gzdopen(fd, "r");
-        if (gzf == NULL) {
+	}
+	gzf = gzdopen(fd, "r");
+	if (gzf == NULL) {
 	    fprintf(LogFo, "Error opening compressed article\n");
 	    return(-1);
 	}
-    	*base = (char *)malloc(tah.ArtLen + tah.HeadLen + 2);
+	*base = (char *)malloc(tah.ArtLen + tah.HeadLen + 2);
 	if (*base == NULL) {
 	    fprintf(LogFo, "Unable to malloc %d bytes for article (%s)\n",
 				tah.ArtLen + tah.HeadLen + 2, strerror(errno));
@@ -280,10 +284,10 @@ MapArticle(int fd, char *fname, char **base, History *h, int *extra, int *artSiz
     }
 
     if (*base == NULL) {
-	fprintf(LogFo, "Unable to map file %s: %s (%d,%d,%d)\n",
+	fprintf(LogFo, "Unable to map file %s: %s (%llu,%d,%d)\n",
 					fname,
 					strerror(errno),
-					(int)(h->boffset - *extra),
+					(off_t)(h->boffset - *extra),
 					 (int)(h->bsize + *extra + 1),
 					*extra
 	);
@@ -302,7 +306,7 @@ DumpArticle(char *fname, History *h, const char *msgid)
     int headLen;
     int oldFormat = 0;
     int compressedFormat = 0;
-    SpoolArtHdr *ah;
+    SpoolArtHdr ah;
 
     if ((fd = open(fname, O_RDONLY)) >= 0) {
 	char *base = NULL;
@@ -344,26 +348,19 @@ DumpArticle(char *fname, History *h, const char *msgid)
 	    rv = 1;
 	}
 
-#if	USE_MMAP_BCOPY
-	{
-	    static SpoolArtHdr tah;
-	    bcopy(ptr, &tah, sizeof(tah));
-	    ah = &tah;
-	}
-#else
-	ah = (SpoolArtHdr *)ptr;
-#endif
+	bcopy(ptr, &ah, sizeof(ah));
+
 	if (DebugOpt) {
-	    printf("magic1=%x  magic2=%x\n", (uint8)ah->Magic1,
-							(uint8)ah->Magic2);
+	    printf("magic1=%x  magic2=%x\n", (uint8)ah.Magic1,
+							(uint8)ah.Magic2);
 	}
-	if ((uint8)ah->Magic1 == (uint8)STORE_MAGIC1 &&
-				(uint8)ah->Magic2 == (uint8)STORE_MAGIC2) {
-	    headLen = (uint32)ah->ArtHdrLen;
-	    artSize -= (uint8)ah->HeadLen;
-	    ptr += (uint8)ah->HeadLen;
+	if ((uint8)ah.Magic1 == (uint8)STORE_MAGIC1 &&
+				(uint8)ah.Magic2 == (uint8)STORE_MAGIC2) {
+	    headLen = (uint32)ah.ArtHdrLen;
+	    artSize -= (uint8)ah.HeadLen;
+	    ptr += (uint8)ah.HeadLen;
 	    wireFormat = 0;
-	    if ((uint8)ah->StoreType & STORETYPE_WIRE)
+	    if ((uint8)ah.StoreType & STORETYPE_WIRE)
 		wireFormat = 1;
 	} else {
 	    headLen = artSize;
