@@ -145,7 +145,7 @@ DReadOver(char *data)
 	slash = strchr(gfname, '/');
 
 	snprintf(path, sizeof(path), "%s/%s", PatExpand(GroupHomePat), gfname);
-	printf("%s:%d\t%s", data, artNo, gfname);
+	printf("GROUPINFO:%s:%d\t%s", data, artNo, gfname);
 
 	if ((fd1 = open(path, O_RDONLY)) < 0) {
 	    printf("\t(array open fail)\n");
@@ -212,22 +212,28 @@ DReadOver(char *data)
 		return;
 	}
 
-	printf("\tdataFilePos=%d\tsize=%d\n", oa.oa_SeekPos, oa.oa_Bytes);
+	printf("\tdataFilePos=%d\tsize=%d\treceived=%d\tbytes=%d\thash=%x.%x\n",
+			oa.oa_SeekPos, oa.oa_Bytes, oa.oa_TimeRcvd,
+			oa.oa_ArtSize, oa.oa_MsgHash.h1, oa.oa_MsgHash.h2);
 
 	if (ViewData) {
-	    char buf[8192];
-	    int n = oa.oa_Bytes;
+	    static char *buf = NULL;
+	    int buflen = 0;
+	    int n = oa.oa_Bytes + 1;
 	    int r;
 
+	    if (n > buflen)
+		buflen = n;
+		buf = (char *)realloc(buf, buflen);
 	    lseek(fd2, oa.oa_SeekPos, 0);
-	    while ((r = read(fd2, buf, (n > sizeof(buf)) ? sizeof(buf) : n)) > 0) {
-		write(1, buf, r);
+	    while ((r = read(fd2, buf, n)) > 0) {
+		printf("%s", buf);
 		n -= r;
 	    }
 	    close(fd2);
 	}
 	close(fd1);
-	if (artNo < artNoEnd)
+	if (artNo <= artNoEnd)
 	    printf("--------------------------------------------------------------------\n");
     }
     printf("\n");
