@@ -109,21 +109,21 @@ main(int ac, char **av)
             continue;
 
         if (strncmp(s1, "SOUT", 4) == 0) {
-            char *path = strtok(NULL, " \t\n");
-            const char *msgid = MsgId(strtok(NULL, " \t\n"), NULL);
-            char *offsize = strtok(NULL, " \t\n");
-            char *nglist = strtok(NULL, " \t\n");
-            char *dist = strtok(NULL, " \t\n");
-            char *npath = strtok(NULL, " \t\n");
-            char *headOnly = strtok(NULL, " \t\n");
-            const char *artType = strtok(NULL, " \t\n");
-            const char *cSize = strtok(NULL, " \t\n");
+            char *path = strtok(NULL, "\t\n");
+            char *offsize = strtok(NULL, "\t\n");
+            const char *msgid = MsgId(strtok(NULL, "\t\n"), NULL);
+            char *nglist = strtok(NULL, "\t\n");
+            char *dist = strtok(NULL, "\t\n");
+            char *npath = strtok(NULL, "\t\n");
+            char *headOnly = strtok(NULL, "\t\n");
+            const char *artType = strtok(NULL, "\t\n");
+            const char *cSize = strtok(NULL, "\t\n");
             if (path && offsize && msgid && nglist) {
                 int spamArt = 0;
 		if (npath == NULL) npath = "1";
 		if (headOnly == NULL) headOnly = "0";
 		if (artType == NULL) artType = "000000";
-		if (cSize == NULL) cSize = "0";
+		if (cSize != NULL && strcmp(cSize, "0") == 0) cSize = NULL;
 		if (sendStdout)
 		    printf(
 			"SOUTLINE: %s %s %s %s %s HO=%s AT=%s %s\n",
@@ -214,7 +214,7 @@ writeFeedOut(const char *label, const char *file, const char *msgid, const char
 	    sprintf(fe->fe_Buf + fe->fe_BufIdx, "%s %s %s%s%s%s\n",
 		file, msgid, offSize, 
 		(headOnly ? " H" : ""),
-		(cSize != NULL ? " " : ""),
+		(cSize != NULL ? " C" : ""),
 		(cSize != NULL ? cSize : "")
 	    );
 	    fe->fe_BufIdx += strlen(fe->fe_Buf + fe->fe_BufIdx);
@@ -257,7 +257,7 @@ flushFeedOut(Feed *fe)
 	}
 	if (n != fe->fe_BufIdx && fe->fe_Failed == 0) {
 	    fe->fe_Failed = 1;
-	    syslog(LOG_INFO, "failure writing to feed %s", fe->fe_Label);
+	    logit(LOG_INFO, "failure writing to feed %s", fe->fe_Label);
 	}
     }
     fe->fe_BufIdx = 0;
