@@ -47,6 +47,7 @@ int Quiet;
 int ForceRealTime = 0;
 int  TxBufSize = 0;
 int  RxBufSize = 0;
+int  QoS = 0;
 char *OutboundIpStr = "-nop";
 
 int
@@ -56,7 +57,7 @@ main(int ac, char **av)
     const char *cFile = NULL;
     int oldFormat = 0;
 
-    openlog("dspoolout", (DebugOpt > 0 ? LOG_PERROR: 0) | LOG_NDELAY | LOG_PID, LOG_NEWS);
+    OpenLog("dspoolout", (DebugOpt > 0 ? LOG_PERROR: 0) | LOG_NDELAY | LOG_PID);
     LoadDiabloConfig(ac, av);
 
     cFile = DNewsfeedsPat;
@@ -122,6 +123,9 @@ main(int ac, char **av)
 		case 'R':
 		    RxBufSize = strtol((*ptr ? ptr : av[++i]), NULL, 0);
 		    break;
+		case 'Q':
+		    QoS = strtol((*ptr ? ptr : av[++i]), NULL, 0);
+		    break;
 		case 'r':
 		    ForceRealTime = 1;
 		    break;
@@ -230,7 +234,7 @@ processDnewsfeeds(const char *qFile)
 	    flags = flags | (li->li_RealTime ? SF_REALTIME : 0);
 	    flags = flags | (li->li_Notify ? SF_NOTIFY : 0);
 	    flags = flags | (li->li_HeadFeed ? SF_HEADFEED : 0);
-	    flags = flags | (li->li_NoCheck ? SF_NOCHECK : 0);
+	    flags = flags | (li->li_Check ? 0 : SF_NOCHECK);
 	    flags = flags | (li->li_GenLines ? SF_GENLINES : 0);
 	    flags = flags | (li->li_PreserveBytes ? SF_PRESBYTES : 0);
 	    flags = flags | (li->li_ArticleStat ? SF_ARTSTAT : 0);
@@ -324,6 +328,7 @@ processDspoolout(const char *fname, char *qFile)
 		    int compress = 0;
 		    int txbufsiz = TxBufSize;
 		    int rxbufsiz = RxBufSize;
+		    int qos = QoS;
 		    int maxstream = 0;
 		    char *obip = NULL;
 		    char *hours = NULL;
@@ -378,6 +383,8 @@ processDspoolout(const char *fname, char *qFile)
 				txbufsiz = strtol(maxqStr + 1, NULL, 0);
 			    } else if (maxqStr[0] == 'R') {
 				rxbufsiz = strtol(maxqStr + 1, NULL, 0);
+			    } else if (maxqStr[0] == 'Q') {
+				qos = strtol(maxqStr + 1, NULL, 0);
 			    } else if (maxqStr[0] == 'M') {
 				maxstream = strtol(maxqStr + 1, NULL, 0);
 			    } else {
@@ -439,6 +446,7 @@ processDspoolout(const char *fname, char *qFile)
 			li.li_StartDelay = delay;
 			li.li_TransmitBuf = txbufsiz;
 			li.li_ReceiveBuf = rxbufsiz;
+			li.li_QoS = qos;
 			li.li_LogArts = logarts;
 			li.li_BindAddress = obip;
 			li.li_MaxStream = 0;
@@ -508,8 +516,10 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
     struct stat st;
     char txBufSizeStr[32];
     char rxBufSizeStr[32];
+    char QoSStr[32];
     char maxStreamStr[32];
     char compressStr[32];
+    char delayStr[32];
 
     if (VerboseOpt)
 	printf("spool_file(%s, %d, %s, %d)\n",
@@ -520,17 +530,23 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 
     sprintf(txBufSizeStr, "-nop");
     sprintf(rxBufSizeStr, "-nop");
+    sprintf(QoSStr, "-nop");
     sprintf(maxStreamStr, "-nop");
     sprintf(compressStr, "-nop");
+    sprintf(delayStr, "-nop");
 
     if (li->li_TransmitBuf > 0 && li->li_TransmitBuf < 16 * 1024 * 1024)
 	sprintf(txBufSizeStr, "-T%d", li->li_TransmitBuf);
     if (li->li_ReceiveBuf > 0 && li->li_ReceiveBuf < 16 * 1024 * 1024)
 	sprintf(rxBufSizeStr, "-R%d", li->li_ReceiveBuf);
+    if (li->li_QoS > 0 && li->li_QoS <= 0xff )
+	sprintf(QoSStr, "-Q%d", li->li_QoS);
     if (li->li_MaxStream > 0)
 	sprintf(maxStreamStr, "-M%d", li->li_MaxStream);
     if (li->li_Compress > 0)
 	sprintf(compressStr, "-Z%d", li->li_Compress);
+    if (li->li_DelayFeed > 0)
+	sprintf(delayStr, "-W%d", li->li_DelayFeed);
 
     /*
      * Outbound article logging.
@@ -595,10 +611,9 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 	{
 	    int tries = 100;
 	    int32 dt;
+	    struct stat st;
 
 	    while (tries > 0) {
-		struct stat st;
-
 		sprintf(newFile, "%s.S%05d", spoolFile, endSeq);
 		if (stat(newFile, &st) < 0)
 		    break;
@@ -640,7 +655,8 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 		    }
 		}
 
-		if (rename(spoolFile, newFile) == 0) {
+		if (stat(spoolFile, &st) == 0 && st.st_size > 0 &&
+					rename(spoolFile, newFile) == 0) {
 		    ++endSeq;
 		    if (!Quiet)
 			printf("dspoolout: add %s\n", newFile);
@@ -650,6 +666,12 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 		    if (endSeq == begSeq)
 			AddNode(spoolFile);
 		}
+		if (flags & SF_REALTIME) {
+		    int fd = xopen(O_RDWR|O_CREAT, 0644, spoolFile);
+		    if (fd != -1) {
+			close(fd);
+		    }
+		}
 		newTime = time(NULL);
 	    } else {
 		AddNode(spoolFile);
@@ -696,7 +718,7 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 	if (flags & SF_NOBATCH)
 	    tries = 0;
 
-	for (look = 0; look < 10 && tries &&
+	for (look = 0; look < 32 && tries &&
 			begSeq + look < endSeq - li->li_QueueSkip; ++look) {
 	    int use = begSeq + look;
 	    int fd;
@@ -732,7 +754,7 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 		    sprintf(templateFile, "%s.S%%05d", spoolFile);
 		    sprintf(seqName, "%d", use);
 		    if (VerboseOpt)
-			    printf("%s %s%s %s%s %s%s %s%s %s%s %s %s %s %s %s %s %s %s %s %s %s %s\n",
+			    printf("%s %s%s %s%s %s%s %s%s %s%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
 				"dnewslink",
 				"-h", li->li_HostName,
 				"-b", templateFile,
@@ -752,13 +774,23 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 				outboundIpStr,
 				txBufSizeStr,
 				rxBufSizeStr,
+				QoSStr,
+				delayStr,
 				maxStreamStr
 			    );
 		    if (ForReal && fork() == 0) {
 			char binPath[128];
 
-			if (li->li_StartDelay > 0)
-			    sleep(li->li_StartDelay);
+			if (li->li_StartDelay > 0) {
+			    if (xflock(fd, XLOCK_EX|XLOCK_NB) == 0) {
+				sleep(li->li_StartDelay);
+				xflock(fd, XLOCK_UN);
+			    } else {
+				if (DebugOpt)
+				    printf(" qfile already locked for delay\n");
+				exit(0);
+			    }
+			}
 
 			if (li->li_Priority > 0)
 			    nice(li->li_Priority);
@@ -784,6 +816,8 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 				outboundIpStr,
 				txBufSizeStr,
 				rxBufSizeStr,
+				QoSStr,
+				delayStr,
 				maxStreamStr,
 				NULL
 			    );
@@ -817,7 +851,7 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 			printf("dspoolout: run realtime %s\n", spoolFile);
 
 		    if (VerboseOpt)
-			printf("%s %s%s %s%s %s%s %s%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
+			printf("%s %s%s %s%s %s%s %s%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
 				"dnewslink",
 				"-h", li->li_HostName,
 				"-b", spoolFile,
@@ -830,7 +864,7 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 				((flags & SF_HEADFEED) ?
 				    ((flags & SF_PRESBYTES) ? "-HB" : "-H") :
 				    "-nop"),
-				((flags & SF_NOCHECK) ? "-I" : "-nop"),
+				(((flags & SF_NOCHECK)||(li->li_Check==2)) ? "-I" : "-nop"),
 				((flags & SF_GENLINES) ? "-L" : "-nop"),
 				((flags & SF_ARTSTAT) ? "-x" : "-nop"),
 				compressStr,
@@ -838,6 +872,7 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 				outboundIpStr,
 				txBufSizeStr,
 				rxBufSizeStr,
+				delayStr,
 				maxStreamStr
 			    );
 		    if (ForReal && fork() == 0) {
@@ -846,6 +881,9 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 			if (li->li_StartDelay > 0)
 			    sleep(li->li_StartDelay);
 
+			if (li->li_Priority > 0)
+			    nice(li->li_Priority);
+
 			snprintf(binPath, sizeof(binPath), "%s/dbin/dnewslink", NewsHome);
 
 			    /*
@@ -865,7 +903,7 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 				((flags & SF_HEADFEED) ?
 				    ((flags & SF_PRESBYTES) ? "-HB" : "-H") :
 				    "-nop"),
-				((flags & SF_NOCHECK) ? "-I" : "-nop"),
+				(((flags & SF_NOCHECK)||(li->li_Check==2)) ? "-I" : "-nop"),
 				((flags & SF_GENLINES) ? "-L" : "-nop"),
 				((flags & SF_ARTSTAT) ? "-x" : "-nop"),
 				compressStr,
@@ -873,6 +911,7 @@ spool_file(char *spoolFile, int pass, NewslinkInfo *li, int flags)
 				outboundIpStr,
 				txBufSizeStr,
 				rxBufSizeStr,
+				delayStr,
 				maxStreamStr,
 				NULL
 			    );
