@@ -95,6 +95,8 @@ Usage(void)
     fprintf(stderr, "  -NB           = sync begin numbers\n");
     fprintf(stderr, "  -NE           = sync end numbers\n");
     fprintf(stderr, "  -NX           = sync end numbers but only upwards\n");
+    fprintf(stderr, "  -u            = Log into remotehost using this password\n");
+    fprintf(stderr, "  -U            = Log into remotehost using this username\n");
     fprintf(stderr, "  -XE           = sync NX field - feeder sync\n");
     fprintf(stderr, "  -XX           = sync NX but only upwards\n");
     fprintf(stderr, "  -n#           = if we sync end number then increase local value by remote+#\n");
@@ -112,6 +114,8 @@ main(int ac, char **av)
     char *wild = NULL;
     char *dbfile = NULL;
     char *grpfile = NULL;
+    char *username = NULL;
+    char *password = NULL;
     int port = 119;
 
     LoadDiabloConfig(ac, av);
@@ -138,9 +142,9 @@ main(int ac, char **av)
 	    if ((p = strrchr(ptr, ':')) != NULL) {
 		*p++ = 0;
 		BindPort = p;
-		BindHost = ptr;
+		BindHost = strdup(SanitiseAddr(ptr));
 	    } else {  
-		BindHost = ptr;
+		BindHost = strdup(SanitiseAddr(ptr));
 	    }
 	    break;
 	case 'C':
@@ -208,6 +212,12 @@ main(int ac, char **av)
 	    ptr = (*ptr) ? ptr : av[++i];
 	    port = strtol(ptr, NULL, 0);
 	    break;
+	case 'U':
+	    username = (*ptr) ? ptr : av[++i];
+	    break;
+	case 'u':
+	    password = (*ptr) ? ptr : av[++i];
+	    break;
 	case 'w':
 	    wild = (*ptr) ? ptr : av[++i];
 	    break;
@@ -337,7 +347,7 @@ main(int ac, char **av)
      * Connect to remote host
      */
 
-    if (host)
+    if (host != NULL)
     {
 	int fd;
 
@@ -373,7 +383,7 @@ main(int ac, char **av)
      * startup
      */
 
-    if (host)
+    if (host != NULL)
     {
 	char *line;
 
@@ -387,6 +397,40 @@ main(int ac, char **av)
 	}
     }
 
+    /*
+     * login
+     */
+
+    if (host != NULL && username != NULL) {
+	char *line;
+	char cmd[4096];
+
+	sprintf(cmd, "authinfo user %s",username);
+	switch(CommandResponse(cmd, &line)) {
+	    case 381:
+		printf("authinfo accepted: %s\n", line);
+		break;
+	    default:
+		printf("authinfo denied: %s\n", line);
+		exit(1);
+	}
+    }
+
+    if (host != NULL && password != NULL) {
+	char *line;
+	char cmd[4096];
+
+	sprintf(cmd, "authinfo pass %s",password);
+	switch(CommandResponse(cmd, &line)) {
+	    case 281:
+		printf("authinfo accepted: %s\n", line);
+		break;
+	    default:
+		printf("authinfo denied: %s\n", line);
+		exit(1);
+	}
+    }
+
     /*
      * mode reader
      */
