@@ -24,6 +24,14 @@ char *FindHeader(const char *name, char *def);
 
 Header *HBase = NULL;
 Header **PHe = &HBase;
+char *argv[10] = { PGP_PATH, PGP_ARG0, "-f", NULL };
+
+void
+Usage(void)
+{
+    printf("Usage: pgpverify [-d] [pgp_path] [pgp_args]\n");
+    exit(1);
+}
 
 int
 main(int ac, char **av)
@@ -38,6 +46,43 @@ main(int ac, char **av)
     char *signature = NULL;
     pid_t pid = 0;
     int fds[3] = { -1, -1, -1 };
+    int i;
+    int argcount = 0;
+    int argend = 0;
+
+    for (i = 1; i < ac; ++i) {
+	char *ptr = av[i];
+
+	if (*ptr != '-' || argend) {
+	    argend = 1;
+	    if (argcount > 8)
+		break;
+	    argv[argcount++] = ptr;
+	    if (argcount == 0) {
+		if (strrchr(ptr, '/') == NULL)
+		    argv[argcount++] = ptr;
+		else
+		    argv[argcount++] = strrchr(ptr, '/') + 1;
+	    }
+	    argv[argcount] = NULL;
+	    continue;
+	}
+	ptr += 2;
+	switch(ptr[-1]) {
+	case '-':
+	    argend = 1;
+	    break;
+	case 'd':
+	    if (*ptr)
+		DebugOpt = strtol(ptr, NULL, 0);
+	    else
+		DebugOpt = 1;
+	    break;
+	default:
+	    fprintf(stderr, "illegal option: %s\n", ptr - 2);
+	    Usage();
+	}
+    }
 
     /*
      * Scan headers in message
@@ -111,7 +156,6 @@ main(int ac, char **av)
      */
 
     {
-	char *argv[] = { PGP_PATH, PGP_ARG0, "-f", NULL };
 	char nhtmp[PATH_MAX];
 	char *env[2];
 
@@ -150,8 +194,11 @@ main(int ac, char **av)
 
 	fprintf(fo, "\n");
 
-	while (fgets(buf, sizeof(buf), stdin) != NULL)
+	while (fgets(buf, sizeof(buf), stdin) != NULL) {
+	    if (buf[0] == '-')
+		fputs("- ", fo);
 	    fputs(buf, fo);
+	}
 
 	/*
 	 * blank line, then PGP signature
@@ -187,8 +234,12 @@ main(int ac, char **av)
 		r= 3;
 		break;
 	    }
-	    if ((good=strstr(buf, "Good signature from user"))) {
-		char *tok = strtok(good + 24, " \"\n");
+	    if ((good=strstr(buf, "Good signature from"))) {
+		char *tok;
+		good = strchr(good + 19, '"');
+		if (good == NULL)
+		    break;
+		tok = strtok(good + 1, " \"\n");
 		if (tok[0]) {
 		    r = 0;
 		    printf("%s\n", tok);
